// phat_example.C
//
// $Id: phat_example.C,v 1.3 1999/02/18 04:10:07 decowski Exp $
//
// This macro shows how to access the four different data containers stored in 
// a ROOT format file.  The macro extracts simple information from the data  
// and returns that information in a graphical format. 

// access and use Phobos data.  The user should work through this example
// while referring to a C++ book and Phat and ROOT documentation.

// Obviously, lines beginning with "//" are comments and are not
// executed.
// This macro was written by Mike Reuter at UIC in mid-June 1998.
// Modifications were made by Steve Manly in Sept-Oct 1998. 

// Must enclose macro in "{}".           
{

  // This line guarantees that we start with a clean slate and should
  // be the first statement in most macros.
  gROOT->Reset();

  // This is the command that opens the ROOT format file where the data
  // is stored. Directory should be modified to users location where the 
  // file is stored.  If you happen to have a different file, change the
  // filename too.  TPhDST is the Phobos DST interface class.
  TPhDST dst("$PHATHOME/data/hijing_2cen.adat.root");


  // "printf" is a C command causing the words "dst->ls()" to be
  // written to the root window. "/n" forces a line break before
  // output from TPhDST is written to the root window. 
  // WARNING:  This example uses the C command "printf".  C++ gurus
  // argue that you should use "cout" instead because it is more robust.
  printf("dst->ls();\n");

  // The ls command displays the objects stored inside the file or class.
  // TPhDST includes a ROOT class called TFILE that defines the ls
  // function.  "ls" is defined/used elsewhere in ROOT too.  The
  // useage thruout ROOT is to list objects stored in the file.  Here, the
  // file is a member of the TPhDST class called "dst".  
  dst->ls();

  // force a line break in the output 
  printf("\n");

  // The next two commands setup a pointer to the event stored in this file.
  // This particular file only contains one event.  GetNextEvent is a ROOT
  // function that goes to the next event so that the new pointer, "event",
  // points to it.
  TPhEvent *event;
  event = dst->GetNextEvent();

  // Print "event->ls("C") to ROOT window, then line break.
  printf("event->ls("C");\n");

  // The ls command with the option C displays the objects stored in the
  // TPhEvent class.  See the sub-section Data Storage in the PhAT 
  // Documentation for more information on the data containers.
  event->ls("C");

  // Force line break.
  printf("\n");

  // The next ten commands are ROOT commands to setup a canvas and create 
  // four pads within that canvas.  These pads will be used to display the
  // information retrieved from each data container. 
  // To find out more about TCanvas,
  // see "classes and members reference guide" in the ROOT web documentation
  // and click on "TCanvas", then click on "class description".  You'll 
  // want to do the same for "TPad".
  // Instead of "class description" you can also pick "source code".  There 
  // you will find useful documentation as well.
  // Below a canvas is created with the top left corner position chosen
  // to be at 50, 50 and the width and height given by the last two numbers,
  // respectively (in pixels).
  // The fill color behind the pads on the canvas is set by the second
  // command.  You can find what color corresponds to what number arguement
  // by going to the main ROOT web page (root.cern.ch) and picking
  // "search the web site".  Type "setfillcolor" and request a search.
  // Numerous places where SetFillColor is used in ROOT will be returned.
  // For example, pick "a simple fitting example". This will take you to 
  // a page listing for a simple routine.  If you look down the page you
  // will find SetFillColor used.  It is in hypertext, so click on it. 
  // That will take you to a class called TAttFill.  If you page down this
  // file or click on "class description", you will find a color-number
  // mapping for SetFillColor.
  // OR ... do it the easy way!  Go to the "View" menu item in the canvas and 
  // pick "Colors".  It will return a canvas filled with color numbers
  // and corresponding colors.
  TCanvas *c1 = new TCanvas("c1", "PhAT EXample", 50, 50, 800, 800);
  c1->SetFillColor(4);

  // For fun you can comment out the above two lines and uncomment the
  // two below to see what happens.  Isn't vi fun??
  // TCanvas *c1 = new TCanvas("c1", "PhAT Example", 50, 50, 1000, 80);
  // c1->SetFillColor(6);

  // The next four commands define the pads on the canvas.
  // The arguements are (see TPad documentation in ROOT)
  // "name, title, xlow, ylow, xup, yup, color"
  // The last arguement is color.  You can go back to the color-number
  // map introduced above to see that the pads have the correct colors.
  // The 3rd and 5th arguements give the range of the pad (in percent
  // of full range) in x.  The 4th and 6th arguements give the same in y.
  // Once you think this makes sense, uncomment the two lines below
  // for "pad5" and see if it does what you think it should.  
  // The 5th pad you will see should be whitish (color=19).  There will
  // be nothing in it because it is not used in the rest of the
  // program.
  TPad *pad1 = new TPad("pad1", "PMCRawHitFECn0s", 0.01, 0.5, 0.49,0.99, 41);
  TPad *pad2 = new TPad("pad2", "PMCHitArraysFromHits", 
			0.5, 0.5, 0.99, 0.99, 30);
  TPad *pad3 = new TPad("pad3", "PMCHits", 0.01, 0.01, 0.49, 0.49, 7);
  TPad *pad4 = new TPad("pad4", "PMCTracks", 0.5, 0.01, 0.99, 0.49, 3);
  // TPad *pad5 = new TPad("pad5", "MCTracks", 0.2, 0.35, 0.4, 0.75, 19);

  // Draw the pads.
  pad1->Draw();
  pad2->Draw();
  pad3->Draw();
  pad4->Draw();
  // pad5->Draw();

  //  The following code assumes that you have read documentation and
  // know the names of the data containers.
  // Go to Phobos computing web page -> phat link -> phat documentation.
  // Some of the language is rather opaque to C++ beginners, but
  // hang in there, it will come.

// MCRawHitBuffers

  // This command makes the first pad available for drawing.  
  // You are setting the path for the commands to refer to pad 1.
  // Alternate form is c1->cd(1)
  pad1->cd();

  // A shorter alternative to some of the above code is:  
  // c1->Divide(2,2);  (make four pads in canvas:2x2)
  // c1->cd(1);  (work on top left pad, go clockwise with increasing number)
  // c1->(); or c1->cd(0);  (go to the original canvas if desired)
  // (now go to first pad and change color) 
  // c1->cd(1);
  // gPad->SetFillColor(3);  
  // (note that there is no need to draw the pads before filling them here)


  // In this data file, the MCRawHitBuffers object does not exist.
  // Let's create a warning sign because of this.
  // Arguements are (start x range, start y range, end x range,
  // end y range, text for label, option).
  // You can find this on the ROOT web pages: search for tpavelabel.
  // Ranges are in percent of full range (referring to pad1 size now).
  // The option sets different types of shading.
  // The new label is an object called "warning" that belongs to the
  // tpavelabel class.
  TPaveLabel *warning = new TPaveLabel(0.01, 0.4, 0.99, 0.6 ,
			      "PMCRawHitFECn0s not available", "BR");
  // Color the label.
  warning->SetFillColor(33);
  warning->Draw();

  // This command does just what it says.  It draws the previous set of 
  // commands to the selected pad.
  c1->Update();

// MCHitArrays

  // This command makes the second pad available for drawing. 
  pad2->cd();

  // The next two commands make a string with the detector name we want to 
  // use.  
  // Fortran folks might notice the input name is only 4 characters
  // but the declaration is for 5.  Thank C++ for that and refer to
  // your C++ book for more info.
  Char_t sensorname[5];
  sprintf(sensorname, "PE01");

  // The next four commands show the step down procedure necessary to obtain
  // information from named objects.  The end result is the pointer to the 
  // sensor that we wish to extract information about.  
  TPhHashContainer *HitContainer;
  HitContainer = (TPhHashContainer *)event->GetObject("PMCHitArraysFromHits");
  TPhHitArray *hitarray;
  hitarray = (TPhHitArray *)HitContainer->GetObject(sensorname);

  if(!hitarray) {
    // Sensor is not found, pointer comes back zero or negative, 
    // therefore "not hitarray" is true and execute this code.
    TPaveLabel *notfound = new TPaveLabel(0.01, 0.4, 0.99, 0.6 ,
                           "Sensor not found", "BR");
    notfound->SetFillColor(33);
    notfound->Draw();
    c1->Update();
  } 
  else {
    // Sensor is found, pointer comes back positive, "not hitarray"
    // is false, execute this code instead of above.
    Char_t titlestr[20];
    Char_t rowstr[20];
    Char_t colstr[20];
    Char_t hitstr[20];


    // The next two lines can be used to dump some information
    // about the sensor we have chosen.
    TPhDetector *det = TPhDetector::ReadLatestGeometry();
    det->GetSensor("PE01")->Print();

  // These statements take something (name, number of rows, number
  // of columns, number of hits) from the data object pointed to
  // by hitarray and write a message for each. 
  // Recall from C that "%s" is a conversion character for a character
  // string and "%d" is a conversion character for an integer.
    sprintf(titlestr, "Sensor %s", hitarray->GetSensorName());
    sprintf(rowstr, "Rows   : %d", hitarray->GetRows());
    sprintf(colstr, "Columns:%d", hitarray->GetColumns());
    sprintf(hitstr, "# Hits : %d", hitarray->GetNHits());

  // The next 12 commands creates labels and draws them on the second pad.
  // Again, I have varied the color for fun.  I have also varied the size
  // and shading options.  Looking at the results I think it is obvious
  // that T(B) stands for Top(Bottom) and R(L) stands for Right(Left).
    TPaveLabel *namelab = new TPaveLabel(0.1, 0.76, 0.9, 0.99 , titlestr, "BR");
    namelab->SetFillColor(3);
    namelab->Draw();
    TPaveLabel *rowlab = new TPaveLabel(0.2, 0.55, 0.7, 0.7 , rowstr, "TR");
    rowlab->SetFillColor(4);
    rowlab->Draw();
    TPaveLabel *collab = new TPaveLabel(0.1, 0.3, 0.9, 0.45 , colstr, "BL");
    collab->SetFillColor(5);
    collab->Draw();
    TPaveLabel *hitlab = new TPaveLabel(0.2, 0.01, 0.7, 0.25 , hitstr, "TL");
    hitlab->SetFillColor(6);
    hitlab->Draw();
    c1->Update();  
  }

// MCHits

  pad3->cd();

  // Declare histogram for output. TH1F is a class for 1-D histograms
  // where a float is used to increment the sum of weights for each channel.
  // Arguements of TH1F are (reference to histo, title of histo,
  // number of bins, x value of low edge, high edge x value).
  // THdp is a histogram class, where d=dimension (1,2,3) and p
  // is precision (C=char, S=short, F=Float, D=Double).  See the ROOT
  // overview if you don't believe me.
  TH1F *mchithisto = new TH1F("mchithisto", "Energy of All Hits", 100, 0, 0.005);
  
  // Objects without names must be handled differently from those with names.
  // We still must obtain a pointer to the object TPhMCHit 
  TPhObjectContainer *MCHitContainer;
  TPhMCHit *mchit;

  // To loop over the list of objects, we need to create an iterator.  The 
  // following commands perform this procedure. 
  MCHitContainer = (TPhObjectContainer *)event->GetObject("PMCHits");
  TIter MCHitIter(MCHitContainer->GetIterator());
 
  // This is how to loop over the list.  The condition keeps the while loop
  // running until after the last entry is read.
  while(mchit = (TPhMCHit *)MCHitIter()) {

    // Fill the declared histogram.
    mchithisto->Fill(mchit->GetDe());
  } 
  mchithisto->SetFillColor(42);

  // One can do patterned fills too!  See the same file where you
  // found the color map (class TAttFill).  Uncomment the following line
  // of code and see what happens.
//  mchithisto->SetFillStyle(3003);

  // Draw the histogram. 
  mchithisto->Draw();
  c1->Update();

// MCTracks
  
  pad4->cd();

  // Declare histogram for output.
  TH1D *mctrackhisto = new TH1D("mctrackhisto", "Momentum of individual Tracks", 100, 0, 2.5);
  
  // The same rules that applied for MCHits apply here as well.
  TPhObjectContainer *MCTrackContainer;
  TPhMCTrack *mctrack;

  MCTrackContainer = (TPhObjectContainer *)event->GetObject("PMCTracks");
  TIter MCTrackIter(MCTrackContainer->GetIterator());

  while(mctrack = (TPhMCTrack *)MCTrackIter()) {
    mctrackhisto->Fill(mctrack->GetP());
  }

  mctrackhisto->SetFillColor(12);
  mctrackhisto->Draw();
  c1->Update();

}




