#include <math.h>
#include <stdlib.h>
#include "TPhBT98DST.h"

#include "TPhObjectContainer.h"

ClassImp(TPhBT98DST)

////////////////////////////////////////////////////////////////
//
// TPhBT98DST is a PHOBOS class that reads DST from Test Beam '98
// run and creates TPhEvent with properly linked TPhRawHitFECn0
// objects in TB98RawHitFECn0s container.
// TPhEvent provides member function to open and close one
// file and to loop over all events in the file using the 
// GetNextEvent() member function.
//
// $Id: TPhBT98DST.cxx,v 1.4 1999/02/24 23:36:00 olszewsk Exp $
//
////////////////////////////////////////////////////////////////

#define DEBUG

TPhBT98DST::TPhBT98DST()
{
// Constructor. Set fNumEvent and fFileName to NULL

    fNumEvent = 0; 
    fRunNo = 0;
    fBytesLost = 0;
    fFileName[0] = '\0'; 
    fDInput = NULL;
    fVerbose = 0;
    fInitialized = kFALSE;
}

TPhBT98DST::TPhBT98DST(Char_t *FileName, Int_t skip, Int_t runno)
{
// Constructor. Set fNumEvent and fRootFile to NULL
// and open the file *FileName

    fNumEvent  = 0;
    fRunNo = runno;
    fBytesLost = 0; 
    strncpy(fFileName,FileName,100);
    fDInput = NULL;  
    fVerbose = 0;
    fInitialized = kFALSE;

    // create the first buffer immediately
    if(Open(fFileName)) 
      if(SkipEvent(skip)) fInitialized = kTRUE;
}

TPhBT98DST::~TPhBT98DST()
{
// Destructor. Close the file properly if it is still open
   Close();
}

Bool_t TPhBT98DST::Open(Char_t *FileName)
{
// Open the DST file *FileName. Return kFALSE if the 
// file can not be found.

  Reset();

  if(fVerbose)
    printf("Open DST file %s\n",FileName);
  fDInput = fopen(FileName, "rb");
  if( !fDInput ) {
     if( fVerbose )
	Warning("Open", "Couldn't open file %s\n");
     return (fInitialized = kFALSE);
  }
  return (fInitialized = kTRUE);
}

Bool_t TPhBT98DST::Close()
{
// Close the current file. Return kFALSE
// if fDInput is !=0. 

  if(fDInput) {
    fclose(fDInput);
  } else {
    if(fVerbose)
      Warning("Close", "input file not opened yet, can't close.\n");
    return kFALSE;
  }
  Reset();
  return kTRUE;
}

void TPhBT98DST::ClearBuffer()
{
    // clear buffer(s) 
    memset(&fBuffer,0,kTB98BufferSize);
}

void TPhBT98DST::Reset()
{
  // clear buffer(s) 
  // may be used to start reading from another file
    ClearBuffer();

    fNumEvent  = 0;
    fBytesLost = 0; 
    fDInput = NULL; 
    fInitialized = kFALSE; 
}

Bool_t TPhBT98DST::SkipEvent(Int_t skip)
{
  Int_t i;

  while(skip--) {
    fNumEvent++;
    for(i=0;i<8;i++) {
      if(!ReadRepeaterBlock()) {
	Warning("SkipEvent","error reading data file\n");
	return kFALSE;
      }
    }
  }
  return kTRUE;
}

Bool_t TPhBT98DST::ReadRepeaterBlock()
{
   ssize_t siz;

   siz=fread(&fBuffer,1,kTB98BufferSize,fDInput);
   if ((siz < kTB98BufferSize) && !feof(fDInput)) {
      Error("ReadRepeaterBlock", "error reading all requested bytes from file %s, got %d of %d",
            fFileName, siz, kTB98BufferSize);
      return kFALSE;
   }
   fBytesRead  += siz;

   return kTRUE;
}

TPhEvent *TPhBT98DST::GetNextEvent(Int_t skip)
{
// Read 8 repeater data block in sequence (checking for the repeater
// number), create and fill two TPhRawHitFECn0 clone objects for each repeater, 
// remove one of the strings in each of them, so that only one string is
// left in each of the FEC objects.
// Then put the objects into the TPhEvent's "TB98RawHitFECn0" 
// container, and return a pointer to the TPhEvent object. 
// If the skip argument is provided, skip over the corresponding
// number event blocks before reading an event.
//
// Note that the memory allocated inside this routine has 
// to be freed by the program calling GetNextEvent().

  Int_t i,j,eventnumber;
  Int_t totalwords=0;
  Int_t maxsrch=0,eodpos=0,cstrnum=0;

  // prepare for conversion and read the raw byte stream from TB98 data file
  Int_t string,repeater;
  Char_t eventname[30];
  TPhEvent *event = NULL;
  TPhObjectContainer *rawcont = NULL;
  TPhRawHitFECn0TB98 *rawhit = NULL, *rawhitclone = NULL;

  if(!SkipEvent(skip)) return NULL;

  string=0; repeater=0;
  ClearBuffer();
  for(i=0;i<8;i++) {
    if( ReadRepeaterBlock() ) {
      if(feof(fDInput)) return NULL;
      repeater = i;

      // fill FEC with 1 repeater info (2 strings)
      // - from 1 FEC with wrong string positions make 
      //   2 FECs each with one string at the correct position
      rawhit = new TPhRawHitFECn0TB98(fBuffer, kTB98BufferSize);
      // make a clone object
      rawhitclone = (TPhRawHitFECn0TB98 *) rawhit->Clone();    
      
      // set the proper FEC number for this string
      rawhit->SetFECNo(kFECNo[string]);
      // remove 2nd string from one FEC
      if(!rawhit->RemoveString(kStringRem[string])) return NULL;
      // move 1st to the proper position
      if(!rawhit->MoveString(kStringIn[string],kStringOut[string])) return NULL;
      string++; string = string%16;
      
      // set the proper FEC number for this string
      rawhitclone->SetFECNo(kFECNo[string]);
      // remove 1st string from 2nd FEC
      if(!rawhitclone->RemoveString(kStringRem[string])) return NULL;
      // move 2nd to the proper position
      if(!rawhitclone->MoveString(kStringIn[string],kStringOut[string])) return NULL;
      string++; string = string%16;
      
      // create new TPhEvent if repeater number = 0
      if(repeater==0) {
	eventnumber = rawhit->GetEventNumber();
	sprintf(eventname,"Event_%d",eventnumber);
	event = new TPhEvent(fRunNo,fNumEvent);
	event->SetName(eventname);
	event->SetTitle("Test Beam 98");
	
	fNumEvent++;
        if(fVerbose) printf("%d events processed\n",fNumEvent);
	
	// create and add a container with raw hits to the event object
	rawcont = new TPhObjectContainer("RawHitFECn0s", "Test Beam 98");
	event->AddObject(rawcont);
      }
      rawcont->Add(rawhit);
      rawcont->Add(rawhitclone);
      // ----------------------------------
      // return pointer to the filled event
      if(repeater==7) return event;
      // ----------------------------------
    } else {
      Error("GetNextEvent","error reading repeater %d block in Event no %d\n",i,fNumEvent);
      return NULL;
    }
  }
  return NULL;
}







