#include "TPhHit.h"
#include <math.h>
#include <iostream.h>
#include <stdlib.h>

#include "TPhDetector.h"
#include "TPhSubDetector.h"
#include "TPhSiSensor.h"

ClassImp(TPhHit)

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhHit                                                         //
//                                                                   //
//    TPhHit is a PHOBOS data class for storing information for      //
//    one detector hit. This class has been designed as a most basic //
//    class. Extension for MC hits and hits used in reconstruction   //
//    for spectrometer, vertex or multiplicity detectors require     //
//    creation of classes derived from TPhHit.                       //
//                                                                   //
//                                                                   //
///////////////////////////////////////////////////////////////////////

TPhHit::TPhHit()
{
  fSensorLabel = 0;
  fMarker = 0;
  fLayer = 0;
  fLabel = -1;  // negative value for not initialized hit
  fX = 0.0;
  fY = 0.0;
  fZ = 0.0;
  fPadX = -1;   // negative value for unknown pad
  fPadY = -1;
  fDe = 0.0;
  fField = 0.0;
}

TPhHit::~TPhHit()
{
  if(fMarker) delete fMarker;
}

Float_t TPhHit::CalcDist45()
{
  // Calculate distance from the hit to the 45 deg. line
  // Dist45 is equal 2*distance**2, but is negative for Z > abs(X)

  Float_t x, z, dist45;

  x = GetX();
  z = GetZ();

  if(fabs(x) == fabs(z))  return 0.0;
  if(x > 0.0)
  {
     dist45 = z-x;
     dist45 *= dist45;
     if(z > x)  dist45 = -dist45;
  }
  else
  {
     dist45 = z+x;
     dist45 *= dist45;
     if(z > -x)  dist45 = -dist45;
  }
  return dist45;
}

void TPhHit::ls(Option_t *option)
{
  // List the hit object content
  TObject::IncreaseDirLevel();
  IndentLevel();
  cout << "Class:" << IsA()->GetName() << " is a hit in Sensor " 
       << GetSensorLabel() << " at " << GetX() << "," << GetY() 
       << "," << GetZ() << endl;
  TObject::DecreaseDirLevel();
}

void TPhHit::Print(Option_t *option)
{
  // List the hit object content

  printf(" - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -" \
	 " - - - - - - - \n");
  printf("%s - Hit labeled: %d in sensor no: %5d layer no: %5d at " \
	 "%10.5f %10.5f %10.5f\n PadX %5d  PadY %5d  De %7.3e  Dt %7.3e " \
	 "By %7.3e\n", IsA()->GetName(), fLabel, 
	 fSensorLabel, fLayer, fX, fY, fZ, fPadX, fPadY, fDe, fDt, fField);
}

TPhSensor* TPhHit::GetSensor()
{
  // Return the sensor this hit belongs to.
  // If the detector or sensor is unavailable return NULL.

  TPhDetector *Det;

  if((Det = TPhDetector::GetInstance()) != NULL)
    return Det->GetSensor(GetSensorLabel());
  else 
    return NULL;
}
	
void TPhHit::Draw(Option_t *Option)
{
  // Draw the hit in the current view 
  // 
  // If the hit is in a detector, then draw it only when the subdetector
  // is also on. 
  // If the hit does not belong to any detector, then draw it with a 
  // different color. 

  Color_t color = 5; // yellow

  TPhDetector *det = TPhDetector::GetInstance();
  TPhSubDetector *SubDet = det->GetSubDetectorOfSensor(GetSensorLabel());

  if(SubDet) {
    if(!SubDet->IsOn())
      return;
  }
  else 
    color = 2; // red

  fMarker = new TPolyMarker3D(1, 8);
  fMarker->SetPoint(0, fX, fY, fZ);
  fMarker->SetMarkerColor(color);
  fMarker->SetMarkerSize(0.5);
  fMarker->Draw();
}
