#include <iostream.h>
#include "TClass.h"
#include "TPhHit.h"
#include "TPhRecHit.h"
#include "TPhTrack.h"

ClassImp(TPhRecHit)

////////////////////////////////////////////////////////////////////////
//                                                                    //
//    TPhRecHit                                                       //
//                                                                    //
//    TPhRecHit is the PHOBOS hit class used to store "reconstructed" //
//    hits (merged signals in neighbour pads)                         // 
//                                                                    //
////////////////////////////////////////////////////////////////////////

TPhRecHit::TPhRecHit()
{
  // Default constructor

  fDist45  = 0.0;
  fNPads   = 0;
  fNTracks = 0;
  fLabel   = 0;
}

TPhRecHit::TPhRecHit(TPhHit *rawhit)
{
  // Constructor copying information from an existing hit

  fSensorLabel = rawhit->GetSensorLabel();
  fLayer = rawhit->GetLayer();
  fLabel = rawhit->GetLabel();
  fX = rawhit->GetX();
  fY = rawhit->GetY();
  fZ = rawhit->GetZ();
  fPadX = rawhit->GetPadX();
  fPadY = rawhit->GetPadY();
  fDe = rawhit->GetDe();
  fDt = rawhit->GetDt();

  fDist45 = 0.0;
  fNPads = 1;
  fNTracks = 0;
}


Float_t TPhRecHit::CalcDist45()
{
  // Calculate distance from the hit to the 45 deg. line
  // Dist45 is equal 2*distance**2, but is negative for Z > abs(X)

  Float_t x, z;

  if(fDist45 != 0.0)  return  fDist45;

  x = fX;
  z = fZ;

  if(fabs(x) == fabs(z))  return 0.0;

  if(x > 0.0)
  {
     fDist45 = z-x;
     fDist45 *= fDist45;
     if(z > x)  fDist45 = -fDist45;
  }
  else
  {
     fDist45 = z+x;
     fDist45 *= fDist45;
     if(z > -x)  fDist45 = -fDist45;
  }
  return fDist45;
}

Bool_t TPhRecHit::MergeHit(TPhRecHit *hit)
{
   // Merge an additional hit to the present hit
   // False returend if the two hits can not be merged

   Float_t w, w1, w2;
   Int_t addpads;

   if((fNPads + (addpads = hit->GetNPads())) > kNPadsMax)  return kFALSE;
   if((fPadX+fNPads) != hit->GetPadX())  return kFALSE;

   fDe += hit->GetDe();
   if(fDt > hit->GetDt())  fDt = hit->GetDt();

   w1 = fNPads;
   w2 = addpads;
   w = w1 + w2;
   fX = ((fX*w1 + hit->GetX()*w2)/w);
   fY = ((fY*w1 + hit->GetY()*w2)/w);
   fZ = ((fZ*w1 + hit->GetZ()*w2)/w);
   fField = ((fField*w1 + hit->GetField()*w2)/w);

   fNPads += addpads;
   fDist45 = 0.0;

   return kTRUE;
}

Bool_t TPhRecHit::AddHit(TPhRecHit *hit)
{
   // Add an additional hit to the present hit
   // False returend if the two hits have different PadX or PadY

   if((fPadX != hit->GetPadX()) || (fPadY != hit->GetPadY()))  return kFALSE;

   fDe += hit->GetDe();
   if(fDt > hit->GetDt())  fDt = hit->GetDt();

   return kTRUE;
}


void TPhRecHit::Print(Option_t *option)
{
  // List the hit object info
  // Use "number" as an option to indicate how many sublevels should be listed

  printf("-------------------------------------------------------------------------------\n");
  printf("RecHit: merged %d pads\n", fNPads);
  TPhHit::Print(option);
}

void TPhRecHit::ls(Option_t *option)
{
  // List Object. option is still unused.

  TObject::IncreaseDirLevel();
  IndentLevel();
  cout << "Class:" << IsA()->GetName() << " is a merged hit in Sensor " 
       << GetSensorLabel() << " at " << GetX() << "," << GetY() 
       << "," << GetZ() << endl;
  TObject::DecreaseDirLevel();
}
