#include <iostream.h>
#include <string.h>
#include "TPhFECDetector.h"
#include "TPhChannelCalTB98.h"
#include "TPhStringCalTB98.h"

ClassImp(TPhFECDetector)

////////////////////////////////////////////////////////////////////////////
//                                                                        //
// The FECDetector class stores a list of the FEC objects describing all  //
// the Phobos electronics channels                                        //
//                                                                        //
// Author: Andrzej Olszewski                                              //
// $Id: TPhFECDetector.cxx,v 1.2 1999/03/05 11:32:33 olszewsk Exp $
//                                                                        //
////////////////////////////////////////////////////////////////////////////

TPhFECDetector::TPhFECDetector() 
{
  // Default constructor.

  fFECList = NULL;
  fStringList = NULL;
  fVersion = 0;
  fVerbose = 0;
}

TPhFECDetector::TPhFECDetector(Char_t *name) : TNamed(name, name)
{
// Normal FECDetector constructor.

  fFECList = NULL;
  fStringList = NULL;
  fVersion = 0;
  fVerbose = 0;
}

TPhFECDetector::~TPhFECDetector()
{
  // Normal destructor

  if(fFECList) {
    fFECList->Delete();
    delete fFECList;
  }
  if(fStringList) {
    delete fStringList;
  }
}

Bool_t TPhFECDetector::IsThere(const Int_t fecno)
{
  // Check if the FEC object of a given number exists in the FECDetector
  // 
  // Return: kTRUE if sensor exists 
  //         kFALSE if it does not. 

  if(!fFECList) return kFALSE;
  if(fFECList->At((Int_t) fecno))
    return kTRUE;
  else
    return kFALSE;
}

void TPhFECDetector::Print() const
{
  // Print information about this FECDetector and all the FEC objects inside it. 

  TPhFEC *fec;

  if(fFECList) printf("FECDetector: %s\n", GetName());

  TIter NextFEC(fFECList);
  while((fec = (TPhFEC *) NextFEC())) fec->Print();
}
 
void TPhFECDetector::ls(Option_t *Option) const
{
  // Print information about the FEC objects

  IndentLevel();
  cout << "FECDetector "<< GetName() << endl;
  TObject::IncreaseDirLevel();
  fFECList->ls(Option);
  
  TObject::DecreaseDirLevel();
}

Bool_t TPhFECDetector::AddFEC(TPhFEC *fec) 
{
  // Put FEC at position i (with "i" number)
  Int_t fecno;

  if(!fFECList) fFECList = new TObjArray(kMaxFECs);

  fecno = fec->GetFECNo();
  if(fFECList->At(fecno)) return kFALSE;

  fFECList->AddAtAndExpand(fec,fecno);
  return kTRUE;
}

TPhFEC *TPhFECDetector::GetFEC(const Int_t i) const
{
  // Return the FEC at position i (with "i" number)

  if(fFECList) {
    if(fFECList->GetSize()<=i) return NULL;
    return (TPhFEC*) fFECList->At(i);
  }
  return NULL;
}

TPhString* TPhFECDetector::GetString(const Int_t fecno, const Int_t strno) const 
{
  //
  TPhFEC *fec;

  fec = GetFEC(fecno);
  if(fec) 
    return fec->GetString(strno);
  else
    return NULL;
}

Bool_t TPhFECDetector::ReadinStringCalibrationTB98File(Char_t *filename)
{
  // Read text file with calibration info
  // Build calibration objects, first the channel ones, 
  // then transform them into larger objects for strings 
  // and add the summary calibration objects to pointers at string objects
  Bool_t currstatus, status = kTRUE;

  if( !ReadCalibrationTB98File(filename) ) {
    if(fVerbose) 
      Error("ReadinStringCalibrationTB98File",
	    "Initialization of calibration parameters failed\n");
    return kFALSE;
  }

  Int_t numstrings,fecno,strno;
  TPhStringCalTB98  *strcal;
  TPhString *string;
  TPhFEC *fec;

  for(fecno=0; fecno<kMaxFECs; fecno++) {
    fec = GetFEC(fecno);
    if(!fec) 
      continue;
    numstrings = fec->GetNumStrings();
    for(strno=0; strno<numstrings; strno++) {
      string = fec->GetString(strno);
      if(!string) 
	continue;
      strcal = new TPhStringCalTB98();
      currstatus = string->MakeStringCalObject(strcal);
      if(!currstatus) 
	delete strcal;
      status = status && currstatus;
    }
  }
  // commented out for now, to allow use of calibration objects
  // created at the channel and at the string level
  //  DeleteCalChannelArrays();
  return status;
}

Bool_t TPhFECDetector::ReadCalibrationTB98File(Char_t *filename)
{
  // Read text file with calibration info
  // Build calibration objects and add them to lists in string objects

  //  transformations defined in TPhBT98DST.h
  //  const Int_t kFECNo[16]      = {8,0,8,0,8,0,8,0,9,1,9,1,9,1,9,1}; 
  //  const Int_t kStringRem[16]  = {5,0,4,1,5,0,4,1,5,0,4,1,5,0,4,1};
  //  const Int_t kStringIn[16]   = {0,5,1,4,0,5,1,4,0,5,1,4,0,5,1,4};
  //  const Int_t kStringOut[16]  = {0,1,1,0,2,3,3,2,0,1,1,0,2,3,3,2}; 

  // FEC and String numbers for any combination of repeator/string number 
  // that could come from TB98 data (-1 means - should not occure)
  const Int_t kFECNo[8][6]    = {{ 8,-1,-1,-1,-1, 0},{-1, 8,-1,-1, 0,-1}, //repeator 0-1
				 { 8,-1,-1,-1,-1, 0},{-1, 8,-1,-1, 0,-1}, //repeator 2-3
                                 { 9,-1,-1,-1,-1, 1},{-1, 9,-1,-1, 1,-1}, //repeator 4-5
				 { 9,-1,-1,-1,-1, 1},{-1, 9,-1,-1, 1,-1}};//repeator 6-7
  const Int_t kStrNo[8][6]    = {{ 0,-1,-1,-1,-1, 1},{-1, 1,-1,-1, 0,-1}, //repeator 0-1
				 { 2,-1,-1,-1,-1, 3},{-1, 3,-1,-1, 2,-1}, //repeator 2-3
                                 { 0,-1,-1,-1,-1, 1},{-1, 1,-1,-1, 0,-1}, //repeator 4-5
				 { 2,-1,-1,-1,-1, 3},{-1, 3,-1,-1, 2,-1}};//repeator 6-7

  const Char_t kNoPars = 4;
  Char_t line[300], formula[100];
  Int_t i, parno, parpointsno;
  Float_t linearity;
  Int_t repno, chno, fecno, strno, strphno;
  Float_t par[kNoPars];
  FILE *calfile;
  TPhChannelCalTB98 *chancal;
  TPhString *string;
  
  calfile = fopen(filename, "r");

  if(calfile == NULL) {
    if(fVerbose) 
      Error("ReadCalibrationTB98File",
	    "Can not open file with calibration parameters.\n");
    return kFALSE;
  }
  // Read the ASCII File.
  // For now, discard the text of the formula used for fitting
  if (fscanf(calfile, "%99[ -~]", formula) == EOF) return kFALSE;

  // Read no of formula params, calibration points, board linearity factor
  // 990318 HP change format for testbeam calibration base
  if (fscanf(calfile, "%d %d %f ", &parno, &parpointsno, &linearity) < 3) {
    //  if (fscanf(calfile, "%*[^0-9] %d %*[^0-9] %d %*[^0-9] %f ", 
    //     &parno, &parpointsno, &linearity) < 3) {
    fclose(calfile);
    return kFALSE;
  }

  // Read calibration points
  TArrayF calpoints(parpointsno);
  //  Char_t *pointp =  (Char_t *) &line[0];
  //  Char_t **point = (Char_t **) &pointp;
  Char_t *token;
  Char_t  ntoken;
  if (fscanf(calfile, "%300[ -~]", line) == EOF) {
    fclose(calfile);
    return kFALSE;
  }
  token = strtok(line," ");
  for(i=0; i<parpointsno; i++) {
    //    token = strsep(point," ");
    if(!token) {
      fclose(calfile);
      return kFALSE;
    }
    sscanf(token,"%f",&(calpoints[i]));
    token = strtok(NULL," ");
  }
  // Read and discard the header line
  fscanf(calfile, " %300[ -~]", line);

  while ((ntoken=fscanf(calfile, "%d %d %d %f %f %f %f", 
         &repno, &chno, &strno, par, par+1, par+2, par+3)) != EOF) {
    if(ntoken < 7) {
      fclose(calfile);
      return kFALSE;
    }
    if(repno<0 || repno>7 || strno<0 || strno>5) {
      Error("ReadCalibrationTB98File","unexpected repeator(%d)/string(%d) combination.\n",
                                       repno,strno);
      fclose(calfile);
      return kFALSE;
    }
    // correct channel nos to start from 0 for each string
    if(chno > 767) chno -= 768;
    if(chno<0 || chno>767) {
      Error("ReadCalibrationTB98File","unexpected channel number: %d.\n", chno);
      fclose(calfile);
      return kFALSE;
    }
    chancal = new TPhChannelCalTB98(parno, parpointsno, linearity,
				    par, calpoints.GetArray());
    fecno = kFECNo[repno][strno];
    strphno = kStrNo[repno][strno];
    if(fecno==-1 || strphno==-1) {
      Error("ReadCalibrationTB98File","unexpected repeator(%d)/string(%d) combination.\n",
                                       repno,strno);
      delete chancal;
      fclose(calfile);
      return kFALSE;
    }
    string = GetString(fecno,strphno);
    if(!string) {
      Error("ReadCalibrationTB98File",
	    "string object has not been found for FEC=%d String=%d.\n",fecno,strno);
      delete chancal;
      fclose(calfile);
      return kFALSE;
    }
    string->AddChannelCalObject(chno, chancal);
  }
  fclose(calfile);

  // don't do this check for TB98, since here FECDetector 
  // initialized from the standard SensorDBMap is not compatible
  // with the calibration data base
  //  return CalibrationCheck();
  return kTRUE;
}

Bool_t TPhFECDetector::CalibrationCheck() 
{
  // Check compatibility between FECs/Strings structures 
  // and the initialized calibration objects 
  Int_t numstrings,numchannels,fecno,strno,channo;
  TPhChannelCalAbs *chancal;
  TPhString *string;
  TPhFEC *fec;

  for(fecno=0; fecno<kMaxFECs; fecno++) {
    fec = GetFEC(fecno);
    if(!fec) 
      continue;
    numstrings = fec->GetNumStrings();
    for(strno=0; strno<numstrings; strno++) {
      string = fec->GetString(strno);
      if(!string) 
	continue;
      numchannels = string->GetStringSize();
      for(channo=0; channo<numchannels; channo++) {
	chancal = string->GetChannelCalObject(channo);
	if(!chancal) return kFALSE;
	if(fVerbose>1) chancal->ls();
      }
    }
  }
  return kTRUE;
}

Bool_t TPhFECDetector::CompatibilityCheck(TPhHashContainer *container) 
{
  // Check compatibility between FECs structures and event's raw hit buffers

  Int_t fecnumber,hitstrlength,fecstrlength,i;
  Bool_t check=kTRUE;
  TPhString *string;
  TPhRawHitFECn0 *fecrawbuffer;
  TPhFEC *fec;

  if(!container) {
    Error("CompatibilityCheck","NULL container pointer");
    return kFALSE;
  }
  TIter nextHitBuffer(container->GetIterator());
  while((fecrawbuffer = (TPhRawHitFECn0 *) nextHitBuffer())) {
    fecnumber = fecrawbuffer->GetFECNum();
    fec = GetFEC(fecnumber);
    if(!fec) {
      if(fVerbose>1) 
	Error("CompatibilityCheck","FEC no %d for container %s " \
	      "not found in the FECDetector structure\n",fecnumber,container);
      check = kFALSE;
      continue;
    }
    for(i=0;i<kNumStrings;i++) {
      hitstrlength = fecrawbuffer->GetRawHitArraySize(i);
      if(!hitstrlength) { 
	string = fec->GetString(i);
	if(!string) {
	  if(fVerbose>1) 
	    Error("CompatibilityCheck","FEC no %d has no information on string no %d\n",
		  fecnumber,i);
	  check = kFALSE;
	  continue;
	}
	fecstrlength = string->GetStringSize();
	if(fecstrlength != hitstrlength) {
	  if(fVerbose>1) 
	    Error("CompatibilityCheck","FEC %d string %d has different length %d then %d " \
		  "in the %s container\n", fecnumber,i, fecstrlength, hitstrlength,
		  container->GetName());
	  check = kFALSE;
	}
      }
    }
  }
  return check;
}

Bool_t TPhFECDetector::CompatibilityCheck(TPhRawHitFECn0 *fecrawbuffer) 
{
  // Check compatibility between FECs structures and event's raw hit buffers

  Int_t fecnumber,hitstrlength,fecstrlength,i;
  Bool_t check=kTRUE;
  TPhString *string;
  TPhFEC *fec;

  if(!fecrawbuffer) {
    Error("CompatibilityCheck","NULL FEC raw buffer pointer");
    return kFALSE;
  }
  fecnumber = fecrawbuffer->GetFECNum();
  fec = GetFEC(fecnumber);
  if(!fec) {
    if(fVerbose>1) 
      Error("CompatibilityCheck","FEC no %d " \
	    "not found in the FECDetector structure\n",fecnumber);
    return kFALSE;
  }
  for(i=0;i<kNumStrings;i++) {
    hitstrlength = fecrawbuffer->GetRawHitArraySize(i);
    if(!hitstrlength) { 
      string = fec->GetString(i);
      if(!string) {
	if(fVerbose>1) 
	  Error("CompatibilityCheck","FEC no %d has no information on string no %d\n",
		fecnumber,i);
	check = kFALSE;
	continue;
      }
      fecstrlength = string->GetStringSize();
      if(fecstrlength != hitstrlength) {
	if(fVerbose>1) 
	  Error("CompatibilityCheck","FEC %d string %d has different length %d then %d " \
		"in the %s container\n", fecnumber,i, fecstrlength, hitstrlength);
	check = kFALSE;
      }
    }
  }
  return check;
}

Bool_t TPhFECDetector::PedPreProcess(TPhRawHitFECn0 *fecevent)     
{ 
  //
  TPhFEC *fec = (TPhFEC *) GetFEC(fecevent->GetFECNum());
  if(fec) 
    return fec->PedPreProcess(fecevent); 
  return kFALSE; 
}

Bool_t TPhFECDetector::PedProcess(TPhRawHitFECn0 *fecevent)        
{ 
  //
  TPhFEC *fec = (TPhFEC *) GetFEC(fecevent->GetFECNum());
  if(fec) 
    return fec->PedProcess(fecevent); 
  return kFALSE; 
}

Bool_t TPhFECDetector::CMNProcess(TPhRawHitFECn0 *fecevent)  
{ 
  //
  TPhFEC *fec = (TPhFEC *) GetFEC(fecevent->GetFECNum());
  if(fec) 
    return fec->CMNProcess(fecevent); 
  return kFALSE; 
}

Bool_t TPhFECDetector::NoiseProcess(TPhRawHitFECn0 *fecevent)      
{ 
  //
  TPhFEC *fec = (TPhFEC *) GetFEC(fecevent->GetFECNum());
  if(fec) 
    return fec->NoiseProcess(fecevent); 
  return kFALSE; 
}

Bool_t TPhFECDetector::ZeroSuppress(TPhRawHitFECn0 *fecevent)       
{ 
  //
  TPhFEC *fec = (TPhFEC *) GetFEC(fecevent->GetFECNum());
  if(fec) 
    return fec->ZeroSuppress(fecevent); 
  return kFALSE; 
}

Bool_t TPhFECDetector::Calibrate(TPhAbsRawHitFEC *fecevent) 
{ 
  //
  TPhFEC *fec = (TPhFEC *) GetFEC(fecevent->GetFECNum());
  if(fec) 
    return fec->Calibrate(fecevent); 
  return kFALSE; 
}

Bool_t TPhFECDetector::CalibrateOnStrings(TPhAbsRawHitFEC *fecevent) 
{ 
  //
  TPhFEC *fec = (TPhFEC *) GetFEC(fecevent->GetFECNum());
  if(fec) 
    return fec->CalibrateOnStrings(fecevent); 
  return kFALSE; 
}

void TPhFECDetector::Calibrate(Bool_t &status)
{
  //
  Bool_t fecstatus = kTRUE;;
  status = kTRUE;

  fFECList->ForEach(TPhFEC,Calibrate)(fecstatus);
  status = status && fecstatus; 

  return;
}
