#include "math.h"
#include "TPhLayer.h"
#include "TPhEvent.h"
#include "TPhBoxSiSensor.h"
#include "TPhHit.h"
#include "TView.h"
#include "TCanvas.h"

ClassImp(TPhLayer)

////////////////////////////////////////////////////////////////
//
// The TPhLayer class provides an interface to the PHOBOS
// geometry and calibration database (which so far resides in
// a couple of ASCII files).
// The TPhLayer object manages a list of TPhSensor objects
// which hold the actual geometry information
// So far the TPhLayer class does not deal with the vertex
// and multiplicity counters. This upgrade will involve renaming
// some of existing functions and data members. 
//
// Integrating the TOF elements should be straightforward.
// We only need to add an fTofSlatList data member and provide
// some ASCII file containing the TOF geometry
//
//
// $Id: TPhLayer.cxx,v 1.8 1999/03/12 07:23:30 decowski Exp $
//
////////////////////////////////////////////////////////////////

TPhLayer::TPhLayer()
{
// Constructor. Sets fSensorList to NULL. To fill the sensorlist,
// the AddSensor() member function has to be called from the 
// PHOBOS subdetector containing the layer

    fSensorList     = NULL;
}

TPhLayer::~TPhLayer()
{
  // Default destructor
  //

  SafeDelete(fSensorList);
}

void TPhLayer::AddSensor(TPhSensor *sensor)
{
// Add a Si sensor element to this layer

    if(fSensorList == NULL)
    {
        fSensorList = new TObjArray();
    }
    fSensorList->Add(sensor);
}

void TPhLayer::Draw(Option_t *option)
{
// Draw all the Sensors in this layer

  fSensorList->Draw(option);
}

void TPhLayer::DeleteHits()
{
// For the time being, the hits are owned by the Sensor elements.
// TPhLayer::DeleteHits() loops over all sensors and asks
// them to delete the hits in their hitbuffers
// If you are absolutely sure about what you are doing,
// you can also use TPhLayer::ClearHits() to just
// clear the sensor hitlists without deleting the hits
// and then take care of deleting the hits yourself in
// some other place.

   TPhSensor *sensor;

   TIter NextSensor(fSensorList);
   while((sensor = (TPhSensor *) NextSensor()) != NULL)
   {
        sensor->DeleteHits();
   }
  
}

void TPhLayer::ClearHits()
{
// TPhLayer::ClearHits() loops over all sensors and asks
// them to clear their hitbuffers, without actually deleting
// the hits. if you don't delete the hits yourself somewhere
// else, this results in a beautiful mempry leak.

   TPhSensor *sensor;

   TIter NextSensor(fSensorList);
   while((sensor = (TPhSensor *) NextSensor()) != NULL)
   {
        sensor->ClearHits();
   }
  
}

void TPhLayer::AddHit(TPhHit *hit)
{
  // Add a hit to the relevant sensor. The sensor is looked up, using the
  // GetSensorLabel() function of the TPhHit.

    TPhSensor *sensor;

    sensor = GetSensor(hit->GetSensorLabel());
    sensor->GetHitList()->Add(hit);
}

Int_t TPhLayer::MakeHitArraysOutOfHits(Char_t *HitConName, 
				       Char_t *HitArrayConName)
{
 // Iterate over all sensors to create the HitArrays. Note that the sensors
 // actually create the HitArrays.
 //
 //  Return: The number of actually created HitArrays.
 
   TPhSensor *sensor;
   Int_t counter = 0;
 
   TIter NextSensor(fSensorList);
   while((sensor = (TPhSensor *) NextSensor()) != NULL) 
     counter += sensor->MakeHitArraysOutOfHits(HitConName, HitArrayConName);
 
   return counter;
}

Int_t TPhLayer::MakeHitsOutOfHitArrays(Char_t *HitArrayConName, 
				       Char_t *HitConName)
{
 // Iterate over all sensors to create the Hits. Note that the sensors
 // actually create the Hits (check if the sensor class has the method needed).
 //
 //  Return: The number of actually created hits.
 
   TPhSensor *sensor;
   Int_t counter = 0;
 
   TIter NextSensor(fSensorList);
   while((sensor = (TPhSensor *) NextSensor()) != NULL) {
     if( sensor->InheritsFrom("TPhBoxSiSensor") )
       counter += 
	 ((TPhBoxSiSensor *) sensor)->MakeHitsOutOfHitArray(HitArrayConName, HitConName);
     else continue;
   }
   return counter;
}

