#include <iostream.h>
#include <iomanip.h>
#include "TCanvas.h"
#include "TArrow.h"
#include "TPhDetector.h"
#include "TPhSiSensor.h"
#include "TPhString.h"
#include "TPhHitArray.h"
#include "TPhRawHitFECs0.h"

ClassImp(TPhString)

/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Description of the string of electronic channels structure and          //
// their properties.                                                       //
// FEC is divided into strings of channels, that may come from different   //
// sensors. A string may come from from several chips, each of them        //
// may be connected to a various number of pads.                           //
//                                                                         //
// $Id: TPhString.cxx,v 1.5 1999/03/12 15:42:49 olszewsk Exp $
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

TPhString::TPhString()
{
  // Default constructor

  fFECNo = 0;
  fStringNo = 0;
  fStringSize = 0; 
  fDoHistograms = kFALSE;
  fNoChips = 0;
  fPedPreMean = 0; 
  fMode = kPedPreMode;

  fPedPreHistoChannels  = NULL;   
  fPedMeanHistoChannels = NULL;  
  fPedRMSHistoChannels  = NULL;   
  fCMNEventHistoChannels = NULL;  
  fCMNMeanHistoChannels = NULL;  
  fCMNFHistoChannels    = NULL;     
  fNoiseHistoChannels   = NULL;    
  fZSSHistoChannels     = NULL;      
  fCalHistoChannels     = NULL;      

  fPedPreHistos  = NULL;  
  fPedMeanHistos = NULL; 
  fPedRMSHistos  = NULL;  
  fCMNEventHistos= NULL; 
  fCMNMeanHistos = NULL; 
  fCMNFHistos    = NULL;    
  fNoiseHistos   = NULL;   
  fZSSHistos     = NULL;     
  fCalHistos     = NULL;     

  fPedPreSHisto  = NULL;   
  fPedMeanSHisto = NULL;  
  fPedRMSSHisto  = NULL;   
  fCMNMeanSHisto = NULL;  
  fCMNFSHisto    = NULL;     
  fNoiseSHisto   = NULL;    
  fZSSSHisto     = NULL;      

  MIN_NUM_PED_EVENTS   = 300;
  MIPADC_CUTOFF        =  25;
  MIN_CHANNELS_CMN_EST =   5;
  MIN_NUM_NOISE_EVENTS = 300;
  THRESH_CMN =             6;
  THRESH_MULTIPLIER =      3;

  fVerbose = 0;
}

TPhString::TPhString(Int_t fecno, Int_t stringno, Int_t stringsize, 
		     Int_t chipno, Int_t *chipsizes) 
{
  // Normal constructor 

  fFECNo = fecno;
  fStringNo = stringno;
  fDoHistograms = kFALSE;
  if(stringsize) 
    fStringSize = stringsize;
  else           
    fStringSize = kMaxLength;
  if(chipno && chipsizes) {
    fNoChips = chipno;
    if( !BuildChipList(chipno, chipsizes) ) {
      Error("TPhString","Constructor for string %d in FEC no %d" \
            " called with inconsistent chip parameters\n",fecno, stringno);
      exit(0);
    }
  }

  fPedPreMean = 0; 
  fPedMean = NULL; 
  fPedRMS  = NULL; 
  fCMNMean = NULL; 
  fCMNFlag = NULL; 
  fNoise   = NULL;        
  
  fEvents = NULL;   
  fSum    = NULL;           
  fSumSq  = NULL; 

  fPedPreHistoChannels  = NULL;   
  fPedMeanHistoChannels = NULL;  
  fPedRMSHistoChannels  = NULL;   
  fCMNEventHistoChannels = NULL;  
  fCMNMeanHistoChannels = NULL;  
  fCMNFHistoChannels    = NULL;     
  fNoiseHistoChannels   = NULL;    
  fZSSHistoChannels     = NULL;      
  fCalHistoChannels     = NULL;      

  fPedPreHistos  = NULL;  
  fPedMeanHistos = NULL; 
  fPedRMSHistos  = NULL;  
  fCMNEventHistos = NULL; 
  fCMNMeanHistos = NULL; 
  fCMNFHistos    = NULL;    
  fNoiseHistos   = NULL;   
  fZSSHistos     = NULL;     
  fCalHistos     = NULL;     

  fPedPreSHisto  = NULL;   
  fPedMeanSHisto = NULL;  
  fPedRMSSHisto  = NULL;   
  fCMNMeanSHisto = NULL;  
  fCMNFSHisto    = NULL;     
  fNoiseSHisto   = NULL;    
  fZSSSHisto     = NULL;      

  fMode = kPedPreMode;

  MIN_NUM_PED_EVENTS   = 300;
  MIPADC_CUTOFF        =  25;
  MIN_CHANNELS_CMN_EST =   5;
  MIN_NUM_NOISE_EVENTS = 300;
  THRESH_CMN           =   6;
  THRESH_MULTIPLIER    =   3;

  fVerbose = 0;
}

TPhString::~TPhString()
{
  // Default destructor

    if(fCalChannelArray) delete fCalChannelArray;
    if(fChips)   delete fChips;
    if(fPedMean) delete fPedMean;
    if(fPedRMS)  delete fPedRMS;
    if(fCMNMean) delete fCMNMean;
    if(fCMNFlag) delete fCMNFlag;
    if(fNoise)   delete fNoise;
    if(fEvents)  delete fEvents;
    if(fSum)     delete fSum;
    if(fSumSq)   delete fSumSq;

    if(fPedPreHistoChannels)  delete fPedPreHistoChannels;   
    if(fPedMeanHistoChannels) delete fPedMeanHistoChannels;  
    if(fPedRMSHistoChannels)  delete fPedRMSHistoChannels;   
    if(fCMNEventHistoChannels)delete fCMNEventHistoChannels;  
    if(fCMNMeanHistoChannels) delete fCMNMeanHistoChannels;  
    if(fCMNFHistoChannels)    delete fCMNFHistoChannels;     
    if(fNoiseHistoChannels)   delete fNoiseHistoChannels;    
    if(fZSSHistoChannels)     delete fZSSHistoChannels;      
    if(fCalHistoChannels)     delete fCalHistoChannels;      

    if(fPedPreHistos)  delete fPedPreHistos;  
    if(fPedMeanHistos) delete fPedMeanHistos; 
    if(fPedRMSHistos)  delete fPedRMSHistos;  
    if(fCMNEventHistos)delete fCMNEventHistos; 
    if(fCMNMeanHistos) delete fCMNMeanHistos; 
    if(fCMNFHistos)    delete fCMNFHistos;    
    if(fNoiseHistos)   delete fNoiseHistos;   
    if(fZSSHistos)     delete fZSSHistos;     
    if(fCalHistos)     delete fCalHistos;     

    if(fPedPreSHisto)  delete fPedPreSHisto;  
    if(fPedMeanSHisto) delete fPedMeanSHisto; 
    if(fPedRMSSHisto)  delete fPedRMSSHisto;  
    if(fCMNMeanSHisto) delete fCMNMeanSHisto; 
    if(fCMNFSHisto)    delete fCMNFSHisto;    
    if(fNoiseSHisto)   delete fNoiseSHisto;   
    if(fZSSSHisto)     delete fZSSSHisto;     
}

Bool_t TPhString::BuildChipList(Int_t chipno, Int_t *chipsizes)
{
  // Build a list of chip objects according to parameters given 
  // in the call to constructor
  Int_t i, strsize=0;
  Bool_t err=kFALSE;

  if(fChips) delete fChips;
  fChips = new TObjArray(chipno);

  for(i=0; i<chipno; i++) {
    if(chipsizes[i]<1) 
      err=kTRUE;
    strsize += chipsizes[i];
    if(strsize>fStringSize) 
      err = kTRUE;
    fChips->Add(new TPhChip(fFECNo,fStringNo,i,chipsizes[i]));
  }
  if( strsize!=fStringSize ) 
    Warning("BuildChipList","In string %d from FEC no %d:\n" \
	    "  sum of chip channels less then the string length.\n",fFECNo, fStringNo);
  if(err) {
    delete fChips;
    return kFALSE;
  }
  return kTRUE;
}

TPhChip* TPhString::GetChip(Int_t no)         
{ 
  // return pointer to no chip object

  if(fChips) 
    return (TPhChip *) fChips->At(no); 
  else return NULL;
}


TPhChannelCalAbs* TPhString::GetChannelCalObject(Int_t no)
{
  // return pointer to no TPhChannelCalAbs calibration object

  if(fCalChannelArray) 
    return (TPhChannelCalAbs *) fCalChannelArray->At(no); 
  else return NULL; 
}

void TPhString::DeletePedPreArrays()
{
  // Delete arrays storing pedestal mean and sigma

  if(fPedPreMean) delete fPedMean;
}

void TPhString::CreatePedPreArrays()
{
  // Create arrays storing pedestal mean and sigma

  DeletePedPreArrays();
  fPedPreMean = new TArrayD(fStringSize);
  fPedPreMean->Reset();
}

void TPhString::DeletePedArrays()
{
  // Delete arrays storing pedestal mean and sigma

  if(fPedMean) delete fPedMean;
  if(fPedRMS)  delete fPedRMS;
}

void TPhString::CreatePedArrays()
{
  // Create arrays storing pedestal mean and sigma

  DeletePedArrays();
  fPedMean = new TArrayF(fStringSize);
  fPedRMS  = new TArrayD(fStringSize);
  fPedMean->Reset();
  fPedRMS->Reset();
}

void TPhString::DeleteCMNArrays()
{
  // Delete arrays with CMN mean value and validity flag

  if(fCMNMean) delete fCMNMean;
  if(fCMNFlag) delete fCMNFlag;
}

void TPhString::CreateCMNArrays()
{
  // Create arrays with CMN mean value and validity flag

  if(!fCMNMean) fCMNMean = new TArrayF(fNoChips);
  if(!fCMNFlag) fCMNFlag = new TArrayS(fNoChips);
  fCMNMean->Reset();
  fCMNFlag->Reset();
}

void TPhString::DeleteNoiseArrays()
{
  // Delete arrays with noise levels

  if(fNoise) delete fNoise;
}

void TPhString::CreateNoiseArrays()
{
  // Create arrays with noise levels

  DeleteNoiseArrays();
  fNoise = new TArrayD(fStringSize);
  fNoise->Reset();
}

void TPhString::DeleteZZArrays()
{
  // Delete arrays with zero suppressed raw hits

  if(fZSSHitChan) delete fZSSHitChan;
  if(fZSSHitADC) delete fZSSHitADC;
}

void TPhString::CreateZZArrays()
{
  // Create arrays for storing zero suppressed hits

  if(!fZSSHitChan) fZSSHitChan = new TArrayS(fStringSize);
  if(!fZSSHitADC)  fZSSHitADC  = new TArrayF(fStringSize);
  fZSSHitChan->Reset();
  fZSSHitADC->Reset();
}

void TPhString::DeleteSuppArrays()
{
  //
  if(fEvents) delete fEvents;
  if(fSum)    delete fSum;
  if(fSumSq)  delete fSumSq;
}

void TPhString::CreateSuppArrays()
{
  //
  DeleteSuppArrays();
  fEvents = new TArrayI(fStringSize);
  fSum    = new TArrayF(fStringSize);
  fSumSq  = new TArrayD(fStringSize);
  fEvents->Reset();
  fSum->Reset();
  fSumSq->Reset();
}

Int_t TPhString::PedPreProcess(FECRawHit_t *hitstring)
{
  // Compute avarage of channel contents for the whole string 
  // as a first stage of pedestal level estimation
  // Return -1 when string length inconsistent with the call
  // Return  0 if minimum number of events got processed 
  // Return >0 (number of events) 
  //            when one has not processed enough events yet

  if(fMode!=kPedPreMode) {
    fMode=kPedPreMode;
    CreatePedPreArrays();
    CreateSuppArrays();
  }
  if(!fPedPreMean) CreatePedPreArrays();
  if(!fEvents)     CreateSuppArrays();

  // First part of Praadeep pedestal calculation routine 
  //    A.O. converted to single event mode and PHAT requirements
  //

  //#define  MIN_NUM_PED_EVENTS 300
  //#define  MIPADC             50

  Int_t      k;
  Int_t      loEvents=0;

  for(k=0; k<fStringSize; k++) {

    //......................................................................
    // Make 1st pass through the data : take ALL events
    // as pedestal events 
    // to get rough idea of where pedestal is

    ((*fSum)[k] += hitstring[k].ADC);
    
    ((*fEvents)[k] = (*fEvents)[k] + 1);
    
    // Do we have sufficient number of pedestal events 
    // if not, set channel number as a return code
    if((*fEvents)[k]<MIN_NUM_PED_EVENTS)
      loEvents = k;
    else {
      (*fPedPreMean)[k] = ((Double_t) (*fSum)[k] /
			   (Double_t) (*fEvents)[k]);
      
      if(fDoHistograms) {
	if(fPedPreHistoChannels) {
	  if((*fPedPreHistoChannels)[k]) 
	    ((TH1F *)fPedPreHistos->At(k))->Fill((*fPedPreMean)[k]);
	  if((*fPedPreHistoChannels)[kMaxLength]==-1) 
	    ((TH1F *)fPedPreHistos->At(kMaxLength))->Fill((*fPedPreMean)[k]);
	}
      }
    }
  } 

  return(loEvents);
}


Int_t  TPhString::PedProcess(FECRawHit_t *hitstring)
{
  // Add pedestal info from this string and 
  // Compute pedestal avarage and sigma if number of events > MIN_NUM_PED_EVENTS
  // Return -1 when string length inconsistent with the call
  // Return  0 if minimum number of events got processed 
  // Return >0 (number of events) 
  //            when one has not processed enough events yet

  if(fMode!=kPedMode) {
    fMode=kPedMode;
    CreateSuppArrays();
    CreatePedArrays();
  }
  if(!fEvents)  CreateSuppArrays();
  if(!fPedMean) CreatePedArrays();

  // Second part of Praadeep pedestal calculation routine 
  //    A.O. converted to single event mode and PHAT requirements
  //..........................................................................
  //
  // Purpose: Calculate Pedestals from Raw ADC data for given string
  //
  //
  // Parameters:
  //              1) MIN_NUM_PED_EVENTS = 300
  //                 - minimum number of events to calculate pedestal for 1 chan
  //              
  //              2) MIPADC = 50
  //                 - approximate ADC value for 1 MIP
  // 
  // Return codes:
  //              0  = All OK 
  //              >0 = Channel number for which there is insufficient number of
  //                   Pedestal events (hope channel # 0 is ok!) 
  //                   Pedestals for remaining channels are NOT calculated!
  // 
  // Notes:
  //              i) The routine below doesn't care if gRawADC[..] are from a 
  //                 Pedestal-Run or from a Beam-Run. In case there isn't 
  //                 sufficient (300) number of pedestal events for any
  //                 channel we return appropriate error code.
  // 
  //              iii) ALL array indices begin from 0!
  //
  //................................................................................


  Int_t      k;
  Float_t    cont=0;
  Int_t      lowChannel=0;

  //......................................................................
  // gPedMean Calculation for all channels

  for(k=0; k<fStringSize; k++) {

    //......................................................................
    // Make 2nd pass through data  : only take anything less than
    // 1 MIP ADC as a pedestal!
    
    //anything less than 1 MIP is pedestal

//    if( TMath::Abs( hitstring[k].ADC-(*fPedPreMean)[k] ) < MIPADC) {
// 280299 HP 
// 0403099 AO removed 0.5 factor after introducing MIPADC_CUTOFF
    if( TMath::Abs( hitstring[k].ADC-(*fPedPreMean)[k] ) < MIPADC_CUTOFF) {
	((*fSum)[k] += hitstring[k].ADC);
	
	((*fSumSq)[k] +=
	 (Double_t) (hitstring[k].ADC)  * (Double_t) (hitstring[k].ADC));
	
	((*fEvents)[k] += 1);
    }
         
    // Do we have sufficient number of pedestal events for this
    // channel? if not, return with channel number as return code
    if((*fEvents)[k]<MIN_NUM_PED_EVENTS) 
      lowChannel=k;
    else {
      //990303 mpd&hp, from Double to Float
      Float_t tempMean;    
      tempMean=((Float_t) (*fSum)[k]) /
			  ((Float_t) (*fEvents)[k]);
      
      //990304 P.S.
      (*fPedRMS)[k]= 
	TMath::Sqrt ( ((*fSumSq)[k] / (Double_t)(*fEvents)[k]) -
	              (Double_t)(tempMean*tempMean) );

      // and our pedestal for this channel is ready
      (*fPedMean)[k]= tempMean;
 
      if(fDoHistograms) {
	if(fPedMeanHistoChannels) {
	  cont = (*fPedMean)[k];
	  if((*fPedMeanHistoChannels)[k]) 
	    ((TH1F *)fPedMeanHistos->At(k))->Fill(cont);
	  if((*fPedMeanHistoChannels)[kMaxLength]==-1) 
	    ((TH1F *)fPedMeanHistos->At(kMaxLength))->Fill(cont);
	}
	if(fPedRMSHistoChannels) {
	  cont = (*fPedRMS)[k];
	  if((*fPedRMSHistoChannels)[k]) 
	    ((TH1F *)fPedRMSHistos->At(k))->Fill(cont);
	  if((*fPedRMSHistoChannels)[kMaxLength]==-1) 
	    ((TH1F *)fPedRMSHistos->At(kMaxLength))->Fill(cont);
	}
      }
    } 
  }
  
  // and off we go..
  return(lowChannel);
  
}


Int_t  TPhString::CMNProcess(FECRawHit_t *hitstring)
{
  // Compute CMN avarages in each of the chip channel ranges for this event
  // Return <0 if structures are incompatible
  // Return  0 if everything is OK 
  // Return >0 number of chips with irregular CMN

  CreateCMNArrays();

  // Praadeep 6 sigma procedure for CMN calculations 
  //    A.O. converted to single event mode and PHAT requirements
  //    Threshold value THRESH_CMN (6) parametrized
  //......................................................................
  //
  // Purpose: Calculate Common Mode Noise from Raw ADC data for given string
  //          given the Pedestals
  //
  // Parameters:
  //              1) MIN_CHANNELS_CMN_EST   =5
  //                 -minimum number of empty channels in a chip for
  //                  CMN estimate to be valid
  // 
  // Return codes:
  //              0  = All OK 
  //              >0 = Number of Chips (not their ID's!) for which the
  //                   CMN estimate is irregular (see gCMNFlag above)
  //                   However, ALL CMN estimates ARE calculated
  // 
  // Notes:
  //              i) The routine below is an implementation of the 
  //                 ' THRESH_CMN (6) Sigma CMN correction Algorithm '
  //                 It is theoretically supposed to give acceptably
  //                 accurate CMN estimates upto the limit of
  //                   a - 95% occupancy and,
  //                   b - CMN upto THRESH_CMN (6) times the intrinsic detector noise
  //                                
  //              ii) ALL array indices begin from 0!
  //
  //......................................................................

  //#define MIN_CHANNELS_CMN_EST  5

  Int_t      i,j,k,rc=0;
  Int_t      channelsperchip, firstchipchannel;
  Float_t    chipMin, chip2ndMin, cmnEstimate;
  Int_t meanCount;
  TPhChip   *chip = NULL;
  //990303 mpd&hp Int_t -> Float_t
  TArrayF    signal(kMaxLength);

  firstchipchannel = 0;
  channelsperchip  = 0;
  for(i=0;i<fNoChips;i++)
    {
      //......................................................................
      // Pass 1 through data : look for chipMin
      
      chipMin=4095; // max 12 bit value

      firstchipchannel += channelsperchip;

      chip = (TPhChip *) fChips->At(i);
      if(!chip) return -1;

      channelsperchip = chip->GetChipSize();
      for (j=0; j<channelsperchip; j++)
	{
	  k=firstchipchannel+j; 
	  
	  // Subtract Pedestal from RawADC
	  signal[k] = hitstring[k].ADC-(*fPedMean)[k];

	  if (signal[k] < chipMin) 
	    chipMin = signal[k];
	}
      
      //......................................................................
      // Pass 2 through data : apply +/- THRESH_CMN*Noise cut on each channel
      //                       to look for 'empty' channels
      
      if(fDoHistograms) {
	if(fCMNEventHistoChannels) {
	  if((*fCMNEventHistoChannels)[i]) 
	    ((TH1F *)fCMNEventHistos->At(i))->Reset();
	  if((*fCMNEventHistoChannels)[kMaxChipNumber]==-1) 
	    ((TH1F *)fCMNEventHistos->At(kMaxChipNumber))->Reset();
	}
      }
      
      cmnEstimate = 0;
      meanCount=0;
      for (j=0; j<channelsperchip; j++) {
	k=firstchipchannel+j; 
	
	// apply cut with respect to chipMin
//	if (signal[k] < chipMin+THRESH_CMN*(*fPedRMS)[k])  
//	if (TMath::Abs(signal[k]) < chipMin+THRESH_CMN*(*fPedRMS)[k])  
	if (TMath::Abs(signal[k]) < THRESH_CMN*(*fPedRMS)[k])  
	  {
	    // this is an empty channel and can be used for
	    // our estimate: 
	    if(fDoHistograms) {
	      if(fCMNEventHistoChannels) {
		if((*fCMNEventHistoChannels)[i]) 
		  ((TH1F *)fCMNEventHistos->At(i))->Fill(signal[k]);
		if((*fCMNEventHistoChannels)[kMaxChipNumber]==-1) 
		  ((TH1F *)fCMNEventHistos->At(kMaxChipNumber))->Fill(signal[k]);
	      }
	    }
	    cmnEstimate+=signal[k];
	    meanCount++;
	  }
      }
      
      // were there at least (5) empty channels?
      if(meanCount >MIN_CHANNELS_CMN_EST) 
	{
	  // go ahead and make a valid CMN estimate
	  (*fCMNMean)[i]=cmnEstimate/meanCount;
	  (*fCMNFlag)[i]=0;
	  if(fDoHistograms) {
	    if(fCMNMeanHistoChannels) {
	      if((*fCMNMeanHistoChannels)[i]) 
		((TH1F *)fCMNMeanHistos->At(i))->Fill((*fCMNMean)[i]);
	      if((*fCMNMeanHistoChannels)[kMaxChipNumber]==-1) 
		((TH1F *)fCMNMeanHistos->At(kMaxChipNumber))->Fill((*fCMNMean)[i]);
	    }
	    if(fCMNFHistoChannels) {
	      if((*fCMNFHistoChannels)[i]) 
		((TH1F *)fCMNFHistos->At(i))->Fill(0);
	      if((*fCMNFHistoChannels)[kMaxChipNumber]==-1) 
		((TH1F *)fCMNFHistos->At(kMaxChipNumber))->Fill(0);
	    }
	  }
	}
      else  // for most cases, will not go down here
	{
	  //............................................................
	  // we're in trouble, too much occupancy or 1-2 funny channels
	  // are way down below
	  // solution: make another pass through the data to determine
	  // second minimum (next-to-lowest)
	  
	  chip2ndMin=4095; // max 12 bit value
	  
	  for (j=0; j<channelsperchip; j++)
	    {
   	      k=firstchipchannel+j; 
	      
	      if (signal[k] > chipMin && signal[k] < chip2ndMin ) 
		chip2ndMin = signal[k];
	    }
	  
	  cmnEstimate = 0;
	  meanCount = 0;
	  
	  for (j=0; j<channelsperchip; j++)
	    {
   	      k=firstchipchannel+j; 
	      
	      // apply cut with respect to chip2ndMin
	      //990303 mpd&hp: same problem as above
	      //       if (TMath::Abs(signal[k]) < chip2ndMin+THRESH_CMN*(*fPedRMS)[k]) 
		if (TMath::Abs(signal[k]) < THRESH_CMN*(*fPedRMS)[k])  
		{
		  // this is an empty channel and can be used for
		  // our estimate: 
		  if(fDoHistograms) {
		    if(fCMNEventHistoChannels) {
		      if((*fCMNEventHistoChannels)[i]) 
			((TH1F *)fCMNEventHistos->At(i))->Fill(signal[k]);
		      if((*fCMNEventHistoChannels)[kMaxChipNumber]==-1) 
			((TH1F *)fCMNEventHistos->At(kMaxChipNumber))->Fill(signal[k]);
		    }
		  }
		  cmnEstimate+=signal[k];
		  meanCount++;
		}
	    }
	  
	  // after this recalculation, were there at least (5) empty channels?
	  if(meanCount >MIN_CHANNELS_CMN_EST) 
	    {
	      // go ahead and make a valid CMN estimate
	      (*fCMNMean)[i]=cmnEstimate/meanCount;
	      (*fCMNFlag)[i]=1;
	      if(fDoHistograms) {
		if(fCMNMeanHistoChannels) {
		  if((*fCMNMeanHistoChannels)[i]) 
		    ((TH1F *)fCMNMeanHistos->At(i))->Fill((*fCMNMean)[i]);
		  if((*fCMNMeanHistoChannels)[kMaxChipNumber]==-1) 
		    ((TH1F *)fCMNMeanHistos->At(kMaxChipNumber))->Fill((*fCMNMean)[i]);
		}
		if(fCMNFHistoChannels) {
		  if((*fCMNFHistoChannels)[i]) 
		    ((TH1F *)fCMNFHistos->At(i))->Fill(1);
		  if((*fCMNFHistoChannels)[kMaxChipNumber]==-1) 
		    ((TH1F *)fCMNFHistos->At(kMaxChipNumber))->Fill(1);
		}
	      }
	    }
	  else
	    {
	      //............................................................
	      // there is something seriously wrong!
	      rc++;
	      // there is really 
	      // a) less than (5) channels empty i.e. >97% occupancy
	      //          or
	      // b) more than (5) channels acting funny  
	      // 
	      // for the TestBeam98 the following 'last-resort' 
	      // works because (a) is excluded and (b) is taken care of
	      // by applying ABSOLUTE THRESH_CMN*sigma cut.
	      // 
	      // in Real DAQ routine, this chip would NOT be zero-suppressed
	      //  
	      
	      cmnEstimate = 0;
	      meanCount = 0;
	      
	      for (j=0; j<channelsperchip; j++)
		{
		  k=firstchipchannel+j; 
		  
		  // apply ABSOLUTE +/- THRESH_CMN Sigma cut
		  if (TMath::Abs(signal[k]) < THRESH_CMN*(*fPedRMS)[k]) 
		    {
		      // this is an empty channel and can be used for
		      // our estimate: 
		      if(fDoHistograms) {
			if(fCMNEventHistoChannels) {
			  if((*fCMNEventHistoChannels)[i]) 
			    ((TH1F *)fCMNEventHistos->At(i))->Fill(signal[k]);
			  if((*fCMNEventHistoChannels)[kMaxChipNumber]==-1) 
			    ((TH1F *)fCMNEventHistos->At(kMaxChipNumber))->Fill(signal[k]);
			}
		      }
		      cmnEstimate+=signal[k];
		      meanCount++;
		    }
		}
	      
	      // after this 3rd recalculation,  how many empty channels?
	      if(meanCount >MIN_CHANNELS_CMN_EST) 
		{
		  // go ahead and make a valid CMN estimate
		  (*fCMNMean)[i]=cmnEstimate/meanCount;
		  (*fCMNFlag)[i]=-1;
		  if(fDoHistograms) {
		    if(fCMNMeanHistoChannels) {
		      if((*fCMNMeanHistoChannels)[i]) 
			((TH1F *)fCMNMeanHistos->At(i))->Fill((*fCMNMean)[i]);
		      if((*fCMNMeanHistoChannels)[kMaxChipNumber]==-1) 
			((TH1F *)fCMNMeanHistos->At(kMaxChipNumber))->Fill((*fCMNMean)[i]);
		    }
		    if(fCMNFHistoChannels) {
		      if((*fCMNFHistoChannels)[i]) 
			((TH1F *)fCMNFHistos->At(i))->Fill(-1);
		      if((*fCMNFHistoChannels)[kMaxChipNumber]==-1) 
			((TH1F *)fCMNFHistos->At(kMaxChipNumber))->Fill(-1);
		    }
		  }
		}
	    }
	}
    }
  
  // and off we go..
  return(rc);
}


Int_t  TPhString::NoiseProcess(FECRawHit_t *hitstring)
{
  // Add noise info from this string and 
  // compute noise avarage and sigma if number of events > MIN_NUM_NOISE_EVENTS
  // Return -1 when string length inconsistent with the call
  // Return  0 if minimum number of events got processed 
  // Return >0 (number of events) 
  //            when one has not processed enough events yet
  //
  // Call the CMN-calculation function (CMNProcess for example) 
  // to calculate the Common Mode Noise first before calling this procedure

  if(fMode!=kNoiseMode) {
    fMode=kNoiseMode;
    CreateNoiseArrays();
    CreateSuppArrays();
  }
  if(!fEvents)  CreateSuppArrays();
  if(!fNoise)   CreateNoiseArrays();

  // Praadeep noise estimation procedure
  //    A.O. converted to single event mode and PHAT requirements
  //......................................................................
  //
  // Purpose: Calculate Noise from Raw ADC and Pedestal data for given string
  //
  //
  // Parameters:
  //              1) MIN_NUM_NOISE_EVENTS = 300
  //                 - minimum number of events to calculate noise for 1 chan
  //              
  //              2) MIPADC = 50
  //                 - approximate ADC value for 1 MIP
  // 
  // Return codes:
  //              0  = All OK 
  //              >0 = Channel number for which there is insufficient number of
  //                   Noise events (hope channel #0 is ok!)
  //                   Noise for Remaining channels is NOT calculated 
  // Notes:
  // 
  //              i) The gNoise[..] is calculated by first correcting the
  //                 the Pedestals for common-mode-noise and then calculating
  //                 the sigma of the cmn-corrected pedestals.
  //                 The routine doesn't care if the gRawADC[..] provided
  //                 are from Pedestal-Run or Beam-Run.
  //
  //              ii)ALL array indices begin from 0!
  //
  //......................................................................

  //#define  MIN_NUM_NOISE_EVENTS 300

  Int_t      i,j,k;
  Int_t      lowChannel=0;
  //990303 mpd&hp: double->float, Int -> float
  Float_t   tempMean;

  Float_t    noise;
  Int_t      channelsperchip, firstchipchannel;
  TPhChip   *chip = NULL;
 
  // Correct all channel RawADC's for CMN and recalcuate sums & sum of sq's
  firstchipchannel = 0;
  channelsperchip  = 0;
  for(i=0;i<fNoChips;i++) {
    firstchipchannel += channelsperchip;
    chip = (TPhChip *) fChips->At(i);
    if(!chip) return -1;
    channelsperchip = chip->GetChipSize();
    for (j=0; j<channelsperchip; j++) {
      k=firstchipchannel+j;
      
//      anything less than 1 MIP is pedestal
//      if( TMath::Abs(hitstring[k].ADC-(*fPedMean)[k])<MIPADC) {
// 280299 HP
      //anything less than THRESH_CMN*Pedestal_noise is pedestal
      if( TMath::Abs(hitstring[k].ADC-(*fPedMean)[k])<THRESH_CMN*(*fPedRMS)[k]) {
	// Sum of CMN corrected RawADC
        noise = hitstring[k].ADC-(*fPedMean)[k]-(*fCMNMean)[i];
	(*fSum)[k]+=noise;
	
	// the Sum of squares of CMN corrected RawADC
	(*fSumSq)[k]+= noise * noise;
	
	(*fEvents)[k]++;
      }
    }
  }

  for(k=0; k<fStringSize; k++) {
    // Do we have sufficient number of noise events for this
    // channel ? if not, return with channel number as error code
    if((*fEvents)[k]<MIN_NUM_NOISE_EVENTS)
      lowChannel = k;
    else {
      tempMean= (*fSum)[k] / (Float_t)(*fEvents)[k];
      
      // Noise of this channel is the sigma of CMN-corrected pedestals
      /*      (*fNoise)[k]= 
	TMath::Sqrt( (Double_t)((*fSumSq)[k] - 
				(tempMean*tempMean*(*fEvents)[k])) /
		     ((Double_t) (*fEvents)[k]) );
		     */
      // 990318 HP use a simple version: expect mean noise to be zero
      (*fNoise)[k]= 
	TMath::Sqrt( ((*fSumSq)[k])/
		     ((Float_t) (*fEvents)[k]) );
      
      if(fDoHistograms) {
	if(fNoiseHistoChannels) {
	  if((*fNoiseHistoChannels)[k]) 
	    ((TH1F *)fNoiseHistos->At(k))->Fill((*fNoise)[k]);
	  if((*fNoiseHistoChannels)[kMaxLength]==-1) 
	    ((TH1F *)fNoiseHistos->At(kMaxLength))->Fill((*fNoise)[k]);
	}
      }
    }
  }
      
  // and off we go..
  return(lowChannel);
}

Int_t TPhString::ZeroSuppress(FECRawHit_t *hitstring)
{
  // ZeroSuppress RawADC data for this string
  //  given Pedestals+Noise for every channel and Common Mode Noise for
  //  every chip
  // Return -1 when string length inconsistent with the call
  // Return >=0 number of hits 
  if(!fPedMean || !fNoise) {
    Error("ZeroSupress","Pedestals and/or Noise not found for this string.\n");
    return -1;
  }
  if(fMode!=kZeroSuppMode) fMode=kZeroSuppMode;
  CreateZZArrays();

  // Praadeep zero-suppression procedure 
  //    A.O. converted to single event mode and PHAT requirements
  //......................................................................
  //
  // Purpose: ZeroSuppress RawADC data for given string
  //          given Pedestals+Noise for every channel and Common Mode Noise for
  //          every chip
  //
  // Parameters:
  //              1) THRESH_MULTIPLIER
  //                 - ADC is how many times more than noise => hit?
  // 
  // Return codes:
  //              >=0  = Number of hits in the string
  //
  // Notes:
  //              i) ALL array indices begin from 0!
  //
  //              ii) Each RawADC value is cmpared to (THRESH_MULTIPLIER*Noise)
  //                  for that channel to decide if it is a hit or not
  //......................................................................
 

  //#define  THRESH_MULTIPLIER  3

  Int_t      i,j,k;
  Int_t      numHits=0;
  
  //990303 mpd&hp Int -> Float
  Float_t      signal;

  Int_t      channelsperchip, firstchipchannel;
  TPhChip   *chip = NULL;
 
  firstchipchannel = 0;
  channelsperchip  = 0;
  for(i=0; i<fNoChips; i++)
    {
      firstchipchannel += channelsperchip;
      chip = (TPhChip *) fChips->At(i);
      if(!chip) return -1;
      channelsperchip = chip->GetChipSize();
      // do we zero-suppress this chip (see gCMNFlag's in CMNCalc.c)
      if((*fCMNFlag)[i]!=-1)
	{
	  for (j=0; j<channelsperchip; j++) {
	    k=firstchipchannel+j;
	    
	    signal=hitstring[k].ADC-(*fPedMean)[k]-(*fCMNMean)[i];
	    
	    // using abs(..) here because want to see anything that is
	    // interesting, including negative hits caused by wrong polarity
	    // of bias voltages!
	    if( TMath::Abs(signal) > (THRESH_MULTIPLIER * (*fNoise)[k]) )
	      {
		// this is a hit, store it!
		(*fZSSHitChan)[numHits]=k;
		
		(*fZSSHitADC)[numHits]=signal;
		
		numHits++;

		if(fDoHistograms) {
		  if(fZSSHistoChannels) {
		    if((*fZSSHistoChannels)[k]) 
		      ((TH1F *)fZSSHistos->At(k))->Fill(signal);
		    if((*fZSSHistoChannels)[kMaxLength]==-1) 
		      ((TH1F *)fZSSHistos->At(kMaxLength))->Fill(signal);
		  }
		}
	      }
	    else
	      {
		// this is an empty channel, use it to update the noise
		// Heinz's Phd formula
		//
		// Noise_new^2 =  Noise_old^2*(w2-1) + Signal^2 
		//               -------------------------------
		//                             w2
		// we take w2 = 64 (time constant of the low-pass filter)
		
		(*fNoise)[k] = TMath::Sqrt(((*fNoise)[k] * (*fNoise)[k] * 63
				        + (signal*signal)) / 64);

		//and pedestal 
                (*fPedMean)[k] = (*fPedMean)[k] + signal/64;

		if(fDoHistograms) {
		  if(fNoiseHistoChannels) {
		    if((*fNoiseHistoChannels)[k]) 
		      ((TH1F *)fNoiseHistos->At(k))->Fill((*fNoise)[k]);
		    if((*fNoiseHistoChannels)[kMaxLength]==-1) 
		      ((TH1F *)fNoiseHistos->At(kMaxLength))->Fill((*fNoise)[k]);
		  }
		  if(fPedMeanHistoChannels) {
		    if((*fPedMeanHistoChannels)[k]) 
		      ((TH1F *)fPedMeanHistos->At(k))->Fill((*fPedMean)[k]);
		    if((*fPedMeanHistoChannels)[kMaxLength]==-1) 
		      ((TH1F *)fPedMeanHistos->At(kMaxLength))->Fill((*fPedMean)[k]);
		  }
		}
	      }
	  }
	}
      else // this Chip had a funny CMN estimate, don't Zero Suppress it!
	{
	  for (j=0; j<channelsperchip; j++) {
	    k=firstchipchannel+j;
	      
	    signal=hitstring[k].ADC-(*fPedMean)[k]-(*fCMNMean)[i];
	    
	    (*fZSSHitChan)[numHits]=k;
	    
	    (*fZSSHitADC)[numHits]=signal;
	    
	    numHits++;
	  }
	}
    }
  
  // and we're done...
  return(numHits);
}

Int_t TPhString::ZeroSuppress(FECRawHit_t *inhitstring, TPhRawHitFECs0 *rawBufs0)
{
  // ZeroSuppress RawADC data for this string
  //  given Pedestals+Noise for every channel and Common Mode Noise for
  //  every chip
  // Return -1 when string length inconsistent with the data
  // Return >=0 number of hits 
  if(!fPedMean || !fNoise) {
    Error("ZeroSupress","Pedestals and/or Noise not found for this string.\n");
    return -1;
  }
  if(fMode!=kZeroSuppMode) fMode=kZeroSuppMode;

  // Praadeep zero-suppression procedure 
  //    A.O. converted to single event mode and PHAT requirements
  //......................................................................
  //
  // Purpose: ZeroSuppress RawADC data for given string
  //          given Pedestals+Noise for every channel and Common Mode Noise for
  //          every chip
  //
  // Parameters:
  //              1) THRESH_MULTIPLIER
  //                 - ADC is how many times more than noise => hit?
  // 
  // Return codes:
  //              >=0  = Number of hits in the string
  //
  // Notes:
  //              i) ALL array indices begin from 0!
  //
  //              ii) Each RawADC value is cmpared to (THRESH_MULTIPLIER*Noise)
  //                  for that channel to decide if it is a hit or not
  //......................................................................
 
  //#define  THRESH_MULTIPLIER  3

  Int_t      i,j,k;
  Int_t      numHits=0;
  Float_t    signal;

  Int_t      channelsperchip, firstchipchannel;
  TPhChip   *chip = NULL;
  FECRawHit_t supHit;
 
  firstchipchannel = 0;
  channelsperchip  = 0;
  for(i=0; i<fNoChips; i++)
    {
      firstchipchannel += channelsperchip;
      chip = (TPhChip *) fChips->At(i);
      if(!chip) return -1;
      channelsperchip = chip->GetChipSize();
      // do we zero-suppress this chip 
      if((*fCMNFlag)[i]!=-1)
	{
	  for (j=0; j<channelsperchip; j++) {
	    k=firstchipchannel+j;
	    
	    signal=inhitstring[k].ADC-(*fPedMean)[k]-(*fCMNMean)[i];
	    
	    // using abs(..) here because want to see anything that is
	    // interesting, including negative hits caused by wrong polarity
	    // of bias voltages!
	    if( TMath::Abs(signal) > (THRESH_MULTIPLIER * (*fNoise)[k]) )
	      {
		// this is a hit, store it!

		//990303 mpd&hp: rounding the signal to an Int_t (as stored in supHit)
		supHit.ADC = TMath::Nint(signal);
		supHit.Channel = k;
		rawBufs0->Add(fStringNo, &supHit);

		numHits++;

		if(fDoHistograms) {
		  if(fZSSHistoChannels) {
		    if((*fZSSHistoChannels)[k]) 
		      ((TH1F *)fZSSHistos->At(k))->Fill(signal);
		    if((*fZSSHistoChannels)[kMaxLength]==-1) 
		      ((TH1F *)fZSSHistos->At(kMaxLength))->Fill(signal);
		  }
		}
	      }
	    else
	      {
		// this is an empty channel, use it to update the noise
		//
		// Heinz's Phd formula
		//
		// Noise_new^2 =  Noise_old^2*(w2-1) + Signal^2 
		//               -------------------------------
		//                             w2
		// we take w2 = 64 (time constant of the low-pass filter)
		
		(*fNoise)[k] = TMath::Sqrt( ( (*fNoise)[k]*(*fNoise)[k]*63
				        + (signal*signal)) / 64);

		//and pedestal 
		(*fPedMean)[k] = (*fPedMean)[k] + signal/64;

		if(fDoHistograms) {
		  if(fNoiseHistoChannels) {
		    if((*fNoiseHistoChannels)[k]) 
		      ((TH1F *)fNoiseHistos->At(k))->Fill((*fNoise)[k]);
		    if((*fNoiseHistoChannels)[kMaxLength]==-1) 
		      ((TH1F *)fNoiseHistos->At(kMaxLength))->Fill((*fNoise)[k]);
		  }
		  if(fPedMeanHistoChannels) {
		    if((*fPedMeanHistoChannels)[k]) 
		      ((TH1F *)fPedMeanHistos->At(k))->Fill((*fPedMean)[k]);
		    if((*fPedMeanHistoChannels)[kMaxLength]==-1) 
		      ((TH1F *)fPedMeanHistos->At(kMaxLength))->Fill((*fPedMean)[k]);
		  }
		}
	      }
	  }
	}
      else // this Chip had a funny CMN estimate, don't Zero Suppress it!
	{
	  for (j=0; j<channelsperchip; j++) {
	    k=firstchipchannel+j;
	      
	    signal=inhitstring[k].ADC-(*fPedMean)[k]-(*fCMNMean)[i];

	    //990303 mpd&hp: rounding the signal to an Int_t (as stored in supHit)
	    supHit.ADC = TMath::Nint(signal);
	    supHit.Channel = k;
	    rawBufs0->Add(fStringNo, &supHit);

	    numHits++;
	  }
	}
    }
  
  // and we're done...
  return(numHits);
}

Bool_t TPhString::AddChannelCalObject(Int_t channo, TPhChannelCalAbs *chobj)
{
  // Add a calibration object to the list 

  if(channo>fStringSize) return kFALSE;
  if(!fCalChannelArray) fCalChannelArray = new TObjArray(fStringSize);

  chobj->SetVerbose(fVerbose);

  if( fCalChannelArray->At(channo) ) return kFALSE;
  fCalChannelArray->AddAt(chobj,channo);

  return kTRUE;
}

Bool_t TPhString::MakeStringCalObject(TPhStringCalAbs *strcal)
{
  // Move information from channel calibration objects into a
  // summary string calibration object for this string 
  Int_t channo;
  TPhChannelCalAbs *chancal;

  if(!fCalChannelArray) return kFALSE;

  // Calibration already initialized
  // (may happen if reading parameters from several files)
  if(fCalStringObj) return kFALSE;

  fCalStringObj = strcal;
  fCalStringObj->SetVerbose(fVerbose);
  for(channo=0; channo<fStringSize; channo++) {
    chancal = (TPhChannelCalAbs *) fCalChannelArray->At(channo);
    if( !chancal ) {
      if(fVerbose) 
	Warning("MakeStringCalObject","No channel calibration object for " \
		"FEC = %d Str = %d Chan = %d\n", fFECNo, fStringNo, channo);
      fCalStringObj = NULL;
      return kFALSE;
    }
    if( !fCalStringObj->Add(channo,chancal) ) {
      if(fVerbose) 
	Warning("MakeStringCalObject","Error adding channel calibration object for " \
		"FEC = %d Str = %d Chan = %d\n", fFECNo, fStringNo, channo);
      fCalStringObj = NULL;
      return kFALSE;
    }
  }
  
  return kTRUE;
}

Bool_t TPhString::Calibrate(FECRawHit_t *hitstring, Int_t strlength)
{
  // This function looks for non-zero entries in FECRawHit_t structures 
  // and calibrates them 
  Bool_t             check=kTRUE;
  Int_t              entry;
  Short_t            channel;
  // 990319 HP change ADC from Short_t to Float_t
  Float_t            ADC;
  TPhChannelCalAbs  *calobj;
 
  if(!fCalChannelArray) {
    if( fVerbose ) 
      Warning("Calibrate", "Array with calibration objects does not exist " \
	      "for FEC=%d, String=%d\n", \
	      fFECNo, fStringNo);
    return kFALSE;
  }

  for(entry=0; entry<strlength; entry++) {
    //990318 HP pass ADC as float for calibration
    ADC = (Float_t)hitstring[entry].ADC;
    channel = hitstring[entry].Channel; 
    // one has to check all entries (there may be holes inside)
    if( !(ADC || channel) ) 
      continue;
    calobj = (TPhChannelCalAbs *) fCalChannelArray->At(channel);
    if(!calobj) {
      if( fVerbose ) 
	Warning("Calibrate", "Couldn't find calibration object " \
			     "for FEC=%d, String=%d, Chan=%d\n", \
			     fFECNo, fStringNo, channel);
      check = kFALSE;
    }

    check = check && calobj->Calibrate(ADC);
    // 990318 do proper rounding after calibration to convert back to short
    hitstring[entry].ADC = TMath::Nint(ADC);

    if(fDoHistograms) {
      if(fCalHistoChannels) {
	if((*fCalHistoChannels)[channel]) 
	  ((TH1F *)fCalHistos->At(channel))->Fill(ADC);
	if((*fCalHistoChannels)[kMaxLength]==-1) 
	  ((TH1F *)fCalHistos->At(kMaxLength))->Fill(ADC);
      }
    }
  }

  return check;
}

Bool_t TPhString::CalibrateOnString(FECRawHit_t *hitstring, Int_t strlength)
{
  // This function looks for non-zero entries in FECRawHit_t structures 
  // and calibrates them using the TPhStringCalAbs class object
  Bool_t check=kTRUE;
  Int_t entry, ADC, channel;

  if(!fCalStringObj) {
    if( fVerbose ) 
      Warning("CalibrateOnStrings", "Calibration object does not exist " \
	      "for FEC=%d, String=%d\n", \
	      fFECNo, fStringNo);
    return kFALSE;
  }
  check = fCalStringObj->Calibrate(hitstring, strlength);

  for(entry=0; entry<strlength; entry++) {
    ADC = hitstring[entry].ADC;
    channel = hitstring[entry].Channel; 
    if(!(ADC || channel)) 
      continue;
    
    if(fDoHistograms) {
      if(fCalHistoChannels) {
	if((*fCalHistoChannels)[channel]) 
	  ((TH1F *)fCalHistos->At(channel))->Fill(ADC);
	if((*fCalHistoChannels)[kMaxLength]==-1) 
	  ((TH1F *)fCalHistos->At(kMaxLength))->Fill(ADC);
      }
    }
  }
  
  return check;
}

void TPhString::Calibrate(Bool_t &status)
{
  // After Zero Suppression procedure has been run on raw data 
  // this function looks for non-zero entries in ZSS arrays
  // and calibrates them using individual channel calibration objects
  if( !(fZSSHitChan && fZSSHitADC) ) {
    if( fVerbose ) 
      Warning("Calibrate","no data found for calibration, FEC=%d String=%d.\n",
	      fFECNo,fStringNo);
    status = kTRUE;
    return;
  }
  if(fMode==kCalibratedMode) {
    if( fVerbose ) 
      Error("Calibrate","data already calibrated, will not repeat it, FEC=%d String=%d.\n",
	    fFECNo,fStringNo);
    status = kFALSE;
    return;
  }
  fMode=kCalibratedMode;

  Bool_t     check=kTRUE;
  Int_t      entry, noentries;
  Short_t    channel;
  // 990319 HP change ADC from Short_t to Float_t
  Float_t    ADC;
  TPhChannelCalAbs *calobj;
 
  if(!fCalChannelArray) {
    if( fVerbose ) 
      Warning("Calibrate", "Array with calibration objects does not exist " \
	      "for FEC=%d, String=%d\n", \
	      fFECNo, fStringNo);
    status = kFALSE;
    return;
  }

  noentries = fZSSHitChan->GetSize();
  for(entry=0; entry<noentries; entry++) {
    //    ADC = TMath::Nint((*fZSSHitADC)[entry]);
    // use ADC as float for calibration
    ADC = (*fZSSHitADC)[entry];
    channel = (*fZSSHitChan)[entry]; 
    if( !(ADC || channel) ) 
      break;
    calobj = (TPhChannelCalAbs *) fCalChannelArray->At(channel);
    if(!calobj) {
      if( fVerbose ) 
	Warning("Calibrate", "Couldn't find calibration object " \
			     "for FEC=%d, String=%d, Chan=%d\n", \
			     fFECNo, fStringNo, channel);
      check = kFALSE;
    }
    check = check && calobj->Calibrate(ADC);
    (*fZSSHitADC)[entry] = ADC;
    if(fDoHistograms) {
      if(fCalHistoChannels) {
	if((*fCalHistoChannels)[channel]) 
	  ((TH1F *)fCalHistos->At(channel))->Fill(ADC);
	if((*fCalHistoChannels)[kMaxLength]==-1) 
	  ((TH1F *)fCalHistos->At(kMaxLength))->Fill(ADC);
      }
    }
  }

  status = check;
  return;
}

void TPhString::MakeHitArrays(TPhHashContainer *HitArrayCont)
{
  // Create/fill hit arrays for data in zero-suppression buffers
  // Loop over entries in the zero-suppressed buffer 
  Int_t entry, noentries;
  Int_t ADC, channel;
  Int_t raw, col;
  Char_t SensorName[10];
  TPhSensorMapDB *map;
  TPhDetector    *det;
  TPhSensor      *sensor;
  TPhHitArray    *HitArray;

  det = TPhDetector::GetInstance();
  map = det->GetSensorMapDB();

  if(!fZSSHitChan || !fZSSHitADC) 
    return;

  noentries = fZSSHitChan->GetSize();
  for(entry=0; entry<noentries; entry++) {

    //990303 mpd&hp changed to rounding instead of truncating!!!
    ADC = TMath::Nint((*fZSSHitADC)[entry]);
    channel = (*fZSSHitChan)[entry]; 
    if(!ADC && !channel) 
      break;
    if(map->Lookup(fFECNo, fStringNo, channel, SensorName, &raw, &col)) {
      HitArray = NULL;
      HitArray = (TPhHitArray*) HitArrayCont->GetObject(SensorName);
      if( !HitArray ) {
	sensor = det->GetSensor(SensorName);
	if( sensor ) {
	  if( sensor->InheritsFrom("TPhSiSensor") ) {
	    HitArray = new TPhHitArray(sensor, SensorName, 
				       "Filled from strings zero-suppress buffers");
	    HitArrayCont->Add(HitArray);
	  } else {
	    if( fVerbose ) 
	      Warning("MakeHitArrays",
		      "Attempt to make HitArrays for a non TPhSiSensor object!");
	    sensor = NULL;
	  }
	} else {
	  if( fVerbose ) 
	    Warning("MakeHitArrays",
		    "Sensor name %s given by TPhSensorMapDB::Lookup " \
		    "not found in the geometry\n");
	}
      }
      if( HitArray ) HitArray->SetADC( raw, col, ADC );
    } else {
      if( fVerbose ) Warning("MakeHitArrays",
			     "Couldn't find sensor,row,column numbers " \
			     "for FEC=%d, String=%d, Chan=%d\n", \
			     fFECNo, fStringNo, channel);
    }
  }

  return;
}

void  TPhString::AddPedPreHistogram(Int_t no, Float_t xlow, Float_t xhigh, Int_t nbins)
{
  // Add a histogram request to a list of channels 
  // Create a histogram and add it to the list
  TH1F *histo;
  Char_t name[20];

  if(!fPedPreHistoChannels)   {
    fPedPreHistoChannels = new TArrayC(kMaxLength+1);
    fPedPreHistoChannels->Reset();
  }
  if(!fPedPreHistos) fPedPreHistos = new TObjArray(kMaxLength+1);

  sprintf(name,"PedPre_%d_%d_%d",fFECNo,fStringNo,no);
  histo = new TH1F(name,"Pedestal pre-evaluation",nbins,xlow,xhigh);
  if(no<0) {
    (*fPedPreHistoChannels)[kMaxLength] = -1;
    fPedPreHistos->AddAt(histo, kMaxLength);
  } else {
    (*fPedPreHistoChannels)[no] = kTRUE;
    fPedPreHistos->AddAt(histo,no);
  }
  return;
}

void  TPhString::AddPedMeanHistogram(Int_t no, Float_t xlow, Float_t xhigh, Int_t nbins)
{
  // Add a histogram request to a list of channels 
  // Create a histogram and add it to the list
  TH1F *histo;
  Char_t name[20];

  if(!fPedMeanHistoChannels)  {
    fPedMeanHistoChannels = new TArrayC(kMaxLength+1);
    fPedMeanHistoChannels->Reset();
  }
  if(!fPedMeanHistos) fPedMeanHistos = new TObjArray(kMaxLength+1);

  sprintf(name,"PedMean_%d_%d_%d",fFECNo,fStringNo,no);
  histo = new TH1F(name,"Pedestal mean evaluation",nbins,xlow,xhigh);
  if(no<0) {
    (*fPedMeanHistoChannels)[kMaxLength] = -1;
    fPedMeanHistos->AddAt(histo, kMaxLength);
  } else {
    (*fPedMeanHistoChannels)[no] = kTRUE;
    fPedMeanHistos->AddAt(histo,no);
  }

  return;
}

void  TPhString::AddPedRMSHistogram(Int_t no, Float_t xlow, Float_t xhigh, Int_t nbins)
{
  // Add a histogram request to a list of channels 
  // Create a histogram and add it to the list
  TH1F *histo;
  Char_t name[20];

  if(!fPedRMSHistoChannels)   {
    fPedRMSHistoChannels = new TArrayC(kMaxLength+1);
    fPedRMSHistoChannels->Reset();
  }
  if(!fPedRMSHistos) fPedRMSHistos = new TObjArray(kMaxLength+1);

  sprintf(name,"PedRMS_%d_%d_%d",fFECNo,fStringNo,no);
  histo = new TH1F(name,"Pedestal RMS evaluation",nbins,xlow,xhigh);
  if(no<0) {
    (*fPedRMSHistoChannels)[kMaxLength] = -1;
    fPedRMSHistos->AddAt(histo, kMaxLength);
  } else {
    (*fPedRMSHistoChannels)[no] = kTRUE;
    fPedRMSHistos->AddAt(histo,no);
  }

  return;
}

void  TPhString::AddCMNEventHistogram(Int_t no, Float_t xlow, Float_t xhigh, Int_t nbins)
{
  // Add a histogram request to a list of channels (chips)
  // Create a histogram and add it to the list
  // This histogram gives ADC-pedestal distribution for a single event 
  // and another distribution of the differences used for CMN average
  // (input to the CMN mean calculations)
  TH1F *histo;
  Char_t name[20];

  if(!fCMNEventHistoChannels)   {
    fCMNEventHistoChannels = new TArrayC(kMaxChipNumber+1);
    fCMNEventHistoChannels->Reset();
  }
  if(!fCMNEventHistos) fCMNEventHistos = new TObjArray(kMaxChipNumber+1);

  sprintf(name,"ADC-PED_%d_%d_%d",fFECNo,fStringNo,no);
  histo = new TH1F(name,name,nbins,xlow,xhigh);
  if(no<0) {
    (*fCMNEventHistoChannels)[kMaxChipNumber] = -1;
    fCMNEventHistos->AddAt(histo, kMaxChipNumber);
  } else {
    (*fCMNEventHistoChannels)[no] = kTRUE;
    fCMNEventHistos->AddAt(histo,no);
  }

  return;
}

void  TPhString::AddCMNMeanHistogram(Int_t no, Float_t xlow, Float_t xhigh, Int_t nbins)
{
  // Add a histogram request to a list of channels 
  // Create a histogram and add it to the list
  TH1F *histo;
  Char_t name[20];

  if(!fCMNMeanHistoChannels)   {
    fCMNMeanHistoChannels = new TArrayC(kMaxChipNumber+1);
    fCMNMeanHistoChannels->Reset();
  }
  if(!fCMNMeanHistos) fCMNMeanHistos = new TObjArray(kMaxChipNumber+1);

  sprintf(name,"CMNMean_%d_%d_%d",fFECNo,fStringNo,no);
  histo = new TH1F(name,"CMN mean evaluation",nbins,xlow,xhigh);
  if(no<0) {
    (*fCMNMeanHistoChannels)[kMaxChipNumber] = -1;
    fCMNMeanHistos->AddAt(histo, kMaxChipNumber);
  } else {
    (*fCMNMeanHistoChannels)[no] = kTRUE;
    fCMNMeanHistos->AddAt(histo,no);
  }

  return;
}

void  TPhString::AddCMNFHistogram(Int_t no, Float_t xlow, Float_t xhigh, Int_t nbins)
{
  // Add a histogram request to a list of channels 
  // Create a histogram and add it to the list
  TH1F *histo;
  Char_t name[20];

  if(!fCMNFHistoChannels)   {
    fCMNFHistoChannels = new TArrayC(kMaxChipNumber+1);
    fCMNFHistoChannels->Reset();
  }
  if(!fCMNFHistos) fCMNFHistos = new TObjArray(kMaxChipNumber+1);

  sprintf(name,"CMNF_%d_%d_%d",fFECNo,fStringNo,no);
  histo = new TH1F(name,"CMN flags",nbins,xlow,xhigh);
  if(no<0) {
    (*fCMNFHistoChannels)[kMaxChipNumber] = -1;
    fCMNFHistos->AddAt(histo, kMaxChipNumber);
  } else {
    (*fCMNFHistoChannels)[no] = kTRUE;
    fCMNFHistos->AddAt(histo,no);
  }

  return;
}

void  TPhString::AddNoiseHistogram(Int_t no, Float_t xlow, Float_t xhigh, Int_t nbins)
{
  // Add a histogram request to a list of channels 
  // Create a histogram and add it to the list
  TH1F *histo;
  Char_t name[20];

  if(!fNoiseHistoChannels)   {
    fNoiseHistoChannels = new TArrayC(kMaxLength+1);
    fNoiseHistoChannels->Reset();
  }
  if(!fNoiseHistos) fNoiseHistos = new TObjArray(kMaxLength+1);

  sprintf(name,"Noise_%d_%d_%d",fFECNo,fStringNo,no);
  histo = new TH1F(name,"Noise evaluation",nbins,xlow,xhigh);
  if(no<0) {
    (*fNoiseHistoChannels)[kMaxLength] = -1;
    fNoiseHistos->AddAt(histo, kMaxLength);
  } else {
    (*fNoiseHistoChannels)[no] = kTRUE;
    fNoiseHistos->AddAt(histo,no);
  }

  return;
}

void  TPhString::AddZSSHistogram(Int_t no, Float_t xlow, Float_t xhigh, Int_t nbins)
{
  // Add a histogram request to a list of channels 
  // Create a histogram and add it to the list
  TH1F *histo;
  Char_t name[20];

  if(!fZSSHistoChannels) {
    fZSSHistoChannels = new TArrayC(kMaxLength+1);
    fZSSHistoChannels->Reset();
  }
  if(!fZSSHistos) fZSSHistos = new TObjArray(kMaxLength+1);

  sprintf(name,"ZSS_%d_%d_%d",fFECNo,fStringNo,no);
  histo = new TH1F(name,"Zero suppressed",nbins,xlow,xhigh);
  if(no<0) {
    (*fZSSHistoChannels)[kMaxLength] = -1;
    fZSSHistos->AddAt(histo, kMaxLength);
  } else {
    (*fZSSHistoChannels)[no] = kTRUE;
    fZSSHistos->AddAt(histo,no);
  }

  return;
}

void  TPhString::AddCalHistogram(Int_t no, Float_t xlow, Float_t xhigh, Int_t nbins)
{
  // Add a histogram request to a list of channels 
  // Create a histogram and add it to the list
  TH1F *histo;
  Char_t name[20];

  if(!fCalHistoChannels)   {
    fCalHistoChannels = new TArrayC(kMaxLength+1);
    fCalHistoChannels->Reset();
  }
  if(!fCalHistos) fCalHistos = new TObjArray(kMaxLength+1);

  sprintf(name,"Cal_%d_%d_%d",fFECNo,fStringNo,no);
  histo = new TH1F(name,"Calibrated",nbins,xlow,xhigh);
  if(no<0) {
    (*fCalHistoChannels)[kMaxLength] = -1;
    fCalHistos->AddAt(histo, kMaxLength);
  } else {
    (*fCalHistoChannels)[no] = kTRUE;
    fCalHistos->AddAt(histo,no);
  }

  return;
}

void TPhString::DeleteHistograms()
{
  // Delete histograms, definitions and lists

    if(fPedPreHistoChannels)  { delete fPedPreHistoChannels;  fPedPreHistoChannels  = NULL; }  
    if(fPedMeanHistoChannels) { delete fPedMeanHistoChannels; fPedMeanHistoChannels = NULL; } 
    if(fPedRMSHistoChannels)  { delete fPedRMSHistoChannels;  fPedRMSHistoChannels  = NULL; } 
    if(fCMNEventHistoChannels){ delete fCMNEventHistoChannels;fCMNEventHistoChannels = NULL;}  
    if(fCMNMeanHistoChannels) { delete fCMNMeanHistoChannels; fCMNMeanHistoChannels = NULL; }  
    if(fCMNFHistoChannels)    { delete fCMNFHistoChannels;    fCMNFHistoChannels    = NULL; }   
    if(fNoiseHistoChannels)   { delete fNoiseHistoChannels;   fNoiseHistoChannels   = NULL; } 
    if(fZSSHistoChannels)     { delete fZSSHistoChannels;     fZSSHistoChannels     = NULL; }  
    if(fCalHistoChannels)     { delete fCalHistoChannels;     fCalHistoChannels     = NULL; } 

    if(fPedPreHistos)  { fPedPreHistos->Delete();  delete fPedPreHistos;  fPedPreHistos = NULL;  }
    if(fPedMeanHistos) { fPedMeanHistos->Delete(); delete fPedMeanHistos; fPedMeanHistos = NULL; }
    if(fPedRMSHistos)  { fPedRMSHistos->Delete();  delete fPedRMSHistos;  fPedRMSHistos = NULL;  }
    if(fCMNEventHistos){ fCMNEventHistos->Delete();delete fCMNEventHistos;fCMNEventHistos = NULL;} 
    if(fCMNMeanHistos) { fCMNMeanHistos->Delete(); delete fCMNMeanHistos; fCMNMeanHistos = NULL; } 
    if(fCMNFHistos)    { fCMNFHistos->Delete();    delete fCMNFHistos;    fCMNFHistos = NULL;    }
    if(fNoiseHistos)   { fNoiseHistos->Delete();   delete fNoiseHistos;   fNoiseHistos = NULL;   } 
    if(fZSSHistos)     { fZSSHistos->Delete();     delete fZSSHistos;     fZSSHistos = NULL;     }
    if(fCalHistos)     { fCalHistos->Delete();     delete fCalHistos;     fCalHistos = NULL;     }

    if(fPedPreSHisto)   { delete fPedPreSHisto;  fPedPreSHisto  = NULL; }
    if(fPedMeanSHisto)  { delete fPedMeanSHisto; fPedMeanSHisto = NULL; }
    if(fPedRMSSHisto)   { delete fPedRMSSHisto;  fPedRMSSHisto  = NULL; }
    if(fCMNMeanSHisto)  { delete fCMNMeanSHisto; fCMNMeanSHisto = NULL; }
    if(fCMNFSHisto)     { delete fCMNFSHisto;    fCMNFSHisto    = NULL; }
    if(fNoiseSHisto)    { delete fNoiseSHisto;   fNoiseSHisto   = NULL; }
    if(fZSSSHisto)      { delete fZSSSHisto;     fZSSSHisto     = NULL; }

    return;
}

TH1F* TPhString::GetHistogram(TObjArray *list, Int_t no)
{
  // Get pointer to specified histogram
  Int_t size;

  if(list) {
    if(no<0) {
      size = list->GetSize();
      return (TH1F *) list->At(size-1);
    } 
    return (TH1F *) list->At(no);
  }
  else return NULL;
}

void  TPhString::DrawHistogram(TObjArray *list, Int_t no)
{
  // Draw specified histogram in the current pad
  TH1F *histo;
  
  histo = GetHistogram(list, no);
  if(histo) histo->Draw();

  return;
}

void  TPhString::DrawCMNEventHistogram(Int_t no)
{
  // Draw specified histogram in the current pad
  // --------------------------------------------
  // ! Memory leak left - arrow is not deleted, !
  // ! to stay visible on the histogram         !
  // --------------------------------------------
  TArrow *ar1;
  TH1F *histo;
  Float_t mean, max;
  
  histo = GetHistogram(fCMNEventHistos, no);
  if(histo) {
    histo->Draw();
    mean = (Float_t ) (*fCMNMean)[no];
    max = histo->GetMaximum();
    ar1 = new TArrow(mean, 0, mean, max/6.0);
    ar1->SetLineWidth(3);
    ar1->SetLineColor(2);
    ar1->SetAngle(30);
    ar1->SetOption("<");
    ar1->Draw();
  }

  return;
}

void  TPhString::DrawPedPreSHisto()
{
  // Draw summary histogram - current image of the array contents
  // Pedestal pre-evaluation

  CreatePedPreSHisto();
  fPedPreSHisto->Draw();

  return;
}
  
void  TPhString::DrawPedMeanSHisto()
{
  // Draw summary histogram - current image of the array contents
  // Pedestal mean evaluation

  CreatePedMeanSHisto();
  fPedMeanSHisto->Draw();

  return;
}
  
void  TPhString::DrawPedRMSSHisto()
{
  // Draw summary histogram - current image of the array contents
  // Pedestal RMS evaluation

  CreatePedRMSSHisto();
  fPedRMSSHisto->Draw();

  return;
}
  
void  TPhString::DrawCMNMeanSHisto()
{
  // Draw summary histogram - current image of the array contents
  // CMN mean evaluation

  CreateCMNMeanSHisto();
  fCMNMeanSHisto->Draw();

  return;
}
  
void  TPhString::DrawCMNFSHisto()
{
  // Draw summary histogram - current image of the array contents
  // CMN flags

  CreateCMNFSHisto();
  fCMNFSHisto->Draw();

  return;
}
  
void  TPhString::DrawNoiseSHisto()
{
  // Draw summary histogram - current image of the array contents
  // Noise evaluation

  CreateNoiseSHisto();
  fNoiseSHisto->Draw();

  return;
}

void  TPhString::DrawZSSSHisto()
{
  // Draw summary histogram - current image of the array contents
  // Zero suppression/calibration evaluation

  CreateZSSSHisto();
  fZSSSHisto->Draw();

  return;
}
  
TH1F *TPhString::CreatePedPreSHisto()
{
  // Create summary histogram - current image of the array contents
  // Pedestal pre-evaluation
  Int_t size;
  Stat_t cont;
  Char_t name[20];

  if(!fPedPreMean) return NULL;
  size = fPedPreMean->GetSize();  
  if(!fPedPreSHisto) {
    sprintf(name,"PedPre_Summ_%d_%d",fFECNo,fStringNo);
    fPedPreSHisto = new TH1F(name,name,size,0,size);
  } 
  while(size--) {
    cont = (Stat_t) (*fPedPreMean)[size];
    fPedPreSHisto->SetBinContent(size+1, cont);
  }
  cont = (Stat_t) (*fPedPreMean)[0];
  fPedPreSHisto->SetBinContent(1, cont);

  return fPedPreSHisto;
}
  
TH1F *TPhString::CreatePedMeanSHisto()
{
  // Create summary histogram - current image of the array contents
  // Pedestal mean evaluation
  Int_t size;
  Stat_t cont;
  Char_t name[20];

  if(!fPedMean) return NULL;
  size = fPedMean->GetSize();  
  if(!fPedMeanSHisto) {
    sprintf(name,"PedMean_Summ_%d_%d",fFECNo,fStringNo);
    fPedMeanSHisto = new TH1F(name,name,size,0,size);
  } 
  while(size--) {
    cont = (Stat_t) (*fPedMean)[size];
    fPedMeanSHisto->SetBinContent(size+1, cont);
  }
  cont = (Stat_t) (*fPedMean)[0];
  fPedMeanSHisto->SetBinContent(1, cont);

  return fPedMeanSHisto;
}
  
TH1F *TPhString::CreatePedRMSSHisto()
{
  // Create summary histogram - current image of the array contents
  // Pedestal RMS evaluation
  Int_t size;
  Stat_t cont;
  Char_t name[20];

  if(!fPedRMS) return NULL;
  size = fPedRMS->GetSize();  
  if(!fPedRMSSHisto) {
    sprintf(name,"PedRMS_Summ_%d_%d",fFECNo,fStringNo);
    fPedRMSSHisto = new TH1F(name,name,size,0,size);
  } 
  while(size--) {
    cont = (Stat_t) (*fPedRMS)[size];
    fPedRMSSHisto->SetBinContent(size+1, cont);
  }
  cont = (Stat_t) (*fPedRMS)[0];
  fPedRMSSHisto->SetBinContent(1, cont);

  return fPedRMSSHisto;
}
  
TH1F *TPhString::CreateCMNMeanSHisto()
{
  // Create summary histogram - current image of the array contents
  // CMN mean evaluation
  Int_t size;
  Stat_t cont;
  Char_t name[20];

  if(!fCMNMean) return NULL;
  size = fCMNMean->GetSize();  
  if(!fCMNMeanSHisto) {
    sprintf(name,"CMNMean_Summ_%d_%d",fFECNo,fStringNo);
    fCMNMeanSHisto = new TH1F(name,name,size,0,size);
  } 
  while(size--) {
    cont = (Stat_t) (*fCMNMean)[size];
    fCMNMeanSHisto->SetBinContent(size+1, cont);
  }
  cont = (Stat_t) (*fCMNMean)[0];
  fCMNMeanSHisto->SetBinContent(1, cont);

  return fCMNMeanSHisto;
}
  
TH1F *TPhString::CreateCMNFSHisto()
{
  // Create summary histogram - current image of the array contents
  // CMN flags
  Int_t size;
  Stat_t cont;
  Char_t name[20];

  if(!fCMNFlag) return NULL;
  size = fCMNFlag->GetSize();  
  if(!fCMNFSHisto) {
    sprintf(name,"CMNFlag_Summ_%d_%d",fFECNo,fStringNo);
    fCMNFSHisto = new TH1F(name,name,size,0,size);
  } 
  while(size--) {
    cont = (Stat_t) (*fCMNFlag)[size];
    fCMNFSHisto->SetBinContent(size+1, cont);
  }
  cont = (Stat_t) (*fCMNFlag)[0];
  fCMNFSHisto->SetBinContent(1, cont);

  return fCMNFSHisto;
}
  
TH1F *TPhString::CreateNoiseSHisto()
{
  // Create summary histogram - current image of the array contents
  // Noise evaluation
  Int_t size;
  Stat_t cont;
  Char_t name[20];

  if(!fNoise) return NULL;
  size = fNoise->GetSize();  
  if(!fNoiseSHisto) {
    sprintf(name,"Noise_Summ_%d_%d",fFECNo,fStringNo);
    fNoiseSHisto = new TH1F(name,name,size,0,size);
  } 
  while(size--) {
    cont = (Stat_t) (*fNoise)[size];
    fNoiseSHisto->SetBinContent(size+1, cont);
  }
  cont = (Stat_t) (*fNoise)[0];
  fNoiseSHisto->SetBinContent(1, cont);

  return fNoiseSHisto;
}
  
TH1F *TPhString::CreateZSSSHisto()
{
  // Create summary histogram - current image of the array contents
  // Zero suppression/calibration evaluation
  Int_t size, chno, step=-1;
  Stat_t cont;
  Char_t name[20];

  if(!fZSSHitADC) return NULL;
  size = fZSSHitADC->GetSize();  
  if(!fZSSSHisto) {
    sprintf(name,"ZSS_Summ_%d_%d",fFECNo,fStringNo);
    fZSSSHisto = new TH1F(name,name,size,0,size);
  } else fZSSSHisto->Reset();
  while(++step<size) {
    chno = (Int_t) (*fZSSHitChan)[step];
    cont = (Stat_t) (*fZSSHitADC)[step];
    if( !(chno || ((Bool_t) cont)) ) break;
    fZSSSHisto->SetBinContent(chno+1, cont);
  }

  return fZSSSHisto;
}
  
void  TPhString::ls(Option_t *option) 
{
  //
    IndentLevel();
    cout << IsA()->GetName() << " Name: " << GetName() << 
      " No: " << fStringNo << " Size: " << fStringSize << " Chips: " << fNoChips << endl;
    if(strstr(option, "D") || strstr(option, "d")) {
      cout << "   " << " PMeans:  " << fPedMean    << " PRMS     " << fPedRMS << endl <<
	      "   " << " CMNMean: " << fCMNMean    << " CMNFlag: " << fCMNFlag << endl <<
	      "   " << " Noise:   " << fNoise      << endl <<
	      "   " << " ZZChan:  " << fZSSHitChan << " ZZADC:   " << fZSSHitADC << endl;
    }
    TObject::IncreaseDirLevel();

    if(strstr(option, "C") || strstr(option, "c"))
       fChips->ls(option);

    if(strstr(option, "D") || strstr(option, "d")) 
      fChips->ls(option);
    TObject::DecreaseDirLevel();
}

void TPhString::DumpArrayS(TArrayS *ar, Int_t no)
{
  // Print contents of several (no) members of the ar array
  Int_t i;
  Int_t size;
  if(!ar || !(size=ar->GetSize())) return;
  if(!no) no = size;
  if(size < no) no = size;
  for(i=0; i<no; i++) {
    cout << setw(5) << i << ". " << setw(8) << (*ar)[i] << " ";
    if(!((i+1)%5)) cout << endl;
  }
  cout << endl;
  return;
}

void TPhString::DumpArrayF(TArrayF *ar, Int_t no)
{
  // Print contents of several (no) members of the ar array
  Int_t i;
  Int_t size;
  if(!ar || !(size=ar->GetSize())) return;
  if(!no) no = size;
  if(size < no) no = size;
  for(i=0; i<no; i++) {
    cout << setw(5) << i << ". " << setw(8) << (*ar)[i] << " ";
    if(!((i+1)%5)) cout << endl;
  }
  cout << endl;
  return;
}

void TPhString::DumpArrayD(TArrayD *ar, Int_t no)
{
  // Print contents of several (no) members of the ar array
  Int_t i;
  Int_t size;
  if(!ar || !(size=ar->GetSize())) return;
  if(!no) no = size;
  if(size < no) no = size;
  for(i=0; i<no; i++) {
    cout << setw(5) << i << ". " << setw(8) << (*ar)[i] << " ";
    if(!((i+1)%5)) cout << endl;
  }
  cout << endl;
  return;
}

void TPhString::DumpArrayI(TArrayI *ar, Int_t no)
{
  // Print contents of several (no) members of the ar array
  Int_t i;
  Int_t size;
  if(!ar || !(size=ar->GetSize())) return;
  if(!no) no = size;
  if(size < no) no = size;
  for(i=0; i<no; i++) {
    cout << setw(5) << i << ". " << setw(8) << (*ar)[i] << " ";
    if(!((i+1)%5)) cout << endl;
  }
  cout << endl;
  return;
}

void  TPhString::Print(Option_t *option) 
{
  //

}
