#include "TPhSubDetector.h"
#include "TPhSpectSensor.h"
#include "TPhLayer.h"
#include "TPhHit.h"
#include "TPhSensor.h"

#include <iostream.h>

ClassImp(TPhSubDetector)
//////////////////////////////////////////////////////////////////////////
//
// The SubDetector class gives access to the various pieces of the Phobos 
// layers. There are the following SubDetectors:
//         "RingMult" Ring Multiplicity detectors
//         "Octagon" Octagon Mult detector detector
//         "Vertex" Vertex detector
//         "SpectP" Positive Spectrometer arm
//         "SpectN" Negative Spectrometer arm
//         "TOFP" Positive Time of flight detector
//         "TOFN" Positive Time of flight detector
//         "Trigger" Trigger detectors
//
// These subdetectors are currently defined in TPhASCIIData when the 
// geometry is read in. 
//
// Author: Patrick Decowski
// $Id: TPhSubDetector.cxx,v 1.11 1999/03/12 08:05:05 decowski Exp $
//
//////////////////////////////////////////////////////////////////////////

TPhSubDetector::TPhSubDetector() 
{
  // Default constructor.

  fTurnedOn = kTRUE;   // Turn the SubDetector ON by default
  fLayerList = NULL;
  fSensorList = NULL;
}

TPhSubDetector::TPhSubDetector(const Char_t *name) : TNamed(name, name)
{
// Normal SubDetector constructor.

  fTurnedOn = kTRUE; // Turn the SubDetector ON by default
  fLayerList = NULL;
  fSensorList = NULL;
}

TPhSubDetector::~TPhSubDetector()
{
  // Normal destructor

  if(fLayerList) {
    fLayerList->Delete();
    delete fLayerList;
  }
}

void TPhSubDetector::AddLayer(TPhLayer *Layer)
{
  // Add a layer to the Subdetector. Used in the TPhASCIIData class. 

  if(!fLayerList)
    fLayerList = new TObjArray();
  fLayerList->Add(Layer);
}

void TPhSubDetector::AddHit(TPhHit *Hit)
{
  // Add a hit to the specified layer

  TIter NextLayer(fLayerList);
  TPhLayer *Layer;
  // Find the layer with the same layernumber as the one in the hit. 
  while ((Layer = (TPhLayer*) NextLayer()) != NULL) {
    if (Layer->GetLabel() == Hit->GetLayer())
      Layer->AddHit(Hit);
  }
}

void TPhSubDetector::AddSensor(TPhSensor *sensor)
{
  // Add a sensor to this layer
  TPhLayer *Layer;

  // Declare the object array if necessary
  if(!fLayerList)
    fLayerList = new TObjArray();
  // find the layer according to the number or make a new one if it is 
  // not found
  if((Layer = (TPhLayer*) fLayerList->At(sensor->GetLayer())) == NULL) {
    Layer = new TPhLayer();
    Layer->SetLabel(sensor->GetLayer());
    fLayerList->Add(Layer);
  }

  Layer->AddSensor(sensor);
}


void TPhSubDetector::BuildSensorList()
{
  // Build a hashtable of sensors in the subdetector out of the sensors
  // contained in the layers inside. 

  if(!fLayerList || fSensorList)
    return;

  fSensorList = new THashTable();

  // Iterate over all layers and add the sensors to the list
  TIter NextLayer(fLayerList);
  TPhLayer *Layer;
  TPhSensor *Sensor;
  while ((Layer = (TPhLayer*) NextLayer()) != NULL) {
    TIter NextSensor(Layer->GetSensorList());
    while ((Sensor = (TPhSensor*) NextSensor()) != NULL)
      fSensorList->Add(Sensor);
  }
}

void TPhSubDetector::Draw(Option_t *option)
{
  // Draw all layers in this subdetector

  // Is the SubDetector turned on? 
  if(!IsOn()) return;

  if(fLayerList) {
    fLayerList->ForEach(TPhLayer,Draw)(option);
  }
}

Bool_t TPhSubDetector::IsThere(const Char_t* Name)
{
  // Check if the sensor of a given name exists in the subdetector
  // 
  // Return: kTRUE if sensor exists 
  //         kFALSE if it does not. 

  if(!fSensorList)
    BuildSensorList();
  // if there still aren't any sensors, then we give up. 
  if(!fSensorList) 
    return kFALSE;
  if(fSensorList->FindObject(Name) != NULL)
    return kTRUE;
  else
    return kFALSE;
}

TIterator* TPhSubDetector::GetSensorIter(const Bool_t dir)
{
  // Return an iterator over all sensors inside the subdetector. 
  // Builds the SensorList if neccessary
  
  if(!fSensorList)
    BuildSensorList();
  return fSensorList->MakeIterator(dir);
}

void TPhSubDetector::CalcMaxSensorHeight()
{
  // Calcute max sensor height. Use GetSensorHeight() to retrieve the value. 

    TPhLayer  *layer;
    TPhSensor *sensor;
    TPhSpectSensor *spectSensor;

    Float_t sensorHeight;

    fMaxSensorHeight = 0;    
   
    TIter NextLayer(GetLayerList());
    while((layer = (TPhLayer *) NextLayer()) != NULL)
    {
        TIter NextSensor(layer->GetSensorList());
        while((sensor = (TPhSensor *) NextSensor()) != NULL)
        {
	  // Make sure it is a Spectrometer sensor.
	  if(sensor->InheritsFrom(TPhSpectSensor::Class())) {
	    spectSensor = (TPhSpectSensor*) sensor;
	    sensorHeight = spectSensor->GetSensorHeight();
	    if(sensorHeight > fMaxSensorHeight) 
	      fMaxSensorHeight = sensorHeight;
	  }
        }
    }
}

void TPhSubDetector::Print() const
{
  // Print information about this SubDetector and all the sensors inside it. 

  TPhLayer *layer;
  TPhSensor *sensor;

  // Is the SubDetector turned on? 
  if(!IsOn()) return;

  if(GetLayerList()) printf("SubDetector: %s\n", GetName());
  TIter NextLayer(GetLayerList());
  while((layer = (TPhLayer *) NextLayer()) != NULL) {
    TIter NextSensor(layer->GetSensorList());
    while((sensor = (TPhSensor *) NextSensor()) != NULL)
      sensor->Print();
  }
}
 
void TPhSubDetector::ls(Option_t* /* Option */) const
{
  // Print information about the geometry tree

  // Is the SubDetector turned on? 
  if(!IsOn()) return;
  
  IndentLevel();
  cout << "SubDetector "<< GetName() << endl;
  TObject::IncreaseDirLevel();
  // need to still do this!
  //fLayerList->ls(option);
  
  TObject::DecreaseDirLevel();
  

}

Int_t TPhSubDetector::MakeHitArraysOutOfHits(Char_t *HitConName, 
					     Char_t *HitArrayConName)
{
  // Loop over all subdetectors and make the HitArrays

  // Is the SubDetector turned on? 
  if(!IsOn()) return 0;

  Int_t NumHitArrays = 0;
  TIter NextLayer(fLayerList);
  TPhLayer *Layer;
  while ((Layer = (TPhLayer*) NextLayer()) != NULL)
    NumHitArrays += Layer->MakeHitArraysOutOfHits(HitConName, 
						  HitArrayConName);

  return NumHitArrays;
}

Int_t TPhSubDetector::MakeHitsOutOfHitArrays(Char_t *HitArrayConName, 
					     Char_t *HitConName)
{
  // Loop over all subdetectors and make the Hits

  // Is the SubDetector turned on? 
  if(!IsOn()) return 0;

  Int_t NumHits = 0;
  TIter NextLayer(fLayerList);
  TPhLayer *Layer;
  while ((Layer = (TPhLayer*) NextLayer()) != NULL)
    NumHits += Layer->MakeHitsOutOfHitArrays(HitArrayConName, HitConName);

  return NumHits;
}

TPhLayer *TPhSubDetector::GetLayer(const Int_t i) const
{
  // Return the layer at position i

  if(fLayerList)
    return (TPhLayer*) fLayerList->At(i);
  else 
    return NULL;
}

void TPhSubDetector::DeleteHits()
{
  if(fLayerList) {
    fLayerList->ForEach(TPhLayer,DeleteHits)();
  }
}

void TPhSubDetector::ClearHits()
{
  if(fLayerList) {
    fLayerList->ForEach(TPhLayer,ClearHits)();
  }
}


void TPhSubDetector::TurnOn()
{
  // Turn the SubDetector on. 

  fTurnedOn = kTRUE;
}

void TPhSubDetector::TurnOff()
{
  // Turn the SubDetector off. 

  fTurnedOn = kFALSE;
}

