//*-- Author : patrick decowski

#include <string.h>
#include <stdio.h>

#include "TPhToscaMagnet.h"
#include "TMath.h"
#include "TDatime.h"
#include "TPolyLine3D.h"
#include "TRandom.h"
#include "TObjArray.h"
#include "TFile.h"

ClassImp(TPhToscaMagnet)

//////////////////////////////////////////////////////////////////////////////
// This class implements the simulation of the magnetic field into ROOT. 
// The normal user only has to do the following:
//
// TPhToscaMagnet *magnet;
// TFile f("PhobosMagnet.root");
// magnet = (TPhToscaMagnet *) f.Get("magnet");
// f.Close();
//
// The bfield at a specific point can then be calculated:
//
// Float_t x, y, z, Bx, By, Bz;
// magnet->GetField(x, y, z, &Bx, &By, &Bz);
//
// $Id: TPhToscaMagnet.cxx,v 1.2 1999/01/11 19:02:24 decowski Exp $
//
//////////////////////////////////////////////////////////////////////////////

  TPhToscaMagnet::TPhToscaMagnet()
{
// Phobos Magnet default constructor

}

TPhToscaMagnet::~TPhToscaMagnet()
{
// Phobos Magnet default destructor

  if(fPolyLine3D)
    fPolyLine3D->Delete();

  delete [] fFineGrid[0];
  delete [] fFineGrid[1];
  delete [] fFineGrid[2];

  delete [] fCoarseGrid[0];
  delete [] fCoarseGrid[1];
  delete [] fCoarseGrid[2];
}


void TPhToscaMagnet::MakeFieldMap(const Char_t* FileNameFineGrid,
		const Char_t* FileNameCoarseGrid, const Char_t* version)
{
// This function processes the ASCII valued field maps and loads them into
// an array in memory. It only needs to be called once, to produce a root 
// file. 
// The field map consists of two different grids supplied by TOSCA:
// 1. a fine grid that takes care of the magnetic field inside the 
// spectrometer. 
// 2. a coarser grid that extends the grid to the outer parts of the 
// magnet. 

  Int_t i, j, offset;
  TDatime date;

// Copy the filenames for keeps
  strcpy(fFNameFine, FileNameFineGrid);
  strcpy(fFNameCoarse, FileNameCoarseGrid);

// Store the date in TDatime format
  date.Set();
  fDate = date.Get();

  for(i = 0; i < 3; i++) 
    fFineGrid[i] = (Float_t*)new Float_t[200000];

  if((fFineNGridPoints = TPhToscaMagnet::ReadGrid(fFNameFine,
     fFineGrid, fMinFine, fMaxFine, fDeltaFine)) == -1) {
    Error("MakeFieldMap", "Fieldmap failed");
    return;
  }

  for(i = 0; i < 3; i++) 
    fCoarseGrid[i] = (Float_t*) new Float_t[50000];

  if((fCoarseNGridPoints = TPhToscaMagnet::ReadGrid(fFNameCoarse,
    fCoarseGrid, fMinCoarse, fMaxCoarse, fDeltaCoarse)) == -1) {
    Error("MakeFieldMap", "Fieldmap failed");
    return;
  }

//Store the version info
  strcpy(fVersion, version);

//Calculate the number of grid points along the three axis
  for(i = 0; i < 3; i++) {
    fNPointsFine[i] = 
      (Int_t)((fMaxFine[i] - fMinFine[i]) / fDeltaFine[i] + 1);
    fNPointsCoarse[i] = 
      (Int_t)((fMaxCoarse[i] - fMinCoarse[i]) / fDeltaCoarse[i] + 1);
  }

  fGridNodesFine = (Float_t*) new Float_t[fNPointsFine[0] 
			   + fNPointsFine[1] + fNPointsFine[2]];
  fGridNodesCoarse = (Float_t*) new Float_t[fNPointsCoarse[0] 
			   + fNPointsCoarse[1] + fNPointsCoarse[2]];
  offset = 0;

  for(j = 0; j < 3; j++) {
    for(i = 0; i < fNPointsFine[j]; i++) {
      fGridNodesFine[i + offset] = i * fDeltaFine[j] + fMinFine[j];
    }
    offset += fNPointsFine[j];
  }

  offset = 0;
  for(j = 0; j < 3; j++) {
    for(i = 0; i < fNPointsCoarse[j]; i++) 
      fGridNodesCoarse[i + offset] = i * fDeltaCoarse[j] + fMinCoarse[j];
    offset += fNPointsCoarse[j];
  }
}

Bool_t TPhToscaMagnet::GetField(Float_t* Coord, Float_t* B)
{
// This function gives the magnetic field. It calculates the field 
// by interpolating between two adjacent gridpoints. 
// It returns kTRUE for a valid field point and kFALSE for a field point
// outside the area.

  Int_t  i;

  if(Coord[0] < fMaxFine[0] && Coord[1] < fMaxFine[1] 
     && Coord[2] < fMaxFine[2]) {

//We are in the fine mesh region.    
    Interpolate(Coord, B, fNPointsFine, fGridNodesFine, fFineGrid);

//Translate from Gaus to Tesla
    for(i=0;i<3;i++)
        B[i] *= 0.0001;
  }
  else if (Coord[0] < fMaxCoarse[0] && Coord[1] < fMaxCoarse[1] 
	   && Coord[2] < fMaxCoarse[2]) {

//We are in the coarse mesh region.    
    Interpolate(Coord, B, fNPointsCoarse, fGridNodesCoarse, fCoarseGrid);

//Translate from Gaus to Tesla
    for(i=0;i<3;i++)
        B[i] *= 0.0001;
  }

  else {
    Warning("GetField","Point (%g, %g, %g) is outside of mesh\n",
	   Coord[0], Coord[1], Coord[2]);
    B[0] = 0; B[1] = 0; B[2] = 0;
    return kFALSE;
  }

  return kTRUE;

}

Bool_t TPhToscaMagnet::GetField(Float_t X, Float_t Y, Float_t Z,
		      Float_t* Bx, Float_t* By, Float_t* Bz)
{
// Another method of accessing the B-field.

  Float_t Coord[3];
  Float_t B[3];
  Int_t ret_value;

  Coord[0] = X;
  Coord[1] = Y;
  Coord[2] = Z;

  ret_value = TPhToscaMagnet::GetField(Coord, B);

  *Bx = B[0];
  *By = B[1];
  *Bz = B[2];
  
  return ret_value;
}

void TPhToscaMagnet::Print(Float_t x, Float_t y, Float_t z)
{
// Small routine meant for diagnostics: it will 
// print the B-field for a given set of coordinates.

  Float_t Bx, By, Bz;
  TPhToscaMagnet::GetField(x, y, z, &Bx, &By, &Bz);
  printf("(%g, %g, %g) ---> Bx: %g, By: %g, Bz: %g\n", x, y, z, Bx, By, Bz);

}

void TPhToscaMagnet::ls()
{
// Print some general info about the field map.

  Int_t date, time;

  TDatime::GetDateTime(fDate, date, time);
  printf("===============================================================\n");
  printf("\nFieldmap Version: %s from %d @ %d\n\n", GetVersion(), date, time);
  printf("===============================================================\n");
  printf("\n Fine Grid\n");
  printf("-----------\n");
  printf("Filename: %s\n", fFNameFine);
  printf("Distance between gridpoints: %g %g %g\n", 
	 fDeltaFine[0], fDeltaFine[1], fDeltaFine[2]);
  printf("Minimum: %g %g %g\n", fMinFine[0], fMinFine[1], fMinFine[2]);
  printf("Maximum: %g %g %g\n\n", fMaxFine[0], fMaxFine[1], fMaxFine[2]);
  printf(" Coarse Grid\n");
  printf("-------------\n");
  printf("Filename: %s\n", fFNameCoarse);
  printf("Distance between gridpoints: %g %g %g\n", 
	 fDeltaCoarse[0], fDeltaCoarse[1], fDeltaCoarse[2]);
  printf("Minimum: %g %g %g\n", fMinCoarse[0], fMinCoarse[1], fMinCoarse[2]);
  printf("Maximum: %g %g %g\n", fMaxCoarse[0], fMaxCoarse[1], fMaxCoarse[2]);
}

void TPhToscaMagnet::Draw3D(Float_t Length, Float_t Normalization, Float_t Rib)
{
// Draw a 3D representation of the magnetic field. Note that for this to 
// function correctly you must have a view all setup.

  Float_t x, y, z;
  Float_t halfX, halfY, halfZ;
  Float_t XRnd, YRnd, ZRnd;
  Float_t Bx, By, Bz;
  Float_t Babs;
  Color_t Color;

  Int_t i;
  TPolyLine3D *Arrow;

  if(!fPolyLine3D)
    fPolyLine3D = new TObjArray();
  else
    fPolyLine3D->Delete();

  for(x = fMinCoarse[0]; x < fMaxCoarse[0]; x += Rib) {
    for(y = fMinCoarse[1]; y < fMaxCoarse[1]; y += Rib) {
      for(z = fMinCoarse[2]; z < fMaxCoarse[2]; z += Rib) {
	GetField(x, y, z, &Bx, &By, &Bz);
	Babs = sqrt(Bx*Bx + By*By + Bz*Bz);

	//if(Babs < 1 && Babs > 0.01) 

	for(i = 0; i < TMath::Nint(Babs/Normalization) + 1; i++) {
	  XRnd = Rib * gRandom->Rndm() - Rib/2 + x;
	  YRnd = Rib * gRandom->Rndm() - Rib/2 + y;
	  ZRnd = Rib * gRandom->Rndm() - Rib/2 + z;
	
	  halfX = Bx / Babs * Length / 2;
	  halfY = By / Babs * Length / 2;
	  halfZ = Bz / Babs * Length / 2;

	  Arrow = new TPolyLine3D(2);
	  Arrow->SetPoint(0, XRnd + halfX, YRnd + halfY, ZRnd + halfZ);
	  Arrow->SetPoint(1, XRnd - halfX, YRnd - halfY, ZRnd - halfZ);

	  if(Babs != 0) {
	    Color = TMath::Nint(TMath::Log(Babs)) + 7 ;
	    if (Color > 1) { 
	      Arrow->SetLineColor(Color);
	      Arrow->Draw();
	      fPolyLine3D->Add(Arrow);
	    }
	    else
	      delete Arrow;
	  }
	}
      }
    }
  }
}

void TPhToscaMagnet::MakeVRML(Float_t Length, Float_t Normalization,
			 Float_t Rib)
{
// Make a VRML file that is suitable to be viewed with a VRML 
// viewer. The file name will be the version file name with 
//  extension wrl. 

  Float_t x, y, z;
  Float_t halfX, halfY, halfZ;
  Float_t XRnd, YRnd, ZRnd;
  Float_t Bx, By, Bz, Babs;

  UInt_t i;

  FILE *handle;

  Char_t filename[200];
  UInt_t color;

// Lookup colormap to get the right colors for the magnetic field intensity.
  struct { Float_t R, G, B; } Colormap[] = {
		     {1, 0, 0}, {0, 1, 0.12}, {0.22, 0, 1},
		     {1, 0.33, 0}, {0, 1, 0.45}, {0.55, 0, 1},
		     {1, 0.67, 0}, {0, 1, 0.78}, {0.88, 0, 1},
		     {1, 1, 0}, {0, 0.88, 1}, {1, 0, 0.77},
		     {0.67, 1, 0}, {0, 0.55, 1}, {1, 0, 0.43},
		     {0.33, 1, 0}, {0, 0.22, 1}
  };
  
  strcpy(filename, fFNameFine);
  strcat(filename, ".wrl");

// Open file and write a preamble (this is not a comment! The viewer actually
// needs it)
  handle = fopen(filename , "w") ;
  fprintf(handle, "#VRML V1.0 ascii\n");

  for(x = 0; x < fMaxCoarse[0]; x += Rib) {
    for(y = 0; y < fMaxCoarse[1]; y += Rib) {
      for(z = 0; z < fMaxCoarse[2]; z += Rib) {
	GetField(x, y, z, &Bx, &By, &Bz);
	Babs = sqrt(Bx*Bx + By*By + Bz*Bz);

	for(i = 0; i < (UInt_t) Babs/Normalization; i++) {
	  XRnd = Rib * gRandom->Rndm() - Rib/2 + x;
	  YRnd = Rib * gRandom->Rndm() - Rib/2 + y;
	  ZRnd = Rib * gRandom->Rndm() - Rib/2 + z;
	
	  halfX = Bx / Babs * Length / 2;
	  halfY = By / Babs * Length / 2;
	  halfZ = Bz / Babs * Length / 2;

// Calculate the RGB values for the color map. 
	  color = (UInt_t)(Babs/23000 * 16);
// 	  printf("color: %d ->%g %g %g\n", color, 
// 		 Colormap[color].R, Colormap[color].G, Colormap[color].B );
// Encode the B-field into VRML statements. 
	  fprintf(handle,
	  "Coordinate3 {\n\tpoint [ %g %g %g,\n\t\t %g %g %g ] \n\t}\n",
	  XRnd + halfX, YRnd + halfY, ZRnd + halfZ,
	  XRnd - halfX, YRnd - halfY, ZRnd - halfZ);
	  fprintf(handle,
	  "Material { emissiveColor %g %g %g } \n",
		  Colormap[color].R, Colormap[color].G, Colormap[color].B );
	  fprintf(handle,
	  "IndexedLineSet { coordIndex [ 0, 1, -1 ] }\n\n");
	}
      }
    }
  }
  fclose(handle);
}

void TPhToscaMagnet::Streamer(TBuffer &b)
{
// Stream an object of class TPhToscaMagnet.
// This function is an adaptation of the automatically produced method. 

  UInt_t SumPoints;

   if (b.IsReading()) {
      Version_t v = b.ReadVersion();
      if(v) { }

      TObject::Streamer(b); 

      b.ReadStaticArray(fVersion);
      b >> fDate;
      b.ReadStaticArray(fFNameFine);
      b.ReadStaticArray(fFNameCoarse);
      b.ReadStaticArray(fDeltaFine);
      b.ReadStaticArray(fDeltaCoarse);
      b.ReadStaticArray(fMinFine);
      b.ReadStaticArray(fMaxFine);
      b.ReadStaticArray(fMinCoarse);
      b.ReadStaticArray(fMaxCoarse);
      b.ReadStaticArray(fNPointsFine);
      b.ReadStaticArray(fNPointsCoarse);

      SumPoints = fNPointsFine[0] + fNPointsFine[1] + fNPointsFine[2];
      fGridNodesFine = new Float_t[SumPoints];
      b.ReadFastArray(fGridNodesFine, SumPoints);

      SumPoints = fNPointsCoarse[0] + fNPointsCoarse[1] + fNPointsCoarse[2];
      fGridNodesCoarse = new Float_t[SumPoints];
      b.ReadFastArray(fGridNodesCoarse, SumPoints);

      b >> fFineNGridPoints;
      fFineGrid[0] = new Float_t[fFineNGridPoints];
      b.ReadFastArray(fFineGrid[0], fFineNGridPoints);
      fFineGrid[1] = new Float_t[fFineNGridPoints];
      b.ReadFastArray(fFineGrid[1], fFineNGridPoints);
      fFineGrid[2] = new Float_t[fFineNGridPoints];
      b.ReadFastArray(fFineGrid[2], fFineNGridPoints);
      b >> fCoarseNGridPoints;
      fCoarseGrid[0] = new Float_t[fCoarseNGridPoints];
      b.ReadFastArray(fCoarseGrid[0], fCoarseNGridPoints);
      fCoarseGrid[1] = new Float_t[fCoarseNGridPoints];
      b.ReadFastArray(fCoarseGrid[1], fCoarseNGridPoints);
      fCoarseGrid[2] = new Float_t[fCoarseNGridPoints];
      b.ReadFastArray(fCoarseGrid[2], fCoarseNGridPoints);
   } else {
      b.WriteVersion(TPhToscaMagnet::IsA());
      TObject::Streamer(b);
      b.WriteArray(fVersion, 30);
      b << fDate;
      b.WriteArray(fFNameFine, 50);
      b.WriteArray(fFNameCoarse, 50);
      b.WriteArray(fDeltaFine, 3);
      b.WriteArray(fDeltaCoarse, 3);
      b.WriteArray(fMinFine, 3);
      b.WriteArray(fMaxFine, 3);
      b.WriteArray(fMinCoarse, 3);
      b.WriteArray(fMaxCoarse, 3);
      b.WriteArray(fNPointsFine, 3);
      b.WriteArray(fNPointsCoarse, 3);

      SumPoints = fNPointsFine[0] + fNPointsFine[1] + fNPointsFine[2];
      b.WriteFastArray(fGridNodesFine, SumPoints);
      SumPoints = fNPointsCoarse[0] + fNPointsCoarse[1] + fNPointsCoarse[2]; 
      b.WriteFastArray(fGridNodesCoarse, SumPoints);

      b << fFineNGridPoints;
      b.WriteFastArray(fFineGrid[0], fFineNGridPoints);
      b.WriteFastArray(fFineGrid[1], fFineNGridPoints);
      b.WriteFastArray(fFineGrid[2], fFineNGridPoints);
      b << fCoarseNGridPoints;
      b.WriteFastArray(fCoarseGrid[0], fCoarseNGridPoints);
      b.WriteFastArray(fCoarseGrid[1], fCoarseNGridPoints);
      b.WriteFastArray(fCoarseGrid[2], fCoarseNGridPoints);
   }
}


/*
-------------------------private member functions-------------------
*/

Int_t TPhToscaMagnet::ReadGrid(const Char_t* FileName,
       Float_t** Grid, Float_t* Min, Float_t* Max, Float_t* Delta)
{

// This is the actual function that reads in the data for each mesh.
  FILE* handler;
  Float_t X, Y, Z;
  Float_t BArray[3][400000]; // Current max size, should be sufficient.

  Float_t Pot;
  Char_t dum[20];
  UInt_t i, j, k, num, numPoints;
  Bool_t FirstRowY, FirstRowZ;
  UInt_t xPoints, yPoints, zPoints;

  // Open grid.
  if ((handler = fopen(FileName, "r")) == NULL) {
    printf("TPhToscaMagnet::ReadGrid: Couldn't open file %s\n", FileName);
    return -1;
  }

// Skip over header
  fscanf(handler, "%s %s %s %s %s %s %s",
	 dum, dum, dum, dum, dum, dum, dum);

// Start reading points 
  i = 0;
  yPoints = 0; zPoints = 0;
// We need this to figure out what the spacing between points is
  FirstRowY = kTRUE;
  FirstRowZ = kTRUE;
  
  while(fscanf(handler, "%f %f %f %f %f %f %f", &X, &Y, &Z,
	       &BArray[0][i], &BArray[1][i], &BArray[2][i], &Pot) != EOF) {

// Figure out minimum of the grid. TOSCA starts at the minimum 
// values of the grid and increases to the maximum 
    if(i == 0) {
      Min[0] = X;
      Min[1] = Y;
      Min[2] = Z;
    }

    if(FirstRowY && Z == Min[2]) yPoints++;
    if(X != Min[0]) FirstRowY = kFALSE;      

    if( (FirstRowZ && Z != Min[2]) || i == 0)
      zPoints++;
    else
      FirstRowZ = kFALSE;      

    i++;
  }

  numPoints = i;

// This is the maximum in the grid
  Max[0] = X;
  Max[1] = Y;
  Max[2] = Z;

// Adjust for counting wrongly, ugly but works
  yPoints-=2;
  zPoints--;
  xPoints =  i / ((yPoints+1) * (zPoints+1)) - 1;

// Calculate the distance between the gridpoints
  Delta[0] = (Max[0] - Min[0]) / xPoints;
  Delta[1] = (Max[1] - Min[1]) / yPoints;
  Delta[2] = (Max[2] - Min[2]) / zPoints;

  fclose(handler);

// We now have to account for the fact that the array was read in with 
// Z as the least significant index. we have to swap this, to account for
// the usual convention, where X is the least significant index.
  xPoints++; yPoints++; zPoints++;
  for(num = 0; num < 3; num++)
    for(i = 0; i < xPoints; i++)
      for(j = 0; j < yPoints; j++)
 	for(k = 0; k < zPoints; k++)
 	  Grid[num][k*xPoints*yPoints + j*xPoints + i] =
 	    BArray[num][i*yPoints*zPoints + j*zPoints + k];

//return the number of points read in.
  return numPoints;
}

void TPhToscaMagnet::Interpolate(Float_t *Coord, Float_t *B, Int_t *NPoints,
			    Float_t *Nodes, Float_t **Grid)
{
// Interpolate the field inbetween grid points.

  Float_t XCoord[3];

// we interpolate the field in the positive quadrant and then set the 
// right sign in the end.
  XCoord[0] = TMath::Abs(Coord[0]);
  XCoord[1] = TMath::Abs(Coord[1]);
  XCoord[2] = Coord[2];

// Interpolate the field in this region. 
  B[0] = TPhToscaMagnet::Fint(3, XCoord, NPoints, Nodes, (Float_t*)Grid[0]);
  B[1] = TPhToscaMagnet::Fint(3, XCoord, NPoints, Nodes, (Float_t*)Grid[1]);
  B[2] = TPhToscaMagnet::Fint(3, XCoord, NPoints, Nodes, (Float_t*)Grid[2]);

// make sure that the B-fld has the correct sign for all the quadrants
// The B-fld was calculated for the first, positive quadrant. 

// This was the way to call TMath::Sign(float) up to Root V1.02
//    B[0] = B[0] * TMath::Sign(Coord[1]);
//    B[1] = B[1] * TMath::Sign(Coord[0]);

  // Starting Root V1.03:
   B[0] = TMath::Sign(B[0], Coord[1]);
   B[1] = TMath::Sign(B[1], Coord[0]);

}

Float_t TPhToscaMagnet::Fint(Int_t narg, Float_t *arg, Int_t *nent,
			Float_t *ent, Float_t *table)
{
// This is the CERNLIB FINT interpolation function, converted from FORTRAN
// into C using f2c and only marginally touched, so that it is easy to call
// from C functions. 
// fint.F -- translated by f2c (version 19960717).
// Revision 1.1.1.1  1996/02/15 17:48:36  mclareni
// Kernlib

    float ret_val, r__1;

    /* Local variables */
    int loca, locb, locc, ndim, lmin, lmax;
    float h__;
    int i__, k, n;
    float x;
    int index[32], istep, knots, ishift;
    float weight[32], eta;


/*   INTERPOLATION ROUTINE. AUTHOR C. LETERTRE. */
/*   MODIFIED BY B. SCHORR, 1.07.1982. */

    /* Parameter adjustments */
    --table;
    --ent;
    --nent;
    --arg;

    /* Function Body */
    ret_val = 0.;
    lmax = 0;
    istep = 1;
    knots = 1;
    index[0] = 1;
    weight[0] = 1.;
    for (n = 1; n <= narg; ++n) {
	x = arg[n];
	ndim = nent[n];
	loca = lmax;
	lmin = lmax + 1;
	lmax += ndim;
	if (ndim > 2) {
	    goto L10;
	}
	if (ndim == 1) {
	    goto L100;
	}
	h__ = x - ent[lmin];
	if (h__ == 0.) {
	    goto L90;
	}
	ishift = istep;
	if (x - ent[lmin + 1] == 0.) {
	    goto L21;
	}
	ishift = 0;
	eta = h__ / (ent[lmin + 1] - ent[lmin]);
	goto L30;
L10:
	locb = lmax + 1;
L11:
	locc = (loca + locb) / 2;
	if ((r__1 = x - ent[locc]) < (float)0.) {
	    goto L12;
	} else if (r__1 == 0) {
	    goto L20;
	} else {
	    goto L13;
	}
L12:
	locb = locc;
	goto L14;
L13:
	loca = locc;
L14:
	if (locb - loca > 1) {
	    goto L11;
	}
/* Computing MIN */
	loca = TMath::Min( TMath::Max(loca, lmin) , (lmax - 1));
	ishift = (loca - lmin) * istep;
	eta = (x - ent[loca]) / (ent[loca + 1] - ent[loca]);
	goto L30;
L20:
	ishift = (locc - lmin) * istep;
L21:
	for (k = 1; k <= knots; ++k) {
	    index[k - 1] += ishift;
	}
	goto L90;
L30:
	for (k = 1; k <= knots; ++k) {
	    index[k - 1] += ishift;
	    index[k + knots - 1] = index[k - 1] + istep;
	    weight[k + knots - 1] = weight[k - 1] * eta;
	    weight[k - 1] -= weight[k + knots - 1];
	}
	knots <<= 1;
L90:
	istep *= ndim;
L100:
	;
    }
    for (k = 1; k <= knots; ++k) {
	i__ = index[k - 1];
	ret_val += weight[k - 1] * table[i__];
    }
    return ret_val;
} /* fint */





