#ifndef _TPhPadMap_INCLUDED_
#define _TPhPadMap_INCLUDED_

////////////////////////////////////////////////////////////////////////
// TPhPadMap is a data container class that stores arrays of infor-
// mation in cells associated either with physical sensor pads or
// bins in physics variables, and which has methods to map informa-
// tion from one representation to the other.  An arbitrary number
// of variables, such as raw ADC counts, reconstructed occupancy,
// and multiplicity, can be stored and mapped in parallel for effi-
// ciency.
//
// File: TPhPadMap.h
//
// Date     Vers  Description
// 22-Jan-99 2.0  Change SetNCells to delete and reallocate when 
//                changing # cells (RV)
//
// $Id: TPhPadMap.h,v 1.9 1999/01/22 22:47:27 verdier Exp $
//
////////////////////////////////////////////////////////////////////////

// 13-Jan-99 2.0  Remove GetHist.cxx defaults; add GetStats and 
//                GetIntegral methods; change SetNcells to SetNCells
//                for consistency (RV)
// 31-Dec-98 2.0  Add new GetGraph and GetHist methods (RV)
// 16-Nov-98 2.0  Add size of TPhSensGeo to allow streaming (RV)
// 19-Aug-98 1.0  Correct vertex handling (RV)
// 11-Aug-98 1.0  Add SetVertex method (RV)
// 14-Jul-98 1.0  Generalized GetGraph and GetHist methods (RV)
// 09-Jul-98 1.0  Add Print method (RV)
// 03-Jul-98 1.0  GetGraph and GetHist arguments changed (RV)
// 01-Jun-98 1.0  Add fNEvents, fMaxEvents, fSelect access (RV)
// 09-May-98 0.3  Add GetHist method (RV)
// 08-May-98 0.3  Add GetGraph method, GetNvars->GetNVars (RV)
// 19-Feb-98 0.3  Removed GetDispList, function is in TPhMul::Recon (RV)
// 09-Feb-98 0.3  Added method Select replacing Match (RV)
// 22-Jan-98 0.3  Added methods GetDispList, Match, ... (RV)
// 19-Jan-98 0.2  Added method GetVal, changed Loc to GetLoc (RV)
// 07-Nov-97 0.2  Added methods GetDLoc and SetData (RV)
// 02-Nov-97 0.2  Added method GetEtaPhi (RV)
// 13-Oct-97 0.1  Creation (Robin Verdier)
////////////////////////////////////////////////////////////////////////

#include "TPhDispDef.h"
#include "TPhSensDef.h"
#include "TPhSensGeo.h"

// forward declarations: ROOT Classes

class TGraphErrors;
class TH2F;
class TObject;

// PHOBOS Classes

class TPhVertex;

class TPhPadMap : public TObject

{   

public:

  TPhPadMap();

  TPhPadMap(Int_t id, Char_t *name = NULL);

  TPhPadMap(Int_t id, Char_t *name, TPhDispDef* s, Int_t nvars, Float_t val);

  TPhPadMap(Int_t id, Char_t *name, TPhSensDef* s, Int_t nvars, Float_t val);

  TPhPadMap(Int_t id, Char_t *name, Int_t type,
    Int_t nxcells, Float_t xmin, Float_t xsize,
    Int_t nycells, Float_t ymin, Float_t ysize,
    Float_t xorig, Float_t yorig, Float_t zorig, 
    Float_t u11, Float_t u21, Float_t u31, 
    Float_t u12, Float_t u22, Float_t u32, 
    Float_t u13, Float_t u23, Float_t u3, 
    Float_t dist);

// Constructors will get data from other pad maps or sensors
//  TPhPadMap(TPhPadMap *Orig);
//  TPhPadMap(TPhSensor *Sensor);

  virtual ~TPhPadMap();

// In the Get and Set methods, ixy = 0 for X and 1 for Y coordinate

//  struct   F2_t { Float_t data[2]; } 

  Float_t  GetDist();

// GetDLoc returns a pointer to to the floating word at indices ix, 
// iy of variable iv, and the array dl of pointer increments to 
// the data at iv+1, ix+1, and iy+1.
// GetDLoc returns zero if any index is out of range.

  Float_t *GetDLoc (Int_t iv, Int_t ix, Int_t iy, Int_t dl[]);

  void     GetEtaPhi(Float_t ix, Float_t iy, Float_t etaphi[3]);
// I was not able to make the following static:
//static void GetEtaPhi(TPhPadMap *map, Int_t ix, Int_t iy, Float_t etaphi[2]);

// General GetGraph returns a pointer to a TGraphErrors with data accumulated
// from one or more selected TPhPadMaps.
  TGraphErrors *GetGraph
( TPhPadMap **maps,    // Array of maps to be searched for selector
  Int_t nMaps,         // Number of maps to search; see GetHist for spec values
  Char_t *graphName,   // Name of TGraphErrors for access via gROOT->GetObject
  Char_t *sel,         // Selector string such as "MPA3" or "MPA*"
  Int_t iMap = -1,     // If >= 0, ignore selector and use maps[iMap]
  Int_t iVar = 0,      // Index of plotted variable, defined in TPhMul.h
  Int_t iVarErr = -1,  // Index of error variable
  Float_t norm = 1.0,  // Normalizer, e.g. 360.0 to integrate on phi
// nXBins or nYBins should be 1; otherwise, we treat nYBins as 1.
  Int_t nXBins = 0,    // Number of X-bins in graph
  Int_t nYBins = 0,    // Number of Y-bins in graph
  Int_t nXMinSkip = 0, // Number of low X-bins to omit
  Int_t nYMinSkip = 0, // Number of low Y-bins to omit
  Int_t nXMaxSkip = 0, // Number of high X-bins to omit
  Int_t nYMaxSkip = 0  // Number of high Y-bins to omit
);

// Specialized GetGraph returns a pointer to a TGraphErrors with data from 
// the TPhPadMap whose GetGraph is called.
  TGraphErrors *GetGraph
( Char_t *graphName,   // Name of TGraphErrors for access via gROOT->GetObject
  Int_t iVar = 0,      // Index of plotted variable, defined in TPhMul.h
  Int_t iVarErr = -1,  // Index of error variable
  Float_t norm = 1.0,  // Normalizer, e.g. 360.0 to integrate on phi
// nXBins or nYBins should be 1; otherwise, we treat nYBins as 1.
  Int_t nXBins = 0,    // Number of X-bins in graph
  Int_t nYBins = 0,    // Number of Y-bins in graph
  Int_t nXMinSkip = 0, // Number of low X-bins to omit
  Int_t nYMinSkip = 0, // Number of low Y-bins to omit
  Int_t nXMaxSkip = 0, // Number of high X-bins to omit
  Int_t nYMaxSkip = 0  // Number of high Y-bins to omit
);

// Specialized GetGraph uses OBSOLETE call form for backwards compatibility
  TGraphErrors *GetGraph
( TPhPadMap **maps, // Array of maps to be searched for selector
  Char_t *graphName,// Name of TGraphErrors for access via gROOT->GetObject
  Char_t *sel,      // Selector string such as "MPA3" or "MPA*"
  Int_t iVar,       // Index of plotted variable, defined in TPhMul.h
  Int_t iVarErr,    // Index of error variable, add 100 for linear combination
  Float_t norm,     // Normalizer, e.g. 360.0 to integrate on phi
// nXBins or nYBins should be 1; otherwise, we treat nYBins as 1.
  Int_t nXBins,     // Number of X-bins in histo
  Int_t nYBins,     // Number of Y-bins in histo
  Int_t nXMinSkip,  // Number of low X-bins to omit
  Int_t nYMinSkip,  // Number of low Y-bins to omit
  Int_t nXMaxSkip,  // Number of high X-bins to omit
  Int_t nYMaxSkip   // Number of high Y-bins to omit
);

// General GetHist returns a pointer to a TH2F with data accumulated 
// from selected TPhPadMaps.  Alternate methods are more specialized.
// Use:  TH2F *h1 = m1->GetHist(maps, nMaps, ROOTname, selector, iMap,
//         iVar, iVarErr, norm, ...)
// Ex:   TPhMul mp;
//       mp.Process ...  // See runmul.C script
//       TPhPadMap **maps = mp.GetMaps()
//       TH2F *h1 = TPhPadMap::GetHist(maps,mp.NSensAct,"s1","MOC*",-1,0)
//       h1->Draw("lego2")
// It is the CALLER'S responsibility to delete the TH2F.
  TH2F *GetHist
( TPhPadMap **maps,    // Array of maps to be searched for selector
  Int_t nMaps,         // Number of maps to search
  Char_t *histName,    // Name of TH2F for access via gROOT->GetObject
  Char_t *sel,         // Selector string such as "MPA3" or "MPA*"
  Int_t iMap = -1,     // If >= 0, ignore selector and use maps[iMap]
  Int_t iVar = 0,      // Index of histo'd variable, defined in TPhMul.h
  Int_t iVarWt = -1,   // Index of weight variable, add 100 for square weight,
                       //  -1 for weight 1.0
  Float_t norm = 1.0,  // Normalizer, e.g. 360 to integrate on phi
  Int_t nXBins = 0,    // Number of X-bins in histo
  Int_t nYBins = 0,    // Number of Y-bins in histo
  Int_t nXMinSkip = 0, // Number of low X-bins to omit
  Int_t nYMinSkip = 0, // Number of low Y-bins to omit
  Int_t nXMaxSkip = 0, // Number of high X-bins to omit
  Int_t nYMaxSkip = 0  // Number of high Y-bins to omit
);

// Specialized GetHist returns a pointer to a TH2F with data accumulated 
// from the TPhPadMap whose GetHist is called.
  TH2F *GetHist
( Char_t *histName,    // Name of TH2F for access via gROOT->GetObject
  Int_t iVar = 0,      // Index of histo'd variable, defined in TPhMul.h
  Int_t iVarWt = -1,   // Index  of weight variable, add 100 for square weight,
                       // -1 for weight 1.0
  Float_t norm = 1.0,  // Normalizer, e.g. 360 to integrate on phi
  Int_t nXBins = 0,    // Number of X-bins in histo
  Int_t nYBins = 0,    // Number of Y-bins in histo
  Int_t nXMinSkip = 0, // Number of low X-bins to omit
  Int_t nYMinSkip = 0, // Number of low Y-bins to omit
  Int_t nXMaxSkip = 0, // Number of high X-bins to omit
  Int_t nYMaxSkip = 0  // Number of high Y-bins to omit
);

// Specialized GetHist omits two arguments, for temporary compatibility
// with OBSOLETE invocations
  TH2F *GetHist
( TPhPadMap **maps,    // Array of maps to be searched for selector
  Char_t *histName,    // Name of TH2F for access via gROOT->GetObject
  Char_t *sel,         // Selector string such as "MPA3" or "MPA*"
  Int_t iVar,          // Index of histo'd variable, defined in TPhMul.h
  Int_t iVarWt = -1,   // Index  of weight variable, add 100 for square weight,
                       // -1 for weight 1.0
  Float_t norm = 1.0,  // Normalizer, e.g. 360 to integrate on phi
  Int_t nXBins = 0,    // Number of X-bins in histo
  Int_t nYBins = 0,    // Number of Y-bins in histo
  Int_t nXMinSkip = 0, // Number of low X-bins to omit
  Int_t nYMinSkip = 0, // Number of low Y-bins to omit
  Int_t nXMaxSkip = 0, // Number of high X-bins to omit
  Int_t nYMaxSkip = 0  // Number of high Y-bins to omit
);

  Int_t    GetId();

  enum { EunSpec = -999993 };

  Float_t  GetIntegral
  ( Int_t iVar,              // Variable #, e.g., 13 for d2N/dEta/dPhi rec
    Int_t iVarRMS = -1,      // RMS error on var
    Float_t xMin = EunSpec,  // Range of integration
    Float_t yMin = EunSpec,
    Float_t xMax = EunSpec,
    Float_t yMax = EunSpec
  );

  Float_t  GetMin(Int_t ixy);

  Char_t  *GetName();

  Int_t    GetMaxEvents();

  Int_t    GetNEvents();

  Int_t    GetNCells(Int_t ixy);

  Int_t    GetNVars();

  Float_t  GetSize(Int_t ixy);

  Int_t    GetStats
// GetStats returns the number of words stored in stats.
// Statistics in this version are: [0] integral(Var(x,y)); [1] <Var>;
// [2] <x>; [3] <y>; [4] xRms; [5] yRms; [6] x-y correlation coefficient;
// [7]-[6] rms assigned to [0]-[6], if iVarRMS >= 0.  These may change.
( Int_t    iVar,           // Variable #, e.g., 13 for d2N/dEta/dPhi rec
  Int_t    iVarRMS = -1,   // RMS error on var
  Int_t    nStats = 1,     // number of statistics to return
  Float_t *stats = NULL,   // User buffer to receive stats
  Float_t  xMin = EunSpec, // Range of integration
  Float_t  yMin = EunSpec,
  Float_t  xMax = EunSpec,
  Float_t  yMax = EunSpec
);

  Int_t    GetType();

// GetVal returns the floating word at indices ivar, ix, iy, or 0 if 
// any index is out of range.  Unlike GetLoc, GetVar does not signal
// an out-of-range error.

  Float_t  GetVal(Int_t ivar, Int_t ix, Int_t iy);

  Float_t  GetVersion();

// Return a pointer to the floating word at indices i1, i2, i3.
// Return 0 if any index is out of range.

  Float_t *GetLoc (int i1, int i2, int i3);

// Map variables in a vector from a cell in Map to displays (last arg)
// This is a primitive version that will evolve

  void Map(Float_t *v, Int_t isens, Int_t ix, Int_t iy, Float_t xy[2][2][2],
           Int_t *DisUp, TPhPadMap **Disps);

// Print a formatted dump of specified variables to stdout.  Print() prints
// help text.  Bin CENTER values are shown.

  void Print (Int_t firstVar = -1, Int_t lastVar = -1);

// Print a formatted dump of statistical variables to stdout.

  void PrintStats
  (Int_t iVar = -1,   // Variable #, e.g., 13 for d2N/dEta/dPhi rec
   Int_t iVarRMS = -1 // Variable # for assigned RMS err
  );

// Release the data part of the padmap to the heap.  This should
// also be executed by the padmap destructor.

  Float_t *Release();

  static Int_t Select(Char_t *name, Char_t *selectors); // Match name strings

// The set methods will eventually all return a status code.

// Set contents to specified value; return total number of cells set:
  Int_t    SetData (Float_t val);
  void     SetDist(Float_t dist);
  void     SetId(Int_t id);
  void     SetMaxEvents(Int_t nEvents = -1);
  void     SetMin(Int_t ixy, Float_t min);
  void     SetName(Char_t *name);
  void     SetNCells(Int_t nXCells, Int_t nYCells, Float_t val);
  void     SetNEvents(Int_t nev);
// Note that there is no SetNvars method; this can only be set at construction
  void     SetSize(Int_t ixy, Float_t size);
  void     SetType(Int_t type);
  void     SetVersion(Float_t vers);
  void     SetVertex(TPhVertex *v);

private:

// The following struct has to be a TPhSensGeo for both Sensor and Display 
// maps, unless we use a union:

Int_t      fSSize;     // Total number of =>BYTEs<= in fS
TPhSensGeo fS;         //! Geometry information, NO STREAMER FUNCTION
Int_t      fMaxEvents; // Total number of events to include, -1 = unlimited
Int_t      fNEvents;   // Number of events accumulated
Int_t      fNvars;     // Number of variable planes
Char_t     fSelect[2]; // Selector masks (not yet implemented)
Int_t      fSize;      // Total number of words in fData
float     *fData;      //! Data, NO STREAMER FUNCTION

// Statistics information will be added

public:
ClassDef(TPhPadMap,2)      // PHOBOS pad data mapping class

};

#endif
