#include "TObject.h"
#include "TPhDept.h"
#include "TPhDeptV.h" // Default values
#include "TPhSensDef.h"

ClassImp(TPhDept)

////////////////////////////////////////////////////////////////////////////
// TPhDept: Energy deposition class for Phobos analysis
//
// GetDept returns the global value of the energy deposited per track 
// for a particular detector pad by interpolating in a table using
// values obtained from Phobos simulation.
// At present it returns only the phi-averaged value.
//
// File: TPhDept.cxx
//
// Date     Vers  Description
// 16-Dec-98 1.0  Comment-out unused args for inimplemented Fit method (RV)
//
// $Id: TPhDept.cxx,v 1.4 1998/12/21 21:28:13 verdier Exp $
//
////////////////////////////////////////////////////////////////////////////

// 22-Sep-98 1.0  kNSeg -> kNSegE because of name clash (mpd)
// 15-Aug-98 1.0  Change default values filename from .v to V.h (RV)
// 22-May-98 0.3  Implement GetPar and SetPar methods, add TPhSensDef
//                argument to constructor (RV)
// 02-Mar-98 0.3  Add gFitDept for access to user fit functions (RV)
// 02-Feb-98 0.1  Add methods for fitting parameters (RV)
// 12-Dec-97 0.1  Add ReportLevel and Nprint (Robin Verdier)
// 04-Nov-97 0.1  Creation based on M_GetDept (Robin Verdier)

// User fit wrapper functions: this never worked and eventually should die

TPhDept* gFitDept = NULL; // Global pointer to user fit function

//extern "C" Double_t CWrapFit(Double_t *x, Double_t *par);

Double_t CWrapFit(Double_t *x, Double_t *par)
{
// CWrapFit is a stand-alone C program that allows the ROOT TFit 
// classes, which only understand C-type
// functions, to access a user fit function implemented as a C++ class 
// member.  The user must set a global pointer declared as
//   TPhDept* gFitDept = NULL
// to the address of the fit function by calling TPhDept::SetupFit() 
// before each fit.  This prevents accidental attempts to share the 
// global pointer.
// CWrapFit must be declared in the implementation file via
//   extern "C" CWrapFit(Double*x, Double_t *par);
//
// 02-Mar-98  R. Verdier  Method suggested by Patrick Decowski

  static Int_t iprint = 0;
  Int_t Nprint = 5;
  Int_t ReportLevel = 1;
  Double_t Result;

  if (gFitDept == 0 || x == 0 || par == 0 )
  { if (ReportLevel >= 1 && iprint < Nprint)
    { if (x == 0) 
        printf ("CWrapFit: called with null address for x, exiting.\n");
      if (par == 0)
        printf ("CWrapFit: called with null address for par, exiting.\n");
      if (gFitDept == 0)
        printf ("CWrapFit: user function pointer gFitDept not inited by "
        "TPhDept::SetupFit().\n");
      iprint++;
    }
// Return a large value on error; unfortunately the ROOT implementation of
// the fit class doesn't have a provision for passing an error flag.
    Result = -999999.;
  }
  else
  { Result = gFitDept->GetFitValue(x, par);
// Free the global pointer so that other objects can use it
    gFitDept = NULL;
  }
  return Result;
}

  static Int_t iprint = 0;
  Int_t Nprint = 10;
  Int_t ReportLevel = 1;

// PHOBOS Classes

TPhDept::TPhDept(TPhSensDef &sd) { fSensDefs = &sd; }

// TPhDept::TPhDept(FILE *values) {} // UNIMPLEMENTED

Int_t TPhDept::Fit (/*Char_t *sensorName, TPhDept *edep, TPhPadMap **disps, 
                    Int_t varIndex, Int_t varRMSIndex, Char_t *outFileName*/)
{ return Eunimplemented; } // UNIMPLEMENTED

Float_t TPhDept::GetDept(Int_t sensorId, Float_t Eta)
{

// Set iseg to the index of the first segment whose eta is <= the 
// requested eta

//  for (int iseg = 0, int nn = 1; nn < kNSegE-1; nn++) *** fails ***?
  Int_t iseg = 0;
  for (Int_t nn = 1; nn < kNSegE-1; nn++)
    if (Eta >= fF[sensorId][nn][1]) iseg = nn;

// Return value obtained by linear interpolation

  Float_t dFdE = (fF[sensorId][iseg+1][0] - fF[sensorId][iseg][0])
    / (fF[sensorId][iseg+1][1] - fF[sensorId][iseg][1]);
  Float_t Edep = fF[sensorId][iseg][0] + dFdE * (Eta - fF[sensorId][iseg][1]);

  if (ReportLevel >= 3 && iprint < Nprint 
      && sensorId == 28 && Eta < -1.88 && Eta > -1.900) 
  { printf ("TPhDept: for sensor %d and Eta = %8.3f, Edep =%12.4e.\n",
      sensorId, Eta, fF[sensorId][iseg][0] + dFdE * 
      (Eta - fF[sensorId][iseg][1]));
    iprint++;
  }

  return Edep;
}

Int_t TPhDept::GetFitSensorId ()
{ return fFitSensId; // Id of sensor being fitted
}

Int_t   TPhDept::GetFitType (Char_t *sensorName)
// A specific fit type is not yet associated with each sensor
{ return (sensorName)? 1 : 1; }

Double_t TPhDept::GetFitValue (Double_t *x, Double_t *pars)
{
// Fit function for use by the TFit classes, returning the value of the
// function at independent variable x for the values of pars, here the
// values of the function at the FIXED points in the table for the 
// sensor being fitted.
// REQUIRE: The values of pars must increase monotonically.  There must
//   be at least two parameters.
// PROMISE: The returned value will extrapolate from the first x that
//   does not increase
// The argument syntax is prescribed by the ROOT TFit class.
// We should later add the sensor id as a non-varying parameter,
// and eliminate fSensFitId (which is non-sharable).
// This code replicates the interpolation in GetDept, so we should make
// it a method.

  Int_t iseg = 0;
  Int_t sensorId = fFitSensId;
// Find the first segment containing x[0]
  while (iseg <= kNSegE - 2 && x[0] >= fF[sensorId][iseg+1][1]) iseg++;
  Double_t dF = pars[iseg+1] - pars[iseg];
  Double_t dX = fF[sensorId][iseg+1][1] - fF[sensorId][iseg][1];
  return (dX > 0)?
    pars[iseg] + dF / dX * (x[0] - fF[sensorId][iseg][1]) : pars[iseg];
}

Int_t   TPhDept::GetNPars (Int_t sensorId)
{ return (sensorId)? kNSegE : kNSegE; } // Individual fits not yet available

Float_t TPhDept::GetPar (Int_t sensorId, Int_t parIndex)
{ return fF[sensorId][0][parIndex];
}

Float_t TPhDept::GetPar (Char_t *sensorName, Int_t parIndex) 
{ const Float_t FlError = -9.99e20; // Note there's no provision for error
  Int_t is = fSensDefs->GetIndex(sensorName);
  return (is >= 0)? fF[is][0][parIndex] : FlError;
}

void TPhDept::SetFitSensorId (Int_t sensorId)
{ // Select sensor to be fitted.  This is non-reentrant and should be changed.
  fFitSensId = (sensorId < kNSensMax)? sensorId : -1;
}

void TPhDept::SetPar (Int_t sensorId, Int_t parIndex, Float_t value)
{ fF[sensorId][0][parIndex] = value;
}

void TPhDept::SetPar (Char_t *sensorName, Int_t parIndex, Float_t value) 
{ Int_t is = fSensDefs->GetIndex(sensorName);
  if (is >= 0) fF[is][0][parIndex] = value;
}

void TPhDept::SetupFit()
{ 
  // Assign pointer to this object to the global variable gFitDept, first
  // checkint that no other instance of the same class is also trying to
  // to fit.  That is the price you pay for using globals.

  if (gFitDept == NULL)
    gFitDept = this;
  else
  { if (ReportLevel >= 1 && iprint < Nprint)
    { printf("TPhDept/SetupFit: can't assign gFitDept, already in use.\n");
      iprint++;
    }
  }
}
