#include "TPhOnContainer.h"

// ROOT classes

#include "TH1.h"
#include "TObjArray.h"
#include "TCanvas.h"
#include "TPad.h"
#include "TPaveText.h"
#include "TStyle.h"
#include "TAttText.h"



ClassImp(TPhOnContainer)
  ///////////////////////////////////////////////////////////////////////////
  // 
  // Container Class to manage groups of online histograms. Garanties 
  // same behaviour for e.g. plotting and accessing histograms. Most of the 
  // work will be done by the derived classes , sofar TPhOnSensorContainer 
  // and TPhOnFECContainer.
  //
  ///////////////////////////////////////////////////////////////////////////


TPhOnContainer::TPhOnContainer()
{
 // Default constructor. Does nothing. 

  fOnArray = NULL;
  fCanvas = NULL;
  fXCanvas = 0;
  fYCanvas = 0;
  fXPosCanvas = 0;
  fYPosCanvas = 0;
}

TPhOnContainer::TPhOnContainer(const Char_t *name):TNamed(name,name)
{
  // constructor
  fCanvas = NULL;
  fOnArray = new TObjArray(1000);
  fXCanvas = 800;
  fYCanvas = 600;
  fXPosCanvas = 10;
  fYPosCanvas = 10;
}


TPhOnContainer::~TPhOnContainer()
{
  // normal destructor, deletes objects in array 
  if(fOnArray) {
    //delete entries in objarray
    fOnArray->Delete();
    // delete objarray
    delete fOnArray;
  }
  SafeDelete(fCanvas);
}

void TPhOnContainer::Fill()
{
  // will be overwritten by derived class
 Warning("Fill","this method must be over written");
}

Bool_t TPhOnContainer::Check()
{
  // will be overwritten by derived class
 Warning("Check","this method must be over written");
 return kFALSE;
}


Int_t TPhOnContainer::Init()
{
  // will be overwritten by derived class
 Warning("Init","this method must be over written");

 return 0;
}

void TPhOnContainer::Reset()
{
  // will be overwritten by derived class
 Warning("Reset","this method must be over written");
}

void TPhOnContainer::ListHisto(Option_t *option)
{
  // List histograms, possible options are:
  // "ALL" , "EMPTY" all empty histos, "FILLED" all histos with entries>0
  // will be overwritten by derived class
  Warning("ListHisto","this method must be over written");
}

Int_t TPhOnContainer::DivideCanvas(Int_t fNPad)
{
  // distribute pads in fCanvas according to number of needed pads.

  if(!fCanvas) {
    fCanvas = new TCanvas("online monitoring","online display",fXPosCanvas, 
                     fYPosCanvas, fXCanvas, fYCanvas);
  }

  if(fNPad <=2){
        fCanvas->Divide(1,2,0.01,0.01,0);  
  }
  else if(fNPad >2 && fNPad <= 4){
        fCanvas->Divide(2,2,0.01,0.01,0);  
  }
  else if(fNPad >4 && fNPad <=9) {
    fCanvas->Divide(3,3,0.01,0.01,0);  
  }
  else if(fNPad>9 && fNPad<=12) {
    fCanvas->Divide(3,4,0.01,0.01,0);     
  }
  else if(fNPad>12 && fNPad<=16) {
    fCanvas->Divide(4,4,0.01,0.01,0);     
  }
  else {
    printf("WARNING:TPhOnContainer::Draw too many pads for canvas\n");
  }

  return 0;
}

void TPhOnContainer::CloseCanvas(){
  if(fCanvas){
    SafeDelete(fCanvas);    
    fCanvas = NULL;
  }
  return;
}






