#include "TPhOnFEC.h"

// ROOT classes

#include "TH1.h"
#include "TObjArray.h"
#include "TCanvas.h"

// Phat classes

#include "TPhDetector.h"
#include "TPhSensorMapDB.h"
#include "TPhSensorMapDBFECItem.h"
#include "TPhOnString.h"


ClassImp(TPhOnFEC)
  ///////////////////////////////////////////////////////////////////////////
  // 
  // Online Class to manage groups of histograms related to one FEC. 
  // It books, fills and draws histograms for all strings of this FEC.
  // The histograms themselves are stored in the TPhOnString objects. 
  //
  // Warning: sofar the filling is not yet implemented due to missing data 
  // (and data format). This class shows at least how the bookkeeping is done. 
  ///////////////////////////////////////////////////////////////////////////


TPhOnFEC::TPhOnFEC()
{
 // Default constructor. 
  fNChanRead = 0;                 
  fNEvent = 0; 
  fFECNr = 0;
  fCanvas = NULL;
  fStringArray = NULL;
}

//-----------------------------------------------------------------------
TPhOnFEC::TPhOnFEC(Int_t FECNr):TNamed()
{
  // constructor; to book histograms use Init()

  // built & set name
  Char_t name[256];
  sprintf(name,"OnFEC_%d",FECNr);
  SetName(name);
  SetTitle(name);
  TPhDetector *fdet = TPhDetector::GetInstance();
  if(!fdet){
    printf("WARNING: TPhOnFEC:  Detector db not connencted\n");
    printf("return\n");
    return;
  }
  TPhSensorMapDB *fsensormapdb   = fdet->GetSensorMapDB();
  if(!fsensormapdb){
    Warning("TPhOnFEC","no sensor map db in detector object");
    return;
  }
  TPhSensorMapDBFECItem *fitem = fsensormapdb->GetFEC(fFECNr);
  fNString = fitem->GetNStrings();
  fStringArray = new TObjArray(fNString);
  fNChanRead = 0;                 
  fNEvent = 0; 
  fFECNr = FECNr;
  fCanvas = NULL;

}

//-----------------------------------------------------------------------
TPhOnFEC::~TPhOnFEC()
{
  // normal destructor;
  fStringArray->Delete();
  delete fStringArray;
}

//-----------------------------------------------------------------------
void TPhOnFEC::Fill()
{
  // fill booked histograms for this FEC and its strings
  // has to be implemented as soon as data format exist 

}


//-----------------------------------------------------------------------
Int_t TPhOnFEC::Init()
{
  // initialize histograms for all strings of this FEC
  if(!fStringArray) fStringArray = new TObjArray(fNString);
  Int_t String;
  TPhOnString *onstring;
  for (String=0; String<fNString; String++)
    {
      onstring = new TPhOnString(fFECNr, String);
      onstring->Init();
      fStringArray->AddAt(onstring, String);
    }
  return 1;
}

//-----------------------------------------------------------------------
void TPhOnFEC::Reset()
{
  // reset all histograms
   TPhOnString *onstring; 
   if(!fStringArray)
     {
       Warning("Reset","fStringArray doesn't exist  - return");
       return;
     }
   Int_t String;
   for(String=0; String<fNString; String++)
     {  
       onstring = (TPhOnString*) fStringArray->At(String);
       onstring->Reset();
     }   
}
//-----------------------------------------------------------------------
void TPhOnFEC::Draw(Int_t StringNr)
{
  // Draw all histograms belonging to the string  NString
  if(!fStringArray)
    {
      Warning("Draw","fStringArray doesn't exist  - return");
      return;
    }
  if(StringNr>fStringArray->GetLast()) 
    {
      Warning("Draw","StringNr out of range  - return");
      return;
    }
  TPhOnString *onstring = (TPhOnString*)fStringArray->At(StringNr);
  if(!fCanvas) fCanvas = new TCanvas("fec","fec",10, 10, 600, 700);
  fCanvas->Clear();
  fCanvas->Divide(2,2);
  fCanvas->cd(1);
  onstring->Draw("pedes");
  fCanvas->cd(2);
  onstring->Draw("noise");
  fCanvas->cd(3);
  onstring->Draw("status");
  fCanvas->cd(4);
  onstring->Draw("calib");
}

//-----------------------------------------------------------------------
void TPhOnFEC::Draw(Int_t StringNr, Option_t *option)
{
  // Draw one specific histogram, options are "pedes","noise","status","calib"
  // for string nr StringNr

  TString opt = option;
  opt.ToLower();

  if(!fStringArray)
    {
      Warning("Draw","fStringArray doesn't exist  - return");
      return;
    }
  if(StringNr>fStringArray->GetLast()) 
    {
      Warning("Draw","StringNr out of range  - return");
      return;
    }
  TPhOnString *onstring = (TPhOnString*) fStringArray->At(StringNr);
  fCanvas->cd();
  if(opt.Contains("pedes")){
    onstring->Draw("pedes");}
  else if(opt.Contains("noise")){
    onstring->Draw("noise");}
  else if(opt.Contains("status")){
    onstring->Draw("status");}
  else if(opt.Contains("calib")){
    onstring->Draw("calib");}
  else
    Warning("Draw","option doesn't exist");
}

//-----------------------------------------------------------------------
void TPhOnFEC::ListHisto(Option_t *option)
{
  // List histograms, possible options are:
  // "ALL" , "EMPTY", "FILLED" 
  
  if(fStringArray->GetEntries()==0 || !fStringArray) {
    Warning("ListHisto","fStringArray empty or not existend - return");
    return;
  }
  Int_t ii;
  TPhOnString *onstring;
  Int_t nstrings = fStringArray->GetLast();
  TString opt = option;
  opt.ToLower();
  if (opt.Contains("filled"))
    {
      for (Int_t i=0; i<nstrings; i++)
	{
	  onstring = (TPhOnString*) fStringArray->At(i);
	  if(onstring)
	    {
	      if(onstring->GetEntries() > 0)
		{
		  ii++;
		  printf(" %s\n",onstring->GetName());
		}
	    }
	}
      printf(" %d non empty histograms \n",ii);
    }
  //*****************************************************
  else   if (opt.Contains("empty"))
    {
      for (Int_t i=0; i<nstrings; i++)
	{
	  onstring = (TPhOnString*) fStringArray->At(i);
	  if(onstring)
	    {
	      if(onstring->GetEntries() == 0)
		{
		  ii++;
		  printf(" %s\n",onstring->GetName());
		}
	    }
	}
      printf(" %d  empty histograms \n",ii);
    }

  //*****************************************************
  else if (opt.Contains("all"))
    {
      for (Int_t i=0; i<nstrings; i++)
	{
	  onstring = (TPhOnString*) fStringArray->At(i);
	  if(onstring)
	    {
	      printf(" %s\n",onstring->GetName());
	    }
	}
      printf(" %d histograms in array \n",ii);
    }
}
//_________________________-------------------------------------------
Bool_t TPhOnFEC::Check(Option_t *option)
{
  // performs checks on group of histograms to detect unusual detector 
  // performance. the checks themselves are defined in TPhOnSensor. 
  // sofar it only checks whether all booked histograms are filled. 

  if(fStringArray->GetEntries()==0 || !fStringArray) {
    Warning("Check","fStringArray empty or not existend - return");
    return kFALSE;
  }
  TString opt = option;
  opt.ToLower();
  if (opt.Contains("all"))
    {      
      Int_t i;
      TPhOnString *onstring;
      for (i=0; i<=fStringArray->GetLast(); i++)
	{
	  onstring = (TPhOnString*)fStringArray->At(i);
	  if(onstring) {
	    if(onstring->GetEntries("pedes")==0) return kFALSE;
	    if(onstring->GetEntries("noise")==0) return kFALSE;
	    if(onstring->GetEntries("status")==0) return kFALSE;
	    if(onstring->GetEntries("calib")==0) return kFALSE;
	  }
	}
    }

  return kTRUE;
}

Bool_t TPhOnFEC::Check(Int_t StringNr, Option_t *option)
{
  // performs checks on a single histogram of sensor sname to detect unusual 
  // detector performance. the check itself is defined in TPhOnSensor. 
  // options are "all","adc","hit","noise"

  if(fStringArray->GetLast()<fNString || !fStringArray) {
    Warning("Check","StringNr or fStringArray not existend - return");
    return kFALSE;
  }

  TPhOnString *onstring = (TPhOnString*) fStringArray->At(StringNr);
  if(!onstring) {
    Warning("Check","no histograms for that string -return");
    return kFALSE;}
  TString opt = option;
  opt.ToLower();
  if (opt.Contains("all"))
    {
      if(onstring->GetEntries("noise")==0) return kFALSE;
      if(onstring->GetEntries("pedes")==0) return kFALSE;
      if(onstring->GetEntries("status")==0) return kFALSE;
      if(onstring->GetEntries("calib")==0) return kFALSE;
    }
  else if(opt.Contains("pedes")) {
    if(onstring->GetEntries("pedes")==0) return kFALSE;}
  else if(opt.Contains("noise")) {
    if(onstring->GetEntries("noise")==0) return kFALSE;}
  else if(opt.Contains("status")) {
    if(onstring->GetEntries("status")==0) return kFALSE;}
  else if(opt.Contains("calib")) {
    if(onstring->GetEntries("calib")==0) return kFALSE;}
  else {
    Warning("Check","option unkown - return");
    return kFALSE;}
  return kTRUE;
}





