#include "TPhOnSensorContainer.h"

// ROOT classes

#include "TH1.h"
#include "TObjArray.h"
#include "TCanvas.h"
#include "TPad.h"
#include "TPaveText.h"
#include "TStyle.h"
#include "TAttText.h"
#include "TSeqCollection.h"

// Phat classes

#include "TPhSensor.h"
#include "TPhOnSensor.h"
#include "TPhSiSensor.h"
#include "TPhDetector.h"
#include "TPhSubDetector.h"

ClassImp(TPhOnSensorContainer)
  ///////////////////////////////////////////////////////////////////////////
  // 
  // Online Class to manage groups of histograms. It books, fills and draws 
  // online histograms for e.g. all sensors or sensors of a given subdetector.
  // The histograms themselves are stored in the TPhOnSensor objects. 
  //
  ///////////////////////////////////////////////////////////////////////////


TPhOnSensorContainer::TPhOnSensorContainer()
{
 // Default constructor. Does nothing. 

  fDetector   = TPhDetector::GetInstance();
}

TPhOnSensorContainer::TPhOnSensorContainer(const Char_t *name):TPhOnContainer(name)
{
  // constructor; to book histograms use Init()
  fDetector   = TPhDetector::GetInstance();
  if(!fDetector){
    printf("WARNING: TPhOnSensorContainer:  Detector db not connencted\n");
    printf("return\n");
    return;
  }
  fOnArray = new TObjArray(1000);
}


TPhOnSensorContainer::~TPhOnSensorContainer()
{
  // normal destructor; fOnArray will be handled by the 
  // destructor of TPhOnContainer
}

void TPhOnSensorContainer::Fill(Option_t *option)
{
  // fill booked histograms for the complete detector; 
  // sofar all histograms (adc,hit,noise) of the sensors will be filled

  TString opt = option;
  opt.ToLower();
  if (opt.Contains("all"))
    {
      if(!fDetector) {
	Warning("Fill","detector not connected - return");
	return;
      }
      TCollection *sublist = fDetector->GetSubDetectorList();
      TPhSubDetector *subdetector;
      TIter NextSubDetector (sublist);
      while ((subdetector = (TPhSubDetector*) NextSubDetector()) != NULL){
	Fill(subdetector);
      }
    }
  else  Warning("Fill","option doesn't exist");
}


Int_t TPhOnSensorContainer::Fill(TPhSubDetector *subdetector)
{
  
  // fill online histograms for a given subdetector. Sofar, 
  // only subdetectors built of silicon  are included
  if(!subdetector) {
      Warning("Fill","subdetector doesn't exist - return");
      return 0;  }
  if(!fOnArray){
    Warning("Fill","fOnArray doesn't exist - return");
    return 0;
  }

  TPhOnSensor *onsens;
  if ((!strcmp("SpectN",subdetector->GetName())) ||
      (!strcmp("SpectP",subdetector->GetName())) || 
      (!strcmp("Octagon",subdetector->GetName())) ||
      (!strcmp("Vertex",subdetector->GetName())) ||
      (!strcmp("RingMult",subdetector->GetName())) )
    {
      TIter NextSensor(subdetector->GetSensorIter());
      TPhSiSensor *sisensor;
      while ((sisensor = (TPhSiSensor*) NextSensor()) != NULL)
	{
	  onsens = (TPhOnSensor*) fOnArray->At(sisensor->GetLabel());	  
	  if(onsens) onsens->Fill();
	}
    }
  return 1;
}


Int_t TPhOnSensorContainer::Init(Option_t *option)
{
  // initialize histograms for all sensors
  TString opt = option;
  opt.ToLower();
  if (opt.Contains("all"))
    {
      if(!fDetector) {
	Warning("Init","detector not connected - return");
	return 0;
      }
      TCollection *sublist = fDetector->GetSubDetectorList();
      TPhSubDetector *subdetector;
      TIter NextSubDetector (sublist);
      while ((subdetector = (TPhSubDetector*) NextSubDetector()) != NULL){
	Init(subdetector);
      }
    }
  else {
    Warning("Init","option doesn't exist");
  }
  return 1;
}

Int_t TPhOnSensorContainer::Init(TPhSubDetector *subdetector)
{
  // initialize all online histograms for a given subdetector. Sofar, 
  // only subdetectors built of silicon  are included

  if(!subdetector) {
      Warning("Init","subdetector doesn't exist - return");
      return 0;
    }
  if(!fOnArray){
    Warning("Init","fOnArray doesn't exist - return");
    return 0;
  }

  if ((!strcmp("SpectN",subdetector->GetName())) ||
      //      (!strcmp("SpectP",subdetector->GetName())) || 
      (!strcmp("Octagon",subdetector->GetName())) ||
      (!strcmp("Vertex",subdetector->GetName())) ||
      (!strcmp("RingMult",subdetector->GetName())) )
    {
      TIter NextSensor(subdetector->GetSensorIter());
      TPhSiSensor *sisensor;
      while ((sisensor = (TPhSiSensor*) NextSensor()) != NULL)
	{
	  Char_t name[256];
	  sprintf(name,"%son",sisensor->GetName());
	  TPhOnSensor *onsens = new TPhOnSensor(name);
	  onsens->Init();
	  fOnArray->AddAt(onsens,sisensor->GetLabel());
	}
    }
  return 1;
}


void TPhOnSensorContainer::Reset()
{
  // reset all histograms
   TPhOnSensor *onsens; 
   if(!fOnArray)
     {
       Warning("Reset","fOnArray doesn't exist  - return");
       return;
     }
   Int_t i;
   for(i=0; i<fOnArray->GetLast()-1; i++)
     {  
       onsens = (TPhOnSensor*) fOnArray->At(i);
       if(onsens) onsens->Reset();
     }   
}

void TPhOnSensorContainer::Draw(Char_t *sname)
{
  // Draw all histograms belonging to the sensor sname
  if(!fOnArray)
    {
      Warning("Draw","fOnArray doesn't exist  - return");
      return;
    }

  if(!fDetector) {
    Warning("Draw(Char_t sname)", " detector not connected - return");
    return;
  }
  TPhSensor *sensor = fDetector->GetSensor(sname);
  if(!sensor) {
    Warning("Draw","sensor doesn't exist - return");
    return;
  }
  TPhOnSensor *onsensor = (TPhOnSensor*)fOnArray->At(sensor->GetLabel());
  if(!onsensor) {
    Warning("Draw","no histograms for this sensor - return");
    return;
  }  
  Int_t nhisto = onsensor->GetNHistos();
  DivideCanvas(nhisto);
  Int_t i;
  Int_t flagadc = 0;
  Int_t flaghit = 0;
  Int_t flagnoise = 0;
  for (i=1; i<=nhisto; i++)
    {
      fCanvas->cd(i);
      if(onsensor->IsExistend("adc") && flagadc ==0) {
	onsensor->Draw("adc");
	flagadc = 1;}
      else if(onsensor->IsExistend("hit") && flaghit==0) {
	onsensor->Draw("hit");
	flaghit= 1;}
      else if(onsensor->IsExistend("noise") && flagnoise==0) {
	onsensor->Draw("noise");
	flagnoise=1;}
    }
}


void TPhOnSensorContainer::Draw(Char_t *sname, Option_t *option)
{
  // Draw one specific histogram, options are "adc","noise","hit"
  // for sensor sname

  TString opt = option;
  opt.ToLower();

  TPhOnSensor *onsens;
  if(!fDetector) {
    Warning("Draw","detector not connected - return");
    return;
  }
  TPhSensor *sensor = fDetector->GetSensor(sname);
  if(!sensor){
    Warning("Draw","sensor doesn't exist - return");
    return;
  }
  if(!fOnArray)
    { Warning("Draw","fOnArray doesn't exist  - return");
      return;
    }

  onsens = (TPhOnSensor*) fOnArray->At(sensor->GetLabel());
  if(!onsens) {
    Warning("Draw","no histograms for this sensor - return");
    return;
  }
  DivideCanvas(1);
  if (opt.Contains("adc"))
    {onsens->Draw("adc");}
  else if (opt.Contains("hit"))
    {onsens->Draw("hit");}
  else if (opt.Contains("noise"))
    {onsens->Draw("noise");}
  else 
    Warning("Draw(sensor,option)","wrong choice - options are: adc,noise,hit");
}

void TPhOnSensorContainer::ListHisto(Option_t *option)
{
  // List histograms, possible options are:
  // "ALL" , "EMPTY", "FILLED" 
  
  if(fOnArray->GetEntries()==0 || !fOnArray) {
    Warning("ListHisto","fOnArray empty or not existend - return");
    return;
  }
  TPhOnSensor *onsensor;
  Int_t ii = 0;
  Int_t maxhistos = fOnArray->GetLast();
  TString opt = option;
  opt.ToLower();
  if (opt.Contains("filled"))
    {
      for (Int_t i=0; i<maxhistos; i++)
	{
	  onsensor = (TPhOnSensor*) fOnArray->At(i);
	  if(onsensor)
	    {
	      if(onsensor->GetEntries() > 0)
		{
		  ii++;
		  printf(" %s\n",onsensor->GetName());
		}
	    }
	}
      printf(" %d non empty histograms \n",ii);
    }
  //*****************************************************
  else   if (opt.Contains("empty"))
    {
      for (Int_t i=0; i<maxhistos; i++)
	{
	  onsensor = (TPhOnSensor*) fOnArray->At(i);
	  if(onsensor)
	    {
	      if(onsensor->GetEntries() == 0)
		{
		  ii++;
		  printf(" %s\n",onsensor->GetName());
		}
	    }
	}
      printf(" %d  empty histograms \n",ii);
    }

  //*****************************************************
  else if (opt.Contains("all"))
    {
      for (Int_t i=0; i<maxhistos; i++)
	{
	  onsensor = (TPhOnSensor*) fOnArray->At(i);
	  if(onsensor)
	    {
	      printf(" %s\n",onsensor->GetName());
	    }
	}
      printf(" %d histograms in array \n",ii);
    }
}

Bool_t TPhOnSensorContainer::Check(Option_t *option)
{
  // performs checks on group of histograms to detect unusual detector 
  // performance. the checks themselves are defined in TPhOnSensor. 
  // sofar it only checks whether all booked histograms are filled. 

  if(fOnArray->GetEntries()==0 || !fOnArray) {
    Warning("Check","fOnArray empty or not existend - return");
    return kFALSE;
  }
  TString opt = option;
  opt.ToLower();
  if (opt.Contains("all"))
    {
      
      Int_t i;
      TPhOnSensor *onsensor;
      for (i=0; i<=fOnArray->GetLast(); i++)
	{
	  onsensor = (TPhOnSensor*)fOnArray->At(i);
	  if(onsensor) {
	    if(onsensor->GetEntries("adc")==0) return kFALSE;
	    if(onsensor->GetEntries("hit")==0) return kFALSE;
	    if(onsensor->GetEntries("noise")==0) return kFALSE;
	  }
	}
    }
  return kTRUE;
}

Bool_t TPhOnSensorContainer::Check(Char_t *sname, Option_t *option)
{
  // performs checks on a single histogram of sensor sname to detect unusual 
  // detector performance. the check itself is defined in TPhOnSensor. 
  // options are "all","adc","hit","noise"

  if(fOnArray->GetEntries()==0 || !fOnArray) {
    Warning("Check","fOnArray empty or not existend - return");
    return kFALSE;
  }
  if(!fDetector){
    Warning("Check","detector not connected - return");
    return kFALSE;
  }

  TPhSensor *sensor = fDetector->GetSensor(sname);
  if(!sensor){
    Warning("Check","sensor doesn't exist - return");
    return kFALSE;}
  TPhOnSensor *onsensor = (TPhOnSensor*) fOnArray->At(sensor->GetLabel());
  if(!onsensor) {
    Warning("Check","no histograms for that sensor -return");
    return kFALSE;}
  TString opt = option;
  opt.ToLower();
  if (opt.Contains("all"))
    {
      if(onsensor->GetEntries("adc")==0) return kFALSE;
      if(onsensor->GetEntries("hit")==0) return kFALSE;
      if(onsensor->GetEntries("noise")==0) return kFALSE;
    }
  else if(opt.Contains("adc")) {
    if(onsensor->GetEntries("adc")==0) return kFALSE;}
  else if(opt.Contains("hit")) {
    if(onsensor->GetEntries("hit")==0) return kFALSE;}
  else if(opt.Contains("noise")) {
    if(onsensor->GetEntries("noise")==0) return kFALSE;}
  else {
    Warning("Check","option unkown - return");
    return kFALSE;}
  return kTRUE;
}





