       Subroutine PH_OUT
*Documentation_Block
************************************************************************
*
*  Description:
*  User routine called at the end of each event.
*  This version writes  AOU output file and/or transfers information
*  to PHAT memory, may also make histos and print ntuples.
*
*  Arguments:
*  none
*
*  Called by:
*  USOUT
*
*  Calls:
*  GFHITS, GFKINE, HFN, HFILL, PRNTUP, USANAL, VZERO, PIXEL, STOPER
*  SPDEL, CONCATB, UHTOC, ADDPMCPART, ADDPMCHIT
*
*  Implicit inputs, outputs, side effects:
*  Fills histograms
*
*  Filename: PMC_SRC:PH_OUT.FOR
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  24-Nov-98  K.W.       Modifications necessary for sending run and event
*                        header to Phat
*  13-Nov-98  K.W.       Added ISTOP for particles stopping inside a sensor
*                        to the particle header
*   7-Jul-98  K.W.       Track length added to the list of hit parameters
*                        Both 11 and 12 paramaters in hits bank supported
*  24-Feb-98  K.W.       Add ILAST parameter in PIXEL call
*                        to improve merging of hits in the same pixel
*                        add safety against non standard detector types
*  10-Feb-98  K.W.       added interface to PHAT - calls to ADDPMCPART and
*                        ADDPMCHIT in case ISMEM is greater than 0
*                        FIRSTC  moved to a common to enable restart of
*                        simulations in PHAT
*  28-Jan-98  K.W.       warning in the case GFHITS returns too many hits;
*                        a call to GUFLD at the first entry, to initialize GUFLD
*                        before EOH (end of header mark) is written to AOU file
*  27-Nov-97  K.W.       added compact and fixed format of AOU output
*                        (in particle and hit data lines only), formats corrected
*  21-Aug-97  K.W.       pixel number (in x and y direction) added
*                        to hits information in AOU data
*  29-Apr-97  K.W.       Expand write fieldwidths, add IOSTAT
*   4-Mar-97  K.W.       Removed a call to UHITSOUT
*                        Changed AOU file format:
*                        integer detector number in place of name is used
*                        Forced STOP changed into a call to STOPER
*  25-Feb-97  K.W.       Add  End of Header (EOH)  line in AOU file
*  20-Nov-96  K.W.       Corrected origin volume name for secondaries in
*                        SECT=1 mode
*  14-Nov-96  K.W.       Corrected format in the AOU output
*  23-Sep-96  K.W.       Added signal link (set to zero) to make
*                        the format of AOU file identical with digitized data
*  25-Jul-96  R. V.      Define integers for testing /GCLIST/ LSETS
*                          for ANSI compliance
*  21-Jul-96  R. V.      Removed unused de, nh, tlife
*   1-Jul-96  K.W.       The particles and their hits are stored in an
*                        ASCII output file
*  13-Jun-94  R. V.      Allow dynamic specification of detector sets,
*                        controlled by SETS in the control file
*  13-May-94  R. V.      Use all 11 (new) HITS parameters.
*  09-Aug-93  M. Baker   Add Geant PID code to ACCCHK call.
*  29-Jun-93  M. Baker   BUGFIX: Set AMASS for use in TK.
*  29-Jun-93  M. Baker   Add acceptance check, modify Ntuples.
*  28-Jun-93  M. Baker   Protect against P=0, Pt=0, NHITSL>NHTM.
*   7-Jun-93  R. V.      Move IEVENT and Epstate reset to GUOUT
*  14-May-93  R. V.      Multipass mods: add NtPrev to stored track #
*  22-Feb-93  R. V.      Correct directory handling via HCDIR
*   5-Feb-93  R. V.      Add optional call to UHITSOUT for hits text file
*  12-Jan-93  R. V.      Change to separate Hits and Truth ntuples;
*                        COMMON.NTU & COMMON.HST -> NTU.CMN & HST.CMN
*  17-Nov-92  R. V.      Separated PH_OUT
*  21-Jul-92  R. V.      Interchange Pvert(1) & Pvert(1) in Tphi calc.
*   1-Jul-92  R. Verdier Store additional information in ntuple
*  17-Jun-92  C. Parsons Separated from package including other subrs.
*     Apr-92  D. Mcleod  Creation in GPH package as GUOUT
************************************************************************
*End_Documentation_Block

      IMPLICIT NONE

*KEEP,BANGC.
C Event generator parameters                 File: BANGC.CMN
C 20-Sep-96  K. Wozniak Added new parameters to control BANG mode = 2
C                       RSPX, RSPZ, RSPR (removed RMAG)
C 10-Mar-95  K. Wozniak Added new parameters to control track generation
c                       and selection (ETAMIN,ETAMAX, IDMIN,IDMAX ..)
c 22-Jun-94  R. V.      Remove NSEQN storage (see PH_HIT)
c 10-May-94  D. McLeod  Increased MXMUL to 13000 for HIJET events
c 17-Dec-92  R. Verdier Changed name from BANG to BANGC for CMS
c 22-Nov-92  D. Mcleod  Added EXF and NSEQN
      INTEGER MXMUL
      PARAMETER (MXMUL=13000)  ! maximum multiplicity for BANG etc.
      INTEGER IRMUL         !  multiplicity in random particle generation
      INTEGER IRARM         !  particles going to 1 or 2 arms selected
      INTEGER IRPID(2)      !  particle id range: min, max
      REAL RMOM(3)          !  momentum of particles:  min, max and step
                            !  (first step if 1/p is changed)
      REAL REXP             !  mean transv. mom. - for momentum generation
      REAL RETA(2)          !  ETA range: min, max
      REAL RANG(3)          !  starting theta angle and step
      REAL RPHI(2)          !  range of emission phi angle: min, max
      REAL RROT(2)          !  phi initial and additional rotation angle (GEN=0)
      REAL RBXX,RBYY,RBZZ   !  central position of the vertex
      REAL RBXY             !  width of the beam intersection in x-y
      REAL RBDX,RBDY,RBDZ   !  the range of positions of the vertex
C                           !
      REAL RSPX,RSPZ,RSPR   !  position and radius of the circle describing
C                           !  area covered by spectrometer
      REAL PHIR,CZ,ZC,RS    !  working variables for BANG
      COMMON/RHBNG/IRMUL, IRARM, IRPID, RMOM, REXP, RETA, RANG, RPHI,
     1             RROT,
     2             RBXX,RBYY,RBZZ, RBDX, RBDY, RBDZ, RBXY,
     3             RSPX,RSPZ,RSPR, PHIR,CZ,ZC,RS
*KEEP,FLAG.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG.CMN
C   7-Jul-98  K. W.      Added  ITRIG flag (initial numer in TRIG command)
C  27-Nov-97  K. W.      Added IAOUF flag
C  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
C  22-Nov-92  D. McLeod
C   These are set in input cards
      INTEGER ISASC,ISHBO,ISHDO,ISNTU, ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     1        IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB, ITRIG
      REAL WPIXL,WLOSS
      COMMON/RHFLG/ISASC,ISHBO,ISHDO,ISNTU,
     1  ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     2  IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB,WPIXL,WLOSS, ITRIG
*KEEP,GCBANK.
      INTEGER IQ,LQ,NZEBRA,IXSTOR,IXDIV,IXCONS,LMAIN,LR1,JCG
      INTEGER KWBANK,KWWORK,IWS
      REAL GVERSN,ZVERSN,FENDQ,WS,Q
C
      PARAMETER (KWBANK=69000,KWWORK=5200)
      COMMON/GCBANK/NZEBRA,GVERSN,ZVERSN,IXSTOR,IXDIV,IXCONS,FENDQ(16)
     +             ,LMAIN,LR1,WS(KWBANK)
      DIMENSION IQ(2),Q(2),LQ(8000),IWS(2)
      EQUIVALENCE (Q(1),IQ(1),LQ(9)),(LQ(1),LMAIN),(IWS(1),WS(1))
      EQUIVALENCE (JCG,JGSTAT)
      INTEGER       JDIGI ,JDRAW ,JHEAD ,JHITS ,JKINE ,JMATE ,JPART
     +      ,JROTM ,JRUNG ,JSET  ,JSTAK ,JGSTAT,JTMED ,JTRACK,JVERTX
     +      ,JVOLUM,JXYZ  ,JGPAR ,JGPAR2,JSKLT
C
      COMMON/GCLINK/JDIGI ,JDRAW ,JHEAD ,JHITS ,JKINE ,JMATE ,JPART
     +      ,JROTM ,JRUNG ,JSET  ,JSTAK ,JGSTAT,JTMED ,JTRACK,JVERTX
     +      ,JVOLUM,JXYZ  ,JGPAR ,JGPAR2,JSKLT
C
*KEEP,GCFLAG.
      COMMON/GCFLAG/IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT(10),IFINIT(20),NEVENT,NRNDM(2)
      COMMON/GCFLAX/BATCH, NOLOG
      LOGICAL BATCH, NOLOG
C
      INTEGER       IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT,IFINIT,NEVENT,NRNDM
C
*KEEP,GCLIST.
      COMMON/GCLIST/NHSTA,NGET ,NSAVE,NSETS,NPRIN,NGEOM,NVIEW,NPLOT
     +       ,NSTAT,LHSTA(20),LGET (20),LSAVE(20),LSETS(20),LPRIN(20)
     +             ,LGEOM(20),LVIEW(20),LPLOT(20),LSTAT(20)
C
      INTEGER       NHSTA,NGET ,NSAVE,NSETS,NPRIN,NGEOM,NVIEW,NPLOT
     + ,NSTAT,LHSTA,LGET ,LSAVE,LSETS,LPRIN,LGEOM,LVIEW,LPLOT,LSTAT
C
*KEEP,GCNUM.
      COMMON/GCNUM/NMATE ,NVOLUM,NROTM,NTMED,NTMULT,NTRACK,NPART
     +            ,NSTMAX,NVERTX,NHEAD,NBIT
      COMMON /GCNUMX/ NALIVE,NTMSTO
C
      INTEGER      NMATE ,NVOLUM,NROTM,NTMED,NTMULT,NTRACK,NPART
     +            ,NSTMAX,NVERTX,NHEAD,NBIT ,NALIVE,NTMSTO
C
*KEEP,GCVOLU.
      COMMON/GCVOLU/NLEVEL,NAMES(15),NUMBER(15),
     +LVOLUM(15),LINDEX(15),INFROM,NLEVMX,NLDEV(15),LINMX(15),
     +GTRAN(3,15),GRMAT(10,15),GONLY(15),GLX(3)
C
      INTEGER NLEVEL,NAMES,NUMBER,LVOLUM,LINDEX,INFROM,NLEVMX,
     +        NLDEV,LINMX
      REAL GTRAN,GRMAT,GONLY,GLX
*KEEP,HST.
C  File HST.CMN (originally COMMON.HST)
C  Common block for inline plot control.. to generate plots during
C  the GEANT run from the track Ntuples (rather than later via PAW)
C  Its contents are set by control cards in the same file as FFREAD:

C  HIST ID TAGNAME  (short form.. default bins and interval)
C    TAGNAME can be Ntuple tag name or e.g. #35 for 35th entry

C  HIST ID TAGNAME NCHANNELS XMIN XMAX is the long form.
C  HIST ID TAGX NX XMI XMA TAGY NY YMI YMA  scatterplot, args as HBOOK2
C  (The number of arguments distinguishes these three formats.)

C      HCONTROL(I,J), J = 1,NHMX contains the control information:
C  I = 1:  User supplied ID number of plot, floated integer
C  I = 2:  Ntuple entry number (floated), from TAGNAME or # input
C  I = 3:  number of channels (0 if short form card)
C  I = 4-5 or 4-9: arguments as in card formats above, all floating

      INTEGER NHMX,NHSTS
      PARAMETER(NHMX=50)
      REAL HCONTROL
      COMMON/RHHST/HCONTROL(9,NHMX),NHSTS

*KEEP,NTU.
C Common block for Ntuple image and other analysis info.  File NTU.INC
C There may be one Truth Ntuple and/or one Hits Ntuple for each track.
C These must be correlated into events using the event number and other
C event information.
C 10-Jul-95  K. W.      Increased number of planes for spectrometer to 17
C 10-Mar-95  K. W.      Increased number of planes for spectrometer
C 19-Jul-94  R. V.      Update tagnames: Cxx -> Dxx, VOLTERM -> VOLUME
C 10-May-94  R. V.      Modify Ntuples to contain new hits information;
C                       add NTTAGS common for tags; move data to NTUD.INC
C 29-Jun-93  M. Baker   Modify Ntuples
C 10-Jan-93  R.Verdier  Separate Hits and Truth ntuples; rename to NTU.CMN
C    Mar-92  D. Mcleod  Creation as COMMON.NTU
C Number:  Name:    --------------- Description:----------------------------
C>>>>>>>GENERAL INFORMATION<<<<<<<<<<<<<<<<<<
*   1      RUN      run number
*   2      EVENT    event number
*   3      TRACK    track number
*   4      GEN      event generator type; 1 = fixed, 2 = random, etc.
*   5      ANAL     event analysis type; 1 = Ntuple only, 2 = resolution, etc.
*   6      SCAT     ISCAT = IMULS + 3*ILOSS + 9*IPIXL; 1st 2 are in /GCPHYS/
*   7      BIMP     Impact Parameter   ** NOT YET IMPLEMENTED **
C>>>>>>>THE TRUTH<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
*   8      XPROD    track origin
*   9      YPROD       "    "
*   10     ZPROD       "    "
*   11     IDPART   particle type (GEANT code)
*   12     CHRG     particle charge
*   13     TP       particle momentum (at production)
*   14     TK          "     kinetic energy    "
*   15     TPHI        "     azimuth angle
*   16     TTHETA      "     polar angle
C>>>>>>>ANALYSIS RESULTS..FOR THE WHOLE TRACK<<<<<<<<<<<<<<<<<<
*   17     ACCEPT   Acceptance Flag (+/-1=acc. in +/- wing, 2=both, 0=neither)
*   18     NHITSL   number of hits on track, set to a maximum of NHTM
*   19     USR1     spare, for later use
*   20     USR2              "
C>>>>>>>TRACK HITS<<<<<<<<
* The Truth ntuple stops here; the Hits ntuple continues with
* A series of particle history words
* NHITSL groups of 12 words each, at index = 32 + 12*(Ihit-1):
*
*   21     KCASE    creation mechanism
*   22     PARENT   track number
*   23     VOLUME   creation volume name
*   24     XTERM
*   25     YTERM
*   26     ZTERM
*   27     PXTERM
*   28     PYTERM
*   29     PZTERM
*   30     ETERM
*   31     STOP     stop mechanism

*   32+    PL       plane number hit
*   33+    X        Track center in plane in master coords (maybe pixeled)
*   34+    Y          "     "     "   "    "  "      "       "       "
*   35+    Z          "     "     "   "    "  "      "       "       "
*   36+    DE       Energy deposited in the plane
*   37+    DX       Exit minus entry position in master coord system
*   38+    DY        "    "    "    "      "        "    "
*   39+    DZ        "    "    "    "      "        "    "
*   40+    EFIN     track energy at exit from plane
*   41+    TOF      time registered by a TOF detector, in seconds
*   42+    ITRK     track number for this hit
*   43+    IS       Combined information:  IS = IVOL + 1000 * ISEQ
*                   ISEQ = sequential (first = 1) hit number on this track
*                   IVOL = system volume number of the plane
C The maximal number of hits is now NHTM = 17
C There are thus 31 + 12*NHTM or 235 entries in a Hits Ntuple, with zeroes
C after the last hit

      INTEGER NVMCTUP,NVTUP,NTOFF,NHTM
      PARAMETER (NTOFF=31, NVMCTUP=20, NHTM=17)
      PARAMETER (NVTUP=31+12*NHTM)

      REAL    RUN, EVENT, TRACK, GEN, ANAL, SCAT, BIMP
      REAL    XPROD, YPROD, ZPROD, IDPART, CHRG, TP, TK, TPHI, TTHETA
      REAL    ACCEPT, NHITSL, USR1, USR2
      REAL    STOP, PARENT, VOLTERM, XTERM, YTERM, ZTERM, PXTERM, PYTERM
      REAL    PZTERM, ETERM, UNUSED
      REAL    TRK(12,NHTM), TUP(NVTUP)

      COMMON/RHTUP/ RUN, EVENT, TRACK, GEN, ANAL, SCAT, BIMP,
     1  XPROD, YPROD, ZPROD, IDPART, CHRG, TP, TK, TPHI, TTHETA,
     2  ACCEPT, NHITSL, USR1, USR2, STOP, PARENT, VOLTERM, XTERM,
     3  YTERM, ZTERM, PXTERM, PYTERM, PZTERM, ETERM, UNUSED,
     4  TRK

      EQUIVALENCE (TUP(1),RUN)

      COMMON /NTTAGS/ TAGS(NVTUP)
      CHARACTER*8 TAGS
*KEEP,MUPEP.
c MUPEP.CMN: PHOBOS MC multi-pass event processing variables
c 16-Mar-95 K. W.     NeSkip added
c  6-Jun-93 R. V.     Creation
c Multi-pass event processing breaks up each event into groups of
c tracks, in order to avoid overflowing Zebra track storage.
c Common for access to control variables:
      common /MUPEP/ EPState, NtConc, NtpE, NtPrim, NeSkip, NtSkip,
     1  MPind, NtPrev, NtTot, NtMax, NvPrev
c Parameters and data values are assigned in MUPEP.CDA.
c Event processing state: EPState = 0 for first pass,
c 1 for continuing pass, 2 for final pass
      integer EPState
c Current pass index:
      integer MPind
c Maximum number of tracks to process concurrently in a group:
      integer NtConc
c Maximum track number for this group:
      integer NtMax
c Maximum number of tracks per event:
      integer NtpE
c Number of primary and secondary tracks generated before this pass:
      integer NtPrev
c Number of primary tracks and vertices generated:
      integer NtPrim, NvPrev
c Number of input file events to skip initially:
      integer NeSkip
c Number of tracks to skip initially:
      integer NtSkip
c Total number of primary and secondary tracks generated:
      integer NtTot
*KEEP,IO.
C Common block for GPH logical units etc.: file IO.CMN
c 13-Aug-96  R. V.       Added USRBATCHK for KUIP commands for UGCONS
c 21-Jul-96  R. V.       Remove unused record; note CARDDEF now obsolete
c 22-Feb-93  R. V.       Added USRMCNTU and IMCNTU for MC Ntuple
c 14-Nov-92  R. Verdier  Added UGCIN, IUGCIN, CARDDEF and ICARDDEF
c                        for UGC and EUCARD (obsolete) input units
C    Apr-92  C. Parsons  Replacement for original COMMON.CDT
      INTEGER USRGCIN, CARDDEF, USRIN, USROUT, USRMAP, USRNTU, USRZEB,
     *  USRMCNTU, USRBATKUMAC
      PARAMETER (USRIN = 4, USROUT = 16, USRMAP = 18)
      PARAMETER (USRNTU = 15, USRZEB = 17, USRGCIN = 19, CARDDEF = 20)
      PARAMETER (USRMCNTU = 21, USRBATKUMAC = 22)
      INTEGER ICARDDEF, IMCNTU, IUSRGCIN, IUSRIN, IUSROUT, IUSRMAP,
     *  IUSRNTU, IUSRZEB, IUSRAOU, IUSRBATKUMAC
      COMMON /IO/ IUSRIN, IUSROUT, IUSRMAP, IUSRNTU, IUSRZEB, IUSRGCIN,
     *  ICARDDEF, IMCNTU, IUSRAOU, IUSRBATKUMAC
*KEEP,RHSIL.
C RHSIL.CMN: PHOBOS detector plane dimensions, positions, and
c granularity; UGC parameters
c The actual and maximum numbers of detector planes and detector hit
c parameters.  Their default values are defined in RHSILD.INC.
c 12-Dec-97 K. W.       PADSOLD and PADSNEW from UGCSET moved to a common
c  4-Nov-97 K. W.       Increased number of PADS parameters to 10
c                       Increased NSLPM to 800
c  1-Jul-97 K. W.       Added data for GEANT pixelization (GSDVN)
c  1-May-97 R. V.       Increased NSLPM to 500
c  3-MAR-97 K. W.       Removed SPL, added PADS data
C  3-Jan-97 K. W.       Increased NSLPM to 180
c  3-Jul-96 K. W.       Geometry version number added
c  5-Apr-96 D. McL.     Moved data value of UgcpMax to RHSILD
c  ...
c    Mar-92 D.McLeod    Creation
c Storage limit for number of silicon planes:
      INTEGER NSLPM
      PARAMETER (NSLPM=800)
c Storage limit for number of parameters for each hit:
      integer NSipMaxp
      parameter (NSipMaxp = 12)
c Safety limit for number of parameter values that may be entered in
c a UGC text input record:
      integer UgcpMaxp
      parameter (UgcpMaxp = 10)
c Number of silicon planes in use:
      integer NSLP
c Additional information on detectors - pad sizes:
C  DETNAME  - name of the detector volume
C  DIVXNAME - name for division in X direction
C  DIVYNAME - name for division in Y direction
C  DIVZNAME - name for division in Z direction
C  NDIVX    - number of divisions in X direction
C  NDIVY    - number of divisions in Y direction
C  NDIVZ    - number of divisions in Z direction
C  IPADFL   - pad type flag
C  PADn   - four pad parameters
C             (pad size in X, Y, Z direction for IPADFL=1 for example)
      CHARACTER*4 DETNAME(NSLPM), DIVXNAME(NSLPM),
     +             DIVYNAME(NSLPM), DIVZNAME(NSLPM)
      INTEGER NDIVX(NSLPM), NDIVY(NSLPM), NDIVZ(NSLPM)
      REAL PAD1(NSLPM),PAD2(NSLPM),PAD3(NSLPM),PAD4(NSLPM),PAD5(NSLPM)
      REAL PAD6(NSLPM),PAD7(NSLPM),PAD8(NSLPM),PAD9(NSLPM),PAD10(NSLPM)
      INTEGER NDETS, IPADFL(NSLPM)
      LOGICAL PADSOLD, PADSNEW
      COMMON /RHPADS/ NDETS, PADSOLD, PADSNEW,
     +       DETNAME, DIVXNAME, DIVYNAME, DIVZNAME,
     +       NDIVX, NDIVY, NDIVZ,
     +       IPADFL, PAD1, PAD2, PAD3, PAD4, PAD5,
     +               PAD6, PAD7, PAD8, PAD9, PAD10
c Number of silicon planes - compared with final value of NDETS
      COMMON /RHSIL/ NSLP
c UGC parameter information:
      integer UgcpMax
      real UGCpars(UgcpMaxp)
c Declare silicon plane parameters; their default values are specified
c in RHSILD.INC
      integer NSIP
c Names of parameters:
      CHARACTER*4 SINAMH(NSipMaxp)
c Mumber of bits stored for each parameter:
      INTEGER SIBITH(NSipMaxp)
c Multiplicative scale factors and origins, defined so that
c   Stored value = (Parameter value + SIORIG) * SIFACT:
      REAL SIFACT(NSipMaxp), SIORIG(NSipMaxp)
c Common for access to UGC parameters and hit parameter definitions:
      common /UGCPAR/ UgcpMax, UGCpars,
     1                NSIP, SINAMH, SIBITH, SIORIG, SIFACT
      real GEOV  !  geometry version number
      common /ugGEOV/ GEOV
*KEEP,UHITS.
c UHITS.CMN: control variables for UHITSOUT.FOR
c  5-Feb-93  R. Verdier     Creation
      integer draw_mode, HitsOut
      logical draw_hits
      common /Hitsocon/ HitsOut, draw_hits, draw_mode
*KEEP,UJKBUF.
C  PHOBOS JKINE user buffer definitions.          File: UJKBUF.INC
c 18-Jul-94  R. V.  Added NHitsAct
	integer Nwubuf, Nwubmax, NHitsAct
	parameter (Nwubmax = 11)
	common /UJKB/ Nwubuf, Ubuf(Nwubmax), NHitsAct
	real Ubuf
	integer IUBUF(Nwubmax)
	equivalence (IUBUF, UBUF)
c NHitsAct:   The number of hits on the track being propagated
c The user words at present are:
c UBUF(1):    Process in which particle was created (DCAY, BREM etc.)
c     (2):    Parent track number, negative is parent not stored
c             (possible in SECT=1 mode)
c     (3):    Volume number at origin, (4 characters in one integer word)
c             not definded for primaries and undefined for all tracks
c             in SECT=1 mode (integer value 0)
c     (4-6):  X, Y, Z at termination
c     (7-10): Px, Py, Pz, E at termination
c     (11):   Istop at track termination, positive when particle stopped
c             in a sensitive volume, negative otherwize
*KEEP,HITPIX.
C HITPIX.CMN: PHOBOS hits information for AOU output
c   20-Nov-98  K. W.  Added INDX table for hits sorting
c    7-Jul-98  K. W.  Increased NHITPAR to 12, but allow variable NHITSPAR
c                     depending on UGCDATA (in PH_OUT code)
c   10-Feb-98  K. W.  Increased NHHTMAX  parameter,
C                     NHITSPAR parameter defined
c   21-Aug-97  K. W.  Creation
c Storage limit for number of hits on a single track (including
c multiple hits in a plane due to traversing several pixels):
      integer NHHTMAX, NHITSPAR, NHITSBM
      parameter (NHITSPAR=12, NHHTMAX=500)
      parameter (NHITSBM=NHITSPAR*NHHTMAX)

      INTEGER NHHT, IPLHT(NHHTMAX), INDX(NHHTMAX)
      CHARACTER*4 PLHT(NHHTMAX)
      EQUIVALENCE (IPLHT(NHHTMAX),PLHT(NHHTMAX))
      REAL XHT(NHHTMAX), YHT(NHHTMAX), ZHT(NHHTMAX)
      REAL DEHT(NHHTMAX), DTHT(NHHTMAX), TRACKLNG(NHHTMAX)
      REAL DXHT(NHHTMAX), DYHT(NHHTMAX), DZHT(NHHTMAX)
      INTEGER IXPIX(NHHTMAX), IYPIX(NHHTMAX)

      REAL HITSBUF(NHITSBM)
      INTEGER NUMBV(NHHTMAX), ITRATAB(NHHTMAX)

      COMMON /CHITPIX/ NHHT, IPLHT, XHT, YHT, ZHT, DEHT, DTHT,
     +     DXHT, DYHT, DZHT, IXPIX, IYPIX, TRACKLNG,
     +     HITSBUF, NUMBV, ITRATAB, INDX
*KEEP,FIRSTC.
C  PHOBOS "first call" flags,   File FIRSTC.CMN
C  The flags denoting the very first call are put in one common
C  to enable easy restart of simulations in PHAT
C  11-Feb-98  K. W.      Creation
C
      LOGICAL FCGUFLD, FCGUSTEP, FCPHOUT
      COMMON /CFIRSTC/ FCGUFLD, FCGUSTEP, FCPHOUT
*KEND.
c
C     Local definitions:
C     ==================

      CHARACTER*150  AOUBUF1,AOUBUF2, AOUBUFS
      INTEGER IAOULNG

      INTEGER LWRTRMAX
      PARAMETER (LWRTRMAX=30000)
      LOGICAL LWRTR(LWRTRMAX)

      LOGICAL PLNHIT(NSLPM)
      INTEGER IPART,ITRK,IPARENT,NNTRACK,IVOL,NVERT,NUB
      INTEGER I,ID,IX,IY, J1, J2
      INTEGER ILAST, NHHT1, ISTOPSENS
      integer Idet, IIudet, IIuset, ios, Iplane, Iset, Isret, J, Jdh,
     1        Jdp, NHITS, Nhitp, Nsetsa, LSTPLN, ISLPL
c Ndetsa = number of active detectors in each set, determined from
c UGCDATA and from LSETS in /GCLIST/, set by the SETS card.  Its
c dimension should be >= that of LSETS.
      integer Ndetsa(20)
      character*4 Iudet, Iuset, CVOL
      equivalence (Iudet,IIudet), (Iuset,IIuset)
c Equivalences for testing CERN integer-type character strings
      character*4 chall /'ALL'/, challlc /'all'/, chblank /'    '/,
     1  chstar /'*'/
      integer all, alllc, blank, star
      equivalence (all,chall), (alllc,challlc), (blank,chblank),
     1  (star,chstar)
      REAL RDUM, RDAR(20)
      REAL VERT(3),PVERT(4),AMASS,ACHG
      REAL UB(NWUBMAX)
      INTEGER IUB(NWUBMAX)
      EQUIVALENCE (UB,IUB)
      CHARACTER*20 ADUM20
      INTEGER IDUM, NDUM
c Storage for additional information in ntuples, temporarily
c overstoring ntuple information
      REAL UJKBN(NWUBMAX)
      EQUIVALENCE (UJKBN(1), STOP)

      REAL DETHIT(NHITSPAR)
      REAL     ACCCHK
      EXTERNAL ACCCHK

C     Executable Statements
C     =====================
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

****************************************************
*   Loop over all the tracks including secondaries *
****************************************************

      DO 100 ITRK = 1,NTRACK

C       Get the "truth"
        CALL GFKINE(ITRK,VERT,PVERT,IPART,NVERT,UBUF,NUB)

C       Omit nonexistent tracks
        IF(IPART.EQ.0) GOTO 100

C       Fill some of /RHTUP/:
         CALL VZERO(TUP,NVTUP)
         RUN = IDRUN
         EVENT = IEVENT
         BIMP  = 0.0     ! Not Yet Implemented
         XPROD = VERT(1)
         YPROD = VERT(2)
         ZPROD = VERT(3)
         IDPART = IPART
C Get the charge and mass of the particle.
         CALL GFPART( IPART, ADUM20, IDUM, AMASS, ACHG, RDUM,
     >                                                   RDAR, NDUM)
         CHRG = ACHG
c Store the track number for the whole event, not just this pass
         TRACK = ITRK + NtPrev
         GEN = IGEN
         ANAL = IANAL
         SCAT = ISCAT
         TP = SQRT(PVERT(1)**2+PVERT(2)**2+PVERT(3)**2)
         TK = PVERT(4) -AMASS
C MDB: Protect against P=0 or Pt=0.
         IF (ABS(PVERT(1)).GE.0.0001 .OR. ABS(PVERT(2)).GE.0.0001) THEN
            TPHI = ATAN2(PVERT(2),PVERT(1))
         ELSE
            TPHI = 0.0
         ENDIF
         TTHETA = 0.0
         IF (TP.GT.0.0001) THEN
           IF(ABS(PVERT(3)) .LE. TP)  THEN
             TTHETA = ACOS(PVERT(3)/TP)
           ELSE
             IF(PVERT(3) .GT. 0.0)  TTHETA = 0.0
             IF(PVERT(3) .LT. 0.0)  TTHETA = ACOS(-1.0)
           ENDIF
         ENDIF

C Initialize hit counters: Nhitsl is the total hit count, Iplane is
c the cumulative index for all detectors regardless of set
         NHITSL = 0
         LSTPLN = 0
         Iplane = 0

C  Total number of hits on the track
         NHHT = 0

****************************************************************
* Get quantities from HITS.  Cycle over Si plane numbers:      *
****************************************************************

c Get the number of active detector sets, Nsetsa, from GCBANK.
c Utilities to do this ought to be available.

         Nsetsa = IQ(JSET - 1)

         do Iset = 1, Nsetsa

c Get the ASCII set name, Iuset, and the number Ndets of detectors in it
c IIuset is the integer alias of the character string.
           IIuset = IQ(JSET + Iset)
           Jdp = LQ(JSET - Iset)

c On first track only, set the number of detectors used in each set to
c be the number specified in the geometry data file, as modified by
c LSETS, loaded from the SETS line in the control file:
c if SETS(1) is blank, *, ALL or all, then all sets specified will be
c active; otherwise, only detector sets listed on the SETS line will be
c active.  Thus,  SETS none   would turn off all detectors, unless there
c were actually a set called 'none'.  If there is no SETS line, all
c specified sets are active.
           if (TRACK .eq. 1) then
             Ndetsa(Iset) = IQ(Jdp - 1)
             if (LSETS(1) .ne. blank .and. LSETS(1) .ne. star .and.
     1         LSETS(1) .ne. all .and. LSETS(1) .ne. alllc) then
               call GLOOK(Iuset, LSETS, Nsets, Isret)
               if (Isret .le. 0) Ndetsa(Iset) = 0
             endif
           endif

           do Idet = 1, Ndetsa(Iset)

             Iplane = Iplane + 1
             PLNHIT(Iplane) = .FALSE.

c Get the ASCII detector name and check the number of hit parameters
             IIudet = IQ(Jdp + Idet)
             Jdh = LQ(Jdp - Idet)
             Nhitp = IQ(Jdh + 4)
             if(Nhitp .gt. NHITSPAR)  then
                write(*,*) ' ***** PH_OUT: wrong detector type: ',
     +             'hits bank size greater than NHITSPAR:', Nhitp
                write(*, '(''       detector: '', a4)') iiudet
                call stoper('PH_OUT', 'WRONG DETECTOR', 1)
             endif

c Get and store the hits on this detector.
c NOTE: Multiple hits on one plane from one track are ignored here.
             call GFHITS (Iuset, Iudet, 1, Nhitp, NHHTMAX-1, ITRK, 0,
     1                    ITRATAB, NUMBV, HITSBUF, NHITS)

             IF (NHITS .GE. NHHTMAX) THEN
                NHITS = NHHTMAX - 1
                WRITE(*,*) ' *** WARNING IN PH_OUT: GFHITS ',
     +           'RETURNS MORE THAN', NHITS, ' HITS, EXCEEDING ',
     +           'HITS IGNORED'
             ENDIF

             IF (NHITS .gt. 0) THEN

               ISLPL = 0
               DO  J=1,NSLP
                  IF(DETNAME(J) .EQ. IUDET)  ISLPL = J
               ENDDO
               IF(ISLPL .EQ. 0)  WRITE(*,*) ' *** WARNING IN PH_OUT:',
     +            ' NO PADS DATA FOR DETECTOR  ', IUDET

               LSTPLN = Iplane
               PLNHIT(Iplane) = .TRUE.

               NHHT1 = NHHT
               DO  J=1,NHITS
                 DO  I=1,NHITSPAR
                   IF(I .LE. Nhitp)   THEN
                      DETHIT(I) = HITSBUF(Nhitp*(J-1)+I)
                   ELSE
                      DETHIT(I) = 0.0
                   ENDIF
                 ENDDO
                 ILAST = 0
                 IF(J .GT. 1)
     +              ILAST = IFIX(HITSBUF(Nhitp*(J-2)+11)+0.01)
                 CALL PIXEL(ISLPL, DETHIT, ILAST)
               ENDDO

               IF(NHHT1 .LT. NHHT  .AND.
     +                Nhitp .LT. NHITSPAR)  THEN  !  track length meaningless
                 DO  I=NHHT1,NHHT
                   TRACKLNG(I) = 0.0
                 ENDDO
               ENDIF
               IF (NHITSL .lt. NHTM) THEN
                    NHITSL = NHITSL + 1

C Fill the entries for the hit in the Ntuple, starting with cumulative
c plane index, provided the hit storage is not exceeded.
c Q: WOULDN'T IUDET BE A BETTER VARIABLE FOR NTUPLE PLANE TAGGING?

                    TRK(1,NHITSL) = Iplane
                    DO  J = 1, MIN(Nsip, NsipMaxp)
                      IF(J .LT. Nhitp)  THEN
                         TRK(J+1,NHITSL) = HITSBUF(J)
                      ELSE
                         TRK(J+1,NHITSL) = 0.0
                      ENDIF
                    ENDDO

               ENDIF

             ENDIF    !  NHITS .GT. 0

           enddo  ! Idet

         enddo  ! Iset

*************************************************************
*        Decide whether a particle has been accepted        *
*        Note that this acceptance check doesn't check      *
*        detector type                                      *
*************************************************************

         IF (NHITSL.EQ.0) THEN
            ACCEPT = 0.0
         ELSE
            ACCEPT = ACCCHK( TP, IPART, PLNHIT, DETNAME, NHITSL, LSTPLN)
         ENDIF

*****************************************************************
*       Any other quantities added to spares in Ntuples:        *
*****************************************************************

         CALL USANAL


C   Check if there are ancestors of the track with hit(s)
C   that should be written to ASCII file
         IPARENT = UBUF(2)
         IF((ISAOU .EQ. 3  .OR.  ISMEM .EQ. 3)  .AND.
     +                                         ISECT .GE. 2)   THEN
           IF(ITRK .GT. LWRTRMAX)  THEN
              WRITE(*,*) ' PH_OUT: TOO LARGE NUMBER OF TRACKS'
              WRITE(*,*) ' PH_OUT: DECREASE THE  NTCO  PARAMETER'
              WRITE(*,*) ' PH_OUT: OR CHANGE THE SIZE OF TABLE LWRTR'
              CALL STOPER('PH_OUT', 'TOO SMALL INTERNAL TABLES', 1)
           ENDIF
           IF(IPARENT .EQ. 0)  THEN   !  primary track will be written or stored
              LWRTR(ITRK) = .TRUE.
           ELSE
              LWRTR(ITRK) = .FALSE.
           ENDIF

           IF(NHHT .GT. 0)   THEN
             LWRTR(ITRK) = .TRUE.
   51      CONTINUE
             IF(IPARENT .LE. 0)  GO TO 55
             IF(LWRTR(IPARENT-NTPREV))   GO TO 55
               LWRTR(IPARENT-NTPREV) = .TRUE.
               CALL GFKINE(IPARENT-NTPREV, VERT, PVERT, IPART,
     +                      NVERT, UB, NUB)
               NNTRACK = IPARENT
               IPARENT = UB(2)
               IVOL = IUB(3)
               ISTOPSENS = IUB(11)
               IF(ISTOPSENS .LT. 0)  ISTOPSENS = 0
               CALL UHTOC(IVOL, 4, CVOL, 4)
               J = 0
               IF(IAOUF .EQ. 0  .AND.  ISAOU .EQ. 3)  THEN
                 WRITE(AOUBUF1, '('' P '', A4)')  CVOL
                 WRITE(AOUBUF2,
     +            '(I10, I10, I5, I5, 3F16.6, 3F14.3, I4)')
     +            NNTRACK, IPARENT, J, IPART,
     +            PVERT(1),PVERT(2),PVERT(3), VERT, ISTOPSENS
                 CALL SPDEL(AOUBUF2, IAOULNG)
                 CALL CONCATB(AOUBUF1,7, AOUBUF2, IAOULNG, AOUBUFS)
                 IAOULNG = IAOULNG + 7
                 WRITE(IUSRAOU,'(150A1)',IOSTAT = ios)
     +                  (AOUBUFS(I:I), I=1,IAOULNG)
               ELSE IF(ISAOU .EQ. 3)  THEN
                 WRITE(IUSRAOU,
     +            '('' P '', A4, I8, I9, I4, I3, 3F12.6, 3F10.3, I3)',
     +            IOSTAT = ios)
     +            CVOL, NNTRACK, IPARENT, J, IPART,
     +            PVERT(1),PVERT(2),PVERT(3), VERT, ISTOPSENS
               ENDIF
C
               IF(ISMEM .EQ. 3)  THEN
                  CALL ADDPMCTRACK(CVOL, NNTRACK, IPARENT, J, IPART,
     +                         PVERT(1),PVERT(2),PVERT(3),
     +                         VERT(1), VERT(2), VERT(3), ISTOPSENS )
               ENDIF
               GO TO 51

   55         CONTINUE
              CALL GFKINE(ITRK,VERT,PVERT,IPART,NVERT,UBUF,NUB)
            ENDIF   !   NHHT .GT. 0
         ENDIF   !   ISAOU .EQ. 3  ...
C
         IPARENT = UBUF(2)
         ISTOPSENS = IUBUF(11)
         IF(ISTOPSENS .LT. 0)  ISTOPSENS = 0

         IF((ISAOU .GE. 4  .OR.  ISMEM .GE. 4)  .OR.
     1      (IPARENT .EQ. 0  .AND.
     2            (ISAOU .GE. 2  .OR. ISMEM .GE. 2))  .OR.
     3      (NHHT .GT. 0  .AND.
     4            (ISAOU .GE. 1  .OR. ISMEM .GE. 1))  )   THEN

           IVOL = IUBUF(3)
C     find correct name of the origin volume for secondary particles
C     with hits (SECT=1 mode) as there is no such informatinon in KINE bank
           IF(ISECT .EQ. 1  .AND.  IPARENT .NE. 0)   THEN
              CALL GTMEDI(VERT, J)
              IVOL = NAMES(NLEVEL)
           ENDIF
           CALL UHTOC(IVOL, 4, CVOL, 4)

           NNTRACK = ITRK + NTPREV

C    Write the information to AOU file
           IF(IPARENT .EQ. 0  .AND.
     +           ( (ISAOU .GE. 1  .AND.  NHHT .GT. 0)  .OR.
     +                                       ISAOU .GE. 2))  THEN
              IF(IAOUF .EQ. 0)  THEN
                 WRITE(AOUBUF1, '('' P ORIG'')')
                 WRITE(AOUBUF2,
     +            '(I10, I10, I5, I5, 3F16.6, 3F14.3, I4)')
     +            NNTRACK, IPARENT, NHHT, IPART,
     +            PVERT(1),PVERT(2),PVERT(3), VERT, ISTOPSENS
                 CALL SPDEL(AOUBUF2, IAOULNG)
                 CALL CONCATB(AOUBUF1,7, AOUBUF2, IAOULNG, AOUBUFS)
                 IAOULNG = IAOULNG + 7
                 WRITE(IUSRAOU,'(150A1)',IOSTAT = ios)
     +                  (AOUBUFS(I:I), I=1,IAOULNG)
              ELSE
                 WRITE(IUSRAOU,
     +            '('' P ORIG'', I8,I9, I4,I3, 3F12.6, 3F10.3, I3)',
     +            IOSTAT = ios)
     +            NNTRACK, IPARENT, NHHT, IPART,
     +            PVERT(1),PVERT(2),PVERT(3), VERT, ISTOPSENS
              ENDIF
           ELSE IF(ISAOU .GE. 4  .OR.
     +             (NHHT .GT. 0  .AND.  ISAOU .GE. 1))  THEN
              IF(IAOUF .EQ. 0)  THEN
                 WRITE(AOUBUF1, '('' P '', A4)')  CVOL
                 WRITE(AOUBUF2,
     +            '(I10, I10, I5, I5, 3F16.6, 3F14.3, I4)')
     +            NNTRACK, IPARENT, NHHT, IPART,
     +            PVERT(1),PVERT(2),PVERT(3), VERT, ISTOPSENS
                 CALL SPDEL(AOUBUF2, IAOULNG)
                 CALL CONCATB(AOUBUF1,7, AOUBUF2, IAOULNG, AOUBUFS)
                 IAOULNG = IAOULNG + 7
                 WRITE(IUSRAOU,'(150A1)',IOSTAT = ios)
     +                  (AOUBUFS(I:I), I=1,IAOULNG)
              ELSE
                 WRITE(IUSRAOU,
     +           '('' P '', A4, I8, I9, I4, I3, 3F12.6, 3F10.3, I3)',
     +           IOSTAT = ios)
     +           CVOL, NNTRACK, IPARENT, NHHT, IPART,
     +           PVERT(1),PVERT(2),PVERT(3), VERT, ISTOPSENS
              ENDIF
           ENDIF

C    Store information in PHAT memory
           IF(IPARENT .EQ. 0  .AND.
     +          ( (ISMEM .GE. 1  .AND.  NHHT .GT. 0)  .OR.
     +                                       ISMEM .GE. 2) )  THEN
              CVOL = 'ORIG'
              CALL ADDPMCTRACK(CVOL, NNTRACK, IPARENT, NHHT, IPART,
     +                      PVERT(1),PVERT(2),PVERT(3),
     +                      VERT(1), VERT(2), VERT(3), ISTOPSENS  )
           ELSE IF(ISMEM .GE. 4  .OR.
     +             (NHHT .GT. 0  .AND.  ISMEM .GE. 1))  THEN
              CALL ADDPMCTRACK(CVOL, NNTRACK, IPARENT, NHHT, IPART,
     +                      PVERT(1),PVERT(2),PVERT(3),
     +                      VERT(1), VERT(2), VERT(3), ISTOPSENS )
           ENDIF

C   For the particles with hits store the hits
           IF(NHHT .GT. 0)  THEN
             DO  J=1,NHHT
                INDX(J) = J
             ENDDO
             IF(NHHT .GT. 1)  THEN  !  sort hits according to time and
               DO  J=2,NHHT         !  track length
                 J1 = J - 1
                 DO  J2 = J,NHHT
                    IF((DTHT(INDX(J1)) .GT. DTHT(INDX(J2))) .OR.
     +                 ((DTHT(INDX(J1)) .EQ. DTHT(INDX(J2))) .AND.
     +                    (TRACKLNG(INDX(J1)) .GT.
     +                           TRACKLNG(INDX(J2))) )      )  THEN
                       I = INDX(J1)
                       INDX(J1) = INDX(J2)
                       INDX(J2) = I
                    ENDIF
                 ENDDO
               ENDDO
             ENDIF  !  IF(NHHT .GT. 1)

             DO  J1=1,NHHT
              J = INDX(J1)
              IF(ISAOU .GE. 1 .AND.  IAOUF .EQ. 0)  THEN
                 WRITE(AOUBUFS,
     +           '(1X,I6,3F13.4,F12.6,E17.7,3F11.4,2I6,F13.4)' )
     +            IPLHT(J), XHT(J),YHT(J),ZHT(J), DEHT(J), DTHT(J),
     +            DXHT(J),DYHT(J),DZHT(J), IXPIX(J),IYPIX(J),
     +            TRACKLNG(J)
                 CALL SPDEL(AOUBUFS, IAOULNG)
                 WRITE(IUSRAOU,'(150A1)',IOSTAT = ios)
     +                  (AOUBUFS(I:I), I=1,IAOULNG)
              ELSE IF(ISAOU .GE. 1)  THEN
                 WRITE(IUSRAOU,
     +           '(1X,I4,3F11.4,F10.6,E15.7,3F9.4,2I4,F13.4)' )
     +            IPLHT(J), XHT(J),YHT(J),ZHT(J), DEHT(J), DTHT(J),
     +            DXHT(J),DYHT(J),DZHT(J), IXPIX(J),IYPIX(J),
     +            TRACKLNG(J)
              ENDIF

              IF(ISMEM .GE. 1)   THEN
                 CALL ADDPMCHIT(NNTRACK, DETNAME(IPLHT(J)),
     +            IPLHT(J), XHT(J),YHT(J),ZHT(J), DEHT(J), DTHT(J),
     +            DXHT(J),DYHT(J),DZHT(J), IXPIX(J),IYPIX(J),
     +            TRACKLNG(J)  )
              ENDIF

             ENDDO
           ENDIF   !  NHHT .GT. 0
         ENDIF


c Temporarily store user words from JKINE structure in
c Ntuple analysis words

         IF (NUB .GT. 0 .AND. NUB .LE. NWUBMAX) THEN
            DO I = 1, NUB
               UJKBN(I) = UBUF(I)
            ENDDO
         ENDIF

*******************************************************************
* The Ntuple image is now filled.  Output track ntuple if ISNTU
* contains 1 and either the track produced some hits or ISNTU also
* contains 4; output MC Ntuple (the first part of the track ntuple)
* if ISNTU contains 2
*******************************************************************

         if (IAND(ISNTU,1) .ne. 0 .and.
     1      (NHITSL .gt. 0 .or. IAND(ISNTU,4) .ne. 0)) then

c Set directory to the ntuple disk file
           call HCDIR ('//RHTUPF', ' ')
c Set directory to the ntuple in memory
           call HCDIR ('//PAWC/RHTUP', ' ')
           call HFN (1,TUP)
           call HCDIR ('//PAWC', ' ')
           if (ISWIT(2) .GE. 1  .AND.  IDEBUG .EQ. 1) then
* print the track ntuple
             call PRNTUP (IUSROUT, TAGS, TUP, NVTUP,
     1         ' Track Ntuple at end of PH_OUT')
           endif
         endif

         if (IAND(ISNTU,2) .ne. 0) then
           call HCDIR ('//MCTUPF', ' ')
           call HCDIR ('//PAWC/MCTUP', ' ')
           call HFN (2,TUP)
           call HCDIR ('//PAWC', ' ')
           if (ISWIT(2) .GE. 1  .and.  IDEBUG .EQ. 1) then
* print the MC ntuple
             call PRNTUP (IUSROUT, TAGS, TUP, NVMCTUP,
     1         ' MC Ntuple at end of PH_OUT')
           endif
         endif

************************************************************
*  Fill any in-line plots requested by HIST data cards:    *
************************************************************
         IF(NHSTS.NE.0) THEN

            DO I = 1,NHSTS
               ID = NINT(HCONTROL(1,I))
               IX = NINT(HCONTROL(2,I))
               IY = NINT(HCONTROL(6,I))
               IF(IY.EQ.0) THEN
                  CALL HFILL(ID,TUP(IX),1.0,1.0)
               ELSE
                  CALL HFILL(ID,TUP(IX),TUP(IY),1.0)
               ENDIF
            ENDDO

         ENDIF

  100 CONTINUE

*******************************************************
*  Write end-of-event information in Hits text file:  *
*******************************************************

      if(ISAOU .GT. 0  .AND.  epstate .ge. 2)  then
        WRITE(IUSRAOU, '('' EOE    !  END OF EVENT'')' )
      endif
      if(ISMEM .GT. 0  .AND.  epstate .ge. 2)  then
        CALL PMCEOE(IEVENT)
      endif

      RETURN
      END
