      subroutine UGCDEF (key, nargs, atypes, iargs, rargs, lsargs,
     *  ssargs, line, ires)
*Documentation_Block
************************************************************************
*
*  Description:
*  UGCDEF executes geometry and control information definitions for
*  Phobos.
*  UGCDEF receives the information from a UGC input line and passes the
*  specified values of the variables to specialized (e.g., Geant)
*  geometry- and reconstruction-defining subroutines.
*
*  Arguments:
*  atypes: list of argument types: 1 = integer, 2 = real, 3 = string
*  iargs:  array of integer arguments
*  ires:   integer value returned to caller:
*          0 if no errors were encountered;
*          1 if the keyword was not recognized;
*         -1 if the expected number of arguments was not found;
*         -2 if an argument was of the wrong type.
*         -4 if a special argument was of the wrong type, probably
*            a temporary condition until code is generalized.
*  key:    keyword identifying this data
*  line:   storage for string arguments
*  lsargs: array of locations of string arguments in line
*  nargs:  total number of arguments of all types
*  rargs:  array of real arguments
*  ssargs: array of sizes of string arguments
*
*  Called by:
*  UGCIN, after parsing a line of UGC text input with primary key GS.
*
*  Sub-
*  key    Calls:
*  ATT    GSATT  to set volume attributes such as 'inactive';
*  CLOS   GGCLOS to close geometry banks, required by Geant;
*  DET    GSDET  to define a Geant sensitive volume [detector];
*  DETH   GSDETH to define a Geant detector's hit handling parameters;
*  DET    GSDETV to define a sensitive volume, Geant sets NAMESV;
*  DVN    GSDVN  to divide a sensitive voulme into subvolumes (pixels)
*  MATE   GSMATE to define a Geant material;
*  MIXT   GSMIXT to define a Geant material mixture;
*  POS    GSPOS  to position one Geant volume inside another;
*  ROTM   GSROTM to define a rotation matrix;
*  TMED   GSTMED to define a Geant tracking medium;
*  TPAR   GSTPAR to define individual Geant tracking medium parameters;
*  VOLU   GSVOLU to define a Geant volume;
*  Other Geant subroutine calls will be added as required.
*
*  Implicit inputs, outputs, side effects:
*  Geant creates Zebra banks containing all the description data,
*  and the data structures JPART, JMATE, JTMED, JROTM, JVOLUM, and
*  JSET.
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  12-Dec-97  K.W.       call to UGCSENS to support short definitions of
*                        sensitive detector volumes
*  17-Sep-97  K.W.       DVN is not called when PIXL=0
*                        (important for drawing of detectors)
*   2-Jul-97  K.W.       added GS DVN command for division of senvitive volumes
*   2-Mar-97  K.W.       unused IRET, ISET, IDET tables changed to single
*                        variables to protect against overflow numbers
*                        in the UGCDATA input file
*  25-Jul-96  R. V.      ")," -> ")" in writes for ANSI compliance
*  19-Apr-95  A. Trzupek Allow NLMAT <0 in GS MIXT for mix by # atoms
*  13-Jun-94  R. V.      Add GSDETV to enable GLVOLU; store NSIP
*  19-May-94  R. V.      Added GSTPAR
*  28-Dec-93  R. V.      Added GSATT
*  27-Apr-93  R. V.      Move UGCPAR definitions to RHSIL.CMN
*  29-Dec-92  R. V.      Discard returned Geant volume #s if more than
*                        MaxVolDef volumes are defined, since they are
*                        unused; improve error messages.
*  02-Nov-92  R. Verdier Creation
************************************************************************
*End_Documentation_Block

      implicit none

c Argument declarations:

      character*(*) key, line
      integer nargs, atypes(*), iargs(*), lsargs(*), ssargs(*),
     *  ires
      real rargs(*)

c Common declaration for access to hits parameter definition vectors
c used by UGCDEF and UGCSET:

*KEEP,FLAG.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG.CMN
C   7-Jul-98  K. W.      Added  ITRIG flag (initial numer in TRIG command)
C  27-Nov-97  K. W.      Added IAOUF flag
C  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
C  22-Nov-92  D. McLeod
C   These are set in input cards
      INTEGER ISASC,ISHBO,ISHDO,ISNTU, ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     1        IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB, ITRIG
      REAL WPIXL,WLOSS
      COMMON/RHFLG/ISASC,ISHBO,ISHDO,ISNTU,
     1  ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     2  IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB,WPIXL,WLOSS, ITRIG
*KEEP,RHSIL.
C RHSIL.CMN: PHOBOS detector plane dimensions, positions, and
c granularity; UGC parameters
c The actual and maximum numbers of detector planes and detector hit
c parameters.  Their default values are defined in RHSILD.INC.
c 12-Dec-97 K. W.       PADSOLD and PADSNEW from UGCSET moved to a common
c  4-Nov-97 K. W.       Increased number of PADS parameters to 10
c                       Increased NSLPM to 800
c  1-Jul-97 K. W.       Added data for GEANT pixelization (GSDVN)
c  1-May-97 R. V.       Increased NSLPM to 500
c  3-MAR-97 K. W.       Removed SPL, added PADS data
C  3-Jan-97 K. W.       Increased NSLPM to 180
c  3-Jul-96 K. W.       Geometry version number added
c  5-Apr-96 D. McL.     Moved data value of UgcpMax to RHSILD
c  ...
c    Mar-92 D.McLeod    Creation
c Storage limit for number of silicon planes:
      INTEGER NSLPM
      PARAMETER (NSLPM=800)
c Storage limit for number of parameters for each hit:
      integer NSipMaxp
      parameter (NSipMaxp = 12)
c Safety limit for number of parameter values that may be entered in
c a UGC text input record:
      integer UgcpMaxp
      parameter (UgcpMaxp = 10)
c Number of silicon planes in use:
      integer NSLP
c Additional information on detectors - pad sizes:
C  DETNAME  - name of the detector volume
C  DIVXNAME - name for division in X direction
C  DIVYNAME - name for division in Y direction
C  DIVZNAME - name for division in Z direction
C  NDIVX    - number of divisions in X direction
C  NDIVY    - number of divisions in Y direction
C  NDIVZ    - number of divisions in Z direction
C  IPADFL   - pad type flag
C  PADn   - four pad parameters
C             (pad size in X, Y, Z direction for IPADFL=1 for example)
      CHARACTER*4 DETNAME(NSLPM), DIVXNAME(NSLPM),
     +             DIVYNAME(NSLPM), DIVZNAME(NSLPM)
      INTEGER NDIVX(NSLPM), NDIVY(NSLPM), NDIVZ(NSLPM)
      REAL PAD1(NSLPM),PAD2(NSLPM),PAD3(NSLPM),PAD4(NSLPM),PAD5(NSLPM)
      REAL PAD6(NSLPM),PAD7(NSLPM),PAD8(NSLPM),PAD9(NSLPM),PAD10(NSLPM)
      INTEGER NDETS, IPADFL(NSLPM)
      LOGICAL PADSOLD, PADSNEW
      COMMON /RHPADS/ NDETS, PADSOLD, PADSNEW,
     +       DETNAME, DIVXNAME, DIVYNAME, DIVZNAME,
     +       NDIVX, NDIVY, NDIVZ,
     +       IPADFL, PAD1, PAD2, PAD3, PAD4, PAD5,
     +               PAD6, PAD7, PAD8, PAD9, PAD10
c Number of silicon planes - compared with final value of NDETS
      COMMON /RHSIL/ NSLP
c UGC parameter information:
      integer UgcpMax
      real UGCpars(UgcpMaxp)
c Declare silicon plane parameters; their default values are specified
c in RHSILD.INC
      integer NSIP
c Names of parameters:
      CHARACTER*4 SINAMH(NSipMaxp)
c Mumber of bits stored for each parameter:
      INTEGER SIBITH(NSipMaxp)
c Multiplicative scale factors and origins, defined so that
c   Stored value = (Parameter value + SIORIG) * SIFACT:
      REAL SIFACT(NSipMaxp), SIORIG(NSipMaxp)
c Common for access to UGC parameters and hit parameter definitions:
      common /UGCPAR/ UgcpMax, UGCpars,
     1                NSIP, SINAMH, SIBITH, SIORIG, SIFACT
      real GEOV  !  geometry version number
      common /ugGEOV/ GEOV
*KEND.

c Local declarations:

c local debug controls:
c Idebug = 0 for no output, 1 for fatal errors only, ...
      integer idebug /1/, ios, lun /6/
c Storage for volume definitions:
      integer MaxVolDef, nvoldef /0/
      parameter (MaxVolDef = 100)
      integer ivoldef, ivolu(MaxVolDef) /MaxVolDef*0/
      logical FlagTooMany /.true./

c Declarations for keys and validation of input data:
      integer nkeys, maxargs
      parameter (nkeys = 14, maxargs = 20)
      character*8 keys(nkeys)
      character*24 typename(3) /'an integer.',
     * 'a floating point number.', 'a character string.'/
      data keys / 'ATT', 'CLOS', 'DET', 'DETH', 'DETV', 'MATE', 'MIXT',
     *  'POS', 'ROTM', 'TMED', 'TPAR', 'VOLU', 'VOLU', 'DVN' /
      integer iarg, ikey, kk
      integer nargsexp(nkeys), vtypes(maxargs, nkeys)

c A negative expected number of args means that at least one argument
c is variable size, so that the actual number can be known only by
c code specific to the called subroutine; only the first -n args
c are checked for type.

      data nargsexp / 3, 0, 12, 7, 9, 9, -3, 8, 7, 13, 3, 5, -4, 4 /

c Data type code vtype = 1: integer; 2: real; 3: character string.

      data (vtypes (kk, 1), kk=1,3)  / 3, 3, 1 /
      data (vtypes (kk, 3), kk=1,12) / 3, 3, 1, 3, 4*1, 3, 1, 3, 1 /
      data (vtypes (kk, 4), kk=1,7)  / 3, 3, 1, 3, 3, 3, 3 /
      data (vtypes (kk, 5), kk=1,9)  / 3, 3, 3*1, 3, 1, 3, 1 /
      data (vtypes (kk, 6), kk=1,9)  / 1, 3, 5*2, 2*1 /
      data (vtypes (kk, 7), kk=1,3)  / 1, 3, 2 /
      data (vtypes (kk, 8), kk=1,8)  / 3, 1, 3, 2, 2, 2, 1, 3 /
      data (vtypes (kk, 9), kk=1,7)  / 1, 6*2 /
      data (vtypes (kk,10), kk=1,13) / 1, 3, 3*1, 6*2, 2*1 /
      data (vtypes (kk,11), kk=1,3)  / 1, 3, 2 /
      data (vtypes (kk,12), kk=1,5)  / 3, 3, 1, 3, 1 /
      data (vtypes (kk,13), kk=1,4)  / 3, 3, 1, 2 /
      data (vtypes (kk,14), kk=1,4)  / 3, 3, 1, 1 /

c Local definitions unique to Phobos
      integer idiv, iaxis
      character*4 ddname, dmname

c Unused GSDET returns:
      INTEGER ISETDUM, IDETDUM

c End of declarations

c Check if it is one of special commands to define sensitive volume
      call UGCSENS(key, nargs, atypes, iargs, rargs, lsargs,
     *  ssargs, line, ires)
      if(ires .le. 0)   return   !   command found and executed
c                                !   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

      ires = 0

c Check the given key against the list of keywords.
c At completion of loop, ikey will be the index of the found
c keyword; = 0 means none was found, < 0 means one was found that
c failed a validity test.
c Ires will have the flag for the last failure found for the key.
      kk = 0
      ikey = 0
      do while (ikey .le. 0 .and. kk .lt. nkeys)
        kk = kk + 1
        if (key .eq. keys(kk)) then
          ikey = kk
c The following is *** wrong ***, it loses the old ires.
          ires = 0
        endif

c Verify that the number of tokens found is correct.  As multiple
c entries for a keyword are allowed, this and the following
c condition may not be permanent errors.

        if (ikey .gt. 0) then
          if (nargsexp(ikey) .gt. 0 .and. nargs .ne. nargsexp(ikey))
     *    then
            if (idebug .ge. 2) write (lun,'(a,i3,3a,i3,a)',iostat=ios)
     *        ' UGCDEF:', nargs, ' arguments found for keyword ',
     *        key, '; ', nargsexp(ikey), ' expected.'
            ires = -1
          endif
        endif

c Verify that the argument variable types are correct.

        if (ires .eq. 0 .and. ikey .gt. 0) then
          do iarg = 1, iabs(nargsexp(ikey))
            if (vtypes(iarg, ikey) .gt. 0 .and.
     *          atypes(iarg) .ne. vtypes(iarg, ikey)) ires = -2
          enddo
          if (idebug .ge. 2 .and. ires .eq. -2)
     *      write (lun, '(3a,i2,2a)', iostat=ios)
     *        ' UGCDEF: ', line(lsargs(1):lsargs(1)+ssargs(1)-1),
     *        ' argument #', iarg, ' is not ',
     *        typename(iabs(vtypes(iarg, ikey)))
        endif

        if (ires .eq. 0) then

c If no errors so far, call the subroutine for this key:

          if (ikey .eq. 1) then
            call GSATT (line(lsargs(1):lsargs(1)+ssargs(1)-1),
     *          line(lsargs(2):lsargs(2)+ssargs(2)-1), iargs(3))

          elseif (ikey .eq. 2) then
            call GGCLOS

          elseif (ikey .eq. 3) then
            if   (line(lsargs(4):lsargs(4)+ssargs(4)-1) .eq. 'ISET'
     *      .and. line(lsargs(5):lsargs(5)+ssargs(5)-1) .eq. 'IDET')
     *      then
              call GSDET (line(lsargs(1):lsargs(1)+ssargs(1)-1),
     *          line(lsargs(2):lsargs(2)+ssargs(2)-1), iargs(1),
     *          line(lsargs(3):lsargs(3)+ssargs(3)-1), iargs(2),
     *          iargs(3), iargs(4), iargs(5), ISETDUM,
     *          IDETDUM)
            else
              ires = -1
            endif

          elseif (ikey .eq. 4) then
c Temporarily, use only standard detector information tables referenced
c by the names that Geant uses:
            if   (line(lsargs(3):lsargs(3)+ssargs(3)-1) .eq. 'NAMESH'
     *      .and. line(lsargs(4):lsargs(4)+ssargs(4)-1) .eq. 'NBITSH'
     *      .and. line(lsargs(5):lsargs(5)+ssargs(5)-1) .eq. 'ORIG'
     *      .and. line(lsargs(6):lsargs(6)+ssargs(6)-1) .eq. 'FACT')
     *      then
              NSIP = iargs(1)
              call GSDETH (line(lsargs(1):lsargs(1)+ssargs(1)-1),
     *          line(lsargs(2):lsargs(2)+ssargs(2)-1), NSIP,
     *          SINAMH, SIBITH, SIORIG, SIFACT)
            else
              ires = -2
            endif

          elseif (ikey .eq. 5) then
            if   (line(lsargs(3):lsargs(3)+ssargs(3)-1) .eq. 'ISET'
     *      .and. line(lsargs(4):lsargs(4)+ssargs(4)-1) .eq. 'IDET')
     *      then
              call GSDETV (line(lsargs(1):lsargs(1)+ssargs(1)-1),
     *          line(lsargs(2):lsargs(2)+ssargs(2)-1), iargs(1),
     *          iargs(2), iargs(3), ISETDUM, IDETDUM)
            else
              ires = -1
            endif

          elseif (ikey .eq. 6) then
            call GSMATE (iargs(1),
     *        line(lsargs(1):lsargs(1)+ssargs(1)-1), rargs(1),
     *        rargs(2), rargs(3), rargs(4), rargs(5), rargs(2),
     *        iargs(3))

          elseif (ikey .eq. 7) then
            if (ABS(iargs(2)) .eq. 2 .and.
     *        nargs .eq. 3 * ABS(iargs(2)) + 4) then
              call GSMIXT (iargs(1),
     *          line(lsargs(1):lsargs(1)+ssargs(1)-1), rargs(1),
     *          rargs(3), rargs(5), iargs(2), rargs(6))
            else
              ires = -2
            endif

          elseif (ikey .eq. 8) then
            call GSPOS (line(lsargs(1):lsargs(1)+ssargs(1)-1),
     *        iargs(1), line(lsargs(2):lsargs(2)+ssargs(2)-1),
     *        rargs(1), rargs(2), rargs(3), iargs(2),
     *        line(lsargs(3):lsargs(3)+ssargs(3)-1))

          elseif (ikey .eq. 9) then
            call GSROTM (iargs(1), rargs(1), rargs(2), rargs(3),
     *        rargs(4), rargs(5), rargs(6))

          elseif (ikey .eq. 10) then
            call GSTMED (iargs(1),
     *        line(lsargs(1):lsargs(1)+ssargs(1)-1), iargs(2),
     *        iargs(3), iargs(4), rargs(1), rargs(2), rargs(3),
     *        rargs(4), rargs(5), rargs(6), iargs(5), iargs(6))

          elseif (ikey .eq. 11) then
            call GSTPAR (iargs(1),
     *        line(lsargs(1):lsargs(1)+ssargs(1)-1), rargs(1))

          elseif (ikey .eq. 12) then
c If volume shape parameters were specified in a vector, check that
c the name is "SPARS"; this will be generalized later to any recognized
c vector
            if (line(lsargs(3):lsargs(3)+ssargs(3)-1) .eq. 'SPARS')
     *      then
c Protect against storage overflow of returned results.  This informa-
c tion is presently unused; if required in future, the solution is to
c recompile with MaxVolDef increased.
              nvoldef = nvoldef+1
              ivoldef = min(nvoldef, MaxVolDef)
              call GSVOLU (line(lsargs(1):lsargs(1)+ssargs(1)-1),
     *          line(lsargs(2):lsargs(2)+ssargs(2)-1),
     *          iargs(1), Ugcpars(1), iargs(2), ivolu(ivoldef))
            else
c Flag attempt to reference an arbitrary parameter vector, rather
c than the special vector Ugcpars.  This will be generalized later.
              ires = -4
            endif

          elseif (ikey .eq. 13) then
c If volume shape parameters were specified inline, check that the
c # of parameters found agrees with the number specified
            if (nargs .eq. iargs(2)+4) then
              nvoldef = nvoldef+1
              ivoldef = min(nvoldef, MaxVolDef)
              call GSVOLU (line(lsargs(1):lsargs(1)+ssargs(1)-1),
     *          line(lsargs(2):lsargs(2)+ssargs(2)-1),
     *          iargs(1), rargs(1), iargs(2), ivolu(ivoldef))
            else
c Flag nmber of arguments inconsistent with number of shape parameters.
              ires = -1
            endif
          elseif (ikey .eq. 14) then
c Division of a volume
            ddname = line(lsargs(1):lsargs(1)+ssargs(1)-1)
            dmname = line(lsargs(2):lsargs(2)+ssargs(2)-1)
            idiv = iargs(1)
            iaxis = iargs(2)
            if(iaxis .eq. 1)  then
               if(ndivx(ndets) .ne. 0)  go to 90
               if(dmname .eq. detname(ndets))  go to 95
               if(dmname .eq. divyname(ndets))  go to 95
               if(dmname .eq. divzname(ndets))  go to 95
            else if(iaxis .eq. 2)  then
               if(ndivy(ndets) .ne. 0)  go to 90
               if(dmname .eq. detname(ndets))  go to 95
               if(dmname .eq. divxname(ndets))  go to 95
               if(dmname .eq. divzname(ndets))  go to 95
            else if(iaxis .eq. 3)  then
               if(ndivz(ndets) .ne. 0)  go to 90
               if(dmname .eq. detname(ndets))  go to 95
               if(dmname .eq. divxname(ndets))  go to 95
               if(dmname .eq. divyname(ndets))  go to 95
            endif
            write(*,*) ' *** ERROR IN UGCDATA FILE: ',
     +       'GS DVN command: mother volume: ', dmname,
     +       '  not defined in last GS SET PADS or GS DVN command'
            write(*,*) '          (GS DVN has to be right after ',
     +                 'GS SET PADS)'
            go to 96
   90       continue
            write(*,*) ' *** ERROR IN UGCDATA FILE: ',
     +       'GS DVN command: axis: ', iaxis,
     +           '   already divided or the axis number wrong'
            go to 96
   95       continue
            if(iaxis .eq. 1)  then
               ndivx(ndets) = idiv
               divxname(ndets) = ddname
            else if(iaxis .eq. 2)  then
               ndivy(ndets) = idiv
               divyname(ndets) = ddname
            else if(iaxis .eq. 3)  then
               ndivz(ndets) = idiv
               divzname(ndets) = ddname
            endif

            if(IPIXL .gt. 0)
     +               call GSDVN (ddname, dmname, idiv, iaxis)
   96       continue
          endif ! if ikey

        endif ! if ires

c If an error occurred, reset ikey negative and try again
        if (ires .lt. 0) ikey = -iabs(ikey)

      enddo ! while ikey <= 0 and kk <= nkeys

c Flag unrecognized key: this is not necessarily an error, if we pass
c non-UGC control data through UGC.

      if (ikey .le. 0 .and. ires .eq. 0) ires = 1

c Debug: display errors (if idebug >= 1) or success (if idebug >= 3)

      if (ires .eq. 1 .and. idebug .ge. 2) then
        write (lun, '(3a)', iostat=ios) ' UGCDEF: key ', key,
     *    ' unknown, information not processed.'
      elseif (idebug .ge. 1) then
        if (ires .eq. -1) then
          write (lun, '(3a)', iostat=ios) ' UGCDEF: ', key,
     *    ' had an unexpected number of arguments, not processed.'
        elseif (ires .eq. -2) then
          write (lun, '(3a)', iostat=ios) ' UGCDEF: ', key,
     *    ' had an argument of unexpected type, not processed.'
        elseif (ires .eq. -4) then
          write (lun, '(3a)', iostat=ios) ' UGCDEF: ', key,
     *    ' had a special argument of unexpected type, not processed.'
        endif
      elseif (ires .eq. 0 .and. idebug .ge. 2) then
        if (FlagTooMany .and. Nvoldef .gt. MaxVolDef) then
          write (lun, '(3a,i3)', iostat=ios)
     *    ' UGCDEF: more than', MaxVolDef, ' volumes defined, excess',
     *     ' volume indices discarded.'
          FlagTooMany = .false.
        endif
        write (lun, '(3a,i3)', iostat=ios)
     *  ' UGCDEF found ', key, ' = key #', ikey
      endif

      return
      end
