      subroutine UGCSENS(key, nargs, atypes, iargs, rargs, lsargs,
     *  ssargs, line, ires)
*Documentation_Block
************************************************************************
*
*  Description:
*  UGCSENS an extension of UGCDEF to store definitions of sensor types
*  provided by  GS VOLT, GS PADT, GS DIVT, GS GDTH, GS GDTV
*  and executes geometry and control definitions for GS SENS command.
*
*
*  Arguments (the same as UGCDEF):
*  atypes: list of argument types: 1 = integer, 2 = real, 3 = string
*  iargs:  array of integer arguments
*  ires:   integer value returned to caller:
*          0 if no errors were encountered;
*          1 if the keyword was not recognized;
*         -1 if the expected number of arguments was not found;
*         -2 if an argument was of the wrong type.
*         -4 if a special argument was of the wrong type, probably
*            a temporary condition until code is generalized.
*  key:    keyword identifying this data
*  line:   storage for string arguments
*  lsargs: array of locations of string arguments in line
*  nargs:  total number of arguments of all types
*  rargs:  array of real arguments
*  ssargs: array of sizes of string arguments
*
*  Called by:
*  UGCDEF, at the very beginning
*
*  Sub-
*  key
*  VOLT   - stores parameters of volume definition (for GSVOLU call)
*  PADT   - stores information on pixelization
*  DIVT   - stores parameters of volume division (for GSDVN call)
*  GDTH   - stores information for GSDETH call
*  GDTV   - stores information for GSDETV call
*  SENS   - multi-call command, calls: GSVOLU, GSDVN, GSDETH, GSDETV
*           and copies information on pixelization
*           using information provided by previous commands
*
*  Implicit inputs, outputs, side effects:
*  Geant creates Zebra banks containing all the description data,
*  and the data structures JPART, JMATE, JTMED, JROTM, JVOLUM, and
*  JSET.
*
*====================================================================
* UGCSENS enables to substitute 8 lines that define a sensityve detector:
*
* GS VOLU MOA1  'BOX '   4   3.928     1.755    0.0150   3
* GS ROTM 553    0.      0.      90.0   270.0   90.0     0.0    ! MOA1
* GS POS  MOA1  1  ROA1    0.0       0.0         0.0      0  ONLY
* GS SET PADS  MOA1       1      0.2455  0.87750     0.     0.
* GS DVN   2OA1  MOA1       32     1
* GS DVN   3OA1  2OA1        4     2
* GS DETV  BARR  MOA1   1  12000  1000  ISET   1 IDET   1
* GS DETH  BARR  MOA1  11  NAMESH  NBITSH  ORIG  FACT
*
* by  2 lines common for all sensitive detectors:
*
* GS GDTV    1  12000  1000  ISET   1 IDET   1
* GS GDTH   11  NAMESH  NBITSH  ORIG  FACT
*
* three lines common for all detectors with the same wafr and
* pixel sizes as MOA1:
*
*         stype  - (sensor type)
* GS VOLT  1    'BOX '   4   3.928     1.755    0.0150   3
* GS PADT  1       1      0.2455  0.87750     0.     0.
* GS DIVT  1      32   4   0
*
* and then only 3 line specific for  MOA1:
*
*                      stype  3 names of division volumes (x, y, z)
* GS SENS  BARR  MOA1   1     2OA1  3OA1  XXXX
* GS ROTM 553    0.      0.      90.0   270.0   90.0     0.0    ! MOA1
* GS POS  MOA1  1  ROA1    0.0       0.0         0.0      0  ONLY
*
* the command  GS SENS  uses common information defined previously
* by  GS VOLT, GS PADT, GS DIVT for the given sensor type and
* defined for all detecors by  GS GDTV,  GS GDTH
* commands  GS ROTM and  GS POS  are the same as previously
*
* If we have 400 detectors of 10 different wafer types we can
* in places of original  400 * 8  =  3200  lines
* use  only  2  +  3 * 10  +  400 * 3   =   1232  lines
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*   9-Dev-97  K.W.       Creation
*                        first part copied from UGCDEF
************************************************************************
*End_Documentation_Block

      implicit none

c Argument declarations:

      character*(*) key, line
      integer nargs, atypes(*), iargs(*), lsargs(*), ssargs(*),
     *  ires
      real rargs(*)

c Common declaration for access to hits parameter definition vectors
c used by UGCDEF and UGCSET:

*KEEP,FLAG.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG.CMN
C   7-Jul-98  K. W.      Added  ITRIG flag (initial numer in TRIG command)
C  27-Nov-97  K. W.      Added IAOUF flag
C  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
C  22-Nov-92  D. McLeod
C   These are set in input cards
      INTEGER ISASC,ISHBO,ISHDO,ISNTU, ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     1        IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB, ITRIG
      REAL WPIXL,WLOSS
      COMMON/RHFLG/ISASC,ISHBO,ISHDO,ISNTU,
     1  ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     2  IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB,WPIXL,WLOSS, ITRIG
*KEEP,RHSIL.
C RHSIL.CMN: PHOBOS detector plane dimensions, positions, and
c granularity; UGC parameters
c The actual and maximum numbers of detector planes and detector hit
c parameters.  Their default values are defined in RHSILD.INC.
c 12-Dec-97 K. W.       PADSOLD and PADSNEW from UGCSET moved to a common
c  4-Nov-97 K. W.       Increased number of PADS parameters to 10
c                       Increased NSLPM to 800
c  1-Jul-97 K. W.       Added data for GEANT pixelization (GSDVN)
c  1-May-97 R. V.       Increased NSLPM to 500
c  3-MAR-97 K. W.       Removed SPL, added PADS data
C  3-Jan-97 K. W.       Increased NSLPM to 180
c  3-Jul-96 K. W.       Geometry version number added
c  5-Apr-96 D. McL.     Moved data value of UgcpMax to RHSILD
c  ...
c    Mar-92 D.McLeod    Creation
c Storage limit for number of silicon planes:
      INTEGER NSLPM
      PARAMETER (NSLPM=800)
c Storage limit for number of parameters for each hit:
      integer NSipMaxp
      parameter (NSipMaxp = 12)
c Safety limit for number of parameter values that may be entered in
c a UGC text input record:
      integer UgcpMaxp
      parameter (UgcpMaxp = 10)
c Number of silicon planes in use:
      integer NSLP
c Additional information on detectors - pad sizes:
C  DETNAME  - name of the detector volume
C  DIVXNAME - name for division in X direction
C  DIVYNAME - name for division in Y direction
C  DIVZNAME - name for division in Z direction
C  NDIVX    - number of divisions in X direction
C  NDIVY    - number of divisions in Y direction
C  NDIVZ    - number of divisions in Z direction
C  IPADFL   - pad type flag
C  PADn   - four pad parameters
C             (pad size in X, Y, Z direction for IPADFL=1 for example)
      CHARACTER*4 DETNAME(NSLPM), DIVXNAME(NSLPM),
     +             DIVYNAME(NSLPM), DIVZNAME(NSLPM)
      INTEGER NDIVX(NSLPM), NDIVY(NSLPM), NDIVZ(NSLPM)
      REAL PAD1(NSLPM),PAD2(NSLPM),PAD3(NSLPM),PAD4(NSLPM),PAD5(NSLPM)
      REAL PAD6(NSLPM),PAD7(NSLPM),PAD8(NSLPM),PAD9(NSLPM),PAD10(NSLPM)
      INTEGER NDETS, IPADFL(NSLPM)
      LOGICAL PADSOLD, PADSNEW
      COMMON /RHPADS/ NDETS, PADSOLD, PADSNEW,
     +       DETNAME, DIVXNAME, DIVYNAME, DIVZNAME,
     +       NDIVX, NDIVY, NDIVZ,
     +       IPADFL, PAD1, PAD2, PAD3, PAD4, PAD5,
     +               PAD6, PAD7, PAD8, PAD9, PAD10
c Number of silicon planes - compared with final value of NDETS
      COMMON /RHSIL/ NSLP
c UGC parameter information:
      integer UgcpMax
      real UGCpars(UgcpMaxp)
c Declare silicon plane parameters; their default values are specified
c in RHSILD.INC
      integer NSIP
c Names of parameters:
      CHARACTER*4 SINAMH(NSipMaxp)
c Mumber of bits stored for each parameter:
      INTEGER SIBITH(NSipMaxp)
c Multiplicative scale factors and origins, defined so that
c   Stored value = (Parameter value + SIORIG) * SIFACT:
      REAL SIFACT(NSipMaxp), SIORIG(NSipMaxp)
c Common for access to UGC parameters and hit parameter definitions:
      common /UGCPAR/ UgcpMax, UGCpars,
     1                NSIP, SINAMH, SIBITH, SIORIG, SIFACT
      real GEOV  !  geometry version number
      common /ugGEOV/ GEOV
*KEEP,SNS.
C SNS.CMN: PHOBOS detector types information
C          recognized, stored and used in UGCSENS routine
C
c    9-Dec-97  K. W.  Creation
c Storage limit for number of detector types and number of parameters
c in the definition of active volume
      integer SNSMAX,SNSPARMAX
      parameter (SNSMAX=200,SNSPARMAX=6)

      CHARACTER*4 SNSSHAPE(SNSMAX)
      REAL SNSPAR(SNSPARMAX,SNSMAX), SNSPAD(10,SNSMAX)
      INTEGER SNSFILL(SNSMAX), SNSNMED(SNSMAX), SNSIPAD(SNSMAX)
      INTEGER SNSNPAR(SNSMAX)
      INTEGER SNSDIVX(SNSMAX), SNSDIVY(SNSMAX), SNSDIVZ(SNSMAX)
      INTEGER SNSGDTH, SNSVIDT, SNS NEHI, SNSNWDI

      COMMON /CSNS/ SNSGDTH, SNSVIDT, SNS NEHI, SNSNWDI,
     +     SNSSHAPE, SNSNMED, SNSPAR, SNSNPAR, SNSIPAD,
     +     SNSPAD, SNSDIVX, SNSDIVY, SNSDIVZ
*KEND.

c Local declarations:

c Declarations for keys and validation of input data:
      integer nkeys, maxargs
      parameter (nkeys = 6, maxargs = 20)
ccc unused
ccc      character*24 typename(3) /'an integer.',
ccc     * 'a floating point number.', 'a character string.'/

      character*8 keys(nkeys)
      data keys / 'VOLT', 'PADT', 'DIVT', 'GDTH', 'GDTV', 'SENS' /
      integer iarg, ikey, kk
      integer nargsexp(nkeys), vtypes(maxargs, nkeys)

c A negative expected number of args means that at least one argument
c is variable size, so that the actual number can be known only by
c code specific to the called subroutine; only the first -n args
c are checked for type.

      data nargsexp / -4, 12, 4, 5, 7, 6 /

c Data type code vtype = 1: integer; 2: real; 3: character string.

      data (vtypes (kk, 1), kk=1,4)  / 1, 3, 1, 2 /
      data (vtypes (kk, 2), kk=1,12) / 1, 1, 2,2,2,2,2, 2,2,2,2,2 /
      data (vtypes (kk, 3), kk=1,4)  / 1, 1, 1, 1 /
      data (vtypes (kk, 4), kk=1,5)  / 1, 3, 3, 3, 3 /
      data (vtypes (kk, 5), kk=1,7)  / 1, 1, 1, 3, 1, 3, 1 /
      data (vtypes (kk, 6), kk=1,6)  / 3, 3, 1, 3, 3, 3 /

c Local definitions unique to Phobos
      character*4 setname, dtname, dmname
      integer i, idt, isetdum,idetdum, idum

c End of declarations


      ires = 0

c Check the given key against the list of keywords.
c At completion of loop, ikey will be the index of the found
c keyword; = 0 means none was found, < 0 means one was found that
c failed a validity test.
c Ires will have the flag for the last failure found for the key.
      kk = 0
      ikey = 0
      do while (ikey .le. 0 .and. kk .lt. nkeys)
        kk = kk + 1
        if (key .eq. keys(kk))   ikey = kk
      enddo

      if (ikey .eq. 0) then
         ires = 1
         return   !      >>>>>>>>>>>>>>>>>>>>>>>>>>>
      endif


c Verify that the number of tokens found is correct.  As multiple
c entries for a keyword are allowed, this and the following
c condition may not be permanent errors.

      if (ikey .gt. 0) then
        if (nargsexp(ikey) .gt. 0 .and. nargs .ne. nargsexp(ikey))
     *            then
           ires = -1
           write(*,*) ' *** ERROR IN UGCSENS: COMMAND    GS ', KEY,
     +            '   HAS WRONG NUMBER OF PARAMATERS'
        endif
      endif

c Verify that the argument variable types are correct.

      if (ires .eq. 0 .and. ikey .gt. 0) then
         do iarg = 1, iabs(nargsexp(ikey))
            if (vtypes(iarg, ikey) .gt. 0 .and.
     *          atypes(iarg) .ne. vtypes(iarg, ikey)) ires = -2
         enddo
      endif


c If no errors so far, call the subroutine for this key:

      if (ires .eq. 0) then

        if (ikey .eq. 1) then    !   VOLT
           idt = iargs(1)
           if(idt .lt. 1  .or.  idt .gt. snsmax)  then
              write(*,*) ' *** ERROR IN UGCSENS: DETECTOR TYPE',
     +            ' NUMBER WRONG ',  idt, '   (GS VOLT)'
              go to 900
           endif
           if(snsfill(idt) .gt. 0)  then
              write(*,*) ' *** ERROR IN UGCSENS: DETECTOR TYPE',
     +            ' NUMBER ',  idt, '   REDEFINED (GS VOLT)'
           endif
           if(iargs(3) .lt. 0 .or. iargs(3) .gt. snsparmax) then
              write(*,*) ' *** ERROR IN UGCSENS: DETECTOR TYPE',
     +            ' NUMBER ',  idt, '   DEFINITION  (GS VOLT)'
              write(*,*) '     HAS TOO MANY PARAMETERS', iargs(3)
              write(*,*) '     OR UNSUPPORTED VOLUME SHAPE USED ',
     +                   line(lsargs(1):lsargs(1)+ssargs(1)-1)
              go to 900
           endif
           snsshape(idt) = line(lsargs(1):lsargs(1)+ssargs(1)-1)
           snsnmed(idt)  = iargs(2)
           snsnpar(idt)  = iargs(3)
           do  i=1,iargs(3)
              snspar(i,idt) = rargs(i)
           enddo
           snsfill(idt) = 1

        elseif (ikey .eq. 2) then   !   PADT
           idt = iargs(1)
           if(idt .lt. 1  .or.  idt .gt. snsmax)  then
              write(*,*) ' *** ERROR IN UGCSENS: DETECTOR TYPE',
     +            ' NUMBER WRONG ',  idt, '   (GS PADT)'
              go to 900
           endif
           snsipad(idt) = iargs(2)
           do  i=1,10
              snspad(i,idt) = rargs(i)
           enddo

        elseif (ikey .eq. 3) then   !   DIVT
           idt = iargs(1)
           if(idt .lt. 1  .or.  idt .gt. snsmax)  then
              write(*,*) ' *** ERROR IN UGCSENS: DETECTOR TYPE',
     +            ' NUMBER WRONG ',  idt, '    (GS DIVT)'
              go to 900
           endif
           if(iargs(2) .lt. 0  .or.  iargs(2) .gt. 9999  .or.
     +        iargs(3) .lt. 0  .or.  iargs(3) .gt. 9999  .or.
     +        iargs(4) .lt. 0  .or.  iargs(4) .gt. 9999 )  then
                write(*,*) ' *** ERROR IN UGCSENS: WRONG NUMER OF',
     +                     ' DIVISIONS (GS DIVT):',
     +                     iargs(2), iargs(3), iargs(4)
              go to 900
           endif
           snsdivx(idt) = iargs(2)
           snsdivy(idt) = iargs(3)
           snsdivz(idt) = iargs(4)

        elseif (ikey .eq. 4) then   !   GDTH
c Temporarily, use only standard detector information tables referenced
c by the names that Geant uses:
           if   (line(lsargs(1):lsargs(1)+ssargs(1)-1) .eq. 'NAMESH'
     *      .and. line(lsargs(2):lsargs(2)+ssargs(2)-1) .eq. 'NBITSH'
     *      .and. line(lsargs(3):lsargs(3)+ssargs(3)-1) .eq. 'ORIG'
     *      .and. line(lsargs(4):lsargs(4)+ssargs(4)-1) .eq. 'FACT')
     *     then
              snsgdth = iargs(1)
           else
              write(*,*) ' *** ERROR IN UGCSENS: WRONG PARAMETER',
     +                   ' IN  GS GDTH'
              ires = -2
           endif

        elseif (ikey .eq. 5) then   !   GDTV
           if   (line(lsargs(1):lsargs(1)+ssargs(1)-1) .eq. 'ISET'
     *       .and. line(lsargs(2):lsargs(2)+ssargs(2)-1) .eq. 'IDET')
     *     then
              snsvidt = iargs(1)
              snsnehi = iargs(2)
              snsnwdi = iargs(3)
           else
             write(*,*) ' *** ERROR IN UGCSENS: WRONG PARAMETER(S)',
     +                  ' IN  GS GDTV'
             ires = -1
           endif

        elseif (ikey .eq. 6) then   !   SENS
           idt = iargs(1)
           if(idt .lt. 1  .or.  idt .gt. snsmax)  then
              write(*,*) ' *** ERROR IN UGCSENS: DETECTOR TYPE',
     +            ' NUMBER WRONG ',  idt, '    (GS SENS)'
              go to 900
           endif
           if(snsfill(idt) .ne. 1)  then
              write(*,*) ' *** ERROR IN UGCSENS: UNDEFINED DETECTOR',
     +            ' TYPE NUMBER ',  idt, '    (GS SENS)'
              go to 900
           endif
           setname = line(lsargs(1):lsargs(1)+ssargs(1)-1)
           dtname = line(lsargs(2):lsargs(2)+ssargs(2)-1)
           if(padsold)  then
             write(*,*) ' *** ERROR IN UGCSENS - GS SENS  MIXED ',
     +          ' WITH  GS SET IUDET  OR  GS SET SLP'
             go to 900
           else
             padsnew = .TRUE.
             ndets = ndets + 1
             if(ndets .gt. nslpm)
     +          call stoper('UGCSET',
     +          'TOO MANY DETECTORS - INCREASE TABLES SIZE NSLPM',1)
             detname(ndets) = dtname
             ipadfl(ndets) = snsipad(idt)
             pad1(ndets) = snspad(1,idt)
             pad2(ndets) = snspad(2,idt)
             pad3(ndets) = snspad(3,idt)
             pad4(ndets) = snspad(4,idt)
             pad5(ndets) = snspad(5,idt)
             pad6(ndets) = snspad(6,idt)
             pad7(ndets) = snspad(7,idt)
             pad8(ndets) = snspad(8,idt)
             pad9(ndets) = snspad(9,idt)
             pad10(ndets) = snspad(10,idt)

             call GSVOLU (dtname, snsshape(idt), snsnmed(idt),
     +            snspar(1,idt), snsnpar(idt), idum)
             ndivx(ndets) = snsdivx(idt)
             ndivy(ndets) = snsdivy(idt)
             ndivz(ndets) = snsdivz(idt)
             divxname(ndets) = line(lsargs(3):lsargs(3)+ssargs(3)-1)
             divyname(ndets) = line(lsargs(4):lsargs(4)+ssargs(4)-1)
             divzname(ndets) = line(lsargs(5):lsargs(5)+ssargs(5)-1)
             if(IPIXL .gt. 0)  then
               dmname = detname(ndets)
               if(ndivx(ndets) .gt. 0)  then
                 call GSDVN (divxname(ndets), dmname, ndivx(ndets), 1)
                 dmname = divxname(ndets)
               endif
               if(ndivy(ndets) .gt. 0)  then
                 call GSDVN (divyname(ndets), dmname, ndivy(ndets), 2)
                 dmname = divyname(ndets)
               endif
               if(ndivz(ndets) .gt. 0)  then
                 call GSDVN (divzname(ndets), dmname, ndivz(ndets), 3)
                 dmname = divzname(ndets)
               endif
             endif   !   (IPIXL .gt. 0)

             call GSDETV (setname, dtname, snsvidt, snsnehi, snsnwdi,
     +                    ISETDUM, IDETDUM)

             call GSDETH (setname, dtname, snsgdth,
     +            SINAMH, SIBITH, SIORIG, SIFACT)

           endif   !   (padsold)
        endif ! if (ikey .gt. 0)

      endif ! if (ires .eq. 0)

      return

  900 continue
      return
      end
