#include <string.h>
#include <unistd.h>

#include "TSystem.h"
#include "TDatime.h"
#include "TString.h"
#include "TRotMatrix.h"

#include "TPhPMC.h"
#include "TPhInteract.h"
#include "TPhDetector.h"
#include "TPhSensor.h"
#include "TPhLayer.h"
#include "TPhRun.h"
#include "TPhEvent.h"
#include "TPhObjectContainer.h"
#include "TPhMCTrack.h"
#include "TPhMCHit.h"

#include "cfortran.h.3.9"

ClassImp(TPhPMC)

//////////////////////////////////////////////////////////////////////////
//                                                                      //
//   TPhPMC                                                             //
//                                                                      //
//   Phobos PMC interface class                                         //
//                                                                      //
//   TPhPMC provides interface to the Phobos Geant detector simulation  //
//   code called PMC. It is allowed to have only one object of this     //
//   class in memory, and only once the PMC code can be initialized by  //
//   Init(). PMC is controlled by options set in the RHDATA file.       //
//   It works exactly the same as the standalone version, but provides  //
//   output in a Phat TPhEvent object format.                           //
//   The three functions: SwimPMCEvent, SwimPhatEvent, SwimPhatTrack    //
//   take particle characteristics info on input from different sources.//
//   HIJET ascii file in the first case, TPhEvent object in second,     //
//   SwimTrack simulates passage of a particle explicitely specified    //
//   by its vertex-momentum characteristics.                            //
//   A user is expected to create a TPhEvent object and                 //
//   provide its address as a parameter to the Swim...() functions.     //
//   After returning from this function the event will be filled with   //
//   the PMC output information on primary and secondary tracks, and    //
//   hits in the phobos detector elements.                              //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

  //#define DEBUG 1
  //#define DEBUGPAR 1
#undef DEBUG

PROTOCCALLSFSUB1(GZEBRA,gzebra, INT)
#define GZEBRA(A)      CCALLSFSUB1(GZEBRA,gzebra, INT, A)

PROTOCCALLSFSUB1(HLIMIT,hlimit, INT)
#define HLIMIT(A)      CCALLSFSUB1(HLIMIT,hlimit, INT, A)

PROTOCCALLSFSUB2(VZERO,vzero, PINT,INT)
#define VZERO(A1,A2)  CCALLSFSUB2(VZERO,vzero, PINT,INT, A1,A2)

PROTOCCALLSFSUB0(UGINIT,uginit)
#define UGINIT()  CCALLSFSUB0(UGINIT,uginit)

PROTOCCALLSFSUB0(GRUN,grun)
#define GRUN()    CCALLSFSUB0(GRUN,grun)

PROTOCCALLSFSUB0(UGLAST,uglast)
#define UGLAST()  CCALLSFSUB0(UGLAST,uglast)

PROTOCCALLSFSUB3(GETGEANTPMCVER,getgeantpmcver,\
                 PFLOAT,PFLOAT,PFLOAT)
#define GETGEANTPMCVER(A1,A2,A3) CCALLSFSUB3(GETGEANTPMCVER,getgeantpmcver,\
                 PFLOAT,PFLOAT,PFLOAT,A1,A2,A3)

PROTOCCALLSFSUB7(PUTVKINE,putvkine,\
                 INT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT)
#define PUTVKINE(A1,A2,A3,A4,A5,A6,A7)  CCALLSFSUB7(PUTVKINE,putvkine,\
                 INT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,\
                 A1,A2,A3,A4,A5,A6,A7)

PROTOCCALLSFSUB7(PUTAKINE,putakine,\
                 INT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT)
#define PUTAKINE(A1,A2,A3,A4,A5,A6,A7)  CCALLSFSUB7(PUTAKINE,putakine,\
                   INT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,\
                   A1,A2,A3,A4,A5,A6,A7)

PROTOCCALLSFSUB2(GUFLD,gufld,\
                 FLOATV,FLOATV)
#define GUFLD(A1,A2)  CCALLSFSUB2(GUFLD,gufld,\
                   FLOATV,FLOATV,\
                   A1,A2)

PROTOCCALLSFSUB2(GETPMCPNAME,getpmcpname,\
                 PINT,PSTRING)
#define GETPMCPNAME(A1,A2)  CCALLSFSUB2(GETPMCPNAME,getpmcpname,\
                   PINT,PSTRING,\
                   A1,A2)

PROTOCCALLSFSUB3(GETPMCPAR,getpmcpar,\
                 PSTRING,PINT,FLOATV)
#define GETPMCPAR(A1,A2,A3)  CCALLSFSUB3(GETPMCPAR,getpmcpar,\
                   PSTRING,PINT,FLOATV,\
                   A1,A2,A3)

PROTOCCALLSFSUB3(SETPMCPAR,setpmcpar,\
                 PSTRING,PINT,FLOATV)
#define SETPMCPAR(A1,A2,A3)  CCALLSFSUB3(SETPMCPAR,setpmcpar,\
                   PSTRING,PINT,FLOATV,\
                   A1,A2,A3)

#define NWGEAN  3200000
#define NWPAW   400000

typedef struct {
   Bool_t BATCH;
   Bool_t NOLOG;
} GCFLAX_DEF;

#define GCFLAX COMMON_BLOCK(GCFLAX,gcflax)
 COMMON_BLOCK_DEF(GCFLAX_DEF,GCFLAX);

typedef struct 
{
   int IDEBUG;
   int IDEMIN;
   int IDEMAX;
   int ITEST;
   int IDRUN;
   int IDEVT;
   int IEORUN;
   int IEOTRI;
   int IEVENT;
   int ISWIT[10];
   int IFINIT[20];
   int NEVENT;
   int NRNDM[2];
} GCFLAG_DEF;

#define GCFLAG COMMON_BLOCK(GCFLAG,gcflag)
  COMMON_BLOCK_DEF(GCFLAG_DEF,GCFLAG);


typedef struct 
{
   int LUNIT[128];
} GCXLUN_DEF;

#define GCXLUN COMMON_BLOCK(GCXLUN,gcxlun)
  COMMON_BLOCK_DEF(GCXLUN_DEF,GCXLUN);

//temporary list of rotation matrices
const  Int_t    MAXROTMTX = 682;           
static Float_t  gRotMtxList[MAXROTMTX][6];

static TPhRun             *gPMCOutputRun        = NULL;
static Char_t              gPMCEGName[120]      = {'\0'};
static TPhEvent           *gPMCOutputEvent      = NULL;
static TPhMCTrack         *gPMCLatestTrack      = NULL;
static TPhObjectContainer *gPMCTrackContainer   = NULL;
static TPhObjectContainer *gPMCHitContainer     = NULL;
static Int_t               gPMCLatestTrackNo    = 0;
static Int_t               gPMCEventTrackOffset = 0;


extern "C" void AddPMCTrack(Char_t *orig, Int_t num, Int_t parnt, Int_t nhits, \
                   Int_t id, Float_t px, Float_t py, Float_t pz, \
                   Float_t vx, Float_t vy, Float_t vz, Int_t istop) 
{
#ifdef DEBUG
   printf("AddPMCTrack was called with the following arguments:\n ORIG:%s, NUMBER:%d, PARENT:%d, NHITS:%d, ID:%d,\n PX:%f, PY:%f, PZ:%f, Vx:%f, Vy:%f, Vz:%f\n", orig, num, parnt,  nhits, id, px, py, pz, vx, vy, vz, istop);
#endif
   // Check the validity of some of the parameters
   if( id<1 )  id = 1;
   if( !orig ) orig = "";

    TPhPMC *pmcp = NULL;
    pmcp = TPhPMC::GetInstance();
    if(!pmcp) {
#ifdef DEBUG
       printf("No TPhPMC object when AddPMCTrack was called.\n");
#endif
       return;
    }
          
   Int_t gcharge[50]={ 
         0, 1,-1, 0, 1,-1, 0, 1,-1, 0,
         1,-1, 0, 1,-1, 0, 0, 0, 1, 0,
        -1, 0,-1,-1, 0, 0,-1, 0, 1, 0,
         1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 1, 1, 2, 0, 2, 0};

   Float_t gmass[50]={ 
         0, 0.00051099906, 0.00051099906, 0, 0.105658389, 
         0.105658389, 0.1349764, 0.1395700, 0.1395700, 0.497672,
         0.493677, 0.493677, 0.93956563, 0.83827231,  0.83827231, 
         0.497672, 0.54745, 1.115684, 1.18937, 1.19255,
         1.197436, 1.3149, 1.32132, 1.67245, 0.93956563, 
         1.115684, 1.18937, 1.19255, 1.197436, 1.3149,
         1.32132, 1.67245, 0, 0, 0, 
         0, 0, 0, 0, 0,
         0, 0, 0, 0, 1.875613, 
         2.80925, 3.727417, 0, 2.80923, 0};

    gPMCLatestTrack = new TPhMCTrack();
    // TrackOffset counts tracks within Phat event, 
    // while ...trackNo counts tracks within a PMC "event"
    gPMCLatestTrackNo = num;

    gPMCLatestTrack->SetPx((Double_t)px);
    gPMCLatestTrack->SetPy((Double_t)py);
    gPMCLatestTrack->SetPz((Double_t)pz);
    gPMCLatestTrack->SetXProd((Double_t)vx);
    gPMCLatestTrack->SetYProd((Double_t)vy);
    gPMCLatestTrack->SetZProd((Double_t)vz);
    gPMCLatestTrack->SetGPid(id);
    gPMCLatestTrack->SetIStop(istop);
    gPMCLatestTrack->SetOriginName(orig);
    gPMCLatestTrack->SetTrackNo((Int_t)(num+gPMCEventTrackOffset));
    if(parnt) 
      gPMCLatestTrack->SetParentNo((Int_t)(parnt+gPMCEventTrackOffset));
    else
      gPMCLatestTrack->SetParentNo(0);
    gPMCLatestTrack->SetCharge(gcharge[id-1]);
    gPMCLatestTrack->SetMass(gmass[id-1]);

    gPMCTrackContainer->Add(gPMCLatestTrack);
}

extern "C" void AddPMCHit(Int_t trackno, Char_t *detname, Int_t det, \
                 Float_t x, Float_t y, Float_t z, Float_t dE, Float_t dt, \
                 Float_t dx, Float_t dy, Float_t dz, Int_t xpix, Int_t ypix, Float_t tlen) 
{
#ifdef DEBUG
   printf("AddPMCHit was called with the following arguments:\n TRACK NO:%d, DETECTOR:%s, NUMBER:%d, X:%f, Y:%f, Z%f,\n DE:%f, DT:%f, DX:%f, DY:%f, DZ:%f, XPIX:%d, YPIX:%d Track length: %f\n", trackno, detname, det, x, y, z, dE, dt, dx, dy, dz, xpix, ypix, tlen);
#endif

    TPhPMC *pmcp = NULL;
    pmcp = TPhPMC::GetInstance();
    if(!pmcp) {
#ifdef DEBUG
       printf("No TPhPMC object when AddPMCHit was called.\n");
#endif
       return;
    }

    TPhDetector *detector = NULL;
    detector = TPhDetector::GetInstance();
    if( !detector ) {
       pmcp->Error("AddPMCHit",
             "TPhDetector object not available, Layer and sensor nos not set.\n");
    }    

    TPhMCHit       *hit = new TPhMCHit();

    hit->SetX((Float_t)x);
    hit->SetY((Float_t)y);
    hit->SetZ((Float_t)z);
    hit->SetLength((Double_t) tlen);

    TPhSensor *sensor = NULL;
    if(detector) {
        sensor = detector->GetSensor(detname);
        if(!sensor) {
           pmcp->Warning("AddPMCHit", "Unknown sensor with name: %s.\n", detname);
	   return;
	}
        hit->SetLayer( sensor->GetLayer()); 
        hit->SetSensorLabel( sensor->GetLabel());
    }

    if(trackno != gPMCLatestTrackNo) 
       pmcp->Warning("AddPMCHit","Wrong hits assigned to PMC track no: %d\n",
                      gPMCLatestTrackNo);
    hit->SetTrackNo(trackno+gPMCEventTrackOffset);
    hit->SetDe(dE);
    hit->SetDt(dt);
    hit->SetDx(dx);
    hit->SetDy(dy);
    hit->SetDz(dz);
    hit->CalcPsi();

    Float_t vec[3], bfield[3];
    vec[0] = x; vec[1] = y; vec[2] = z; 
    pmcp->PMCGufld(&vec[0], &bfield[0]);
    hit->SetField(bfield[1]);

    Int_t ncolumn;
    ncolumn = sensor->GetColumns();
    if( xpix > ncolumn-1 ) {
      if( pmcp->GetVerbose()>0 )
	printf(" Change in sensor %s PadX = %d (>ncol(%d)-1): set to %d\n",
	       detname,xpix,ncolumn,ncolumn-1);
      xpix = ncolumn - 1;
    }
    if( xpix<0 ) {
      if( pmcp->GetVerbose()>0 )
	printf(" Change in sensor %s PadX = %d (<0): set to 0\n",
	       detname,xpix);
      xpix = 0;
    }
    hit->SetPadX((Int_t)xpix);

    // Changing PMC convention of pixel numbering used in PMC geometry v.4.2
    // to make it in agreement with the convention used in the electronic maps

    Int_t nrow;
    nrow = sensor->GetRows();
    if(detector) {
      if( TMath::Nint(10*detector->GetVersion()) == 42 ) {  //version 4.2 
        if( strncmp(detname, "MOA", 3)==0 ||
            strncmp(detname, "MOB", 3)==0 ||
            strncmp(detname, "MOC", 3)==0 ||
            strncmp(detname, "MOD", 3)==0 ||
            strncmp(detname, "VB",  2)==0   ) 
  	  {
            nrow = sensor->GetRows();
#ifdef DEBUG
            printf(" Change in sensor %s with %d rows : Old Ypix = %d, New Ypix = %d\n",
                     detname,nrow,ypix,nrow-1-ypix);
#endif
            ypix = nrow - 1 - ypix;
 	  }
      }
      if( ypix > nrow-1 ) {
        if( pmcp->GetVerbose()>0 )
           printf(" Change in sensor %s PadY = %d (>nrow(%d)-1): set to %d\n",
                    detname,ypix,nrow,nrow-1);
        ypix = nrow - 1;
      }
      if( ypix<0 ) {
        if( pmcp->GetVerbose()>0 )
           printf(" Change in sensor %s PadY = %d (<0): set to 0\n",
                    detname,ypix);
        ypix = 0;
      }
      hit->SetPadY(ypix);
    }

    gPMCLatestTrack->AddHit(hit);
    gPMCHitContainer->Add(hit);
}

void TPhPMC::AddPhatPMCRInfo(Char_t *runinfo)
{
#ifdef DEBUGPAR
   printf("AddPhatPMCRInfo was called with:\n Run parameter:%s\n", runinfo);
#endif
   Char_t keyname[20], keyvalue[120];
   TPhParKey *parkey = NULL;

   sscanf(runinfo,"%19s %119[ -~]", keyname, keyvalue);
   parkey = new TPhParKey("PHATPMC", keyname, keyvalue, 0);

   if(gPMCOutputRun) gPMCOutputRun->AddParameter(parkey);
   return;
}


extern "C" void AddPMCRInfo(Char_t *runinfo)
{
#ifdef DEBUGPAR
   printf("AddPMCRInfo was called with:\n Run parameter:%s\n", runinfo);
#endif
   Char_t temp[120];
   Char_t ind[2], keyname[20], keyvalue[120];
   TPhParManager *parman = NULL;
   TPhParKey *parkey = NULL;
   Char_t sourceMC[]="MC", sourcePMC[]="PMC";

   if(!gPMCOutputRun) return;

   if(gPMCLatestTrack) {
     printf("AddPMCRInfo: Possible attempt to start new run\n");
     printf("AddPMCRInfo: Adding run info not allowed after simulations have been started\n");
     gSystem->Exit(0);
   }
	    
   // Real generator name is given as a parameter
   // Only after this is known the proper source name is set
   // for other generator parameters
   if(*gPMCEGName=='\0') {
     parman = gPMCOutputRun->GetParManager();
     if(parman) {
       if(parman->GetParTextValue(temp, "GENERATOR", sourceMC)) 
	 strncpy(gPMCEGName,temp,119);
     }
   }

   sscanf(runinfo,"%1s %19s %119[ -~]", ind, keyname, keyvalue);
   // S,M,G are assumed to come from PMC parameters
   if(!strcmp(ind,"S") || !strcmp(ind,"M") || !strcmp(ind,"G")) {
     parkey = new TPhParKey(sourcePMC,keyname,keyvalue);
   }
   // R indicates event generator run parameters
   else if(!strcmp(ind,"R")) {
     if(*gPMCEGName!='\0') 
          parkey = new TPhParKey(gPMCEGName,keyname,keyvalue);
     else parkey = new TPhParKey(sourceMC,keyname,keyvalue);
   }
   // others are ignored
   else return;

   gPMCOutputRun->AddParameter(parkey);
   return;
}

extern "C" void AddPMCEInfo(Char_t *evinfo)
{
#ifdef DEBUGPAR
   printf("AddPMCEInfo was called with:\n Event parameter:%s\n", evinfo);
#endif
   Char_t temp[120];
   Char_t ind[2], keyname[20], keyvalue[120];
   TPhParManager *parman = NULL;
   TPhParKey *parkey = NULL;
   Char_t sourceMC[]="EventGenerator";

   if(!gPMCOutputEvent) return;

   // Real generator name is given as a parameter
   // Only after this is known the proper source name is set
   // for other generator parameters
   if(*gPMCEGName!='\0') {
     parman = gPMCOutputRun->GetParManager();
     if(parman) {
       if(parman->GetParTextValue(temp, "GENERATOR", sourceMC)) 
	 strncpy(gPMCEGName,temp,119);
     }
   }

   sscanf(evinfo,"%1s %19s %119[!-~]", ind, keyname, keyvalue);
   // only H is expected to be seen here
   if(!strcmp(ind,"H")) 
     if(*gPMCEGName!='\0') 
          parkey = new TPhParKey(gPMCEGName,keyname,keyvalue);
     else parkey = new TPhParKey(sourceMC,keyname,keyvalue);
   // others are ignored
   else return;

   gPMCOutputEvent->AddParameter(parkey);
   return;
}

extern "C" void PMCEOE(Int_t evno)
{
#ifdef DEBUGPAR
   printf("PMCEOE was called with:\n Event number:%d\n", evno);
#endif
}

extern "C" void PMCEOH()
{
#ifdef DEBUGPAR
   printf("PMCEOH has been called\n");
#endif
}

extern "C" void MakeDetector(Float_t geover)
{
#ifdef DEBUG
   printf("MakeDetector was called with argument:\n Geometry version no:%f\n", geover);
#endif

    TPhPMC *pmcp;
    pmcp = TPhPMC::GetOrCreateInstance();

    TPhDetector *detector; 
    detector = TPhDetector::CreateInstance();
    detector->SetVersion(geover);
    Char_t  VersionString[20]; 
    sprintf(VersionString, "PMC_V%1.2f", geover);
    detector->SetName(VersionString);
    if(pmcp) {
       pmcp->Warning("MakeDetector","Geometry version %6.3f found in UGCDATA file.", geover);
       pmcp->SetDetectorCreated(kTRUE);
    }
#ifdef DEBUG
      else printf("No TPhPMC object when MakeDetector was called.\n");
#endif
}

extern "C" void AddPMCRot(Int_t rotno, Float_t *rotm, Float_t *rota)
{
  rotno--;
#ifdef DEBUG
   printf("AddPMCRot was called with the following arguments:\n ROTATION MTX NO:%d\n ROTM: %f %f %f %f %f %f %f %f %f\n ROTA:%f %f %f %f %f %f\n",
           rotno, 
           *(rotm),*(rotm+1),*(rotm+2),*(rotm+3),*(rotm+4),*(rotm+5),*(rotm+6),*(rotm+7),*(rotm+8),
           *(rota),*(rota+1),*(rota+2),*(rota+3),*(rota+4),*(rota+5));
#endif

    TPhPMC *pmcp = NULL;;
    pmcp = TPhPMC::GetInstance();
    if(!pmcp) {
       printf("No TPhPMC object present while AddPMCRot was called for matrix no: %d.\n",rotno);
       return;
    }
    if( rotno<0 || rotno > MAXROTMTX-1 ) {
       pmcp->Error("AddPMCRot",
          "Rotation matrix number %d larger then the expected maximum of %d.\n", 
           rotno, MAXROTMTX);
       return;
    }
    // store only 6-element rotation matrices
    for(Int_t i=0; i<6; i++) gRotMtxList[rotno][i] = *(rota+i);
    return;
}

extern "C" void AddPMCDevo(Int_t detno, Char_t *detname, Int_t shapeno, Float_t *center,
			   Int_t nvolpar, Float_t *volpars, Int_t nrotmtx, Int_t *rotnums,
			   Int_t padtype, Float_t *padpars)
{
  // Phat sensor labels are smaller from the GEANT numbers by 1. Sensors start at 0 position in the 
  // detector sensor list, and the first one (the no 1 MOA0 GEANT volume) has label 0.
  // Using THashList as object that handles sensor list it is not easy to start the list from 1,
  // a dummy object would be needed to start the list.
   detno--;
#ifdef DEBUG
   printf("AddPMCDevo was called with the following arguments:\n DET NO:%d, DETECTOR:%s, SHAPE:%d, X:%f, Y:%f, Z:%f,\n VOLUME PARS(5):%f %f %f %f %f No of ROT MATRIX: %d ROT MTX NO:%d\n PAD PARS(1+5): %d %f %f %f %f %f\n\n", detno, detname, shapeno, center[0], center[1], center[2], volpars[0], volpars[1], volpars[2], volpars[3], volpars[4], nrotmtx, rotnums[0]-1, padtype, padpars[0], padpars[1], padpars[2], padpars[3], padpars[4]);
#endif

    TPhDetector *detector = NULL;
    TPhPMC *pmcp = NULL;;
    pmcp = TPhPMC::GetInstance();
    if(pmcp) {
      if(pmcp->GetDetectorCreated()) detector = TPhDetector::GetInstance();
    } else {
#ifdef DEBUG
       printf("No TPhPMC object when AddPMCDevo was called.\n");
#endif
       return;
    }
    if( !detector ) {
       pmcp->Error("AddPMCDevo",
             "TPhDetector object not available, sensor no %d not added.\n", detno);
       return;
    }
    if( nrotmtx>1 ) {
#ifdef DEBUG
       printf("AddPMCDevo: Can not handle more then 1 rotation matrix for sensor no %d.\n", detno);
#endif
       return;
    }
    TRotMatrix *rotmat = NULL;
    Int_t pos = rotnums[0] - 1;

    // Check if matrix is initialized (hopefully not by 9999 number)
    if( fabs(gRotMtxList[pos][0]-9999.0)<1.0e-7 ) {
       pmcp->Error("AddPMCDevo",
          "Rotation matrix no %d not available, sensor no %d not added.\n", 
           pos, detno);
       return;
    }
    Char_t name[40];
    sprintf(name, "Rot_%d", pos);
    rotmat = new TRotMatrix(name, name, 
                 gRotMtxList[pos][0], gRotMtxList[pos][1], gRotMtxList[pos][2], 
                 gRotMtxList[pos][3], gRotMtxList[pos][4], gRotMtxList[pos][5]);

    detector->AddSensor(detno, detname, shapeno, center, nvolpar, volpars, rotmat, 
                        padtype, padpars);
}

FCALLSCSUB12(AddPMCTrack,ADDPMCTRACK,addpmctrack,\
            PSTRING,INT,INT,INT,INT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,INT)

FCALLSCSUB14(AddPMCHit,ADDPMCHIT,addpmchit,\
            INT,PSTRING,INT,\
            FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,INT,INT,FLOAT)

FCALLSCSUB1(AddPMCRInfo,ADDPMCRINFO,addpmcrinfo,\
            PSTRING)

FCALLSCSUB1(AddPMCEInfo,ADDPMCEINFO,addpmceinfo,\
            PSTRING)

FCALLSCSUB0(PMCEOH,PMCEOH,pmceoh)

FCALLSCSUB1(PMCEOE,PMCEOE,pmceoe,\
            INT)

FCALLSCSUB1(MakeDetector,MAKEDETECTOR,makedetector,\
            FLOAT)

FCALLSCSUB3(AddPMCRot,ADDPMCROT,addpmcrot,\
            INT,FLOATV,FLOATV)

FCALLSCSUB10(AddPMCDevo,ADDPMCDEVO,addpmcdevo,\
            INT,PSTRING,INT,FLOATV,INT,FLOATV,INT,INTV,INT,FLOATV)

TPhPMC *gPMC;
Bool_t TPhPMC::fgPMCExists = kFALSE;
Bool_t TPhPMC::fgPMCInit = kFALSE;
Bool_t TPhPMC::fgDetectorCreated = kFALSE; 

TPhPMC *TPhPMC::GetInstance()
{
// Returns the singleton instance of the PMC object. 

    return gPMC;
}

TPhPMC *TPhPMC::CreateInstance()
{
// Returns the singleton instance of the Detector object. If one is already
// present it will be deleted and a new one is (re)created. 
// It doesn't mean that you can reinitialize PMC (code). 
// Init() function can be used only once in a life of a root application.

    if(fgPMCExists)
    {
     fgPMCExists = kFALSE;
     delete gPMC;
    }
    gPMC = new TPhPMC();

    return gPMC;
}

TPhPMC *TPhPMC::GetOrCreateInstance()
{
// Returns the singleton instance of the Detector object. If it is not
// yet initialized, it will be created. 

    if(!fgPMCExists)
    {
        CreateInstance();
    }
    return gPMC;
}

TPhPMC::TPhPMC() 
{
    if(!fgPMCExists)
    {
      fgPMCExists = kTRUE;
      gPMC = this;
      fVerbose = 0;
      fGeantVer = 0;
      fPmcVer = 0;
      fFLDASC = "";
      fGCDEFS = "";           
      fGENEVS = "";           
      fRHAOU =  "";            
      fRHDATA = "";           
      fRHASC =  "";            
      fRHHST =  "";            
      fRHNTU =  "";            
      fMCNTU =  "";            
      fRHZEB =  "";            
      fPMCNotes = NULL;
    } else {
        Error("TPhPMC()", "TPhPMC already exists. Use TPhPMC::GetInstance to access it.");
    }
}

TPhPMC::~TPhPMC()
{
   gPMC = NULL;
   fgPMCExists = kFALSE;

   // If run object exists it must have taken over the notes object
   if(!gPMCOutputRun) SafeDelete(fPMCNotes);
   Close();
}

Bool_t TPhPMC::Init()
{
//    Local variables:
      int NH;

//    Check for the proper setting of links to PMC files
      if( !InitLinks("ALL") ) return kFALSE;

      // Before proceeding, look into dangerous situations
      if( fgPMCInit ) {
        Error("InitLinks","Cannot re-initialize PMC - do not repeat calls to TPhPMC::Init()!");
        return kFALSE;
      }

//    Initialize Memory

      fgPMCInit = kTRUE;
      for(Int_t i=0; i<MAXROTMTX; i++) 
        for(Int_t j=0;       j<6; j++) gRotMtxList[i][j] = 9999.0;

      GZEBRA(NWGEAN);
      NH=-NWPAW;
      HLIMIT(NH);

      VZERO(GCXLUN.LUNIT[0],128);
      GCXLUN.LUNIT[5]=6;
      GCXLUN.LUNIT[6]=6;
      GCXLUN.LUNIT[7]=6;
      GCXLUN.LUNIT[10]=8;
      for(int I=11; I<=18; I++) {
         GCXLUN.LUNIT[I]=6;
      }
      GCXLUN.LUNIT[19]=8;
      GCXLUN.LUNIT[81]=7;
      GCXLUN.LUNIT[82]=7;
      GCXLUN.LUNIT[83]=7;
      GCXLUN.LUNIT[84]=7;
      GCXLUN.LUNIT[89]=7;
      GCXLUN.LUNIT[91]=8;
      GCXLUN.LUNIT[97]=7;

//    Define User Geometry

      UGINIT();
      // Get versions of the GEANT nad PMC codes
      GetVersions();
      GCFLAX.NOLOG = kTRUE;
      GCFLAX.BATCH = kFALSE;
      GCFLAG.IEVENT = 0;
      GCFLAG.NEVENT = 0;

      // List PMC parameters
      if(fVerbose>0) PrintPMCPars();

      // Add parameter definitions specific to PHATPMC to the run object
      AddPhatPMCRInfo(" DEFR  RHDATA Symbolic link to (input) file with control params"); 
      AddPhatPMCRInfo(" DEFR  FLDASC Symbolic link to (input) file with magnetic field map"); 
      AddPhatPMCRInfo(" DEFR  GCDEFS Symbolic link to (input) file with geometry description"); 
      AddPhatPMCRInfo(" DEFR  GENEVS Symbolic link to (input) file with events to be simulated"); 
      AddPhatPMCRInfo(" DEFR  RHAOU  Symbolic link to (output) ASCII file with simulated events"); 
      AddPhatPMCRInfo(" DEFR  RHASC  Symbolic link to (output) file with user output"); 
      AddPhatPMCRInfo(" DEFR  RHHST  Symbolic link to (output) file with histograms"); 
      AddPhatPMCRInfo(" DEFR  RHNTU  Symbolic link to (output) file with ntuples"); 
      AddPhatPMCRInfo(" DEFR  MCNTU  Symbolic link to (output) file with ntuples"); 
      AddPhatPMCRInfo(" DEFR  RHZEB  Symbolic link to (output) file in ZEBRA format"); 
      AddPhatPMCRInfo(" DEFR  FILELINK  Symbolic link changed during phatpmc execution");
      AddPhatPMCRInfo(" DEFR  PARCHANGE  PMC parameter changed during phatpmc execution");

      // Let TPhRun know about the file links
      Char_t str[200];
      sprintf(str," RHDATA %s", fRHDATA.Data());
      AddPhatPMCRInfo(str);
      sprintf(str," FLDASC %s", fFLDASC.Data());
      AddPhatPMCRInfo(str);
      sprintf(str," GCDEFS %s", fGCDEFS.Data());
      AddPhatPMCRInfo(str);
      sprintf(str," GENEVS %s", fGENEVS.Data());
      AddPhatPMCRInfo(str);
      sprintf(str," RHAOU %s", fRHAOU.Data());
      AddPhatPMCRInfo(str);
      sprintf(str," RHASC %s", fRHASC.Data());
      AddPhatPMCRInfo(str);

      // Check where the output is to be sent to, and protest if no otuput is specified.
      Float_t values[10];
      Int_t number;
      Bool_t aou = kFALSE; Bool_t mem = kFALSE;
      GetPMCParValue("SAOU", &number, values);
      if( fabs(values[0])<1.0e-6 ) {
         if(fVerbose) printf("PMC information: SAOU=0, AOU file will not be written.\n");
      } else 
        aou = kTRUE;
      GetPMCParValue("SMEM", &number, values);
      if( fabs(values[0])<1.0e-6 ) {
         if(fVerbose) printf("PMC information: SMEM=0\n- neither TPhDetector object nor Phat events will not be filled.\n");
      }  else 
         mem = kTRUE;

      if( (!aou) && (!mem) ) Warning("Init", 
                "SAOU and SMEM set to 0 in RHDATA control file\n - No output will be produced!");
            
      return kTRUE;
}

Bool_t TPhPMC::InitLinks(Char_t *which)
{
  // Link new files to standard PMC file names.
  // Paths to files can be initialized first with SetPathToFile() function.

    Bool_t linit = kTRUE, llinit = kTRUE;
    const Char_t *ans;

    const Char_t *pmchome = gSystem->Getenv("PMCHOME");
    const Char_t *phathome = gSystem->Getenv("PHATHOME");
    Char_t stdlinksdir[100];
    Char_t filename[100], sname[128];

    if(!which) return kFALSE;

    if ( pmchome ) sprintf(stdlinksdir,"%s",pmchome);
    else if ( phathome )  sprintf(stdlinksdir,"%s/phatpmc",phathome);
    else sprintf(stdlinksdir,".");

    const Char_t *wdir;
    wdir = gSystem->WorkingDirectory();

    // Setting path to PMC ascii output (AOU) file
    if(!strcmp(which,"ALL") || !strcmp(which,"RHAOU")) {
      llinit = kTRUE;
      // case: RHAOU set to path provided by the user 
      if( !fRHAOU.IsNull() ) {
         ans = fRHAOU.Data();
	 // unconditionaly overwrites old RHAOU file if file name
	 // set via SetPathToFile()
         if( SymRead("RHAOU",sname,128) ) {
           gSystem->Unlink("RHAOU");
	   gSystem->Unlink(sname);
	 }
         if( gSystem->Symlink(ans,"RHAOU") ) {
	   //            Error("InitLinks", "Failed to set symbolic link RHAOU to %s.", ans);
            llinit = kFALSE; 
         }
      } 
      // if failed previously or file name unknow so far
      if( !llinit || fRHAOU.IsNull() ) {
        sprintf(filename,"%s/RHAOU",wdir);
        // case: RHAOU file already exists
        if( !gSystem->AccessPathName(filename,kFileExists) ) {
          TPhInteract *iaou = new TPhInteract();
          ans = iaou->Ask("Do you want to overwrite the file pointed by RHAOU? [yes/no]: ");
          if( !strstr(ans,"y") && !strstr(ans,"Y") ) {
             ans = iaou->Ask("Give the new AOU file name: ",".AOU");
             gSystem->Unlink("RHAOU");
             if( gSystem->Symlink(ans,"RHAOU") || !gSystem->AccessPathName(ans,kFileExists) ) {
                Error("InitLinks", "Failed to change symbolic link RHAOU.");
                if(fVerbose > 0) 
                   Warning("InitLinks", "(re)Move a file pointed to by RHAOU before proceeding.");
                llinit = kFALSE; linit = linit && llinit;
             } else fRHAOU = ans;
          } else {
             if( !SymRead("RHAOU",sname,128) ) {
                 gSystem->Unlink("RHAOU");
                 fRHAOU = "RHAOU";
             } else {
               gSystem->Unlink(sname);
               fRHAOU = sname;
             }
          }
          delete iaou;
        } else {
            if( !SymRead("RHAOU",sname,128) ) {
                fRHAOU = "RHAOU";
            } else {
                fRHAOU = sname;
            }
	}
      }
    }

    // Setting a link to the magnetic field map
    // Allowed even after Init() has been called
    if(!strcmp(which,"ALL") || !strcmp(which,"FLDASC")) {
      llinit = kTRUE;
      // case: FLDASC set to path provided by the user 
      if( !fFLDASC.IsNull() ) {
        gSystem->Unlink("FLDASC");
        gSystem->Symlink(fFLDASC.Data(),"FLDASC");
      }
      // check to see if this file exists
      sprintf(filename,"%s/FLDASC",wdir);
      if( gSystem->AccessPathName(filename,kReadPermission) ) {
        sprintf(filename,"%s/dat/%s",stdlinksdir,"std_phomag");
        if( gSystem->AccessPathName(filename,kReadPermission) 
            || gSystem->Symlink(filename,"FLDASC")) {
          Error("InitLinks", "File FLDASC does not exist!");
          if(fVerbose > 0) {
             Warning("InitLinks", 
             "Create a link to the file with magnetic field if parameter MAGT=2.\n", 
             "in other case ignore the FALSE return value of the Init() function.");
          } 
          llinit = kFALSE; linit = linit && llinit;
	} 
        if( llinit ) {
	   fFLDASC = filename;
           if(fVerbose > 0)
             Warning("InitLinks",
               "FLDASC file with magnetic field linked to standard std_phomag!");	
	}
      } else {
         if( !SymRead("FLDASC",sname,128) ) {
             fFLDASC = "FLDASC";
         } else {
             fFLDASC = sname;
         }
      }
    }
      
    // Setting a link to the UGCDATA geometry file
    if(!strcmp(which,"ALL") || !strcmp(which,"GCDEFS")) {
      llinit = kTRUE;
      // case: GCDEFS set to path provided by the user 
      if( !fGCDEFS.IsNull() ) {
        gSystem->Unlink("GCDEFS");
        gSystem->Symlink(fGCDEFS.Data(),"GCDEFS");
      }
      // check to see if this file exists
      sprintf(filename,"%s/GCDEFS",wdir);
      if( gSystem->AccessPathName(filename,kReadPermission) ) {
	sprintf(filename,"%s/dat/%s",stdlinksdir,"std_ugcdata");
        if( gSystem->AccessPathName(filename,kReadPermission) 
            || gSystem->Symlink(filename,"GCDEFS") )  {
          Error("InitLinks", "Link GCDEFS to a file with geometry info before proceeding.");
          llinit = kFALSE; linit = linit && llinit;
        } 
        if( llinit ) {
          fGCDEFS = filename;
          if(fVerbose > 0)
            Warning("InitLinks",
              "GCDEFS file with geometry info linked to standard std_ugcdata!");
	}
      } else {
         if( !SymRead("GCDEFS",sname,128) ) {
            fGCDEFS = "GCDEFS";
         } else {
            fGCDEFS = sname;
         }
      }
    }
      
    // Setting a link to the PMC control file
    if(!strcmp(which,"ALL") || !strcmp(which,"RHDATA")) {
      llinit = kTRUE;
      // case: RHDATA set to path provided by the user 
      if( !fRHDATA.IsNull() ) {
        gSystem->Unlink("RHDATA");
        gSystem->Symlink(fRHDATA.Data(),"RHDATA");
      }
      // check to see if this file exists
      sprintf(filename,"%s/RHDATA",wdir);
      if( gSystem->AccessPathName(filename,kReadPermission) ) {
        sprintf(filename,"%s/dat/%s",stdlinksdir,"std_control");
        if( gSystem->AccessPathName(filename,kReadPermission) 
            || gSystem->Symlink(filename,"RHDATA")) {
          Error("InitLinks", "Link RHDATA to a file with control data info before proceeding.");
          llinit = kFALSE; linit = linit && llinit;
	} 
        if( llinit ) {
          fRHDATA = filename;
          if(fVerbose > 0)
            Warning("InitLinks",
              "RHDATA file with control data info linked to standard std_control!");
	}
      } else {
         if( !SymRead("RHDATA",sname,128) ) {
            fRHDATA = "RHDATA";
         } else {
            fRHDATA = sname;
         }
      }
    }
      
    // Setting a link to the file with HIJET input data
    // Allowed even after Init() has been called.
    if(!strcmp(which,"ALL") || !strcmp(which,"GENEVS")) {
      llinit = kTRUE;
      // case: GENEVS set to path provided by the user 
      if( !fGENEVS.IsNull() ) {
        gSystem->Unlink("GENEVS");
        gSystem->Symlink(fGENEVS.Data(),"GENEVS");
      }
      // check to see if this file exists
      sprintf(filename,"%s/GENEVS",wdir);
      if( gSystem->AccessPathName(filename,kReadPermission) ) {
        sprintf(filename,"%s/dat/%s",stdlinksdir,"std_auaudata");
        if( gSystem->AccessPathName(filename,kReadPermission) 
            || gSystem->Symlink(filename,"GENEVS")) {
          Error("InitLinks", "Link GENEVS to a file with particle info before proceeding.");
          llinit = kFALSE; linit = linit && llinit;
	} 
        if( llinit ) {
         fGENEVS = filename;
          if(fVerbose > 0)
            Warning("InitLinks",
              "GENEVS file with particle info linked to standard std_auaudata!");
	}
      } else {
         if( !SymRead("GENEVS",sname,128) ) {
            fGENEVS = "GENEVS";
         } else {
            fGENEVS = sname;
         }
      }
    }

    // Setting a link to the file with console output
    if(!strcmp(which,"ALL") || !strcmp(which,"RHASC")) {
      // case: RHASC set to path provided by the user 
      if( !fRHASC.IsNull() ) {
        gSystem->Unlink("RHASC");
        gSystem->Symlink(fRHASC.Data(),"RHASC");
      }
      // check to see if this file exists
      sprintf(filename,"%s/RHASC",wdir);
      if( gSystem->AccessPathName("RHASC",kReadPermission) ) {
         if(fVerbose > 0)
            Warning("InitLinks", "File RHASC does not exists - creating.");
         fRHASC = "RHASC";
      } else {
         if( !SymRead("RHASC",sname,128) ) {
            gSystem->Unlink("RHHST");
            fRHASC = "RHASC";
         } else {
            gSystem->Unlink("sname");
            fRHASC = sname;
         }
      }
    }
      
    // Setting a link to the file with histogram output
    if(!strcmp(which,"ALL") || !strcmp(which,"RHHST")) {
      // case: RHHST set to path provided by the user 
      if( !fRHHST.IsNull() ) {
        gSystem->Unlink("RHHST");
        gSystem->Symlink(fRHHST.Data(),"RHHST");
      }
      // check to see if this file exists
      sprintf(filename,"%s/RHHST",wdir);
      if( gSystem->AccessPathName("RHHST",kReadPermission) ) {
         if(fVerbose > 0)
            Warning("InitLinks", "File RHHST does not exists - creating.");
         fRHHST = "RHHST";
      } else {
         if( !SymRead("RHHST",sname,128) ) {
            gSystem->Unlink("RHHST");
            fRHHST = "RHHST";
         } else {
           gSystem->Unlink(sname);
           fRHHST = sname;
         }
      }
    }
      
    // Setting a link to the ntuple output file for tracks with hits
    if(!strcmp(which,"ALL") || !strcmp(which,"RHNTU")) {
      // case: RHNTU set to path provided by the user 
      if( !fRHNTU.IsNull() ) {
        gSystem->Unlink("RHNTU");
        gSystem->Symlink(fRHNTU.Data(),"RHNTU");
      }
      // check to see if this file exists
      sprintf(filename,"%s/RHNTU",wdir);
      if( gSystem->AccessPathName("RHNTU",kReadPermission) ) {
         if(fVerbose > 0)
            Warning("InitLinks", "File RHNTU does not exists - creating.");
         fRHNTU = "RHNTU";
      } else {
         if( !SymRead("RHNTU",sname,128) ) {
            gSystem->Unlink("RHNTU");
            fRHNTU = "RHNTU";
         } else {
           gSystem->Unlink(sname);
           fRHNTU = sname;
         }
      }
    }
      
    // Setting a link to the ntuple output file for all tracks
    if(!strcmp(which,"ALL") || !strcmp(which,"MCNTU")) {
      // case: MCNTU set to path provided by the user 
      if( !fMCNTU.IsNull() ) {
        gSystem->Unlink("MCNTU");
        gSystem->Symlink(fMCNTU.Data(),"MCNTU");
      }
      // check to see if this file exists
      sprintf(filename,"%s/MCNTU",wdir);
      if( gSystem->AccessPathName("MCNTU",kReadPermission) ) {
         if(fVerbose > 0)
            Warning("InitLinks", "File MCNTU does not exists - creating.");
         fMCNTU = "MCNTU";
      } else {
         if( !SymRead("MCNTU",sname,128) ) {
            gSystem->Unlink("MCNTU");
            fMCNTU = "MCNTU";
         } else {
           gSystem->Unlink(sname);
           fMCNTU = sname;
         }
      }
    }
      
    // Setting a link to the RZ-format Zebra bank output file
    if(!strcmp(which,"ALL") || !strcmp(which,"RHZEB")) {
      // case: RHZEB set to path provided by the user 
      if( !fRHZEB.IsNull() ) {
        gSystem->Unlink("RHZEB");
        gSystem->Symlink(fRHZEB.Data(),"RHZEB");
      }
      // check to see if this file exists
      sprintf(filename,"%s/RHZEB",wdir);
      if( gSystem->AccessPathName("RHZEB",kReadPermission) ) {
         if(fVerbose > 0)
            Warning("InitLinks", "File RHZEB does not exists - creating.");
         fRHZEB = "RHZEB";
      } else {
         if( !SymRead("RHZEB",sname,128) ) {
            gSystem->Unlink("RHZEB");
            fRHZEB = "RHZEB";
         } else {
           gSystem->Unlink(sname);
           fRHZEB = sname;
         }
      }
    }
    return linit;
}


void  TPhPMC::SetRunLink(TPhRun *run)
{
  // Set a pointer to run object. It will receive info on run parameters

  gPMCOutputRun = run;
  if(fPMCNotes) gPMCOutputRun->AddNotes(fPMCNotes);

  return;
}


void TPhPMC::Close()
{
  //  Close and clear PMC stores
      UGLAST();
}


void TPhPMC::GetVersions()
{
  // Get Geant and PMC version numbers and store them as TPhPMC object data members
   Float_t geantver, pmcver, detver;
   GETGEANTPMCVER(geantver, pmcver, detver);
   fGeantVer = geantver;
   fPmcVer = pmcver;
   fPmcDetVer = detver;

#ifdef DEBUG
   printf("GetGeantPmcVer received:\n GEANT version:%f, PMC version:%f, Geometry version:%f\n", geantver, pmcver, detver);
#endif   
}


void TPhPMC::AddPMCNotes(Char_t *notes) 
{
  // Add user notes to the PMC notes on the PMC parameters and files used for simulations

  if(!fPMCNotes) 
    fPMCNotes = new TPhNotes("PMCNotes","PMCNotes created by TPhPMC code");  

  fPMCNotes->AddLine(notes);

  return;
}


void TPhPMC::PutVTrack(Int_t id, \
                              Float_t px, Float_t py, Float_t pz, \
                              Float_t vx, Float_t vy, Float_t vz) 
{
   PUTVKINE(id,px,py,pz,vx,vy,vz);
}

void TPhPMC::PutATrack(Int_t id, \
                              Float_t p, Float_t th, Float_t phi, \
                              Float_t vx, Float_t vy, Float_t vz) 
{
   PUTAKINE(id,p,th,phi,vx,vy,vz);
}

void TPhPMC::PutTrack(TPhTrack *track)
{
   Int_t id;
   Float_t px,py,pz,vx,vy,vz;

   id = track->GetGPid();
   px = track->GetPx();
   py = track->GetPy();
   pz = track->GetPz();
   vx = track->GetXProd();
   vy = track->GetYProd();
   vz = track->GetZProd();

   PUTVKINE(id,px,py,pz,vx,vy,vz);
}

Bool_t TPhPMC::SetPathToFile(Char_t *filesymbol, Char_t *filename)
{
  // Set file/path names for some important PMC input/output files
  // Stored in TPhPMC object for user information purpose.
  // Files with magnetic field (FLDASC) and with input MC data (GENEVS) 
  // can be changed even after initialization.

  // After initialization allow changes only for GENEVS and FLDASC
  if( strcmp(filesymbol,"FLDASC") && strcmp(filesymbol,"GENEVS") && fgPMCInit) 
  {
     Warning("SetPathToFile",
         "Change of %s not allowed after PMC initialization", filesymbol);
     return kFALSE;
  }

  // Let TPhRun know about the changes
  Char_t str[200];
  sprintf(str," FILELINK  %s linked to %s",filesymbol, filename);
  AddPhatPMCRInfo(str);

  Int_t one = 1;
  Float_t values[10];
  if( !strcmp(filesymbol,"FLDASC") )     {
    if( gSystem->AccessPathName(filename,kReadPermission) ) {
     Warning("SetPathToFile",
         "No access to %s, FLDASC link not changed", filename);
      return kFALSE;
    } 
      fFLDASC = filename;
      values[0] = 2;
      // Order is important. InitLinks links the file
      // SetPMCParValue reads the map in.
      if( InitLinks(filesymbol) ) 
          return SetPMCParValue("MAGT",&one,values);
      else 
	return kFALSE;                    }
  else if( !strcmp(filesymbol,"GENEVS") ) {
    if( gSystem->AccessPathName(filename,kReadPermission) ) {
     Warning("SetPathToFile",
         "No access to %s, GENEVS link not changed", filename);
      return kFALSE;
    } 
    fGENEVS = filename;
    values[0] = 0;
    // Order is important. 
    // SetPMCParValue closes eventually) opened previously file.
    // InitLinks links the new one.
    SetPMCParValue("GEN ",&one,values);
    return InitLinks(filesymbol); }
  else if( !strcmp(filesymbol,"GCDEFS") ) {
    if( gSystem->AccessPathName(filename,kReadPermission) ) {
     Warning("SetPathToFile",
         "No access to %s, GCDEFS link not changed", filename);
      return kFALSE;
    } 
    fGCDEFS = filename;
    return InitLinks(filesymbol); }
  else if( !strcmp(filesymbol,"RHDATA") ) {
    if( gSystem->AccessPathName(filename,kReadPermission) ) {
     Warning("SetPathToFile",
         "No access to %s, RHDATA link not changed", filename);
      return kFALSE;
    } 
    fRHDATA = filename;
    return InitLinks(filesymbol); }
  // below assume can open non-existing file for writing 
  else if( !strcmp(filesymbol,"RHAOU") ) {
    fRHAOU = filename;
    return InitLinks(filesymbol); }
  else if( !strcmp(filesymbol,"RHASC") ) {
    fRHASC = filename;
    return InitLinks(filesymbol); }
  else if( !strcmp(filesymbol,"RHHST") ) {
    fRHHST = filename;
    return InitLinks(filesymbol); }
  else if( !strcmp(filesymbol,"RHNTU") ) {
    fRHNTU = filename;
    return InitLinks(filesymbol); }
  else if( !strcmp(filesymbol,"MCNTU") ) {
    fMCNTU = filename;
    return InitLinks(filesymbol); }
  else if( !strcmp(filesymbol,"RHZEB") ) {
    fRHZEB = filename;
    return InitLinks(filesymbol); }
  else return kFALSE;
}

Char_t *TPhPMC::GetPMCParName(Int_t *no)
{
  // Get the PMC name of the parameter no.
  // this name can be later used to get/set the parameter value.

   Char_t *pname = new Char_t[5];
   GETPMCPNAME(*no, pname);

#ifdef DEBUG
   printf("GetPMCParName was called for parameter no %d, name: %s\n",*no,pname);
#endif

   return pname;
}

Bool_t TPhPMC::GetPMCParValue(Char_t *pname, Int_t *number, Float_t *values)
{
  // Get the parameter set of values.
   Char_t sym[5]; 
   strcpy(sym,pname);
   GETPMCPAR(sym, *number, values);

#ifdef DEBUG
   printf("GetPMCParValue: parameter %s is set to:",pname);
   for (int i=0; i<*number; i++) printf("%f ",values[i]);
   printf("\n"); 
#endif

   return (*number>0);
}

Bool_t TPhPMC::SetPMCParValue(Char_t *pname, Int_t *number, Float_t *values)
{
   // Set the parameter set of values.
   Char_t sym[5];
   Char_t str[200], stradd[30];
   Bool_t setOK; 
   strcpy(sym,pname);

   // Block change of the SMEM parameter if before Init() was set to 0.
   // I want only original detector object created by TPhPMC to be used for
   // labeling of tracks and hits.
   if( !strcmp(sym,"SMEM") ) {
     Int_t newsmem = (Int_t) values[0];
     if(newsmem && !GetDetectorCreated()) {
       if(fgPMCInit) {
         Error("SetPMCParValue","SMEM change from 0 not allowed after PMC initialization",
                newsmem ); 
         return kFALSE;
       }
     }
   }
   SETPMCPAR(sym, *number, values);
   setOK = (*number>0);
   // inform run object about the changes
   if(setOK) {
     // Let TPhRun know about the changes
     sprintf(str," PARCHANGE %10s changed to: ",pname);
     for(Int_t it=0; it<(*number); it++) {
       if(it>6) {
	 strcat(str," ...");
	 break;
       }
       sprintf(stradd," %10f ",*(values+it));
       strcat(str,stradd);
     }
     AddPhatPMCRInfo(str);
   }
#ifdef DEBUG
   printf("SetPMCParValue: parameter %s is being set to:",pname);
   for (int i=0; i<*number; i++) printf("%f ",values[i]);
   printf("\n"); 
#endif

   return setOK;
}

void TPhPMC::PrintPMCPars()
{
  // Print PMC parameters (those that can be changed by SetPMCParValue() function)
    printf("\n************************************************************************************************\n");
    printf("*                                        PARAMETERS                                            *\n");
    printf("************************************************************************************************\n");
    Char_t *pname; Int_t parnum = 0, parcount = 0; Float_t values[10];
    while(++parnum) {
      pname = GetPMCParName(&parnum);
      if(!(parnum)) break;
      GetPMCParValue(pname,&parcount,values); 
      if(parcount) {
         printf("            %s set to: ",pname);
         for (int i=0; i<parcount; i++) printf("%10f ",values[i]);
         printf("\n");
      }
    }
    printf("*************************************************************************************************\n\n");
}

void TPhPMC::PMCGufld(Float_t *vect, Float_t *b)
{
  // The original PMC function calculating the PHOBOS magnetic field is called here.
  // so that no additional TPhMagnet has to be used together with TPhPMC.

   GUFLD(vect,b);

#ifdef DEBUG
   printf("PMCGufld was called: at X=%f Y=%f Z=%f magnetic field is:%f %f %f\n",
           vect[0],vect[1],vect[2],b[0],b[1],b[2]);
#endif

   return;
}

void TPhPMC::SetTrackBaseNo(Int_t no)
{
  // Start numbering tracks again from 0 

     gPMCLatestTrackNo = 0;
     gPMCEventTrackOffset = no;
}

TPhEvent *TPhPMC::SwimPmcEvent(TPhEvent *outEvent,
                               Char_t *outTrackContName, Char_t *outHitContName)
{
  // Swim tracks from TRIG (set in control file) events read from the GENEVS 
  // (sym link) Hijet file and store tracks and hits in the containers of (one) 
  // outEvent TPhEvent object  (provided by a user), 
  // with names given as parameters to this function.

     TPhDetector *det = NULL;
     det = TPhDetector::GetInstance();
     if( !det ) {
       Error("SwimPmcEvent","TPhDetector object not available, cannot swim particles!");
       return NULL;
     }
     if( TMath::Abs(GetPmcDetVer()-det->GetVersion())>1.0e-5 ) {
       Error("SwimPmcEvent","Current geometry version differs from that during " \
             "PMC initialization, will not swim particles!");
       return NULL;
     }

     Float_t values[10];
     Int_t number;
     GetPMCParValue("GEN ", &number, values);
     if( fabs(values[0]-10)<1.0e-7 ) {
         Error("SwimPmcEvent",
               "GEN=%2f can not be used with this function.",values[0]);
         return NULL;
     }
     if( !outEvent ) {
         if(fVerbose) Warning("SwimPmcEvent","NULL pointer to output TPhEvent object.");
         return NULL;
     }
     if( !outTrackContName ) {
         if(fVerbose) 
            Warning("SwimPmcEvent","Unknown output track container name, set to PMCTracks.");
         outTrackContName = "PMCTracks";
     }
     if( !outHitContName ) {
         if(fVerbose) 
            Warning("SwimPmcEvent","Unknown output hit container name, set to PMCHits.");
         outHitContName = "PMCHits";
     }

     gPMCLatestTrackNo = 0;
     gPMCEventTrackOffset = 0;
     Run(outEvent, outTrackContName, outHitContName);
     return outEvent;
}


TPhEvent *TPhPMC::SwimPhatEvent(TPhEvent *inEvent, Char_t *inTrackContName, 
                  TPhEvent *outEvent, Char_t *outTrackContName, Char_t *outHitContName)
{
  // Swim tracks from the TPhEvent inEvent event (inTrackContName track container 
  // name) and store tracks and hits in the new containers of the outEvent object 
  // (provided by a user), with their names given as parameters to this function.
  // Pointers inEvent and outEvent can be the same.

     TPhDetector *det = NULL;
     det = TPhDetector::GetInstance();
     if( !det ) {
       Error("SwimPhatEvent","TPhDetector object not available, cannot swim particles!");
       return NULL;
     }
     if( TMath::Abs(GetPmcDetVer()-det->GetVersion())>1.0e-5 ) {
       Error("SwimPhatEvent","Current geometry version differs from that during PMC initialization, will not swim particles!");
       return NULL;
     }

     Float_t values[10];
     Int_t number;
     GetPMCParValue("GEN ", &number, values);
     if( number==0 || !(fabs(values[0]-10)<1.0e-7) ) {
         Error("SwimPhatEvent",
               "This function should be used with GEN=10, check the RHDATA control file");
         return NULL;
     }

     if( !inEvent ) {
         if(fVerbose) Warning("SwimPhatEvent","NULL pointer to input TPhEvent object.");
         return NULL;
     }
     if( !inTrackContName ) {
         if(fVerbose) Warning("SwimPhatEvent","Input track container name: %s unknown.",
                               inTrackContName);
         return NULL;
     }
     if( !outEvent ) outEvent = inEvent;
     if( !outTrackContName ) {
         if(fVerbose) 
            Warning("SwimPhatEvent","Unknown output track container name, set to PMCTracks.");
         outTrackContName = "PMCTracks";
     }
     if( !outHitContName ) {
         if(fVerbose) 
            Warning("SwimPhatEvent","Unknown output hit container name, set to PMCHits.");
         outHitContName = "PMCHits";
     }
     TPhObjectContainer *trackCont = NULL;
     if( !(trackCont = (TPhObjectContainer *) inEvent->GetObject(inTrackContName)) ) {
         if(fVerbose) Warning("SwimPhatEvent","Input track container %s not found.",
                               inTrackContName);
         return NULL;
     }

     
     TPhTrack *track;
     TIter nextTrack(trackCont->GetIterator());

     gPMCLatestTrackNo = 0;
     gPMCEventTrackOffset = 0;
     while( (track = (TPhTrack *) nextTrack()) ) {
        PutTrack(track);
        Run( outEvent, outTrackContName, outHitContName);
     }
     return outEvent;
}


TPhMCTrack *TPhPMC::SwimPhatTrack(Int_t gid, Float_t vx, Float_t vy, Float_t vz, 
                                         Float_t px, Float_t py, Float_t pz,
                    TPhEvent *outEvent, Char_t *outTrackContName, Char_t *outHitContName)
{
  // Swim track and store it and its hits in the containers of an outEvent
  // object (provided by a user), with names given as parameters to this function.
  // In addition creates and returns TPhMCTrack object with info only about primary track.

     TPhDetector *det = NULL;
     det = TPhDetector::GetInstance();
     if( !det ) {
       Error("SwimPhatTrack","TPhDetector object not available, cannot swim particles!");
       return NULL;
     }
     if( TMath::Abs(GetPmcDetVer()-det->GetVersion())>1.0e-5 ) {
       Error("SwimPhatTrack","Current geometry version differs from that during PMC initialization, will not swim particles!");
       return NULL;
     }

     Float_t values[10];
     Int_t number;
     GetPMCParValue("GEN ", &number, values);
     if( number==0 || !(fabs(values[0]-10)<1.0e-7) ) {
         Error("SwimPhatTrack",
               "This function should only be used with GEN=10, check the RHDATA control file");
         return NULL;
     }

     if( !outEvent ) {
         if(fVerbose) Warning("SwimPhatTrack","NULL pointer to output TPhEvent object.");
         return NULL;
     }
     if( !outTrackContName ) {
         if(fVerbose) 
            Warning("SwimPhatTrack","Unknown output track container name, set to PMCTracks.");
         outTrackContName = "PMCTracks";
     }
     if( !outHitContName ) {
         if(fVerbose) 
            Warning("SwimPhatTrack","Unknown output hit container name, set to PMCHits.");
         outHitContName = "PMCHits";
     }

     // TrackOffset counts tracks within Phat event, 
     gPMCEventTrackOffset += gPMCLatestTrackNo;    
     if(gPMCEventTrackOffset>1e9) {
       if(fVerbose) {
         Warning("SwimPhatTrack",
                 "Track numbers exceed reasonable value, base set back to 0.\n"); 
         Warning("SwimPhatTrack",
                 "Use SetTrackBaseNo(Int_t) before each new event in your macro.\n");
       }
       gPMCEventTrackOffset = 0;
     }
     // Set track parameters and swim the track
     PutVTrack( gid, px, py, pz, vx, vy, vz);
     Run( outEvent, outTrackContName, outHitContName);

     Int_t parentid;
     TPhMCTrack *outTrack = NULL, *outTrackClone = NULL;
     TIter nextTrack(((TPhObjectContainer *) \
           outEvent->GetObject(outTrackContName))->GetIterator(kIterBackward));
     // Take last track (hopefully the current one) with parent id = 0 (primary);
     while( (outTrack = (TPhMCTrack *) nextTrack()) ) 
       if(!(parentid = outTrack->GetParentNo())) break;
     outTrackClone = outTrack;
     if(outTrack) outTrackClone = (TPhMCTrack *) outTrack->Clone();

     return outTrackClone;
}

TPhMCTrack *TPhPMC::SwimPhatTrack(TPhTrack *inTrack,
                    TPhEvent *outEvent, Char_t *outTrackContName, Char_t *outHitContName)
{
  // Swim track and store it and its hits in the containers of an outEvent
  // object (provided by a user), with names given as parameters to this function.
  // In addition creates and returns pointer to a completely new copy of a 
  // TPhMCTrack object with info only about primary track. 

     TPhDetector *det = NULL;
     det = TPhDetector::GetInstance();
     if( !det ) {
       Error("SwimPhatTrack","TPhDetector object not available, cannot swim particles!");
       return NULL;
     }
     if( TMath::Abs(GetPmcDetVer()-det->GetVersion())>1.0e-5 ) {
       Error("SwimPhatTrack","Current geometry version differs from that during PMC initialization, will not swim particles!");
       return NULL;
     }

     Float_t values[10];
     Int_t number;
     GetPMCParValue("GEN ", &number, values);
     if( number==0 || !(fabs(values[0]-10)<1.0e-7) ) {
         Error("SwimPhatTrack",
               "This function should only be used with GEN=10, check the RHDATA control file");
         return NULL;
     }

     if( !outEvent ) {
         if(fVerbose) Warning("SwimPhatTrack","NULL pointer to output TPhEvent object.");
         return NULL;
     }
     if( !outTrackContName ) {
         if(fVerbose) 
            Warning("SwimPhatTrack","Unknown output track container name, set to PMCTracks.");
         outTrackContName = "PMCTracks";
     }
     if( !outHitContName ) {
         if(fVerbose) 
            Warning("SwimPhatTrack","Unknown output hit container name, set to PMCHits.");
         outHitContName = "PMCHits";
     }

     // TrackOffset counts tracks within Phat event, 
     gPMCEventTrackOffset += gPMCLatestTrackNo;    
     if(gPMCEventTrackOffset>1e9) {
       if(fVerbose) {
         Warning("SwimPhatTrack",
                 "Track numbers exceed reasonable value, base set back to 0.\n"); 
         Warning("SwimPhatTrack",
                 "Use SetTrackBaseNo(Int_t) before each new event in your macro.\n");
       }
       gPMCEventTrackOffset = 0;
     }
     // Set track parameters and swim the track
     PutTrack(inTrack);
     Run( outEvent, outTrackContName, outHitContName);

     Int_t parentid;
     TPhMCTrack *outTrack = NULL, *outTrackClone = NULL;
     TIter nextTrack(((TPhObjectContainer *) \
           outEvent->GetObject(outTrackContName))->GetIterator(kIterBackward));
     // Take last (hopefully current) track with parent id = 0 (primary);
     while( (outTrack = (TPhMCTrack *) nextTrack()) ) 
       if(!(parentid = outTrack->GetParentNo())) break;
     outTrackClone = outTrack;
     if(outTrack) outTrackClone = (TPhMCTrack *) outTrack->Clone();

     return outTrackClone;
}


Bool_t TPhPMC::Run(TPhEvent *event, Char_t *outTrackContName, Char_t *outHitContName)
{
  // Start PMC
  // Repeat ::Run(event) 
  // ...
  // to accumulate more tracks in the same or in the new TPhEvent.
  // They will be added to the outContName container.

      gPMCOutputEvent = event; 
      if(!event->GetObject(outTrackContName)) {
         gPMCTrackContainer = new TPhObjectContainer(outTrackContName);
         gPMCOutputEvent->AddObject(gPMCTrackContainer);
      }
      if(!event->GetObject(outHitContName)) {
         gPMCHitContainer   = new TPhObjectContainer(outHitContName);
         gPMCOutputEvent->AddObject(gPMCHitContainer);
      }
      Int_t itrig;
      Int_t number = 1; Float_t values[10];
      // Process TRIG events by an increase of the top event number counter
      GetPMCParValue("TRIG",&number,values);
      if(number != 1) return kFALSE;
      else itrig = (Int_t) values[0];
      GCFLAG.NEVENT += itrig;
      GRUN();

      return kTRUE;
}


//-------------------functions not present in TUnixSystem--------------------------
Bool_t TPhPMC::SymRead(Char_t *symname, Char_t *filename, Int_t blength)
{
  //read the file name where the symlink is pointing

  int len;

  len = readlink(symname, filename, blength-1);
  if( len < 0 ) return kFALSE;
  filename[len] = '\0';
  return kTRUE;
}

Bool_t TPhPMC::SymChange(Char_t *symname, Char_t *newfilename)
{
  //change file where the symlink is pointing

  Char_t sname[128];
  if( SymRead(symname,sname, 128) ) {
      gSystem->Unlink(symname);
      gSystem->Symlink(newfilename,symname);
      return !gSystem->Symlink(newfilename,symname);
  }
  return kFALSE;
}
