#ifndef _PhStraightHough_INCLUDED_
#define _PhStraightHough_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhStraightHough                                               //
//                                                                   //
//    PHOBOS track selection class                                   //
//                                                                   //
//    TPhStraightHough selects non-conflicting tracks from a list    //
//    of candidate tracks                                            //
//                                                                   //
//    Author  : Gunther Roland (rolandg@vxcern.cern.ch)              //
//    Created : 15/03/97                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
// $Id: TPhStraightHough.h,v 1.2 1998/09/18 03:20:20 decowski Exp $
//                                                                   //
///////////////////////////////////////////////////////////////////////

// ROOT classes

#include "TObject.h"
#include "TObjArray.h"

// Phobos classes

class TPhHit;
class TPhVertex;
class TPhDetector;
class TPhMomentumVector;

// Constants
const Int_t kBinsXZ  = 50;
const Int_t kBinsYZ  = 20;
const Int_t kNLayers = 4;
const Int_t kNArms   = 2;
const Int_t kPosArm  = 0;
const Int_t kNegArm  = 1;

class TPhStraightHough : public TObject {

private:
  void       FillTable(Int_t arm);
  void       FillCandidateTable(Int_t arm);
  void       GetSlopes(TPhHit *hit, Float_t *s1, Float_t *s2);
  void       CalcMinMaxSlopes(); 
  Bool_t     GetBins(Int_t *BinXZ, Int_t *BinYZ, 
		     Float_t SlopeXZ, Float_t SlopeXY, Int_t Arm);
  
public:
  TPhStraightHough();

  virtual ~TPhStraightHough() { };

  void       TransformHits();
  Bool_t     CheckVector(TPhMomentumVector *vec);

  Int_t      GetHitTable(Int_t BinXZ, Int_t BinXY, Int_t  Layer) 
    { return fHits[BinXZ][BinXY][Layer]; }
  void       GetNBins(Int_t *NBinsXZ, Int_t *NBinsYZ) 
    { *NBinsXZ = kBinsXZ; *NBinsYZ = kBinsYZ; }
  void               Reset() { fVertex = NULL; }
  void       SetVerbose(Int_t n)  { fVerbose = n; }

private:
  Int_t        fHits[kBinsXZ][kBinsYZ][kNLayers];  //! Output table of hough transform
  Int_t        fCand[kBinsXZ][kBinsYZ][kNArms];    //! Table of possible straight line tracks in slopeXZ-slopeYZ space
  Int_t        fLimitingLayer;                     //  ID of layer chosen as aperture limit for hough transform
  Float_t      fMinSlopeXZ[kNArms];                //  Minimum slope in XZ using fLimitingLayer as constraint
  Float_t      fMaxSlopeXZ[kNArms];                //  Maximum slope in XZ using fLimitingLayer as constraint
  Float_t      fMinSlopeYZ[kNArms];                //  Minimum slope in YZ using fLimitingLayer as constraint
  Float_t      fMaxSlopeYZ[kNArms];                //  Maximum slope in YZ using fLimitingLayer as constraint
  TPhDetector *fDetector;     //! Detector object for data and geometry access
  TPhVertex   *fVertex;       //! Provides access to vertex pos for transform
  Bool_t       fVerbose;      //! Control debugging output

public:

    ClassDef(TPhStraightHough,0) // StraightHough transform in first few planes
};

#endif



