#include "stdlib.h"
#include "math.h"
#include "TPhMCTrackTemplate.h"
#include "TPhHit.h"
#include "TPhMCHit.h"
#include "TRandom.h"

#include "TPhTrack.h"
#include "TPhMCTrack.h"
#include "TPhDetector.h"
#include "TPhSiSensor.h"

#include "matrix.h"

//mpd SunOS doesn't put finite() into math.h but into ieeefp.h
#ifdef __sun
 #include <ieeefp.h>
#endif

//WinNT does not support finite() according to cc. return always true for now.
#ifdef WIN32
 #error "finite() NOT YET IMPLEMENTED IN WIN32 Phat distribution!"

 int finite(double value)
{ return 1; }
#endif

ClassImp(TPhMCTrackTemplate)


///////////////////////////////////////////////////////////
//
// TPhMCTrackTemplate is a PHOBOS service class that provides
// a chi**2 comparison between a TPhRecTrack defined as a list
// of hits and an ideal particle trajectory in the PHOBOS 
// setup. 
//
////////////////////////////////////////////////////////////


TPhMCTrackTemplate::TPhMCTrackTemplate()
{
// Default constructor. Does nothing

    fNLayers        = 0;
    fMinField       = 0.2;
    fNRows          = 0;
    fMaxCompDist    = 4.0;
    fMinCompEnergy  = 0.5;

    fCovar        = NULL;
    fCovarEntries = NULL;

    fUseMeanDist  = kTRUE;
    fAddPadWidthFlag  = kFALSE;
    fNormEntriesFlag  = kFALSE;
    fNormCovarFlag    = kFALSE;
    fInvertFlag       = kFALSE;
    fPadWidthFactor = 0.0;

    fLastLayer = 0;
    fLastX = 0.0;
    fLastZ = 0.0;

    fDetector     = TPhDetector::GetInstance();
    if(!fDetector)  
       fprintf(stderr, "TPhMCTrackTemplate: Couldn't get detector\n"); 
}

TPhMCTrackTemplate::TPhMCTrackTemplate(Int_t NLayers)
{
// Constructor. Allocate storage for the track parameters 
// in NLayers planes

    Int_t i;

    fMass           = 0;
    fMinField       = 0.2;
    fNRows          = 0;
    fNLayers        = NLayers;
    fMaxCompDist    = 4.0;
    fMinCompEnergy  = 0.5;

    fCovar        = NULL;
    fCovarEntries = NULL;

    fUseMeanDist  = kTRUE;
    fAddPadWidthFlag  = kFALSE;
    fNormEntriesFlag  = kFALSE;
    fNormCovarFlag    = kFALSE;
    fInvertFlag       = kFALSE;
    fPadWidthFactor = 0.0;

    fLastLayer = 0;
    fLastX = 0.0;
    fLastZ = 0.0;

    for(i=0;i<kNMaxRows;i++)
    {
        fEntries[i]     = 0;  
        fLayerToRow[i]  = 0;
        fMeanDe[i]      = 0; 
        fMeanDLength[i] = 0;
        fMeanNPads[i]   = 0;
        fMeanDist45[i]  =  0;
        fMinDist45[i]   =  1000000;
        fMaxDist45[i]   = -1000000;
        fMeanDist[i]    = 0;
        fMeanLength[i]  = 0;
        fMeanR[i]       = 0;    
        fMeanX0[i]      = 0;  
        fMeanZ0[i]      = 0; 
        fPadHeight[i]   = 0; 
        fPadWidth[i]    = 0;
    }

    fDetector     = TPhDetector::GetInstance();
    if(!fDetector)
       fprintf(stderr, "TPhMCTrackTemplate: Couldn't get detector\n");
}

TPhMCTrackTemplate::~TPhMCTrackTemplate()
{
// Destructor. Frees storage for track parameters.

    delete fCovar;   
    delete fCovarEntries;   

}

void TPhMCTrackTemplate::FillTrack(TPhMCTrack *track)
{
// Fill track parametrization with information from *track. 
// In the present implementation only information from ideal 
// tracks without multiple scattering is used to extract the 
// trajectory.

    Int_t   i;
    Int_t   plane;
    Int_t   oldplane;
    Double_t pxz;
    Double_t rb;
    Double_t dx;
    Double_t dz;
    Double_t ww;
    TPhMCHit *hit;
    TPhSiSensor *sensor;
    
    fNRows = 0;
    if(fVerbose)
    {
        printf("fNLayers = %d\n",fNLayers);
    }
    for(i=0;i<kNMaxRows;i++)
    {
        fLayerToRow[i] = -1;
    }
    SetPx(track->GetPx());
    SetPy(track->GetPy());
    SetPz(track->GetPz());
    SetMass(track->GetMass());
    SetCharge(track->GetCharge());
    TIter NextHit(track->GetHitList());
    if(fVerbose)
    {
        printf("%d hits on track \n",
               track->GetHitList()->GetEntries());
    }
    oldplane = -1;
    while( (hit = (TPhMCHit *) NextHit()) )
    {
        plane = hit->GetLayer();
        if(fVerbose)
        {
            printf("hit in plane %d  row %d\n",plane, fNRows);
        }
        if(plane == oldplane)   continue;  // ignore multiple hits in the same layer
        if(plane < oldplane)  break;  // particle can not turn back - rest of track ignored
        if(plane - oldplane > 3)  break;  // particle can not miss two consecutive layers
        if(plane < 0  ||  plane > fNLayers) break;  // template does not include this layer

        oldplane = plane;

        if(plane > fLastLayer)
        {
           fLastLayer = plane;
           fLastX = hit->GetX();
           fLastZ = hit->GetZ();
        }

        sensor = (TPhSiSensor *) fDetector->GetSensor(hit->GetSensorLabel());

        fLayerToRow[plane]  =   fNRows;
        fMeanLength[fNRows] =   hit->GetLength();
        fMeanDe[fNRows]     =   hit->GetDe();
        fMeanDLength[fNRows]     =   hit->CalcDLength();
        fMeanDist45[fNRows] =   hit->CalcDist45();
        fMinDist45[fNRows]  =   fMeanDist45[fNRows];
        fMaxDist45[fNRows]  =   fMeanDist45[fNRows];
        fPadWidth[fNRows]   =   sensor->GetPxlWidth();
        fPadHeight[fNRows]  =   sensor->GetPxlHeight();
        if(fMeanDLength[fNRows] <= sensor->GetThickness())
        {
           fMeanNPads[fNRows]  =   1;
        }
        else
        {
           ww = sensor->GetThickness();
           ww = fMeanDLength[fNRows]*fMeanDLength[fNRows] - ww*ww;
           ww = sqrt(ww) / fPadWidth[fNRows] + 1.0;
           fMeanNPads[fNRows]  =  ww;
        }
        if(hit->GetField() > fMinField)
        {
            pxz = sqrt(GetPx()*GetPx()+GetPz()*GetPz());
            rb  = pxz/(0.3 * hit->GetField()) * 100;
            dx = rb * TMath::Sin(hit->GetPsi());
            dx *= -1;
            dz = rb * TMath::Cos(hit->GetPsi());
            if(GetCharge() < 0)
            {
               dx *= -1;
               dz *= -1;
            }
            fMeanR[fNRows]      =   sqrt(rb*rb);
            fMeanZ0[fNRows]     =   hit->GetZ() + dz;
            fMeanX0[fNRows]     =   hit->GetX() + dx;

            if(fVerbose)
            {
                printf("x,y,z,pxz,ipsi,rb,dx,dz: %g %g %g %g %g %g %g %g\n",
                        hit->GetX(),hit->GetY(),hit->GetZ(),
                        pxz,hit->GetPsi(),rb,dx,dz);
                printf("%d: R,X0,Z0: %g %g %g\n",fNRows,fMeanR[fNRows],fMeanZ0[fNRows],fMeanX0[fNRows]);
            }
            if(fVerbose)
            {
                printf("Template fNRows %d (%d): %g %g %g\n",
                   fNRows,plane,
                  fMeanR[fNRows],
                  fMeanZ0[fNRows],
                  fMeanX0[fNRows]);
             }
        }
        else
        {
            fMeanR[fNRows]      =   0;
            fMeanZ0[fNRows]     =   hit->GetZ();
            fMeanX0[fNRows]     =   hit->GetX();
            if(fVerbose)
                printf("No FLD %d: R,X0,Z0: %g %g %g\n",
                        fNRows,fMeanR[fNRows],fMeanZ0[fNRows],fMeanX0[fNRows]);
        }
        fNRows++;
        if(fNRows >= kNMaxRows)  break;  // never exceed the size of tables
    }
    if(fVerbose)
        printf("Found a total of %d rows for this template\n",fNRows);
    for(i=0;i<fNLayers;i++)
    {
        if(fVerbose)
            printf("Map %d to %d\n",i,fLayerToRow[i]);
    }

}

void TPhMCTrackTemplate::SetUseMeanDist(Bool_t usemeandist)
{
    if(!fCovar)
       fUseMeanDist = usemeandist;
    else
       fprintf(stderr,"TPhMCTrackTemplate::SetUseMeanDist: Covariance matrix already created, flag not changed\n");
}

void TPhMCTrackTemplate::MakeStamp()
{
    Int_t i;
    Int_t n;

    if(fNRows==0)
    {
        fprintf(stderr,"TPhMCTrackTemplate::MakeStamp: Can't make stamp for empty template\n");
        fStamp = 0;
    }
    else
    {
        fStamp = 0;
        for(i=0;i<kNMaxRows;i++)
        {
            if(fLayerToRow[i] != -1)
            {  
               n = (Int_t)pow(2,i);
               fStamp += n;
            }
            if(fVerbose)
               printf("%d: %d %d\n",i,fLayerToRow[i],fStamp);
        }
    }
}

void TPhMCTrackTemplate::AddTrack(TPhMCTrack *track, Bool_t AddToCovar)
{
// Add information from *track to track template 

    Int_t   i;
    Int_t   j;
    Int_t   plane;
    Int_t   row;
    Double_t d[30];
    TPhMCHit *hit;
    Float_t d45;

    if(fNRows < 1)
    {
       fprintf(stderr,"TPhMCTrackTemplate::AddTrack: Call FillTrack first\n");
       return;
    }

    for(i=0;i<fNRows;i++)
    {
        d[i] = 999.9;  //  denotes undefined distance from trajectory
    }
    
    TIter NextHit(track->GetHitList());
    while( (hit = (TPhMCHit *) NextHit()) )
    {
        plane    = hit->GetLayer();
        row      = fLayerToRow[plane];
        if(row > -1)
        {
           d45 = hit->CalcDist45();
           if(fMaxDist45[row] < d45)  fMaxDist45[row] = d45;
           if(fMinDist45[row] > d45)  fMinDist45[row] = d45;

           d[row] = GetDistXZ(hit);
           if(!AddToCovar)
           {
               fMeanDist[row] += d[row]*d[row];
               fEntries[row]++;
           }
        }
    }
    if(AddToCovar)
    {
        if(!fCovar)
        {
            fCovarEntries = (Double_t *) operator new(fNRows*fNRows*sizeof(Double_t));
            fCovar = new matrix(fNRows,fNRows);
            for(i=0;i<fNRows;i++)
                for(j=0;j<fNRows;j++)
                {
                    fCovar->poke(i,j,0);
                    fCovarEntries[fNRows*i+j] = 0;
                }
        }
        AddVector(&d[0]);
    }
}

Int_t TPhMCTrackTemplate::GetRow(Int_t layer)
{
  // Convert layer number to row number

    if(layer >= 0 && layer < fNLayers)  return  fLayerToRow[layer];
    return -1;
}


Double_t TPhMCTrackTemplate::GetMeanDistXZ(Int_t layer)
{
     Int_t row;

     row  = GetRow(layer);
     if(row > -1)
         return fMeanDist[row];
     else
         return 0;
}

void TPhMCTrackTemplate::AddVector(Double_t *vec)
{
// Add vector of hit errors to variance matrix

    Int_t   i;
    Int_t   j;
    Double_t val;
    Double_t v[kNMaxRows];

    for(i=0;i<fNRows;i++)
    {
        if(fVerbose>100)
            printf("%d: %g (%d) %g \n",i,vec[i],finite(vec[i]),fMeanDist[i]);
        v[i] = vec[i];
        if(fUseMeanDist  &&  (vec[i] < 999.0))
        {
           if(fMeanDist[i] > 0.0001) 
           {
               v[i] = vec[i]/fMeanDist[i];
               if(fabs(v[i]) > 990.0)  v[i] = 990.0;
           }
           else
                v[i] = 999.9;
        }
    }
    for(i=0;i<fNRows;i++)
    {
        for(j=0;j<fNRows;j++)
        {
            if((v[i] < 999.0)  &&  (v[j] < 999.0))
            {
               val = v[i]*v[j] + fCovar->peek(i,j);
               if(finite(val))
               {
                   fCovar->poke(i,j,val); 
                   fCovarEntries[fNRows*i+j]++;
               }
               else
               {
                   fprintf(stderr,"TPhMCTrackTemplate::AddVector: %d,%d Found NaN: %g %g %g %g %g %g\n",
                        i,j,vec[i],vec[j],val,fCovar->peek(i,j),fMeanDist[i],fMeanDist[j]);
               }
            }
        }
    }
}

void TPhMCTrackTemplate::NormEntries()
{
// Normalize deviations

    Int_t   row;

    if(fNormEntriesFlag)
    {
        fprintf(stderr,"TPhMCTrackTemplate::NormEntries: already normalized\n");
        return;
    }
   
    fNormEntriesFlag = kTRUE;

    for(row=0;row<fNRows;row++)
    {
        if(fEntries[row] > 0)
        {
           fMeanDist[row] = fMeanDist[row] / fEntries[row];
           fMeanDist[row] = sqrt(fMeanDist[row]);
           if(fMeanDist[row] < 0.001)
           {
              fMeanDist[row] = 0.001;
              if(fVerbose)
                     fprintf(stderr,"TPhMCTrackTemplate::NormEntries: fMeanDist[%d] set to 0.001\n",row);
           }
        }
        else
        {
             fMeanDist[row] = 0.001;
             if(fVerbose)
                     fprintf(stderr,"TPhMCTrackTemplate::NormEntries: Set fMeanDist to default  0.001  for row %d\n",row);
        }
    }
    if(fVerbose)
    {
        for(row=0;row<fNRows;row++)
           printf("row %d: %d %g %g\n",row,fEntries[row],fMeanDe[row],fMeanDist[row]);
        printf("\n");
     }
}

void TPhMCTrackTemplate::NormCovar()
{
// Normalize covariance matrix

    Int_t   row1;
    Int_t   row2;
    Double_t val;

    if(fNormCovarFlag)
    {
        fprintf(stderr,"TPhMCTrackTemplate::NormCovar: already normalized\n");
        return;
    } 
    if(!fCovar)
    {
        fprintf(stderr,"TPhMCTrackTemplate::NormCovar: fCovar == NULL\n");
        return;
    }
    if(!fCovarEntries)
    {
        fprintf(stderr,"TPhMCTrackTemplate::NormCovar: fCovarEntries == NULL\n");
        return;
    }
    if(fVerbose)
    {
        for(row1=0;row1<fNRows;row1++)
        {
            for(row2=0;row2<fNRows;row2++)
            {
                if(fCovarEntries[fNRows*row1+row2])
                {
                    printf("%g ",fCovar->peek(row1,row2));
                }
            }
            printf("\n");
        }
        printf("\n");
        for(row1=0;row1<fNRows;row1++)
        {
            for(row2=0;row2<fNRows;row2++)
            {
                if(fCovarEntries[fNRows*row1+row2])
                {
                    printf("%g ",fCovarEntries[fNRows*row1+row2]);
                }
            }
            printf("\n");
        }
        printf("\n");
    }

    for(row1=0;row1<fNRows;row1++)
    {
        for(row2=0;row2<fNRows;row2++)
        {
            if(fCovarEntries[fNRows*row1+row2])
            {
                val = fCovar->peek(row1,row2) / fCovarEntries[fNRows*row1+row2];
                fCovar->poke(row1,row2,val);
            }
        }
    }
    fNormCovarFlag = kTRUE;
}

void TPhMCTrackTemplate::AddPadWidth(Float_t factor)
{
  // modify the deviations and covariance matrix elements
  // to include the error due to the pad width
  // the pad width is reduced according to the angle at which the 
  // particle entered the sensor and then multiplied by statistical
  // factor (should be equal to 1/sqrt(12))

  Int_t row, i, j;
  Double_t d, dd;
  Double_t newdist[kNMaxRows];

  if(!fNormEntriesFlag)
  {
     fprintf(stderr, "TPhMCTrackTemplate::AddPadWidth: call NormEntries first\n");
     return;
  }
  if(!fNormCovarFlag)
  {
     fprintf(stderr, "TPhMCTrackTemplate::AddPadWidth: call NormCovar first\n");
     return;
  }
  if(fInvertFlag)
  {
     fprintf(stderr, "TPhMCTrackTemplate::AddPadWidth: impossible after InvertCovar\n");
     return;
  }
  if(fAddPadWidthFlag)
  {
     fprintf(stderr, "/TPhMCTrackTemplate::AddPadWidth: operation already done\n");
     return;
  }
  fAddPadWidthFlag = kTRUE;
  fPadWidthFactor = factor;

  if(factor <= 0.0)  return;

  // calculate new deviations
   for(row=0; row<fNRows; row++)
   {
      d = fPadWidth[row];
      if(fMeanNPads[row] > 1.01)
      {
           d = (fMeanNPads[row] - 1.0) * fPadWidth[row] / fMeanDLength[row];
           d = fPadWidth[row] * sqrt(1.0 - d*d);
      }
      d = d  * factor;
      dd = d * d  +  fMeanDist[row]*fMeanDist[row];
      newdist[row] = sqrt(dd);
   }

   if(fCovar)
   {
      if(fUseMeanDist)
      {
         for(i=0; i < fNRows; i++)
         {
            for(j=0; j < fNRows; j++)
            {
               d = fCovar->peek(i, j);
               d = d * fMeanDist[i] * fMeanDist[j];
               d = d / (newdist[i]*newdist[j]);
               fCovar->poke(i, j, d);
            }
            fCovar->poke(i, i, 1.0);
         }
      }
      else   //   !fUseMeanDist
      {
         for(row=0; row < fNRows; row++)
         {
            d = newdist[row] * newdist[row];
            fCovar->poke(row, row, d);
         }
      }
   }   //   if(fCovar)

   for(row=0; row < fNRows; row++)
   {
        fMeanDist[row] = newdist[row];
   }  
  return;
}


void TPhMCTrackTemplate::PrintCovar()
{
// Print covariance matrix

    Int_t row1;
    Int_t row2;

    printf("covar:\n");
    for(row1=0;row1<fNRows;row1++)
    {
        for(row2=0;row2<fNRows;row2++)
        {
            if(fCovar->peek(row1,row2) > 0)
                printf(" %.4f ",fCovar->peek(row1,row2));
            else
                printf("%.4f ",fCovar->peek(row1,row2));
        }
        printf("\n");
    }
    printf("\n");
}

void TPhMCTrackTemplate::InvertCovar()
{
// Invert covariance matrix

  if(!fNormCovarFlag)
  {
     fprintf(stderr, "TPhMCTrackTemplate::AddPadWidth: call NormCovar first\n");
     return;
  }
    
    if(fCovar)
    {
       fCovar->operator~();
       fInvertFlag = !fInvertFlag;
    }
    else
       fprintf(stderr,"TPhMCTrackTemplate::InvertCovar: fCovar == NULL");
}

void TPhMCTrackTemplate::PrintParameters(Int_t plane)
{
    Int_t row;

    row = fLayerToRow[plane];
    if(row > -1)
    {
         printf("Layer %d - row %d: x,z,R,D,E = %g %g %g %g %g\n",
                plane,
                row,
                fMeanX0[row],
                fMeanZ0[row],
                fMeanR[row],
                fMeanDist[row],
                fMeanDe[row]);
    }
    else
    {
        printf("No parameters for plane %d\n",plane);
    }
}

Double_t TPhMCTrackTemplate::GetDistXZ(TPhHit *hit)
{
// Get distance between hit and template in XZ plane

    Int_t   row;

    row = GetRow(hit->GetLayer());
    if(row > -1)   return GetDistXZ(hit,row);
    return 0;
}

Double_t TPhMCTrackTemplate::GetDistXZ(TPhHit *hit,Int_t row)
{
// Get distance between hit and template in XZ plane

    Double_t dx;
    Double_t dz;

    if(fMeanR[row])
    {
        dx = hit->GetX() - fMeanX0[row];
        dz = hit->GetZ() - fMeanZ0[row];
//            printf("%d:%g: dx,dz,r = %g %g %g\n",row,fMeanR[row],dx,dz,sqrt(dx*dx+dz*dz));
        return (sqrt(dx*dx+dz*dz) - fabs(fMeanR[row]));
    }
    else
    {
        dx   = hit->GetX() - fMeanX0[row];
        dz   = hit->GetZ() - fMeanZ0[row];
        if(dz>0)
            return sqrt(dx*dx+dz*dz);
        else
            return -sqrt(dx*dx+dz*dz);
    }
}

Double_t TPhMCTrackTemplate::GetDistLast(Float_t x, Float_t z, Int_t layer)
{
  // Find the distance in XZ plane of the point (x,z) from the 
  // trajectory of the track, assuming the point is in the last layer

   Double_t dx;
   Double_t dz;
   Int_t row;

   row = fLayerToRow[fLastLayer];
   dx   = x - fMeanX0[row];
   dz   = z - fMeanZ0[row];
   if(fMeanR[row])
   {
      return (sqrt(dx*dx+dz*dz) - fabs(fMeanR[row]));
   }
   else
   {
      if(dz>0)
         return sqrt(dx*dx+dz*dz);
      else
         return -sqrt(dx*dx+dz*dz);
   }
}

void TPhMCTrackTemplate::GetLast(Float_t *x, Float_t *z, Int_t *layer)
{
  // return the number of the last layer and the hit position in it

   *layer = fLastLayer;
   *x = fLastX;
   *z = fLastZ;
}

Bool_t TPhMCTrackTemplate::IsCompatible(TPhHit *hit)
{

    Double_t x1;
//    Double_t x2;
    Int_t   row;

    row  = fLayerToRow[hit->GetLayer()];
    if(row == -1)
    {
        if(fVerbose > 1)
            printf("Can't translate layer %d to row\n",hit->GetLayer());
        return kFALSE;
    }
    else
    {
        x1 = fabs(GetDistXZ(hit,row) / fMeanDist[row]);
        if(fVerbose > 1)
        {
            printf("Check hit %g %g %g at %d: %g %g %g - %g %g %g %g\n",
                     hit->GetX(), hit->GetY(), hit->GetZ(),
                     row, fMeanR[row], fMeanX0[row], fMeanZ0[row],
                     GetDistXZ(hit,row), fMeanDist[row], x1, fMaxCompDist);
        }
//        x2 = hit->GetDe()/fMeanDe[row];
//        if(x1 < fMaxCompDist && x2 > fMinCompEnergy)
        if(x1 < fMaxCompDist)
            return kTRUE;
        else
            return kFALSE;
    }

}

Bool_t TPhMCTrackTemplate::IsCompatible(TPhHit *hit, Double_t slopey, Int_t slopeylayer)
{

    Double_t x1, dy;
//    Double_t x2;
    Int_t   row, slopeyrow;

    row  = fLayerToRow[hit->GetLayer()];
    if(row == -1)
    {
        if(fVerbose > 1)
            printf("Can't translate layer %d to row\n",hit->GetLayer());
        return kFALSE;
    }
    else
    {
        x1 = fabs(GetDistXZ(hit,row) / fMeanDist[row]);
        if(fVerbose > 1)
        {
            printf("Check hit %g %g %g at %d: %g %g %g - %g %g %g %g\n",
                     hit->GetX(), hit->GetY(), hit->GetZ(),
                     row, fMeanR[row], fMeanX0[row], fMeanZ0[row],
                     GetDistXZ(hit,row), fMeanDist[row], x1, fMaxCompDist);
        }
//        x2 = hit->GetDe()/fMeanDe[row];
//        if(x1 < fMaxCompDist && x2 > fMinCompEnergy)

        if(x1 > fMaxCompDist) return kFALSE;

        slopeyrow = fLayerToRow[slopeylayer];
        if(slopeyrow == -1)   return kFALSE;

        dy = fabs(hit->GetY() - slopey * fMeanLength[row]);
        dy -= 4.0 * fMeanDist[row];  // always 4 sigma
        dy -= 0.5 * fPadHeight[slopeyrow] * fMeanLength[row] / 
                       fMeanLength[slopeyrow];

        if(dy > fPadHeight[row])   return kFALSE;

        return  kTRUE;

    }
}

void TPhMCTrackTemplate::CalcChi2(TPhTrack *track, Float_t *chi2, Float_t *ndof)
{
    CalcChi2(track, chi2, ndof, kTRUE);
}

void TPhMCTrackTemplate::CalcChi2(TPhTrack *track, Float_t *chi2, Float_t *ndof, Bool_t UseCovar)
{
// Calculate chi**2 between track and template. If UseCovar is
// true, full (inverted) covariance matrix is used.


    Int_t    i;
    Int_t    row;
    Int_t    plane;
    Double_t  dist;
    Double_t  d[kNMaxRows];
    TPhHit  *hit;

    *chi2 = 0; 
    *ndof = 0; 

    if(UseCovar)
    {
        for(i=0; i<kNMaxRows; i++)   d[i] = 999.9;

        TIter NextHit(track->GetHitList());
        while( (hit = (TPhHit *) NextHit()) )
        {
            plane    = hit->GetLayer();
            row      = fLayerToRow[plane];
            if(row > -1)
            {
               d[row] = GetDistXZ(hit, row);
               if(fUseMeanDist) 
               {
                  if(fMeanDist[row] > 0.0001)
                  {
                       d[row] = d[row] / fMeanDist[row];
                       if(fabs(d[row]) > 990.0)  d[row] = 990.0;
                  }
                  else
                       d[row] = 999.9;
               }
            }
        }

        if(fVerbose)
        {
            if(track->GetNHits() < fNRows)
            {
                for(i=0;i<fNRows;i++)
                {
                    if(d[i] < 0)
                        printf("%.3f ",d[i]);
                    else if(d[i] == 0)
                        printf(" ==== ");
                    else
                        printf(" %.3f ",d[i]);
                }
                printf("\n");
                if(fVerbose > 1)
                {
                   for(i=0;i<fNRows;i++)
                   {
                       if(d[i] < 0)
                           printf("%f ", fMeanDist[i]);
                       else if(d[i] == 0)
                           printf(" ==== ");
                       else
                           printf("%f ", fMeanDist[i]);
                   }
                   printf("\n");
                }
            }
        }
       
        CalcChi2(&d[0], chi2, ndof);
    }
    else
    {
        if(fVerbose)
            printf("%d points on track\n",track->GetNHits());
        TIter NextHit(track->GetHitList());
        while( (hit = (TPhHit *) NextHit()) )
        {
            row = fLayerToRow[hit->GetLayer()];
            if(row > -1)
            {
                dist = GetDistXZ(hit) / fMeanDist[row];
                *chi2 += dist*dist;
                *ndof++;
            }
        }
        if(fVerbose)
            printf("chi2, ndof = %g %g\n",*chi2,*ndof);
    }
    if(fVerbose)
        printf("chi2, ndof = %g %g\n",*chi2,*ndof);
}

Bool_t TPhMCTrackTemplate::ReadCovar(Char_t *FileName) 
{
// Read covariance matrix from *FileName in ASCII format

    Int_t    i;
    Int_t    NRows;
    Double_t x;
    FILE *fp;
    Double_t *data;

    fp = fopen(FileName,"r");
    if(fp)
    {
        fscanf(fp,"%d\n",&NRows);
        fCovar = new matrix(NRows,NRows);
        data = fCovar->matval();
        for(i=0;i<NRows*NRows;i++)
        {
            fscanf(fp,"%lf",&x);
            data[i] = x;
        }
        for(i=0;i<NRows;i++)
        {
            fscanf(fp,"%lf",&x);
            fMeanDist[i] = x;
        }
        fclose(fp);
        PrintCovar();
        return kTRUE;
    }
    else
    {
        printf("Can't open %s\n",FileName);
        return kFALSE;
    }
}

Bool_t TPhMCTrackTemplate::WriteCovar(Char_t *FileName) 
{
// Write covariance matrix to *FileName in ASCII format
    UInt_t    i;
    UInt_t    j;
    Int_t    ii;
    Double_t x;
    FILE *fp;

    fp = fopen(FileName,"w");
    if(fp)
    {
        fprintf(fp,"%d\n",fCovar->rowlen());
        for(i=0;i<fCovar->rowlen();i++)
        {
            for(j=0;j<fCovar->rowlen();j++)
            {
                x = fCovar->peek(i,j);
                fprintf(fp,"%f ",x);
            }
            fprintf(fp,"\n");
        }
        for(ii=0;ii<fNRows;i++)
        {
            fprintf(fp,"%g ",fMeanDist[ii]);
        }
        fclose(fp);
        return kTRUE;
    }
    else
    {
        printf("Can't open %s\n",FileName);
        return kFALSE;
    }
}

void TPhMCTrackTemplate::CalcChi2(Double_t *d, Float_t *chi2, Float_t *ndof)
{  // Calculate chi2 for the deviations stored in vector d 
   // using the covariance matrix
   // Missing hits are denoted by deviation = 999.9
   // for them an interpolated value is calculated and used
   // so that the  chi2  is minimized, 
   // for  nmiss = 1  missing hits minimalization is accurate (simple formula)
   // for  nmiss > 1  we should solve a set of  nmiss  equations
   // (probably creating a matrix, inverting it, etc.)
   // now less accurate but faster method is implemented:
   //  - first calculate approximate values of deviations  dn  for the missing 
   //    hits using the deviations from the closest hits only
   //  - then for each missing hit the deviation is calculated as if it would
   //    be the only one missing hit, using  dn values
   //    the deviation stored in  d  and  dn  are modifid
   // we obtain slightly larger chi2 than if we would solve the set of equations
   // we can treat this as an addtional punitive factor, 
   // in any case  ndof  is set to the actual number of hits used,
   // to give proper probability for the calculated chi2

    Int_t    i, j, n1, n2;
    Int_t    nmiss = 0;
    Double_t dchi2=0.0;
    Double_t w;
    Double_t dn[kNMaxRows];

    //  calculate distance in the layers without hits, so that
    //  (almost) minimal possible chi2 will be obtained
   for(i=0; i < fNRows;  i++)
   {
      if(d[i] > 999.0)    nmiss++;
   }
   if(nmiss == 1)   //  accurate calculation is easy
   {
      for(i=0;i<fNRows;i++)
      {
         if(d[i] > 999.0)
         {
            d[i] = 0.0;
            for(j=0; j < fNRows; j++)
               if(j != i)  d[i] += fCovar->peek(i,j) * d[j];
            d[i] = d[i] / fCovar->peek(i,i);
         }
      }
   }   //   if(nmiss == 1)
   if(nmiss > 1)   //  apply approximate calculations
   {
      for(i=0; i < fNRows; i++)
      { //   calculate  dn  vector - a crude approximation of correct deviations
         if(d[i] < 999.0)
            dn[i] = d[i];
         else
         {
            n1 = -1;
            n2 = -1;
            for(j=i; i >=0; i--);
               if(d[i] < 999.0)
               {
                  n1 = j;
                  break;
               }
            for(j=i; i < fNRows; i++)
               if(d[i] < 999.0)
               {  
                  n2 = j;
                  break;
               }
            if(n1 < 0)
            {
               dn[i] = d[n2];
               if(!fUseMeanDist)  dn[i] = fMeanDist[i] * d[n2] / fMeanDist[n2];
            }
            if(n2 < 0)
            {
               dn[i] = d[n1];
               if(!fUseMeanDist)  dn[i] = fMeanDist[i] * d[n1] / fMeanDist[n1];
            }   
            if(n1 >=0  &&  n2 >= 0)
            {
               w = (fMeanLength[i] - fMeanLength[n1]) / (fMeanLength[n2] - fMeanLength[n1]);
               dn[i] = w * d[n1]  + (1.0-w) * d[n2];
               if(!fUseMeanDist)  dn[i] = fMeanDist[i] * (w * d[n1] / fMeanDist[n1] + (1.0-w) * d[n2] / fMeanDist[n2]);
            }
         }   //   else  if(d[i] < 999.0)
      }   //   for(i=0; ...

     //   calculate the more correct values of  d
      for(i=0; i < fNRows; i++)
      {
         if(d[i] > 999.0)
         {
            d[i] = 0.0;
            for(j=0; j < fNRows; j++)
            {
               if(j != i)   d[i] += fCovar->peek(i,j) * dn[j];
            }
            d[i] = -d[i] / fCovar->peek(i,i);
            dn[i] = d[i];
         }
      }   //   for(i=0;  ...
                  
   }  //  if(nmiss > 1)

  // calculate chi2
   for(i=0; i < fNRows; i++)
   {
      for(j=0; j < fNRows; j++)
      {
           dchi2 += fCovar->peek(i,j) * d[i] * d[j];
      }
   }
   *ndof = fNRows - nmiss;
   *chi2 = dchi2;
}

Double_t TPhMCTrackTemplate::GetMeanDist(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMeanDist[row];
     return 0;
}

Double_t TPhMCTrackTemplate::GetMeanLength(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMeanLength[row];
     return 0;
}

Double_t TPhMCTrackTemplate::GetMeanDe(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMeanDe[row];
     return 0;
}

Double_t TPhMCTrackTemplate::GetMeanDLength(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMeanDLength[row];
     return 0;
}

Float_t TPhMCTrackTemplate::GetMeanDist45(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMeanDist45[row];
     return 0;
}

Float_t TPhMCTrackTemplate::GetMinDist45(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMinDist45[row];
     return 0;
}

Float_t TPhMCTrackTemplate::GetMaxDist45(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMaxDist45[row];
     return 0;
}

Float_t TPhMCTrackTemplate::GetMeanNPads(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMeanNPads[row];
     return 0;
}

Double_t TPhMCTrackTemplate::GetMeanR(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMeanR[row];
     return 0;
}

Double_t TPhMCTrackTemplate::GetMeanX0(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMeanX0[row]; 
     return 0;

}

Double_t TPhMCTrackTemplate::GetMeanZ0(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fMeanZ0[row];
     return 0;
}

Float_t TPhMCTrackTemplate::GetPadHeight(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fPadHeight[row];
     return 0;
}

Float_t TPhMCTrackTemplate::GetPadWidth(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fPadWidth[row];
     return 0;
}

Int_t TPhMCTrackTemplate::GetEntries(Int_t layer)
{
    Int_t row;

     row  = GetRow(layer);
     if(row > -1)   return fEntries[row];
     return 0;
}


Bool_t TPhMCTrackTemplate::CloneTemplate(TPhMCTrackTemplate *tmpl)
{

    Int_t i;
    Int_t row;
    Float_t  dd;

    if(CompatibleTemplate(tmpl))
    {
        for(i=0;i<fNLayers;i++)
        {
            row = fLayerToRow[i];
            if(row >=0)
            {
                fMeanDist[row]  = tmpl->GetMeanDist(i);
                dd = fMeanDist45[row] - tmpl->GetMeanDist45(i);
                fMaxDist45[row] = tmpl->GetMaxDist45(i) + dd;
                fMinDist45[row] = tmpl->GetMinDist45(i) + dd;
                fEntries[row]   = tmpl->GetEntries(i);
            }
        }
        fUseMeanDist = tmpl->GetUseMeanDist();
        fPadWidthFactor = tmpl->GetPadWidthFactor();
        fAddPadWidthFlag = tmpl->GetAddPadWidthFlag();
        fNormEntriesFlag = tmpl->GetNormEntriesFlag();
        fNormCovarFlag   = tmpl->GetNormCovarFlag();
        fInvertFlag      = tmpl->GetInvertFlag();
        SetCovar(tmpl->GetCovar());
        return kTRUE;
    }
    else
    {
        printf("TPhMCTrackTemplate::CloneTemplate(): GetNRows() doesn't match: %d %d\n",
                    tmpl->GetNRows(),GetNRows());
        return kFALSE;
    }
}

Bool_t TPhMCTrackTemplate::CompatibleTemplate(TPhMCTrackTemplate *tmpl)
{
    Int_t i;
    Int_t row;

    if(!tmpl)   return kFALSE;
    if(tmpl->GetNRows() != GetNRows())   return  kFALSE;

    for(i=0; i < fNLayers; i++)
    {
        row = fLayerToRow[i];
        if(row >= 0)
        {
            if(tmpl->GetMeanLength(i) == 0)  return  kFALSE;
        }
    }
    return kTRUE;
}

Bool_t TPhMCTrackTemplate::CompatibleTrack(TPhTrack *track)
{
    Int_t plane;
    Int_t hitlayer[kNMaxRows];
    TPhHit *hit;
    TObjArray *hitlist = track->GetHitList();

    if(GetNRows() > hitlist->GetEntries())   return kFALSE;

    for(plane=0; plane < fNRows; plane++)   hitlayer[plane] = 0;

    TIter NextHit(hitlist);
    while( (hit = (TPhHit *) NextHit()) )
    {   
        plane = hit->GetLayer();
        if(fLayerToRow[plane] < 0)    return kFALSE;
        if(fLayerToRow[plane] >= 0)   hitlayer[fLayerToRow[plane]] = 1;
    }

    if(fVerbose> 5)
    {
       for(plane=0; plane < fNRows; plane++)  printf("%3d", hitlayer[plane]);
       printf("\n");
    }
    for(plane=0; plane < fNRows; plane++) 
            if(hitlayer[plane] == 0)     return kFALSE;
    
    return kTRUE;
}
