//
// ======================
// TPhTrackSeedFinder.cxx
// ======================
//
// 19990118.0140 Inkyu.PARK@bnl.gov
//
//
// $Id: TPhTrackSeedFinder.cxx,v 1.10 1999/03/12 15:07:48 icpark Exp $

#include "math.h"

#include "TFile.h"
#include "TKey.h"
#include "TRandom.h"

#include "TPhTrackTemplate.h"
#include "TPhDetector.h"
#include "TPhSubDetector.h"
#include "TPhLayer.h"
#include "TPhHit.h"
#include "TPhRecHit.h"
#include "TPhMerging.h"
#include "TPhTrack.h"
#include "TPhEvent.h"
#include "TPhStuff.h"
#include "TPhObjectContainer.h"
#include "TPhMulguisin.h"
#include "TPhTrackSeedFinder.h"

ClassImp(TPhTrackSeedFinder)

//
// TPhTrackSeedFinder is a part of PHOBOS Spectrometer track reconstruction  
// program. 
//
// Create this object and call a member function RunMulguisin().
// Parameters to be set:
//
// fMergeHit : 
//		0 - no hit merging
//             	1 - hit merging upto 1 hit (So actually the same as 0)
//             	2 - hit merging upto 2 hits
//             	3 - hit merging upto 3 hits
//
// fMulguisinLayer : 
//		0 - use hits upto layer A 
//		1 - use hits upto layer B
//		2 - use hits upto layer C
//		3 - use hits upto layer D (recommendable)
//
// fMulguisinMode :
//		0 - fixed cone direction, fixed cone size
//		1 - movable cone direction, fixed cone size
//		2 - movable cone direction, variable cone size
//		3 - connecting hits (assume 100% hit efficiency)
//
// fMulguisinResol : cone size in eta-phi space
//  
  
TPhTrackSeedFinder::TPhTrackSeedFinder()
{
//
// Constructor : create hit arrays, seed lists
//
  fVerbose	  = 0;
  
  fMergeHit       = 0;
  fMulguisinMode  = 1;
  fMulguisinResol = 0.01;
  fMulguisinLayer = 3;

  fDetector       = TPhDetector::GetInstance();
  if (!fDetector) fprintf(stderr,"TPhTrackSeedFinder: Couldn't get detector object\n");
    fMerge = new TPhMerging();
    fMerge->SetMergingType(1);
    fMerge->SetMaxMergedPads(3);
  for (int i=0;i<kTSFNLayers;i++) fHitA[i]=new TObjArray();
  
  fTrackSeedList  = new TObjArray();
  fCompatibleSeedList = new TObjArray();

}

TPhTrackSeedFinder::~TPhTrackSeedFinder()
{
// 
// Destructor: Clear hit arrays, seed lists, and delete poiters
//
  if (fVerbose) printf("---- ~TPhTrackSeedFinder 0 ----\n");

  for (int i=0;i<kTSFNLayers;i++)
  {
    fHitA[i]->Clear();
    delete fHitA[i];
  }
  
  fTrackSeedList->Clear();
  delete fTrackSeedList;
  
  fCompatibleSeedList->Clear();
  delete fCompatibleSeedList;

  if (fVerbose) printf("---- ~TPhTrackSeedFinder 1 ----\n");
}

void TPhTrackSeedFinder::FillHit()
{// 

  //
  // initialization
  //

  for (int i=0;i<kTSFNLayers;i++) fHitA[i]->Clear();
 
  if (fMergeHit)
  {// only SpecP is considered at this moment
  
    fMerge->Reset('P');
    fMerge->MergeSpecHits('P');
   
    for (int ilayer=0; ilayer<kTSFNLayers; ilayer++)
    {
      int index0, index1;
      int nhits = fMerge->GetSpecHitIndexes(ilayer,&index0,&index1);
      if (nhits == 0) continue;
      for (int idx=index0; idx<=index1; idx++)
      {
        TPhRecHit *h=(TPhRecHit *)fMerge->GetHit(idx);
        fHitA[ilayer]->Add(h);
      }
    }
    return;
  }
  
  //
  // get PHOBOS detector
  //
 
  TPhSubDetector *subdet[2];
  subdet[0]=fDetector->GetSubDetector("SpectP");
  subdet[1]=fDetector->GetSubDetector("SpectN");
  
  for (int arm=0; arm<2; arm++)
  {// ---- loop over spectrometers 0 ----

    if (fVerbose>2)
      printf("Spectrometer %d (%d layers):\n",
	arm,subdet[arm]->GetLayerList()->GetEntries());
    
    TObjArray *LayerList = subdet[arm]->GetLayerList();
    for (int il=0; il<LayerList->GetEntries(); il++)
    {// ---- loop over layers 0 ----

      TPhLayer *hlayer = (TPhLayer *)LayerList->At(il);
      int label=hlayer->GetLabel();
      if (fVerbose>2)
       	printf("Layer %d (%d sensors):",
	  hlayer->GetLabel(), hlayer->GetSensorList()->GetEntries());
     
      TObjArray *SensorList=hlayer->GetSensorList();
      for (int is=0; is<SensorList->GetEntries(); is++)
      {// ---- loop over sensors 0 ----

	TPhSensor *sensor=(TPhSensor *)SensorList->At(is);
        if (!sensor->GetHitList()) continue;
        if (fVerbose>2)
	  printf(" %s(%d)",
	    sensor->GetChLabel(), sensor->GetHitList()->GetEntries());
        
	TObjArray *HitList=sensor->GetHitList();
        for (int ih=0; ih<HitList->GetEntries(); ih++)
        {// ---- loop over hits 0 ----
           TPhHit *hit=(TPhHit *)HitList->At(ih);
           fHitA[label]->Add(hit);
        }// ---- loop over hits 1 ----
      }// ---- loop over sensors 1 ----
      
      if (fVerbose>2) printf("\n");
    
    }// ---- loop over layers 1 ---- 
  }// ---- loop over spectrometers 1 ----
}

void TPhTrackSeedFinder::RunMulguisin(int mode, float resol)
{

  if (fVerbose) printf("---- TPhTrackSeedFinder::RunMulguisin 0 ----\n");

  //
  // initialise Mulguisin
  //
  
  fMulguisinMode=mode;
  fMulguisinResol=resol;

  FillHit();

  if (fTrackSeedList) fTrackSeedList->Clear();

  if (fMulguisinMode<3) 
  {
  
  //
  // looping on hits
  //

  int n_hits = 0;
  for (int i=0;i<=fMulguisinLayer;i++) n_hits += fHitA[i]->GetEntries();
  n_hits += 50;
  
  int    InumC=0;
  int   *IdetC=new int[n_hits];
  float *RentC=new float[n_hits];
  float *RetaC=new float[n_hits];
  float *RphiC=new float[n_hits];
  
  for (int il=0;il<=fMulguisinLayer; il++)
  {

    int nh=fHitA[il]->GetEntries();
    for (int ih=0;ih<nh;ih++)
    {
      TPhHit *hit = (TPhHit *)fHitA[il]->At(ih);  
	
      int layer = hit->GetLayer();
      float x=hit->GetX();
      float y=hit->GetY();
      float z=hit->GetZ();
      float r=sqrt(x*x+y*y+z*z);
      float theta=acos(z/r);
      float phi=atan2(y,x);
      //float rapidity=log(( r+fabs(z) )/sqrt(x*x+y*y));
      //      rapidity=(z>0)?rapidity:-rapidity;  
      float eta=-log(tan(theta/2)); 
	  
      IdetC[InumC]=1<<layer;
      RentC[InumC]=1/r;
      RetaC[InumC]=eta;
      RphiC[InumC]=phi;
      InumC++;
	  
    }
  }
  
  if (fVerbose>1) printf("Number of input hits = %d\n",InumC); 
           
  //
  // Run Mulguisin clustering
  //
  
  TPhMulguisin *mgs = new TPhMulguisin();
  mgs->SetFrame(1);
  mgs->SetMethod(0);
  mgs->SetMode(fMulguisinMode);
  mgs->SetResol(fMulguisinResol);
  
  mgs->Clustering(InumC,IdetC,RentC,RetaC,RphiC);

  int    InumG=mgs->GetInumG();
  int   *IdetG=mgs->GetIdetG();
  float *RentG=mgs->GetRentG();
  float *RetaG=mgs->GetRetaG();
  float *RphiG=mgs->GetRphiG();
      
  if (fVerbose>1) printf("Number of ghosts = %d\n",InumG);
           
  //
  // select track seeds and copy into trackseed list
  //
  
  int    InumJ = 0;
  int   *IdetJ = new int[InumG];
  float *RentJ = new float[InumG];
  float *RetaJ = new float[InumG];
  float *RphiJ = new float[InumG];
	    
  for (int ig=0; ig<InumG; ig++)
  {
    int id=IdetG[ig];
    Bool_t good_jet=1;
    //
    // check whether there exists at least one hit on each frame
    //
    for (int j=1; j<=fMulguisinLayer; j+=2)
    {
      good_jet = good_jet&&(id&(1<<j-1)||id&(1<<j));
    }
    //
    // check whether there are at least 3 hits.
    //
    int nhits=0;
    for (int i=0; i<=fMulguisinLayer; i++) if (id&(1<<i)) nhits++;
    good_jet = good_jet&&(nhits>=3); 
    //
    // 
    //
    if (good_jet)
    {
      IdetJ[InumJ]=IdetG[ig];
      RentJ[InumJ]=RentG[ig];
      RetaJ[InumJ]=RetaG[ig];
      RphiJ[InumJ]=RphiG[ig];
      InumJ++;
      
      float eta_t = RetaG[ig];
      float phi_t = RphiG[ig];
      float theta_t = 2*atan(exp(-eta_t));
      
      TPhTrack *seed = new TPhTrack();

      for (int ilayer=0; ilayer<=fMulguisinLayer; ilayer++)
      { 
        int n_hits = fHitA[ilayer]->GetEntries();
	for (int ihit=0; ihit<n_hits; ihit++)
	{
	  TPhHit *hh=(TPhHit *)fHitA[ilayer]->At(ihit);
          float xh=hh->GetX();
          float yh=hh->GetY();
          float zh=hh->GetZ();
          float rh=sqrt(xh*xh+yh*yh+zh*zh);
          float theta_h=acos(zh/rh);
          float phi_h=atan2(yh,xh);
          float eta_h=-log(tan(theta_h/2));
          float dist = mgs->Rdist(eta_h,phi_h,eta_t,phi_t);   
	  if (dist<fMulguisinResol)
	  {
	    seed->AddHit(hh);
	  }
	}
      }
      seed->SetPx(sin(theta_t)*cos(phi_t));
      seed->SetPy(sin(theta_t)*sin(phi_t));
      seed->SetPz(cos(theta_t));
      seed->SetXProd(0);
      seed->SetYProd(0);
      seed->SetZProd(0);
      fTrackSeedList->Add(seed);
    }
  }
  
  //
  // release memory
  //
  
  delete [] IdetC;
  delete [] RentC;
  delete [] RetaC;
  delete [] RphiC;
  
  delete [] IdetJ;
  delete [] RentJ;
  delete [] RetaJ;
  delete [] RphiJ;

  delete mgs; 
  
  }
  else
  {// ---- Mulguisin 3 ----
  
  TObjArray *fTrack[16];
  fTrack[0] = new TObjArray();
 
  int n0=fHitA[0]->GetEntries(); 
  for (int i0=0; i0<n0; i0++)
  {
    TPhTrack *t = new TPhTrack();
    TPhHit *hit=(TPhHit *)fHitA[0]->At(i0);
    float x=hit->GetX(), y=hit->GetY(), z=hit->GetZ();
    float r=sqrt(x*x+y*y+z*z); if(r==0) printf("?");
    t->SetPx(x/r);
    t->SetPy(y/r);
    t->SetPz(z/r);
    t->SetXProd(0);
    t->SetYProd(0);
    t->SetZProd(0);
    t->AddHit(hit);
    fTrack[0]->Add(t);
  }
  int nt=fTrack[0]->GetEntries();
  if (fVerbose) printf("How many initial tracks? %d\n", nt);
		      
  //
  //
  //

  for (int il=1;il<4;il++)
  {// printf("\n run layer %d (%d tracks)\n",il,fTrack[il-1]->GetEntries());
    fTrack[il] = new TObjArray();
    for (int t0=0; t0<fTrack[il-1]->GetEntries(); t0++)
    {//printf("\n proceed track %d\n",t0); 
      TPhTrack *t= (TPhTrack *) fTrack[il-1]->At(t0);
      float p0x=t->GetPx(), p0y=t->GetPy(), p0z=t->GetPz();
      //float p0=sqrt(p0x*p0x+p0y*p0y+p0z*p0z);
      TPhHit *lh=(TPhHit *)t->GetHitList()->At(t->GetNHits()-1);
      float x0=lh->GetX(), y0=lh->GetY(), z0=lh->GetZ();
      //printf("\n <%f %f %f>",x0,y0,z0);
      int nh=fHitA[il]->GetEntries();
      for (int h1=0; h1<nh; h1++)
      {//printf("\n (%d-%d)",il,h1);
        TPhHit *h=(TPhHit *)fHitA[il]->At(h1);
        float x1=h->GetX(), y1=h->GetY(), z1=h->GetZ();
        //printf("\n <%f %f %f>",x1,y1,z1);
        float dx=x1-x0, dy=y1-y0, dz=z1-z0;
        float dr=sqrt(dx*dx+dy*dy+dz*dz);
        if (dr==0) printf("xx %f %f",x1,x0);
        float p1x=dx/dr, p1y=dy/dr, p1z=dz/dr;
        float dp=acos(p1x*p0x+p1y*p0y+p1z*p0z);
        if (dp<fMulguisinResol)
        {
          TPhTrack *t1 = new TPhTrack();
          t1->SetPx(p1x); 
          t1->SetPy(p1y); 
          t1->SetPz(p1z);
          for (int ihb=0; ihb<t->GetNHits(); ihb++)
          {
            TPhHit *h0=(TPhHit *)t->GetHitList()->At(ihb);
            t1->AddHit(h0);
          }
          t1->AddHit(h);
          fTrack[il]->Add(t1);
        }
      }
    }
    int nt=fTrack[il]->GetEntries();
    if (fVerbose)
      printf("number of tracks after run layer %d =  %d\n",
	il,nt);
  }
  
  fTrackSeedList=fTrack[3];
  
  //
  // Recalculation (This should be done again later by fitting)
  //
  int nts=fTrackSeedList->GetEntries();
  for (int its=0;its<nts;its++)
  {
    TPhTrack *t = (TPhTrack *)fTrackSeedList->At(its);
    TPhHit *h = (TPhHit *)t->GetHitList()->At(0);
    float x=h->GetX(), y=h->GetY(), z=h->GetZ();
    float r=sqrt(x*x+y*y+z*z); if(r==0) printf("?");
    t->SetPx(x/r);
    t->SetPy(y/r);
    t->SetPz(z/r);
  }
  
  }  

  //
  // finalization
  //


  if (fVerbose>1)
    printf("Number of track seeds found = %d\n",
      fTrackSeedList->GetEntries());
  if (fVerbose)
    printf("---- TPhTrackSeedFinder::RunMulguisin 1 ----\n");

}

TObjArray* TPhTrackSeedFinder::GetCompatibleSeedList(TPhMomentumVector *v)
{ 
  
  if (fVerbose>2) printf("\n---- TPhTrackSeedFinder::GetCompatibleSeedList 0 ----");
  
  fCompatibleSeedList->Clear();
  
  int n_seeds = fTrackSeedList->GetEntries();
  for (int is=0; is<n_seeds; is++)
  {
    TPhTrack *s = (TPhTrack *) fTrackSeedList->At(is);
    float x_s = s->GetPx();
    float y_s = s->GetPy();
    float z_s = s->GetPz();
    float theta_s = acos(z_s/sqrt(x_s*x_s + y_s*y_s + z_s*z_s));
    float eta_s = -log(tan(theta_s/2));
     
    float x_v = v->GetPx(); 
    float y_v = v->GetPy(); 
    float z_v = v->GetPz(); 
    float theta_v = acos(z_v/sqrt(x_v*x_v + y_v*y_v + z_v*z_v));
    float eta_v = -log(tan(theta_v/2));
  
    if (fabs(eta_v-eta_s) < fMulguisinResol)
    {
      fCompatibleSeedList->Add(s); 
    }
  }
  return fCompatibleSeedList;
  if (fVerbose>2) printf("\n---- TPhTrackSeedFinder::GetCompatibleSeedList 1 ----");
}

float TPhTrackSeedFinder::GetAngularDist(TPhTrack *t0,TPhTrack *t1)
{
  float p0x=t0->GetPx(), p0y=t0->GetPy(), p0z=t0->GetPz(), p0=t0->GetP();
  float p1x=t1->GetPx(), p1y=t1->GetPy(), p1z=t1->GetPz(), p1=t1->GetP();
  p0x/=p0;p0y/=p0;p0z/=p0;  
  p1x/=p1;p1y/=p1;p1z/=p1;
  return acos(p1x*p0x+p1y*p0y+p1z*p0z);
}
