#include "stdlib.h"
#include "math.h"
#include "TPhTrackTemplate.h"
#include "TPhHit.h"
#include "TPhMCHit.h"
#include "TRandom.h"

#include "TPhTrack.h"
#include "TPhMCTrack.h"

#include "matrix.h"

//mpd SunOS doesn't put finite() into math.h but into ieeefp.h
#ifdef __sun
 #include <ieeefp.h>
#endif

//WinNT does not support finite() according to cc. return always true for now.
#ifdef WIN32
 #error "finite() NOT YET IMPLEMENTED IN WIN32 Phat distribution!"

 int finite(double value)
{ return 1; }
#endif

ClassImp(TPhTrackTemplate)


///////////////////////////////////////////////////////////
//
// TPhTrackTemplate is a PHOBOS service class that provides
// a chi**2 comparison between a TPhMCTrack defined as a list
// of hits and an ideal particle trajectory in the PHOBOS 
// setup. 
//
////////////////////////////////////////////////////////////


TPhTrackTemplate::TPhTrackTemplate()
{
// Default constructor. Does nothing

    fNLayers        = 0;
    fMinField       = 0.2;
    fNRows          = 0;
    fMaxCompDist    = 4.0;
    fMinCompEnergy  = 0.5;

    fCovar        = NULL;
    fCovarEntries = NULL;
}

TPhTrackTemplate::TPhTrackTemplate(Int_t NLayers)
{
// Constructor. Allocate storage for the track parameters 
// in NLayers planes

    Int_t i;
    Int_t row1;
    Int_t row2;

    fMass           = 0;
    fMinField       = 0.2;
    fNRows          = 0;
    fNLayers        = NLayers;
    fMaxCompDist    = 4.0;
    fMinCompEnergy  = 0.5;

    fCovar        = NULL;
    fCovarEntries = NULL;
    for(i=0;i<kMaxRows;i++)
    {
        fEntries[i]    = 0;  
        fLayerToRow[i] = 0;
        fMeanDedx[i]   = 0; 
        fMeanDist[i]   = 0;
        fMeanLength[i] = 0;
        fMeanR[i]      = 0;    
        fMeanX0[i]     = 0;  
        fMeanZ0[i]     = 0; 
        fMeanY[i]      = 0; 
    }
}

TPhTrackTemplate::~TPhTrackTemplate()
{
// Destructor. Frees storage for track parameters.

    delete fCovar;   
    delete fCovarEntries;   

}

void TPhTrackTemplate::FillTrack(TPhMCTrack *track,Bool_t HasMS)
{
// Fill track parametrization with information from *track. 
// In the present implementation only information from ideal 
// tracks without multiple scattering is used to extract the 
// trajectory.

    Int_t   i;
    Int_t   j;
    Int_t   plane;
    Int_t   oldplane;
    Int_t   row;
    Int_t   row1;
    Int_t   row2;
    Double_t pxz;
    Double_t rb;
    Double_t dx;
    Double_t dz;
    Double_t dd;
    Double_t d[30];
    TPhMCHit *hit;

    
    fNRows = 0;
    if(fVerbose)
    {
        printf("fNLayers = %d\n",fNLayers);
    }
    for(i=0;i<kMaxRows;i++)
    {
        fLayerToRow[i] = -1;
    }
    SetPx(track->GetPx());
    SetPy(track->GetPy());
    SetPz(track->GetPz());
    SetMass(track->GetMass());
    SetCharge(track->GetCharge());
    TIter NextHit(track->GetHitList());
    if(fVerbose)
    {
        printf("%d hits on track %x\n",
               track->GetHitList()->GetEntries(),
               track->GetHitList());
    }
    oldplane = -1;
    while(hit = (TPhMCHit *) NextHit())
    {
        plane = hit->GetLayer();
        if(plane == oldplane)   continue;  // ignore multiple hits in the same layer
        oldplane = plane;
        if(fVerbose)
        {
            printf("hit in plane %d\n",plane);
        }
        if(plane<0 || plane > fNLayers)
        {
            fprintf(stderr,"Found plane = %d; can't make template\n",plane);
            exit;
        }
        fLayerToRow[plane] = fNRows;
        if(hit->GetField() > fMinField)
        {
            pxz = sqrt(GetPx()*GetPx()+GetPz()*GetPz());
            rb  = pxz/(0.3 * hit->GetField()) * 100;
            dx = rb * TMath::Sin(hit->GetPsi());
            dx *= -1;
            dz = rb * TMath::Cos(hit->GetPsi());
            if(GetCharge() < 0)
            {
               dx *= -1;
               dz *= -1;
            }
            fMeanR[fNRows]      =   sqrt(rb*rb);
            fMeanZ0[fNRows]     =   hit->GetZ() + dz;
            fMeanX0[fNRows]     =   hit->GetX() + dx;
            fMeanLength[fNRows] =   hit->GetLength();
            fMeanDedx[fNRows]   =   hit->GetDe();
            if(fVerbose)
            {
                printf("x,y,z,pxz,ipsi,rb,dx,dz: %g %g %g %g %g %g %g %g\n",
                        hit->GetX(),hit->GetY(),hit->GetZ(),
                        pxz,hit->GetPsi(),rb,dx,dz);
                printf("%d: R,X0,Z0: %g %g %g\n",fNRows,fMeanR[fNRows],fMeanZ0[fNRows],fMeanX0[fNRows]);
            }
            if(fVerbose)
            {
                printf("Template fNRows %d (%d): %g %g %g\n",
                   fNRows,plane,
                  fMeanR[fNRows],
                  fMeanZ0[fNRows],
                  fMeanX0[fNRows]);
             }
        }
        else
        {
            fMeanR[fNRows]      =   0;
            fMeanZ0[fNRows]     =   hit->GetZ();
            fMeanX0[fNRows]     =   hit->GetX();
            fMeanLength[fNRows] =   hit->GetLength();
            if(fVerbose)
                printf("No FLD %d: R,X0,Z0: %g %g %g\n",
                        fNRows,fMeanR[fNRows],fMeanZ0[fNRows],fMeanX0[fNRows]);
        }
        fNRows++;
    }
    if(fVerbose)
        printf("Found a total of %d rows for this template\n",fNRows);
    for(i=0;i<fNLayers;i++)
    {
        if(fVerbose)
            printf("Map %d to %d\n",i,fLayerToRow[i]);
    }
}

void TPhTrackTemplate::MakeStamp()
{
    Int_t i;
    Int_t n;
    Int_t row;

    if(fNRows==0)
    {
        fprintf(stderr,"Can't make stamp for empty template\n");
        fStamp = 0;
    }
    else
    {
        fStamp = 0;
        for(i=0;i<kMaxRows;i++)
        {
            if(fLayerToRow[i] != -1)
            {  
               n = (Int_t)pow(2,i);
               fStamp += n;
            }
            if(fVerbose)
               printf("%d: %d %d\n",i,fLayerToRow[i],fStamp);
        }
    }
}

void TPhTrackTemplate::AddTrack(TPhMCTrack *track, Bool_t AddToCovar)
{
// Add information from *track to track template 

    Int_t   i;
    Int_t   j;
    Int_t   plane;
    Int_t   row;
    Double_t d[30];
    TPhMCHit *hit;

    for(i=0;i<fNRows;i++)
    {
        d[i] = 1;
    }
    
    TIter NextHit(track->GetHitList());
    while(hit = (TPhMCHit *) NextHit())
    {
        plane    = hit->GetLayer();
        row      = fLayerToRow[plane];
        if(row > -1)
        {
           d[row] = GetDistXZ(hit);
           if(!AddToCovar)
           {
               fMeanDist[row] += d[row]*d[row];
               fEntries[row]++;
           }
        }
    }
    if(AddToCovar)
    {
        if(!fCovar)
        {
            fCovarEntries = (Double_t *) operator new(fNRows*fNRows*sizeof(Double_t));
            fCovar = new matrix(fNRows,fNRows);
            for(i=0;i<fNRows;i++)
                for(j=0;j<fNRows;j++)
                {
                    fCovar->poke(i,j,0);
                    fCovarEntries[fNRows*i+j] = 0;
                }
        }
        AddVector(&d[0]);
    }
}

Double_t TPhTrackTemplate::GetMeanDistXZ(Int_t plane)
{
     Int_t row;

     row  = fLayerToRow[plane];
     if(row > -1)
     {
         return fMeanDist[row];
     }
     else
     {
         return 0;
     }
}

void TPhTrackTemplate::AddVector(Double_t *vec)
{
// Add vector of hit errors to variance matrix

    Int_t   i;
    Int_t   j;
    Double_t val;
    Double_t v[50];

    for(i=0;i<fNRows;i++)
    {
        if(fVerbose>100)
            printf("%d: %g (%d) %g \n",i,vec[i],finite(vec[i]),fMeanDist[i]);
        if(fMeanDist[i] != 0) 
           v[i] = vec[i]/fMeanDist[i];
        else 
           v[i] = 1;
    }
    for(i=0;i<fNRows;i++)
    {
        for(j=0;j<fNRows;j++)
        {
            val = v[i]*v[j] + fCovar->peek(i,j);
            if(finite(val))
            {
                fCovar->poke(i,j,val); 
                fCovarEntries[fNRows*i+j]++;
            }
            else
            {
                fprintf(stderr,"%d,%d Found NaN: %g %g %g %g %g %g\n",
                        i,j,vec[i],vec[j],val,fCovar->peek(i,j),fMeanDist[i],fMeanDist[j]);
            }
        }
    }
}

void TPhTrackTemplate::NormEntries(Bool_t UseMeanDist)
{
// Normalize covariance matrix

    Int_t   row;
    Double_t val;
   
    if(fVerbose)
    {
        if(UseMeanDist)
            printf("Normalize fMeanDist\n");
        else
            printf("Don't normalize fMeanDist\n");
    }

    
    for(row=0;row<fNRows;row++)
    {
        if(fEntries[row] && fMeanDist[row] > 0)
        {
            if(UseMeanDist)
                fMeanDist[row]  = sqrt(fMeanDist[row]/fEntries[row]);
            else
                fMeanDist[row]  = 1.0;
            if(fMeanDist[row] == 0)
            {
                fMeanDist[row] = 0.001;
                fprintf(stderr,"fMeanDist[%d] set to 0.001\n",row);
            }
        }
        else
        {
             fMeanDist[row] = 1;
             fprintf(stderr,"Set fMeanDist to default for row %d\n",row);
        }
    }
    if(fVerbose)
    {
        for(row=0;row<fNRows;row++)
           printf("row %d: %d %g %g\n",row,fEntries[row],fMeanDedx[row],fMeanDist[row]);
        printf("\n");
     }
}

void TPhTrackTemplate::NormCovar()
{
// Normalize covariance matrix

    Int_t   row1;
    Int_t   row2;
    Double_t val;

    
    if(!fCovar)
    {
        fprintf(stderr,"fCovar == NULL\n");
        return;
    }
    if(!fCovarEntries)
    {
        fprintf(stderr,"fCovarEntries == NULL\n");
        return;
    }
    if(fVerbose)
    {
        for(row1=0;row1<fNRows;row1++)
        {
            for(row2=0;row2<fNRows;row2++)
            {
                if(fCovarEntries[fNRows*row1+row2])
                {
                    printf("%g ",fCovar->peek(row1,row2));
                }
            }
            printf("\n");
        }
        printf("\n");
        for(row1=0;row1<fNRows;row1++)
        {
            for(row2=0;row2<fNRows;row2++)
            {
                if(fCovarEntries[fNRows*row1+row2])
                {
                    printf("%g ",fCovarEntries[fNRows*row1+row2]);
                }
            }
            printf("\n");
        }
        printf("\n");
    }

    for(row1=0;row1<fNRows;row1++)
    {
        for(row2=0;row2<fNRows;row2++)
        {
            if(fCovarEntries[fNRows*row1+row2])
            {
                val = fCovar->peek(row1,row2)/fCovarEntries[fNRows*row1+row2];
                fCovar->poke(row1,row2,val);
            }
        }
    }
}


void TPhTrackTemplate::PrintCovar()
{
// Print covariance matrix

    Int_t row1;
    Int_t row2;

    printf("covar:\n");
    for(row1=0;row1<fNRows;row1++)
    {
        for(row2=0;row2<fNRows;row2++)
        {
            if(fCovar->peek(row1,row2) > 0)
                printf(" %.4f ",fCovar->peek(row1,row2));
            else
                printf("%.4f ",fCovar->peek(row1,row2));
        }
        printf("\n");
    }
    printf("\n");
}

void TPhTrackTemplate::InvertCovar()
{
// Invert covariance matrix

    Int_t row1;
    Int_t row2;

    if(fCovar)
       fCovar->operator~();
    else
       fprintf(stderr,"fCovar == NULL");
}

void TPhTrackTemplate::PrintParameters(Int_t plane)
{
    Int_t row;

    row = fLayerToRow[plane];
    if(row > -1)
    {
         printf("Layer %d - row %d: x,y,z,R,D,E = %g %g %g %g %g %g\n",
                plane,
                row,
                fMeanX0[row],
                fMeanY[row],
                fMeanZ0[row],
                fMeanR[row],
                fMeanDist[row],
                fMeanDedx[row]);
    }
    else
    {
        printf("No parameters for plane %d\n",plane);
    }
}

Double_t TPhTrackTemplate::GetDistXZ(TPhHit *hit)
{
// Get distance between hit and template in XZ plane

    Double_t dx;
    Double_t dz;
    Double_t norm;
    Int_t   row;

    row = fLayerToRow[hit->GetLayer()];
    if(row > -1)
    {
         return GetDistXZ(hit,row);
    }
    else
    {
        return 0;
    }
}

Double_t TPhTrackTemplate::GetDistXZ(TPhHit *hit,Int_t row)
{
// Get distance between hit and template in XZ plane

    Double_t dx;
    Double_t dz;
    Double_t norm;

    if(fMeanR[row])
    {
        dx = hit->GetX() - fMeanX0[row];
        dz = hit->GetZ() - fMeanZ0[row];
//            printf("%d:%g: dx,dz,r = %g %g %g\n",row,fMeanR[row],dx,dz,sqrt(dx*dx+dz*dz));
        return (sqrt(dx*dx+dz*dz) - fabs(fMeanR[row]));
    }
    else
    {
        dx   = hit->GetX() - fMeanX0[row];
        dz   = hit->GetZ() - fMeanZ0[row];
        if(dz>0)
            return sqrt(dx*dx+dz*dz);
        else
            return -sqrt(dx*dx+dz*dz);
    }
}

Double_t TPhTrackTemplate::GetDistY(TPhHit *hit)
{
// Get distance between hit and template in ZY plane

    Int_t   row;

    row = fLayerToRow[hit->GetLayer()];
    if(row > -1)
    {
        return fMeanY[row] - hit->GetY();
    }
    else
    {
        return 0;
    }
}

Bool_t TPhTrackTemplate::IsCompatible(TPhHit *hit)
{

    Double_t x1;
    Double_t x2;
    Int_t   row;

    row  = fLayerToRow[hit->GetLayer()];
    if(row == -1)
    {
        if(fVerbose > 1)
            printf("Can't translate layer %d to row\n",hit->GetLayer());
        return kFALSE;
    }
    else
    {
        x1 = fabs(GetDistXZ(hit,row) / fMeanDist[row]);
        if(fVerbose > 1)
        {
            printf("Check hit %g %g %g at %d: %g %g %g - %g %g %g %g\n",
                       hit->GetX(),
                       hit->GetY(),
                       hit->GetZ(),
                       row,
                       fMeanR[row],
                       fMeanX0[row],
                       fMeanZ0[row],
                       GetDistXZ(hit,row),
                       fMeanDist[row],
                       x1, fMaxCompDist);
        }
//        x2 = hit->GetDe()/fMeanDedx[row];
//        if(x1 < fMaxCompDist && x2 > fMinCompEnergy)
        if(x1 < fMaxCompDist)
            return kTRUE;
        else
            return kFALSE;
    }

}

void TPhTrackTemplate::CalcChi2(TPhTrack *track, Float_t *chi2, Float_t *ndof)
{
    CalcChi2(track, chi2, ndof, kTRUE);
}

void TPhTrackTemplate::CalcChi2(TPhTrack *track, Float_t *chi2, Float_t *ndof, Bool_t UseCovar)
{
// Calculate chi**2 between track and template. If UseCovar is
// true, full (inverted) covariance matrix is used.


    Int_t    i;
    Int_t    n;
    Int_t    row;
    Int_t    plane;
    Double_t  dist;
    Double_t  x;
    Double_t  d[30];
    TPhHit  *hit;
    static int first_call = kTRUE;
    FILE *fp;

    *chi2 = 0; 
    *ndof = 0; 

    for(i=0;i<30;i++)
        d[i] = 1;
    if(UseCovar)
    {
        TIter NextHit(track->GetHitList());
        while(hit = (TPhHit *) NextHit())
        {
            plane    = hit->GetLayer();
            row      = fLayerToRow[plane];
            if(row > -1)
            {
               d[row] = GetDistXZ(hit)/fMeanDist[row];
            }
        }
        if(fVerbose)
        {
            if(track->GetNHits() < fNRows)
            {
                for(i=0;i<fNRows;i++)
                {
                    if(d[i] < 0)
                        printf("%.3f ",d[i]);
                    else if(d[i] == 0)
                        printf("      ",d[i]);
                    else
                        printf(" %.3f ",d[i]);
                }
                printf("\n");
            }
        }
       
        CalcChi2(&d[0], chi2, ndof);
    }
    else
    {
        if(fVerbose)
            printf("%d points on track\n",track->GetNHits());
        TIter NextHit(track->GetHitList());
        while(hit = (TPhHit *) NextHit())
        {
            row = fLayerToRow[hit->GetLayer()];
            if(row > -1)
            {
                dist = GetDistXZ(hit)/fMeanDist[row];
                if(dist != 0)
                {
                    *chi2 += dist*dist;
                    *ndof++;
                }
            }
        }
        if(fVerbose)
            printf("chi2, ndof = %g %g\n",*chi2,*ndof);
    }
    if(fVerbose)
        printf("chi2, ndof = %g %g\n",*chi2,*ndof);
}

Bool_t TPhTrackTemplate::ReadCovar(Char_t *FileName) 
{
// Read covariance matrix from *FileName in ASCII format

    Int_t    i;
    Int_t    j;
    Int_t    NRows;
    Double_t x;
    FILE *fp;
    Double_t *data;

    fp = fopen(FileName,"r");
    if(fp)
    {
        fscanf(fp,"%d\n",&NRows);
        fCovar = new matrix(NRows,NRows);
        data = fCovar->matval();
        for(i=0;i<NRows*NRows;i++)
        {
            fscanf(fp,"%f",&x);
            data[i] = x;
        }
        for(i=0;i<NRows;i++)
        {
            fscanf(fp,"%f",&x);
            fMeanDist[i] = x;
        }
        fclose(fp);
        PrintCovar();
        return kTRUE;
    }
    else
    {
        printf("Can't open %s\n",FileName);
        return kFALSE;
    }
}

Bool_t TPhTrackTemplate::WriteCovar(Char_t *FileName) 
{
// Write covariance matrix to *FileName in ASCII format
    Int_t    i;
    Int_t    j;
    Double_t x;
    FILE *fp;

    fp = fopen(FileName,"w");
    if(fp)
    {
        fprintf(fp,"%d\n",fCovar->rowlen());
        for(i=0;i<fCovar->rowlen();i++)
        {
            for(j=0;j<fCovar->rowlen();j++)
            {
                x = fCovar->peek(i,j);
                fprintf(fp,"%f ",x);
            }
            fprintf(fp,"\n");
        }
        for(i=0;i<fNRows;i++)
        {
            fprintf(fp,"%g ",fMeanDist[i]);
        }
        fclose(fp);
        return kTRUE;
    }
    else
    {
        printf("Can't open %s\n",FileName);
        return kFALSE;
    }
}

void TPhTrackTemplate::CalcChi2(Double_t *vec, Float_t *chi2, Float_t *ndof)
{
    Int_t    i;
    Int_t    j;
    Double_t x;
    Double_t dchi2=0.0;

    if(vec[0] == 0)
       vec[0] = vec[1];
    if(vec[fNRows-1]==0)
       vec[fNRows-1] = vec[fNRows-2];
    for(i=1;i<fNRows-1;i++)
    {
        if(vec[i]==0)
        {
            vec[i] = (vec[i-1]+vec[i+1])/2;
        }
    }
    for(i=0;i<fNRows;i++)
    {
        for(j=0;j<fNRows;j++)
        {
            dchi2 += fCovar->peek(i,j) * vec[i] * vec[j];
        }
    }
    *ndof = fNRows;
    *chi2 = dchi2;
}

Double_t TPhTrackTemplate::GetMeanDist(Int_t layer)
{
    Int_t row;

    if(layer >= 0 && layer < fNLayers)
    {
         row = fLayerToRow[layer];
         if(row != -1)
         {
             return fMeanDist[row];
         }
         else
         {
             return 0;
         }
    } 
    else
    {
       return 0;
    }
}
Double_t TPhTrackTemplate::GetMeanLength(Int_t layer)
{
    Int_t row;

    if(layer >= 0 && layer < fNLayers)
    {
         row = fLayerToRow[layer];
         if(row != -1)
         {
             return fMeanLength[row];
         }
         else
         {
             return 0;
         }
    } 
    else
    {
       return 0;
    }
}
Double_t TPhTrackTemplate::GetMeanDedx(Int_t layer)
{
    Int_t row;

    if(layer >= 0 && layer < fNLayers)
    {
         row = fLayerToRow[layer];
         if(row != -1)
         {
             return fMeanDedx[row];
         }
         else
         {
             return 0;
         }
    } 
    else
    {
       return 0;
    }
}
Double_t TPhTrackTemplate::GetMeanR(Int_t layer)
{
    Int_t row;

    if(layer >= 0 && layer < fNLayers)
    {
         row = fLayerToRow[layer];
         if(row != -1)
         {
             return fMeanR[row];
         }
         else
         {
             return 0;
         }
    } 
    else
    {
       return 0;
    }
}
Double_t TPhTrackTemplate::GetMeanX0(Int_t layer)
{
    Int_t row;

    if(layer >= 0 && layer < fNLayers)
    {
         row = fLayerToRow[layer];
         if(row != -1)
         {
             return fMeanX0[row];
         }
         else
         {
             return 0;
         }
    } 
    else
    {
       return 0;
    }
}
Double_t TPhTrackTemplate::GetMeanY(Int_t layer)
{
    Int_t row;

    if(layer >= 0 && layer < fNLayers)
    {
         row = fLayerToRow[layer];
         if(row != -1)
         {
             return fMeanY[row];
         }
         else
         {
             return 0;
         }
    } 
    else
    {
       return 0;
    }
}
Double_t TPhTrackTemplate::GetMeanZ0(Int_t layer)
{
    Int_t row;

    if(layer >= 0 && layer < fNLayers)
    {
         row = fLayerToRow[layer];
         if(row != -1)
         {
             return fMeanZ0[row];
         }
         else
         {
             return 0;
         }
    } 
    else
    {
       return 0;
    }
}
Bool_t TPhTrackTemplate::CloneTemplate(TPhTrackTemplate *tmpl)
{

    Int_t i;
    Int_t row;

    if(CompatibleTemplate(tmpl))
    {
        for(i=0;i<fNLayers;i++)
        {
            row = fLayerToRow[i];
            if(row >=0)
            {
                fMeanDist[row] = tmpl->GetMeanDist(i);
                fMeanDedx[row] = tmpl->GetMeanDedx(i);
            }
        }
        SetCovar(tmpl->GetCovar());
        return kTRUE;
    }
    else
    {
        printf("TPhTrackTemplate::CloneTemplate(): GetNRows() doesn't match: %d %d\n",
                    tmpl->GetNRows(),GetNRows());
        return kFALSE;
    }
}

Bool_t TPhTrackTemplate::CompatibleTemplate(TPhTrackTemplate *tmpl)
{
    Int_t i;
    Int_t row;

    if(!tmpl)   return kFALSE;
    if(tmpl->GetNRows() != GetNRows())   return  kFALSE;

    for(i=0; i < fNLayers; i++)
    {
        row = fLayerToRow[i];
        if(row >= 0)
        {
            if(tmpl->GetMeanLength(i) == 0)  return  kFALSE;
        }
    }
    return kTRUE;
}

Bool_t TPhTrackTemplate::CompatibleTrack(TPhTrack *track)
{
    Int_t plane;
    TPhHit *hit;
    TObjArray *hitlist = track->GetHitList();

    if(GetNRows() > hitlist->GetEntries())   return kFALSE;

    TIter NextHit(hitlist);
    while(hit = (TPhHit *) NextHit())
    {   plane = hit->GetLayer();
        if(fLayerToRow[plane] < 0)   return kFALSE;
    }
    return kTRUE;
}

Bool_t TPhTrackTemplate::NeighbourPoint(Int_t i, Int_t j, Int_t d, Int_t n, Int_t *i1, Int_t *j1)
{
// fuction that enables loop over points arround a point (i,j) in a table.
// Calculates n-th point (i1,j1) at the distance  d  from the center point (i,j)
// end of loop signaled by kFALSE result and by negative indexes i1, j1 
// only when n excedes the number of points at distance  d.  
// Checking of the indexes has be done in the routine calling this function
// (both for too small (negative) and too large indexes)

   *i1 = -1;
   *j1 = -1;
   if((n < 0) || (n >= 8*d))  return kFALSE;

   *i1 = i + d;
   *j1 = j + d - n;
   if(n > 2*d)
   {  *i1 = i + d - (n-2*d);
      *j1 = j - d;
   }
   if(n > 4*d)
   {  *i1 = i - d;
      *j1 = j - d + (n-4*d);
   }
   if(n > 6*d)
   {  *i1 = i - d + (n-6*d);
      *j1 = j + d;
   }
   return kTRUE;
}
