#include <math.h>
#include "stdlib.h"
#include "TPhEvent.h"
#include "TPhRandomEvent.h"
#include "TPhObjectContainer.h"

//*-*-* ROOT includes

#include "TROOT.h"
#include "TMath.h"
#include "TRandom.h"
#include "PhysConst.h"

#include "TPhEvent.h"
#include "TPhTrack.h"


ClassImp(TPhRandomEvent)


///////////////////////////////////////////////////////////////
//
// TPhRandomEvent is a mickey mouse event generator that 
// generates TPhEvent objects including TPhTracks with 
// momentum distributions that mimick those expected at RHIC
// with gaussian rapidity distributions and exponential 
// transverse mass distributions. The events can be written to
// disk and used as input for the TPhSwim mickey mouse GEANT.
//
// Note that the maximum rapidity gap for which this class can
// be used is ~14 - for larger momenta in the lab the storage
// of the momentum components has to be changed to Double_t 
// variables to avoid numerical problems.
//
// $Id: TPhRandomEvent.cxx,v 1.2 1999/03/12 07:10:40 decowski Exp $
//
///////////////////////////////////////////////////////////////


TPhRandomEvent::TPhRandomEvent()
{
// Constructor. Initialize parameters for phase-space 
// distribution of tracks.

    fEventCounter  = 0;
    fTemperature   = 0.2;
    fMult          = 1000;
    fMultWidth     = 100;
    fMeanRap       = 0.0;
    fSigmaRap      = 3.0;
    fK2Pi          = 0.1;
    fP2Pi          = 0.1;
}


TPhEvent *TPhRandomEvent::GetNextEvent()
{

// Create one event with random track population according 
// to the phase-space parameters for TPhRandomEvent. Fills 
// the track list for the event and returns a pointer to 
// an TPhEvent object.

    Int_t i;
    Float_t mult;
    Float_t mass;
    Char_t name[256];

    TPhTrack *track;
    TPhObjectContainer *TrackBuffer = new TPhObjectContainer();

    TrackBuffer->SetName("MCTracks");
    TPhEvent *event = new TPhEvent(0, fEventCounter); 

    fEventCounter++;

    mult = gRandom->Gaus(fMult,fMultWidth);
    if(mult < 0) 
        mult = 0;

    for(i=0;i<mult;i++)
    {
        mass = kPIMASS;
        track = GetNextTrack(mass);
        TrackBuffer->Add(track);
    }

    sprintf(name,"event_%d_%d",event->GetNRun(),event->GetNEvent());
    event->SetName(name);
    sprintf(name,"Phobos Event %d %d",event->GetNRun(),event->GetNEvent());
    event->SetTitle(name);
    event->AddObject(TrackBuffer);
    return event;
}

TPhTrack *TPhRandomEvent::GetNextTrack(Float_t mass)
{
// Create a single TPhTrack object according to the 
// TPhEvent data members fTemperature, fMeanRap etc.


    Float_t rap;
    Float_t konst;
    Float_t mtp;  
    Float_t px,py,pz;
    Float_t pt;
    Float_t phi;
    Float_t mt;
    Float_t therm_mt;
    Float_t therm_montevar;

    TPhTrack *track = new TPhTrack(); 


    konst = 1./(mass*exp(-mass/fTemperature));

    mtp = 0.0;
    while(mtp == 0.0)
    {
        mt = gRandom->Rndm()*2.;
        therm_mt = konst*mt*exp(-(double) mt/(double) fTemperature);
        therm_montevar = gRandom->Rndm();
        if ((therm_montevar <= therm_mt) && (mt > mass) )
        {
               mtp = mt;
        }
    }
    while((rap = fabs(gRandom->Gaus(fMeanRap,fSigmaRap)))>7);
    pt = sqrt(mt*mt - mass*mass);
    phi = (gRandom->Rndm() - 0.5) * 2.0 * TMath::Pi();
    pz = (mt*exp(2.*rap) - mt) / (2. * exp(rap));
    if(gRandom->Rndm() < 0.5)
       pz = -pz;
    px = cos(phi)*pt;
    py = sin(phi)*pt;

    track->SetPx(px);
    track->SetPy(py);
    track->SetPz(pz);

    track->SetXProd(0);
    track->SetYProd(0);
    track->SetZProd(0);
    track->SetMass(mass);
    if(gRandom->Rndm() > 0.5)
       track->SetCharge(-1);
    else
       track->SetCharge(1);

    return track;
}
